@@ -68,7 +68,7 @@ namespace Glest { namespace Game {
 string getGameReadWritePath(string lookupKey) {
 	string path = "";
     if(path == "" && getenv("GLESTHOME") != NULL) {
-        path = getenv("GLESTHOME");
+        path = safeCharPtrCopy(getenv("GLESTHOME"),8096);
         if(path != "" && EndsWith(path, "/") == false && EndsWith(path, "\\") == false) {
             path += "/";
         }
@@ -128,7 +128,7 @@ enum GAME_ARG_TYPE {
 	GAME_ARG_VERBOSE,
 };
 
-bool hasCommandArgument(int argc, wxChar** argv,const string argName,
+bool hasCommandArgument(int argc, wxChar** argv,const string &argName,
 						int *foundIndex=NULL, int startLookupIndex=1,
 						bool useArgParamLen=false) {
 	bool result = false;
@@ -285,16 +285,21 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
     :	wxFrame(NULL, -1, ToUnicode(winHeader),
     	        wxPoint(Renderer::windowX, Renderer::windowY),
 		        wxSize(Renderer::windowW, Renderer::windowH)),
-		        model(NULL), glCanvas(NULL), renderer(NULL),
-		        initTextureManager(true), timer(NULL),
-		        startupSettingsInited(false),
-		        fileDialog(NULL),colorPicker(NULL)
+		        glCanvas(NULL),
+		        renderer(NULL),
+		        timer(NULL),
+		        menu(NULL),
+		        fileDialog(NULL),
+		        colorPicker(NULL),
+		        model(NULL),
+		        initTextureManager(true),
+		        startupSettingsInited(false)
+
 {
 	this->appPath = appPath;
 	Properties::setApplicationPath(executable_path(appPath));
 
 	Config &config = Config::getInstance();
-	string iniFilePath = extractDirectoryPathFromFile(config.getFileName(false));
     //getGlPlatformExtensions();
 
 	isControlKeyPressed = false;
@@ -303,7 +308,7 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 	glCanvas = new GlCanvas(this, args);
 
 #if wxCHECK_VERSION(2, 9, 1)
-	//glCanvas->setCurrentGLContext();
+
 #else
 	glCanvas->SetCurrent();
 #endif
@@ -410,6 +415,7 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 
 #else
 	wxIcon icon;
+	string iniFilePath = extractDirectoryPathFromFile(config.getFileName(false));
 	string icon_file = iniFilePath + "g3dviewer.ico";
 	std::ifstream testFile(icon_file.c_str());
 	if(testFile.good())	{
@@ -423,7 +429,7 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 	if(modelPath != "") {
 		fileDialog->SetPath(ToUnicode(modelPath));
 	}
-    string userData = Config::getInstance().getString("UserData_Root","");
+    string userData = config.getString("UserData_Root","");
     if(userData != "") {
     	endPathWithSlash(userData);
     }
@@ -474,6 +480,13 @@ void MainWindow::setupTimer() {
 }
 
 void MainWindow::setupStartupSettings() {
+	 GLuint err = glewInit();
+	 if (GLEW_OK != err) {
+		fprintf(stderr, "Error [main]: glewInit failed: %s\n", glewGetErrorString(err));
+		//return 1;
+		throw std::runtime_error((char *)glewGetErrorString(err));
+	 }
+
 	renderer= Renderer::getInstance();
 
 	for(unsigned int i = 0; i < autoScreenShotParams.size(); ++i) {
@@ -555,19 +568,14 @@ void MainWindow::init() {
 
 #if wxCHECK_VERSION(2, 9, 3)
 	glCanvas->setCurrentGLContext();
+	//printf("setcurrent #1\n");
 #elif wxCHECK_VERSION(2, 9, 1)
 
 #else
 	glCanvas->SetCurrent();
+	//printf("setcurrent #2\n");
 #endif
 
-	 GLuint err = glewInit();
-	 if (GLEW_OK != err) {
-		fprintf(stderr, "Error [main]: glewInit failed: %s\n", glewGetErrorString(err));
-		//return 1;
-		throw std::runtime_error((char *)glewGetErrorString(err));
-	 }
-
 	//renderer->init();
 
 	//wxCommandEvent event;
@@ -577,7 +585,9 @@ void MainWindow::init() {
 void MainWindow::onPaint(wxPaintEvent &event) {
 	if(!IsShown()) return;
 	
-#if wxCHECK_VERSION(2, 9, 1)
+#if wxCHECK_VERSION(2, 9, 3)
+
+#elif wxCHECK_VERSION(2, 9, 1)
 	glCanvas->setCurrentGLContext();
 #endif
 
@@ -595,12 +605,32 @@ void MainWindow::onPaint(wxPaintEvent &event) {
 //		dc.DestroyClippingRegion();
 //	}
 
+	static float autoScreenshotRender = -1;
+	if(autoScreenShotAndExit == true && autoScreenshotRender >= 0) {
+		anim = autoScreenshotRender;
+	}
 	// notice that we use GetSize() here and not GetClientSize() because
 	// the latter doesn't return correct results for the minimized windows
 	// (at least not under Windows)
 	int viewportW = GetClientSize().x;
 	int viewportH = GetClientSize().y;
 
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("%d x %d\n",viewportW,viewportH);
+
+	if(viewportW == 0 && viewportH == 0) {
+		printf("#1 %d x %d\n",viewportW,viewportH);
+
+		viewportW = GetSize().x;
+		viewportH = GetSize().y;
+
+		if(viewportH > 0) {
+			//viewportH -= menu->GetSize().y;
+			viewportH -= 22;
+		}
+
+		printf("#2 %d x %d\n",viewportW,viewportH);
+	}
+
 #if defined(WIN32)
 	renderer->reset(viewportW, viewportH, playerColor);
 #else
@@ -632,11 +662,12 @@ void MainWindow::onPaint(wxPaintEvent &event) {
 	
 	bool haveLoadedParticles = (particleProjectilePathList.empty() == false || particleSplashPathList.empty() == false);
 
-	if(autoScreenShotAndExit == true) {
+	if(autoScreenShotAndExit == true && viewportW > 0 && viewportH > 0) {
 		printf("Auto exiting app...\n");
 		fflush(stdout);
 
 		autoScreenShotAndExit = false;
+
 		saveScreenshot();
 		Close();
 		return;
@@ -644,6 +675,13 @@ void MainWindow::onPaint(wxPaintEvent &event) {
 	
 	glCanvas->SwapBuffers();
 
+	if(autoScreenShotAndExit == true && viewportW == 0 && viewportH == 0) {
+		autoScreenshotRender = anim;
+
+		printf("Auto exiting desired but waiting for w x h > 0...\n");
+
+		return;
+	}
 	if((modelPathList.empty() == false) && resetAnimation && haveLoadedParticles) {
 		if(anim >= resetAnim && resetAnim > 0) {
 			printf("RESETTING EVERYTHING [%f][%f]...\n",anim,resetAnim);
@@ -728,7 +766,6 @@ void MainWindow::onClose(wxCloseEvent &event){
 	if(glCanvas) glCanvas->Destroy();
 	glCanvas = NULL;
 
-	//delete this;
 	this->Destroy();
 }
 
@@ -1140,7 +1177,7 @@ void MainWindow::loadUnit(string path, string skillName) {
 				const XmlNode *skillsNode= unitNode->getChild("skills");
 				for(unsigned int i = 0; foundSkillName == false && i < skillsNode->getChildCount(); ++i) {
 					const XmlNode *sn= skillsNode->getChild("skill", i);
-					const XmlNode *typeNode= sn->getChild("type");
+					//const XmlNode *typeNode= sn->getChild("type");
 					const XmlNode *nameNode= sn->getChild("name");
 					string skillXmlName = nameNode->getAttribute("value")->getRestrictedValue();
 					if(skillXmlName == lookipForSkillName) {
@@ -1243,9 +1280,7 @@ void MainWindow::loadModel(string path) {
 
             if(timer) timer->Stop();
             delete model;
-            Model *tmpModel= new ModelGl();
-            if(renderer) renderer->loadTheModel(tmpModel, modelPath);
-            model= tmpModel;
+            model = renderer? renderer->newModel(rsGlobal, modelPath): NULL;
 
             statusbarText = getModelInfo();
             string statusTextValue = statusbarText + " animation speed: " + floatToStr(speed * 1000.0) + " anim value: " + floatToStr(anim) + " zoom: " + floatToStr(zoom) + " rotX: " + floatToStr(rotX) + " rotY: " + floatToStr(rotY);
@@ -1449,7 +1484,7 @@ void MainWindow::loadProjectileParticle(string path) {
 			for(std::vector<ParticleSystemTypeProjectile *>::const_iterator it= projectileParticleSystemTypes.begin();
 					it != projectileParticleSystemTypes.end(); ++it) {
 
-				ProjectileParticleSystem *ps = (*it)->create();
+				ProjectileParticleSystem *ps = (*it)->create(NULL);
 
 				if(size > 0) {
 					Vec3f vec = Vec3f(0.f, height / 2.f, 0.f);
@@ -1524,7 +1559,7 @@ void MainWindow::loadSplashParticle(string path) {  // uses ParticleSystemTypeSp
 			std::string unitXML = dir + folderDelimiter + extractFileFromDirectoryPath(dir) + ".xml";
 
 			int size   = 1;
-			int height = 1;
+			//int height = 1;
 
 			if(fileExists(unitXML) == true) {
 				XmlTree xmlTree;
@@ -1534,7 +1569,7 @@ void MainWindow::loadSplashParticle(string path) {  // uses ParticleSystemTypeSp
 				//size
 				size= parametersNode->getChild("size")->getAttribute("value")->getIntValue();
 				//height
-				height= parametersNode->getChild("height")->getAttribute("value")->getIntValue();
+				//height= parametersNode->getChild("height")->getAttribute("value")->getIntValue();
 			}
 
 			// std::cout << "About to load [" << particlePath << "] from [" << dir << "] unit [" << unitXML << "]" << std::endl;
@@ -1558,10 +1593,10 @@ void MainWindow::loadSplashParticle(string path) {  // uses ParticleSystemTypeSp
                                       //ParticleSystemTypeSplash
 			for(std::vector<ParticleSystemTypeSplash *>::const_iterator it= splashParticleSystemTypes.begin(); it != splashParticleSystemTypes.end(); ++it) {
 
-				SplashParticleSystem *ps = (*it)->create();
+				SplashParticleSystem *ps = (*it)->create(NULL);
 
 				if(size > 0) {
-					Vec3f vec = Vec3f(0.f, height / 2.f, 0.f);
+					//Vec3f vec = Vec3f(0.f, height / 2.f, 0.f);
 					//ps->setPos(vec);
 
 					//Vec3f vec2 = Vec3f(size * 2.f, height * 2.f, height * 2.f);   // <------- removed relative projectile
@@ -1983,7 +2018,7 @@ void translateCoords(wxWindow *wnd, int &x, int &y) {
 GlCanvas::GlCanvas(MainWindow *	mainWindow, int *args)
 #if wxCHECK_VERSION(2, 9, 1)
 		: wxGLCanvas(mainWindow, -1, args, wxDefaultPosition, wxDefaultSize, 0, wxT("GLCanvas")) {
-	this->context = new wxGLContext(this);
+	this->context = NULL;
 #else
 		: wxGLCanvas(mainWindow, -1, wxDefaultPosition, wxDefaultSize, 0, wxT("GLCanvas"), args) {
 	this->context = NULL;
@@ -1998,8 +2033,15 @@ GlCanvas::~GlCanvas() {
 
 void GlCanvas::setCurrentGLContext() {
 #ifndef __APPLE__
+
+#if wxCHECK_VERSION(2, 9, 1)
+	if(this->context == NULL) {
+		this->context = new wxGLContext(this);
+	}
+#endif
+	//printf("Set ctx [%p]\n",this->context);
 	if(this->context) {
-		this->SetCurrent(*this->context);
+		wxGLCanvas::SetCurrent(*this->context);
 	}
 #else
 	this->SetCurrent();
@@ -2039,13 +2081,11 @@ END_EVENT_TABLE()
 // ===============================================
 
 bool App::OnInit() {
-	//InterpolationData::setEnableCache(false);
-
 	SystemFlags::VERBOSE_MODE_ENABLED  = false;
-	//Renderer::windowW = 1920;
-	//Renderer::windowH = 1440;
-	//Renderer::windowX= 0;
-	//Renderer::windowY= 0;
+
+#if defined(wxMAJOR_VERSION) && defined(wxMINOR_VERSION) && defined(wxRELEASE_NUMBER) && defined(wxSUBRELEASE_NUMBER)
+	printf("Using wxWidgets version [%d.%d.%d.%d]\n",wxMAJOR_VERSION,wxMINOR_VERSION,wxRELEASE_NUMBER,wxSUBRELEASE_NUMBER);
+#endif
 
 	string modelPath="";
 	string particlePath="";
@@ -2447,8 +2487,6 @@ bool App::OnInit() {
 
 	}
 
-	string appPath = "";
-
 //#if defined(__MINGW32__)
 //		const wxWX2MBbuf tmp_buf = wxConvCurrent->cWX2MB(wxFNCONV(argv[0]));
 //		appPath = tmp_buf;
@@ -2465,12 +2503,12 @@ bool App::OnInit() {
 //#if defined(__MINGW32__)
 #ifdef WIN32
 	const wxWX2MBbuf tmp_buf = wxConvCurrent->cWX2MB(wxFNCONV(exe_path));
-	appPath = tmp_buf;
+	string appPath = tmp_buf;
 
 	std::auto_ptr<wchar_t> wstr(Ansi2WideString(appPath.c_str()));
 	appPath = utf8_encode(wstr.get());
 #else
-	appPath = wxFNCONV(exe_path);
+	string appPath(wxFNCONV(exe_path));
 #endif
 
 //#else
