@@ -14,7 +14,7 @@
 #include "graphics_factory_gl.h"
 #include "graphics_interface.h"
 #include "config.h"
-//#include <png.h>
+#include "platform_common.h"
 #include <memory>
 
 using namespace Shared::Graphics;
@@ -196,9 +196,8 @@ Texture2D * Renderer::getNewTexture2D() {
 	return newTexture;
 }
 
-Model * Renderer::getNewModel() {
-	Model *newModel = modelManager->newModel();
-	return newModel;
+Model * Renderer::newModel(ResourceScope rs,const string &path,bool deletePixMapAfterLoad,std::map<string,vector<pair<string, string> > > *loadedFileList, string *sourceLoader) {
+    return modelManager->newModel(path,deletePixMapAfterLoad,loadedFileList,sourceLoader);
 }
 
 void Renderer::init() {
@@ -328,7 +327,7 @@ void Renderer::reset(int w, int h, PlayerColor playerColor) {
 	glLoadIdentity();
 	glTranslatef(0, -1.5, -5);
 
-	Texture2D *customTexture;
+	Texture2D *customTexture=NULL;
 	switch(playerColor) {
 	case pcRed:
 		customTexture= customTextureRed;
@@ -416,16 +415,9 @@ void Renderer::toggleGrid() {
 	grid= grid? false: true;
 }
 
-void Renderer::loadTheModel(Model *model, string file) {
-	model->setTextureManager(textureManager);
-	model->loadG3d(file);
-	textureManager->init();
-	modelManager->init();
-}
-
 void Renderer::renderTheModel(Model *model, float f) {
 	if(model != NULL){
-		modelRenderer->begin(true, true, !wireframe, &meshCallbackTeamColor);
+		modelRenderer->begin(true, true, !wireframe, false, &meshCallbackTeamColor);
 		model->updateInterpolationData(f, true);
 		modelRenderer->render(model);
 
@@ -462,7 +454,7 @@ void Renderer::renderParticleManager() {
 }
 
 Texture2D * Renderer::getPlayerColorTexture(PlayerColor playerColor) {
-	Texture2D *customTexture;
+	Texture2D *customTexture=NULL;
 	switch(playerColor){
 	case pcRed:
 		customTexture= customTextureRed;
@@ -489,7 +481,7 @@ Texture2D * Renderer::getPlayerColorTexture(PlayerColor playerColor) {
 		customTexture= customTextureMagenta;
 		break;
 	default:
-		assert(false);
+		throw megaglest_runtime_error("Unknown playercolor: " + intToStr(playerColor));
 		break;
 	}
 
