@@ -66,7 +66,7 @@ ProduceTask::ProduceTask(const ResourceType *resourceType) : Task() {
 string ProduceTask::toString() const{
 	string str= "Produce ";
 	if(unitType!=NULL){
-		str+= unitType->getName(true);
+		str+= unitType->getName(false);
 	}
 	return str;
 }
@@ -81,11 +81,11 @@ void ProduceTask::saveGame(XmlNode *rootNode) const {
 	produceTaskNode->addAttribute("unitClass",intToStr(unitClass), mapTagReplacements);
 //	const UnitType *unitType;
 	if(unitType != NULL) {
-		produceTaskNode->addAttribute("unitType",unitType->getName(), mapTagReplacements);
+		produceTaskNode->addAttribute("unitType",unitType->getName(false), mapTagReplacements);
 	}
 //	const ResourceType *resourceType;
 	if(resourceType != NULL) {
-		produceTaskNode->addAttribute("resourceType",resourceType->getName(), mapTagReplacements);
+		produceTaskNode->addAttribute("resourceType",resourceType->getName(false), mapTagReplacements);
 	}
 }
 
@@ -144,7 +144,7 @@ BuildTask::BuildTask(const UnitType *unitType, const Vec2i &pos){
 string BuildTask::toString() const{
 	string str= "Build ";
 	if(unitType!=NULL){
-		str+= unitType->getName(true);
+		str+= unitType->getName(false);
 	}
 	return str;
 }
@@ -157,7 +157,7 @@ void BuildTask::saveGame(XmlNode *rootNode) const {
 
 //	const UnitType *unitType;
 	if(unitType != NULL) {
-		buildTaskNode->addAttribute("unitType",unitType->getName(), mapTagReplacements);
+		buildTaskNode->addAttribute("unitType",unitType->getName(false), mapTagReplacements);
 	}
 //	const ResourceType *resourceType;
 	if(resourceType != NULL) {
@@ -284,8 +284,8 @@ void Ai::init(AiInterface *aiInterface, int useStartLocation) {
 		minWorkerAttackersHarvesting = faction->getAIBehaviorStaticOverideValue(aibsvcMinWorkerAttackersHarvesting);
 		//printf("Discovered overriden static value for AI, scoutResourceRange = %d\n",scoutResourceRange);
 	}
-	if(faction->getAIBehaviorStaticOverideValue(aibsvcMinWorkerAttackersHarvesting) != INT_MAX) {
-		minBuildSpacing = faction->getAIBehaviorStaticOverideValue(aibsvcMinWorkerAttackersHarvesting);
+	if(faction->getAIBehaviorStaticOverideValue(aibsvcMinBuildSpacing) != INT_MAX) {
+		minBuildSpacing = faction->getAIBehaviorStaticOverideValue(aibsvcMinBuildSpacing);
 		//printf("Discovered overriden static value for AI, scoutResourceRange = %d\n",scoutResourceRange);
 	}
 
@@ -355,17 +355,24 @@ void Ai::update() {
 		voteResult->allowSwitchTeam = false;
 
 		const GameSettings *settings =  aiInterface->getWorld()->getGameSettings();
-		// Can only ask the AI player 2 times max per game
-		if(factionSwitchTeamRequestCountCurrent <= 2) {
-			// x% chance the AI will answer yes
-			if(settings->getAiAcceptSwitchTeamPercentChance() >= 100) {
-				voteResult->allowSwitchTeam = true;
-			}
-			else if(settings->getAiAcceptSwitchTeamPercentChance() <= 0) {
-				voteResult->allowSwitchTeam = false;
-			}
-			else {
-				voteResult->allowSwitchTeam = (allowJoinTeam >= (100 - settings->getAiAcceptSwitchTeamPercentChance()));
+
+		// If AI player already lost game they cannot vote
+		if(aiInterface->getWorld()->factionLostGame(aiInterface->getFactionIndex()) == true) {
+			voteResult->allowSwitchTeam = true;
+		}
+		else {
+			// Can only ask the AI player 2 times max per game
+			if(factionSwitchTeamRequestCountCurrent <= 2) {
+				// x% chance the AI will answer yes
+				if(settings->getAiAcceptSwitchTeamPercentChance() >= 100) {
+					voteResult->allowSwitchTeam = true;
+				}
+				else if(settings->getAiAcceptSwitchTeamPercentChance() <= 0) {
+					voteResult->allowSwitchTeam = false;
+				}
+				else {
+					voteResult->allowSwitchTeam = (allowJoinTeam >= (100 - settings->getAiAcceptSwitchTeamPercentChance()));
+				}
 			}
 		}
 
@@ -379,7 +386,7 @@ void Ai::update() {
 	}
 
 	//process ai rules
-	for(int ruleIdx = 0; ruleIdx < aiRules.size(); ++ruleIdx) {
+	for(unsigned int ruleIdx = 0; ruleIdx < aiRules.size(); ++ruleIdx) {
 		AiRule *rule = aiRules[ruleIdx];
 		if(rule == NULL) {
 			throw megaglest_runtime_error("rule == NULL");
@@ -472,7 +479,7 @@ const ResourceType *Ai::getNeededResource(int unitIndex) {
 			const int maxUnitsToHarvestResource = 5;
 
 			vector<int> unitsGettingResource = findUnitsHarvestingResourceType(rt);
-			if(unitsGettingResource.size() <= maxUnitsToHarvestResource) {
+			if((int)unitsGettingResource.size() <= maxUnitsToHarvestResource) {
 				// Now MAKE SURE the unit has a harvest command for this resource
 				// AND that the resource is within eye-sight to avoid units
 				// standing around doing nothing.
@@ -487,7 +494,7 @@ const ResourceType *Ai::getNeededResource(int unitIndex) {
     }
 
     char szBuf[8096]="";
-    snprintf(szBuf,8096,"Unit [%d - %s] looking for resources (not static or consumable)",unit->getId(),unit->getType()->getName().c_str());
+    snprintf(szBuf,8096,"Unit [%d - %s] looking for resources (not static or consumable)",unit->getId(),unit->getType()->getName(false).c_str());
     aiInterface->printLog(3, szBuf);
     snprintf(szBuf,8096,"[resource type count %d] Needed resource [%s].",tt->getResourceTypeCount(),(neededResource != NULL ? neededResource->getName().c_str() : "<none>"));
     aiInterface->printLog(3, szBuf);
@@ -535,7 +542,7 @@ bool Ai::findAbleUnit(int *unitIndex, CommandClass ability, bool idleOnly){
 		return false;
 	}
 	else{
-		*unitIndex= units[random.randRange(0, units.size()-1)];
+		*unitIndex= units[random.randRange(0, (int)units.size()-1)];
 		return true;
 	}
 }
@@ -582,7 +589,7 @@ vector<int> Ai::findUnitsHarvestingResourceType(const ResourceType *rt) {
 				Command *command= unit->getCurrCommand();
 			    const BuildCommandType *bct= dynamic_cast<const BuildCommandType*>(command->getCommandType());
 			    if(bct != NULL) {
-			    	for(unsigned int j = 0; j < bct->getBuildingCount(); ++j) {
+			    	for(int j = 0; j < bct->getBuildingCount(); ++j) {
 						const UnitType *ut = bct->getBuilding(j);
 						if(ut != NULL) {
 							const Resource *r = ut->getCost(rt);
@@ -634,7 +641,7 @@ bool Ai::findAbleUnit(int *unitIndex, CommandClass ability, CommandClass current
 		return false;
 	}
 	else{
-		*unitIndex= units[random.randRange(0, units.size()-1)];
+		*unitIndex= units[random.randRange(0, (int)units.size()-1)];
 		return true;
 	}
 }
@@ -710,7 +717,7 @@ void Ai::addExpansion(const Vec2i &pos) {
 	expansionPositions.push_front(pos);
 
 	//remove expansion if queue is list is full
-	if(expansionPositions.size() > maxExpansions){
+	if((int)expansionPositions.size() > maxExpansions){
 		expansionPositions.pop_back();
 	}
 }
@@ -721,12 +728,13 @@ Vec2i Ai::getRandomHomePosition() {
 		return aiInterface->getHomeLocation();
 	}
 
-	return expansionPositions[random.randRange(0, expansionPositions.size()-1)];
+	return expansionPositions[random.randRange(0, (int)expansionPositions.size()-1)];
 }
 
 // ==================== actions ====================
 
 void Ai::sendScoutPatrol(){
+
 	Vec2i pos;
 	int unit;
 	bool possibleTargetFound= false;
@@ -735,7 +743,8 @@ void Ai::sendScoutPatrol(){
 	        && random.randRange(0, 2) == 1;
 	bool megaResourceAttack=(aiInterface->getControlType() == ctCpuMega || aiInterface->getControlType() == ctNetworkCpuMega)
 			&& random.randRange(0, 1) == 1;
-	if( megaResourceAttack || ultraResourceAttack) {
+
+	if(possibleTargetFound == false && (megaResourceAttack || ultraResourceAttack)) {
 		Map *map= aiInterface->getMap();
 
 		const TechTree *tt= aiInterface->getTechTree();
@@ -776,6 +785,23 @@ void Ai::sendScoutPatrol(){
 			//else printf("is outside map\n");
 		}
 	}
+
+	std::vector<Vec2i> warningEnemyList = aiInterface->getEnemyWarningPositionList();
+	if( (possibleTargetFound == false) && (warningEnemyList.empty() == false)) {
+		//for(int i = (int)warningEnemyList.size() - 1; i <= 0; --i) {
+			//Vec2i &checkPos = warningEnemyList[i];
+			Vec2i &checkPos = warningEnemyList[0];
+			if (random.randRange(0, 1) == 1 ) {
+				pos = checkPos;
+				possibleTargetFound = true;
+				warningEnemyList.clear();
+			} else {
+				aiInterface->removeEnemyWarningPositionFromList(checkPos);
+			}
+			//break;
+		//}
+	}
+
 	if(possibleTargetFound == false){
 		startLoc= (startLoc + 1) % aiInterface->getMapMaxPlayers();
 		pos= aiInterface->getStartLocation(startLoc);
@@ -948,7 +974,7 @@ void Ai::massiveAttack(const Vec2i &pos, Field field, bool ultraAttack){
 
 void Ai::returnBase(int unitIndex) {
     Vec2i pos;
-    std::pair<CommandResult,string> r(crFailUndefined,"");
+    //std::pair<CommandResult,string> r(crFailUndefined,"");
     aiInterface->getFactionIndex();
     pos= Vec2i(
 		random.randRange(-villageRadius, villageRadius),
@@ -956,7 +982,8 @@ void Ai::returnBase(int unitIndex) {
 		                 getRandomHomePosition();
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
-    r= aiInterface->giveCommand(unitIndex, ccMove, pos);
+    //r= aiInterface->giveCommand(unitIndex, ccMove, pos);
+    aiInterface->giveCommand(unitIndex, ccMove, pos);
 
     //aiInterface->printLog(1, "Order return to base pos:" + intToStr(pos.x)+", "+intToStr(pos.y)+": "+rrToStr(r)+"\n");
 }
@@ -990,7 +1017,7 @@ bool Ai::haveBlockedUnits() {
 
 		// If this building is a store
 		if(u->isAlive() && ut->isMobile() && u->getPath() != NULL && (u->getPath()->isBlocked() || u->getPath()->getBlockCount())) {
-			Vec2i unitPos = u->getPos();
+			Vec2i unitPos = u->getPosNotThreadSafe();
 
 			//printf("#1 AI found blocked unit [%d - %s]\n",u->getId(),u->getFullName().c_str());
 
@@ -1031,7 +1058,7 @@ bool Ai::getAdjacentUnits(std::map<float, std::map<int, const Unit *> > &signalA
 
 	bool result = false;
 	Map *map = aiInterface->getMap();
-	Vec2i unitPos = unit->getPos();
+	Vec2i unitPos = unit->getPosNotThreadSafe();
 	for(int i = -1; i <= 1; ++i) {
 		for(int j = -1; j <= 1; ++j) {
 			Vec2i pos = unitPos + Vec2i(i, j);
@@ -1083,7 +1110,7 @@ void Ai::unblockUnits() {
 
 		// If this building is a store
 		if(u->isAlive() && ut->isMobile() && u->getPath() != NULL && (u->getPath()->isBlocked() || u->getPath()->getBlockCount())) {
-			Vec2i unitPos = u->getPos();
+			Vec2i unitPos = u->getPosNotThreadSafe();
 
 			//printf("#2 AI found blocked unit [%d - %s]\n",u->getId(),u->getFullName().c_str());
 
@@ -1131,9 +1158,9 @@ void Ai::unblockUnits() {
 						Vec2i pos= Vec2i(
 							random.randRange(-villageRadius*2, villageRadius*2),
 							random.randRange(-villageRadius*2, villageRadius*2)) +
-											 adjacentUnit->getPos();
+											 adjacentUnit->getPosNotThreadSafe();
 
-						bool canUnitMoveToCell = map->aproxCanMove(adjacentUnit, adjacentUnit->getPos(), pos);
+						bool canUnitMoveToCell = map->aproxCanMove(adjacentUnit, adjacentUnit->getPosNotThreadSafe(), pos);
 						if(canUnitMoveToCell == true) {
 
 							if(ct != NULL) {
@@ -1141,7 +1168,8 @@ void Ai::unblockUnits() {
 									unitGroupCommandId = aiInterface->getWorld()->getNextCommandGroupId();
 								}
 
-								std::pair<CommandResult,string> r = aiInterface->giveCommand(adjacentUnit,ct, pos, unitGroupCommandId);
+								//std::pair<CommandResult,string> r = aiInterface->giveCommand(adjacentUnit,ct, pos, unitGroupCommandId);
+								aiInterface->giveCommand(adjacentUnit,ct, pos, unitGroupCommandId);
 							}
 						}
 					}
