@@ -32,9 +32,163 @@ using namespace Shared::Graphics;
 
 namespace Glest{ namespace Game{
 
-AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useStartLocation) : fp(NULL) {
+// =====================================================
+//	class FactionThread
+// =====================================================
+
+AiInterfaceThread::AiInterfaceThread(AiInterface *aiIntf) : BaseThread() {
+	this->masterController = NULL;
+	this->triggerIdMutex = new Mutex(CODE_AT_LINE);
+	this->aiIntf = aiIntf;
+	uniqueID = "AiInterfaceThread";
+}
+
+AiInterfaceThread::~AiInterfaceThread() {
+	delete triggerIdMutex;
+	triggerIdMutex = NULL;
+}
+
+void AiInterfaceThread::setQuitStatus(bool value) {
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d value = %d\n",__FILE__,__FUNCTION__,__LINE__,value);
+
+	BaseThread::setQuitStatus(value);
+	if(value == true) {
+		signal(-1);
+	}
+
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d\n",__FILE__,__FUNCTION__,__LINE__);
+}
+
+void AiInterfaceThread::signal(int frameIndex) {
+	if(frameIndex >= 0) {
+		static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+		MutexSafeWrapper safeMutex(triggerIdMutex,mutexOwnerId);
+		this->frameIndex.first = frameIndex;
+		this->frameIndex.second = false;
+
+		safeMutex.ReleaseLock();
+	}
+	semTaskSignalled.signal();
+}
+
+void AiInterfaceThread::setTaskCompleted(int frameIndex) {
+	if(frameIndex >= 0) {
+		static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+		MutexSafeWrapper safeMutex(triggerIdMutex,mutexOwnerId);
+		if(this->frameIndex.first == frameIndex) {
+			this->frameIndex.second = true;
+		}
+		safeMutex.ReleaseLock();
+	}
+}
+
+bool AiInterfaceThread::canShutdown(bool deleteSelfIfShutdownDelayed) {
+	bool ret = (getExecutingTask() == false);
+	if(ret == false && deleteSelfIfShutdownDelayed == true) {
+	    setDeleteSelfOnExecutionDone(deleteSelfIfShutdownDelayed);
+	    deleteSelfIfRequired();
+	    signalQuit();
+	}
+
+	return ret;
+}
+
+bool AiInterfaceThread::isSignalCompleted(int frameIndex) {
+	if(getRunningStatus() == false) {
+		return true;
+	}
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(triggerIdMutex,mutexOwnerId);
+	//bool result = (event != NULL ? event->eventCompleted : true);
+	bool result = (this->frameIndex.first == frameIndex && this->frameIndex.second == true);
+
+	//if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] worker thread this = %p, this->frameIndex.first = %d, this->frameIndex.second = %d\n",__FILE__,__FUNCTION__,__LINE__,this,this->frameIndex.first,this->frameIndex.second);
+
+	safeMutex.ReleaseLock();
+	return result;
+}
+
+void AiInterfaceThread::signalQuit() {
+	if(this->aiIntf != NULL) {
+		MutexSafeWrapper safeMutex(this->aiIntf->getMutex(),string(__FILE__) + "_" + intToStr(__LINE__));
+		this->aiIntf = NULL;
+	}
+
+	BaseThread::signalQuit();
+}
+void AiInterfaceThread::execute() {
+    RunningStatusSafeWrapper runningStatus(this);
+	try {
+		//setRunningStatus(true);
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] ****************** STARTING worker thread this = %p\n",__FILE__,__FUNCTION__,__LINE__,this);
+
+		//bool minorDebugPerformance = false;
+		Chrono chrono;
+
+		//unsigned int idx = 0;
+		for(;this->aiIntf != NULL;) {
+			if(getQuitStatus() == true) {
+				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+				break;
+			}
+
+			semTaskSignalled.waitTillSignalled();
+
+			static string masterSlaveOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+			MasterSlaveThreadControllerSafeWrapper safeMasterController(masterController,20000,masterSlaveOwnerId);
+
+			if(getQuitStatus() == true) {
+				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+				break;
+			}
+
+			static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+            MutexSafeWrapper safeMutex(triggerIdMutex,mutexOwnerId);
+            bool executeTask = (frameIndex.first >= 0);
+
+            //if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] frameIndex = %d this = %p executeTask = %d\n",__FILE__,__FUNCTION__,__LINE__,frameIndex.first, this, executeTask);
+
+            safeMutex.ReleaseLock();
+
+            if(executeTask == true) {
+				ExecutingTaskSafeWrapper safeExecutingTaskMutex(this);
+
+				MutexSafeWrapper safeMutex(this->aiIntf->getMutex(),string(__FILE__) + "_" + intToStr(__LINE__));
+
+				this->aiIntf->update();
+
+				safeMutex.ReleaseLock();
+
+				setTaskCompleted(frameIndex.first);
+            }
+
+			if(getQuitStatus() == true) {
+				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+				break;
+			}
+		}
+
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] ****************** ENDING worker thread this = %p\n",__FILE__,__FUNCTION__,__LINE__,this);
+	}
+	catch(const exception &ex) {
+		//setRunningStatus(false);
+
+		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+
+		throw megaglest_runtime_error(ex.what());
+	}
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d\n",__FILE__,__FUNCTION__,__LINE__);
+}
+
+AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex,
+		int useStartLocation) : fp(NULL) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
+	this->aiMutex = new Mutex(CODE_AT_LINE);
+	this->workerThread = NULL;
 	this->world= game.getWorld();
 	this->commander= game.getCommander();
 	this->console= game.getConsole();
@@ -66,7 +220,7 @@ AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useSta
 	//clear log file
 	if(logLevel > 0) {
 #ifdef WIN32
-		fp = _wfopen(Shared::Platform::utf8_decode(aiLogFile).c_str(), L"wt");
+		fp = _wfopen(::Shared::Platform::utf8_decode(aiLogFile).c_str(), L"wt");
 #else
 		fp = fopen(aiLogFile.c_str(), "wt");
 #endif
@@ -75,18 +229,79 @@ AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useSta
 		}
 		fprintf(fp, "MegaGlest AI log file for Tech [%s] Faction [%s] #%d\n\n",this->gameSettings->getTech().c_str(),this->world->getFaction(this->factionIndex)->getType()->getName().c_str(),this->factionIndex);
 	}
+
+
+	if( Config::getInstance().getBool("EnableAIWorkerThreads","true") == true) {
+		if(workerThread != NULL) {
+			workerThread->signalQuit();
+			if(workerThread->shutdownAndWait() == true) {
+				delete workerThread;
+			}
+			workerThread = NULL;
+		}
+		static string mutexOwnerId = string(extractFileFromDirectoryPath(__FILE__).c_str()) + string("_") + intToStr(__LINE__);
+		this->workerThread = new AiInterfaceThread(this);
+		this->workerThread->setUniqueID(mutexOwnerId);
+		this->workerThread->start();
+	}
+
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 }
 
+void AiInterface::init() {
+    world=NULL;;
+    commander=NULL;;
+    console=NULL;;
+    gameSettings=NULL;;
+    timer=0;
+    factionIndex=0;
+    teamIndex=0;
+	redir=false;
+    logLevel=0;
+    fp=NULL;;
+    aiMutex=NULL;
+    workerThread=NULL;
+}
+
 AiInterface::~AiInterface() {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] deleting AI factionIndex = %d, teamIndex = %d\n",__FILE__,__FUNCTION__,__LINE__,this->factionIndex,this->teamIndex);
     cacheUnitHarvestResourceLookup.clear();
 
+	if(workerThread != NULL) {
+		workerThread->signalQuit();
+    	sleep(0);
+    	if(workerThread->canShutdown(true) == true &&
+    			workerThread->shutdownAndWait() == true) {
+			delete workerThread;
+		}
+		workerThread = NULL;
+	}
+
     if(fp) {
     	fclose(fp);
     	fp = NULL;
     }
+
+	delete aiMutex;
+	aiMutex = NULL;
 }
+
+void AiInterface::signalWorkerThread(int frameIndex) {
+	if(workerThread != NULL) {
+		workerThread->signal(frameIndex);
+	}
+	else {
+		this->update();
+	}
+}
+
+bool AiInterface::isWorkerThreadSignalCompleted(int frameIndex) {
+	if(workerThread != NULL) {
+		return workerThread->isSignalCompleted(frameIndex);
+	}
+	return true;
+}
+
 // ==================== main ====================
 
 void AiInterface::update() {
@@ -96,10 +311,15 @@ void AiInterface::update() {
 
 // ==================== misc ====================
 
+bool AiInterface::isLogLevelEnabled(int level) {
+	return (this->logLevel >= level);
+}
+
 void AiInterface::printLog(int logLevel, const string &s){
-    if(this->logLevel >= logLevel) {
+    if(isLogLevelEnabled(logLevel) == true) {
 		string logString= "(" + intToStr(factionIndex) + ") " + s;
 
+		MutexSafeWrapper safeMutex(aiMutex,string(__FILE__) + "_" + intToStr(__LINE__));
 		//print log to file
 		if(fp != NULL) {
 			fprintf(fp, "%s\n", logString.c_str());
@@ -178,7 +398,7 @@ std::pair<CommandResult,string> AiInterface::giveCommand(const Unit *unit, const
 	    char szBuf[8096]="";
 	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
-            unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
+            unit->getId(), unit->getFullName(false).c_str(),unit->getDesc(false).c_str(),
             unit->getFaction()->getIndex());
 
 	    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s\n",szBuf);
@@ -228,7 +448,7 @@ std::pair<CommandResult,string> AiInterface::giveCommand(int unitIndex, const Co
 	    char szBuf[8096]="";
 	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
-            unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
+            unit->getId(), unit->getFullName(false).c_str(),unit->getDesc(false).c_str(),
             unit->getFaction()->getIndex());
 
 	    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s\n",szBuf);
@@ -276,7 +496,7 @@ std::pair<CommandResult,string> AiInterface::giveCommand(int unitIndex, const Co
 	    char szBuf[8096]="";
 	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
-            unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
+            unit->getId(), unit->getFullName(false).c_str(),unit->getDesc(false).c_str(),
             unit->getFaction()->getIndex());
 
 	    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s\n",szBuf);
@@ -324,7 +544,7 @@ std::pair<CommandResult,string> AiInterface::giveCommand(int unitIndex, const Co
 	    char szBuf[8096]="";
 	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
-            unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
+            unit->getId(), unit->getFullName(false).c_str(),unit->getDesc(false).c_str(),
             unit->getFaction()->getIndex());
 
 	    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s\n",szBuf);
@@ -514,30 +734,12 @@ bool AiInterface::isResourceNear(const Vec2i &pos, const ResourceType *rt, Vec2i
 		if(result.x >= 0) {
 			resourcePos = result;
 
-		  	//char szBuf[4096]="";
-			//	    	sprintf(szBuf,"[%s::%s Line: %d] [isresourcenear] pos [%s] resourcePos [%s] fallbackToPeersHarvestingSameResource [%d] rt [%s]",
-			//	    			__FILE__,__FUNCTION__,__LINE__,pos.getString().c_str(),resourcePos.getString().c_str(),fallbackToPeersHarvestingSameResource,rt->getName().c_str());
-
-			//SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"----------------------------------- START [%d] ------------------------------------------------\n",faction->getFrameCount());
-			//SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"%s",szBuf);
-			//SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"------------------------------------ END [%d] -------------------------------------------------\n",faction->getFrameCount());
-
 			if(pos.dist(resourcePos) <= size) {
 				return true;
 			}
 		}
 	}
 
-	//if(faction != NULL) {
-	//	char szBuf[4096]="";
-	//				sprintf(szBuf,"[%s::%s Line: %d] [isresourcenear] pos [%s] resourcePos [%s] fallbackToPeersHarvestingSameResource [%d] rt [%s] getCacheResourceTargetListSize() [%d]",
-	//						__FILE__,__FUNCTION__,__LINE__,pos.getString().c_str(),resourcePos.getString().c_str(),fallbackToPeersHarvestingSameResource,rt->getName().c_str(),faction->getCacheResourceTargetListSize());
-
-	//	SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"----------------------------------- START [%d] ------------------------------------------------\n",faction->getFrameCount());
-	//	SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"%s",szBuf);
-	//	SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"------------------------------------ END [%d] -------------------------------------------------\n",faction->getFrameCount());
-	//}
-
 	return false;
 }
 
@@ -575,7 +777,6 @@ bool AiInterface::getNearestSightedResource(const ResourceType *rt, const Vec2i
 
 	if(canUseResourceType == true) {
 		bool isResourceClose = isResourceNear(pos, rt, resultPos, faction, true);
-		//bool isResourceClose = false;
 
 		// Found a resource
 		if(isResourceClose == true || resultPos.x >= 0) {
@@ -583,8 +784,6 @@ bool AiInterface::getNearestSightedResource(const ResourceType *rt, const Vec2i
 		}
 		else {
 			const Map *map		= world->getMap();
-			//Faction *faction 	= world->getFaction(factionIndex);
-
 			for(int i = 0; i < map->getW(); ++i) {
 				for(int j = 0; j < map->getH(); ++j) {
 					Vec2i resPos = Vec2i(i, j);
@@ -605,8 +804,6 @@ bool AiInterface::getNearestSightedResource(const ResourceType *rt, const Vec2i
 									resultPos= resPos;
 								}
 							}
-
-							//faction->addResourceTargetToCache(resPos,false);
 						}
 					}
 				}
@@ -635,9 +832,23 @@ bool AiInterface::isFreeCells(const Vec2i &pos, int size, Field field){
     return world->getMap()->isFreeCells(pos, size, field);
 }
 
+void AiInterface::removeEnemyWarningPositionFromList(Vec2i &checkPos) {
+	for(int i = (int)enemyWarningPositionList.size() - 1; i >= 0; --i) {
+		Vec2i &pos = enemyWarningPositionList[i];
+
+		if(checkPos == pos) {
+			enemyWarningPositionList.erase(enemyWarningPositionList.begin()+i);
+			break;
+		}
+	}
+}
+
 const Unit *AiInterface::getFirstOnSightEnemyUnit(Vec2i &pos, Field &field, int radius) {
 	Map *map= world->getMap();
 
+	const int CHECK_RADIUS = 12;
+	const int WARNING_ENEMY_COUNT = 6;
+
 	for(int i = 0; i < world->getFactionCount(); ++i) {
         for(int j = 0; j < world->getFaction(i)->getUnitCount(); ++j) {
             Unit * unit= world->getFaction(i)->getUnit(j);
@@ -653,6 +864,37 @@ const Unit *AiInterface::getFirstOnSightEnemyUnit(Vec2i &pos, Field &field, int
                 if(pos.dist(getHomeLocation()) < radius) {
                     printLog(2, "Being attacked at pos "+intToStr(pos.x)+","+intToStr(pos.y)+"\n");
 
+                    // Now check if there are more than x enemies in sight and if
+                    // so make note of the position
+                    int foundEnemies = 0;
+                    std::map<int,bool> foundEnemyList;
+                	for(int aiX = pos.x-CHECK_RADIUS; aiX < pos.x + CHECK_RADIUS; ++aiX) {
+                		for(int aiY = pos.y-CHECK_RADIUS; aiY < pos.y + CHECK_RADIUS; ++aiY) {
+                			Vec2i checkPos(aiX,aiY);
+                			if(map->isInside(checkPos) && map->isInsideSurface(map->toSurfCoords(checkPos))) {
+                				Cell *cAI = map->getCell(checkPos);
+                				SurfaceCell *scAI = map->getSurfaceCell(Map::toSurfCoords(checkPos));
+                				if(scAI != NULL && cAI != NULL && cAI->getUnit(field) != NULL && sc->isVisible(teamIndex)) {
+                					const Unit *checkUnit = cAI->getUnit(field);
+                					if(foundEnemyList.find(checkUnit->getId()) == foundEnemyList.end()) {
+										bool cannotSeeUnitAI = (checkUnit->getType()->hasCellMap() == true &&
+															checkUnit->getType()->getAllowEmptyCellMap() == true &&
+															checkUnit->getType()->hasEmptyCellMap() == true);
+										if(cannotSeeUnitAI == false && isAlly(checkUnit) == false
+												&& checkUnit->isAlive() == true) {
+											foundEnemies++;
+											foundEnemyList[checkUnit->getId()] = true;
+										}
+                					}
+                				}
+                			}
+                		}
+                	}
+                	if(foundEnemies >= WARNING_ENEMY_COUNT) {
+                		if(std::find(enemyWarningPositionList.begin(),enemyWarningPositionList.end(),pos) == enemyWarningPositionList.end()) {
+                			enemyWarningPositionList.push_back(pos);
+                		}
+                	}
                     return unit;
                 }
             }
