@@ -11,7 +11,6 @@
 
 #include "components.h"
 
-#include <cassert>
 #include <algorithm>
 
 #include "metrics.h"
@@ -55,6 +54,7 @@ GraphicComponent::GraphicComponent(std::string containerName, std::string objNam
 	text = "";
 	font = NULL;
 	font3D = NULL;
+	textNativeTranslation = "";
 }
 
 void GraphicComponent::clearRegisteredComponents(std::string containerName) {
@@ -74,7 +74,7 @@ void GraphicComponent::clearRegisterGraphicComponent(std::string containerName,
 }
 
 void GraphicComponent::clearRegisterGraphicComponent(std::string containerName, std::vector<std::string> objNameList) {
-	for(int idx = 0; idx < objNameList.size(); ++idx) {
+	for(int idx = 0; idx < (int)objNameList.size(); ++idx) {
 		GraphicComponent::clearRegisterGraphicComponent(containerName, objNameList[idx]);
 	}
 }
@@ -277,10 +277,12 @@ GraphicLabel::GraphicLabel() {
 	wordWrap = false;
 	centeredW = -1;
 	centeredH = 1;
+	editable = false;
 	editModeEnabled = false;
 	maxEditWidth = -1;
 	maxEditRenderWidth = -1;
 	isPassword = false;
+	texture = NULL;
 }
 
 void GraphicLabel::init(int x, int y, int w, int h, bool centered, Vec3f textColor, bool wordWrap) {
@@ -301,6 +303,10 @@ bool GraphicLabel::mouseMove(int x, int y) {
 		useWidth = (int)lineWidth;
 	}
 
+	if(editable && useWidth<getMaxEditRenderWidth()){
+		useWidth = getMaxEditRenderWidth();
+	}
+
     return
         x > this->x &&
         y > this->y &&
@@ -363,6 +369,7 @@ GraphicListBox::GraphicListBox(std::string containerName, std::string objName)
 : GraphicComponent(containerName, objName), graphButton1(), graphButton2() {
     selectedItemIndex = 0;
     lighted = false;
+    leftControlled = false;
 }
 
 void GraphicListBox::init(int x, int y, int w, int h, Vec3f textColor){
@@ -377,14 +384,28 @@ void GraphicListBox::init(int x, int y, int w, int h, Vec3f textColor){
     lighted=false;
 }
 
+const string & GraphicListBox::getTextNativeTranslation() {
+	if(this->translated_items.empty() == true ||
+			this->selectedItemIndex < 0 ||
+			this->selectedItemIndex >= (int)this->translated_items.size() ||
+			this->items.size() != this->translated_items.size()) {
+		return this->text;
+	}
+	else {
+		return this->translated_items[this->selectedItemIndex];
+	}
+}
+
 //queryes
-void GraphicListBox::pushBackItem(string item){
+void GraphicListBox::pushBackItem(string item, string translated_item){
     items.push_back(item);
+    translated_items.push_back(translated_item);
     setSelectedItemIndex(0);
 }
 
-void GraphicListBox::setItems(const vector<string> &items){
+void GraphicListBox::setItems(const vector<string> &items, const vector<string> translated_items){
     this->items= items;
+    this->translated_items = translated_items;
     if(items.empty() == false) {
     	setSelectedItemIndex(0);
     }
@@ -395,11 +416,66 @@ void GraphicListBox::setItems(const vector<string> &items){
 }
 
 void GraphicListBox::setSelectedItemIndex(int index, bool errorOnMissing){
-    if(errorOnMissing == true) assert(index>=0 && index<items.size());
+	if(errorOnMissing == true && (index < 0 || index >= (int)items.size())) {
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"Index not found in listbox name: [%s] value index: %d size: %lu",this->instanceName.c_str(),index,(unsigned long)items.size());
+		throw megaglest_runtime_error(szBuf);
+	}
     selectedItemIndex= index;
     setText(getSelectedItem());
 }
 
+void GraphicListBox::setLeftControlled(bool leftControlled) {
+	if(this->leftControlled!=leftControlled){
+		this->leftControlled= leftControlled;
+		if(leftControlled==true) {
+			graphButton2.setX(x+graphButton1.getW()-4);
+			graphButton2.setH(graphButton2.getH()-4);
+			graphButton2.setW(graphButton2.getW()-4);
+			graphButton1.setH(graphButton1.getH()-4);
+			graphButton1.setW(graphButton1.getW()-4);
+			graphButton2.setY(graphButton2.getY()+2);
+			graphButton1.setY(graphButton1.getY()+2);
+
+//			graphButton2.setX(x);
+//			graphButton2.setH(graphButton2.getH()/2);
+//			graphButton2.setW(graphButton2.getW()/2);
+//			graphButton1.setH(graphButton1.getH()/2);
+//			graphButton1.setW(graphButton1.getW()/2);
+//			graphButton2.setY(graphButton2.getY()+graphButton1.getH());
+
+		}
+		else {
+			graphButton2.setX(x+w-graphButton2.getW()+4);
+			graphButton2.setH(graphButton2.getH()+4);
+			graphButton2.setW(graphButton2.getW()+4);
+			graphButton1.setH(graphButton1.getH()+4);
+			graphButton1.setW(graphButton1.getW()+4);
+			graphButton2.setY(graphButton2.getY()-2);
+			graphButton1.setY(graphButton1.getY()-2);
+
+
+//			graphButton2.setY(graphButton2.getY()-graphButton1.getH());
+//			graphButton2.setH(graphButton2.getH()*2);
+//			graphButton2.setW(graphButton2.getW()*2);
+//			graphButton1.setH(graphButton1.getH()*2);
+//			graphButton1.setW(graphButton1.getW()*2);
+//			graphButton2.setX(x+w-graphButton2.getW());
+		}
+	}
+}
+
+void GraphicListBox::setX(int x) {
+	this->x= x;
+	graphButton1.setX(x);
+	if(leftControlled==true) {
+		graphButton2.setX(x+graphButton1.getW());
+	}
+	else {
+		graphButton2.setX(x+w-graphButton2.getW());
+	}
+}
+
 void GraphicListBox::setY(int y) {
 	this->y= y;
 	graphButton1.setY(y);
@@ -429,10 +505,13 @@ void GraphicListBox::setSelectedItem(string item, bool errorOnMissing){
 
 	if(iter==items.end()) {
 		if(errorOnMissing == true) {
-			for(int idx = 0; idx < items.size(); idx++) {
+			for(int idx = 0; idx < (int)items.size(); idx++) {
 				SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d]\ninstanceName [%s] idx = %d items[idx] = [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,instanceName.c_str(),idx,items[idx].c_str());
 			}
-			throw megaglest_runtime_error("[" + instanceName +"] Value not found on list box: [" + item + "]");
+
+		    char szBuf[8096]="";
+		    snprintf(szBuf,8096,"Value not found in listbox name: [%s] value: %s",this->instanceName.c_str(),item.c_str());
+			throw megaglest_runtime_error(szBuf);
 		}
 	}
 	else {
@@ -465,6 +544,7 @@ bool GraphicListBox::mouseClick(int x, int y,string advanceToItemStartingWith) {
 			if(advanceToItemStartingWith != "") {
 				for(int i = selectedItemIndex - 1; i >= 0; --i) {
 					string item = items[i];
+					if((int)translated_items.size() > i) item = translated_items[i];
 					if(StartsWith(toLower(item),toLower(advanceToItemStartingWith)) == true) {
 						bFound = true;
 						selectedItemIndex = i;
@@ -472,8 +552,9 @@ bool GraphicListBox::mouseClick(int x, int y,string advanceToItemStartingWith) {
 					}
 				}
 				if(bFound == false) {
-					for(int i = items.size() - 1; i >= selectedItemIndex; --i) {
+					for(int i = (int)items.size() - 1; i >= selectedItemIndex; --i) {
 						string item = items[i];
+						if((int)translated_items.size() > i) item = translated_items[i];
 						//printf("Trying to match [%s] with item [%s]\n",advanceToItemStartingWith.c_str(),item.c_str());
 						if(StartsWith(toLower(item),toLower(advanceToItemStartingWith)) == true) {
 							bFound = true;
@@ -487,14 +568,15 @@ bool GraphicListBox::mouseClick(int x, int y,string advanceToItemStartingWith) {
 				selectedItemIndex--;
 			}
 			if(selectedItemIndex<0){
-				selectedItemIndex=items.size()-1;
+				selectedItemIndex = (int)items.size()-1;
 			}
 		}
 		else if(b2) {
 			bool bFound = false;
 			if(advanceToItemStartingWith != "") {
-				for(int i = selectedItemIndex + 1; i < items.size(); ++i) {
+				for(int i = selectedItemIndex + 1; i < (int)items.size(); ++i) {
 					string item = items[i];
+					if((int)translated_items.size() > i) item = translated_items[i];
 					//printf("Trying to match [%s] with item [%s]\n",advanceToItemStartingWith.c_str(),item.c_str());
 					if(StartsWith(toLower(item),toLower(advanceToItemStartingWith)) == true) {
 						bFound = true;
@@ -505,6 +587,7 @@ bool GraphicListBox::mouseClick(int x, int y,string advanceToItemStartingWith) {
 				if(bFound == false) {
 					for(int i = 0; i <= selectedItemIndex; ++i) {
 						string item = items[i];
+						if((int)translated_items.size() > i) item = translated_items[i];
 						//printf("Trying to match [%s] with item [%s]\n",advanceToItemStartingWith.c_str(),item.c_str());
 						if(StartsWith(toLower(item),toLower(advanceToItemStartingWith)) == true) {
 							bFound = true;
@@ -517,7 +600,7 @@ bool GraphicListBox::mouseClick(int x, int y,string advanceToItemStartingWith) {
 			if(bFound == false) {
 				selectedItemIndex++;
 			}
-			if(selectedItemIndex>=items.size()){
+			if(selectedItemIndex >= (int)items.size()) {
 				selectedItemIndex=0;
 			}
 		}
@@ -706,11 +789,14 @@ bool GraphicCheckBox::mouseMove(int x, int y){
 
 bool GraphicCheckBox::mouseClick(int x, int y){
 	bool result=GraphicComponent::mouseClick( x,  y);
-	if(result)
-    	if(value) 
+	if(result == true) {
+    	if(value) {
     		value=false;
-    	else
+    	}
+    	else {
     		value=true;
+    	}
+	}
     return result;
 }
 
@@ -918,7 +1004,7 @@ void PopupMenu::init(string menuHeader,std::vector<string> menuItems) {
 	}
 
 	int offsetH = (yStartOffset - y);
-	int maxH = (offsetH + ((menuItems.size() -1 ) * (textHeight + textHeightSpacing)));
+	int maxH = (offsetH + (((int)menuItems.size() -1 ) * (textHeight + textHeightSpacing)));
 	if(maxH >= h) {
 		h = maxH;
 		y= (metrics.getVirtualH()-h)/2;
