@@ -54,6 +54,7 @@ public:
 protected:
     int x, y, w, h;
     string text;
+    string textNativeTranslation;
 	Font2D *font;
 	Font3D *font3D;
 	bool enabled;
@@ -94,6 +95,7 @@ public:
 	virtual int getW() const				{return w;}
 	virtual int getH() const				{return h;}
 	virtual const string &getText() const	{return text;}
+	virtual const string &getTextNativeTranslation() { return this->textNativeTranslation;}
 	virtual Font2D *getFont() 				{return font;}
 	virtual Font3D *getFont3D() 			{return font3D;}
 	virtual bool getEnabled() const			{return enabled;}
@@ -105,6 +107,7 @@ public:
 	virtual void setW(int w)					{this->w= w;}
 	virtual void setH(int h)					{this->h= h;}
 	virtual void setText(const string &text)	{this->text= text;}
+	virtual void setTextNativeTranslation(const string &text)	{this->textNativeTranslation= text;}
 	virtual void setFont(Font2D *font)			{this->font= font;}
 	virtual void setFont3D(Font3D *font)		{this->font3D= font;}
 	virtual void setEnabled(bool enabled)		{this->enabled= enabled;}
@@ -146,6 +149,7 @@ private:
 
 	vector<int> textCharLength;
 	bool isPassword;
+	Texture2D *texture;
 
 public:
 	GraphicLabel();
@@ -186,6 +190,8 @@ public:
 	void setMaxEditRenderWidth(int value) { maxEditRenderWidth = value; }
 	int getMaxEditRenderWidth() const { return maxEditRenderWidth; }
 
+	void setTexture(Texture2D *value) { texture = value; }
+	Texture2D *getTexture() const { return texture; }
 };
 
 // ===========================================================
@@ -231,15 +237,17 @@ public:
 private:
     GraphicButton graphButton1, graphButton2;
     vector<string> items;
+    vector<string> translated_items;
     int selectedItemIndex;
     bool lighted;
 	Vec3f textColor;
+	bool leftControlled;
 	
 public:
 	GraphicListBox(std::string containerName="", std::string objName="");
     void init(int x, int y, int w=defW, int h=defH, Vec3f textColor=GraphicComponent::customTextColor);
     
-	int getItemCount() const				{return items.size();}
+	int getItemCount() const				{return (int)items.size();}
 	string getItem(int index) const			{return items[index];}
 	int getSelectedItemIndex() const		{return selectedItemIndex;}
 	string getSelectedItem() const			{return items[selectedItemIndex];}
@@ -247,21 +255,26 @@ public:
 	GraphicButton *getButton2() 			{return &graphButton2;}
 	bool getLighted() const					{return lighted;}
     void setLighted(bool lighted)			{this->lighted= lighted;}
+	bool getLeftControlled() const					{return leftControlled;}
+    void setLeftControlled(bool leftControlled);
 	Vec3f getTextColor() const				{return textColor;}
     void setTextColor(Vec3f color)			{this->textColor= color;}
 
-    void pushBackItem(string item);
-    void setItems(const vector<string> &items);
+    void pushBackItem(string item, string translated_item="");
+    void setItems(const vector<string> &items, const vector<string> translated_items=vector<string>());
 	void setSelectedItemIndex(int index, bool errorOnMissing=true);
     void setSelectedItem(string item, bool errorOnMissing=true);
     void setEditable(bool editable);
 
     bool hasItem(string item) const;
 
-	virtual void setY(int y);
-    
+    virtual void setX(int x);
+    virtual void setY(int y);
+
     virtual bool mouseMove(int x, int y);
     virtual bool mouseClick(int x, int y, string advanceToItemStartingWith="");
+
+    virtual const string &getTextNativeTranslation();
 };
 
 // ===========================================================
@@ -292,7 +305,7 @@ public:
 	bool getAutoWordWrap() const { return autoWordWrap; }
 	void setAutoWordWrap(bool value) { autoWordWrap = value; }
 
-	int getButtonCount() const				{return buttons.size();}
+	int getButtonCount() const				{return (int)buttons.size();}
 	GraphicButton *getButton(int index) 	{return buttons[index];}
 	string getHeader() const				{return header;}
 
