@@ -37,11 +37,12 @@ using namespace Shared::PlatformCommon;
 
 namespace Shared { namespace Graphics {
 	class VideoPlayer;
-}}
+}};
 
 namespace Glest{ namespace Game{
 
 class GraphicMessageBox;
+class ServerInterface;
 
 enum LoadGameItem {
 	lgt_FactionPreview 	= 0x01,
@@ -58,14 +59,10 @@ enum LoadGameItem {
 //
 //	Main game class
 // =====================================================
-class Game: public ProgramState, public FileCRCPreCacheThreadCallbackInterface, public CustomInputCallbackInterface {
+class Game: public ProgramState, public FileCRCPreCacheThreadCallbackInterface,
+            public CustomInputCallbackInterface, public ClientLagCallbackInterface {
 public:
 	static const float highlightTime;
-	enum Speed{
-		sFast,
-		sNormal,
-		sSlow
-	};
 
 private:
 	typedef vector<Ai*> Ais;
@@ -91,11 +88,22 @@ private:
     Vec2i mouseCellPos;
 
 	int updateFps, lastUpdateFps, avgUpdateFps;
+	int framesToCatchUpAsClient;
+	int framesToSlowDownAsClient;
+	int receivedTooEarlyInFrames[GameConstants::networkSmoothInterval];
+	int framesNeededToWaitForServerMessage[GameConstants::networkSmoothInterval];
 	int totalRenderFps, renderFps, lastRenderFps, avgRenderFps,currentAvgRenderFpsTotal;
 	uint64 tickCount;
 	bool paused;
+	bool pauseRequestSent;
+	bool resumeRequestSent;
+	bool pauseStateChanged;
+	bool pausedForJoinGame;
+	bool pausedBeforeJoinGame;
+
 	bool gameOver;
 	bool renderNetworkStatus;
+	bool renderInGamePerformance;
 	bool showFullConsole;
 	bool setMarker;
 	bool cameraDragAllowed;
@@ -106,7 +114,7 @@ private:
 	bool camUpButtonDown;
 	bool camDownButtonDown;
 
-	Speed speed;
+	int speed;
 	GraphicMessageBox mainMessageBox;
 	GraphicMessageBox errorMessageBox;
 
@@ -189,13 +197,27 @@ private:
 	std::vector<std::pair<int,NetworkCommand> > replayCommandList;
 
 	std::vector<string> streamingVideos;
-	Shared::Graphics::VideoPlayer *videoPlayer;
+	::Shared::Graphics::VideoPlayer *videoPlayer;
 	bool playingStaticVideo;
 
 	Unit *currentCameraFollowUnit;
 
 	std::map<int,HighlightSpecialUnitInfo> unitHighlightList;
 
+	MasterSlaveThreadController masterController;
+
+	bool inJoinGameLoading;
+	bool initialResumeSpeedLoops;
+
+	bool quitGameCalled;
+	bool disableSpeedChange;
+
+	std::map<int,FowAlphaCellsLookupItem> teamFowAlphaCellsLookupItem;
+	std::map<string,int64> gamePerformanceCounts;
+
+	bool networkPauseGameForLaggedClientsRequested;
+	bool networkResumeGameForLaggedClientsRequested;
+
 public:
 	Game();
     Game(Program *program, const GameSettings *gameSettings, bool masterserverMode);
@@ -239,7 +261,7 @@ public:
 
 	Uint64 getTickCount()					{return tickCount;}
 	bool getPaused();
-	void setPaused(bool value, bool forceAllowPauseStateChange=false);
+	void setPaused(bool value, bool forceAllowPauseStateChange,bool clearCaches,bool joinNetworkGame);
 	void tryPauseToggle(bool pause);
 	void setupRenderForVideo();
 	void saveGame();
@@ -280,11 +302,11 @@ public:
 	void endPerformanceTimer();
 	Vec2i getPerformanceTimerResults();
 
-	static Texture2D * findFactionLogoTexture(const GameSettings *settings, Logger *logger=NULL,string factionLogoFilter="loading_screen.*", bool useTechDefaultIfFilterNotFound=true);
-	static string findFactionLogoFile(const GameSettings *settings, Logger *logger=NULL, string factionLogoFilter="loading_screen.*");
-	static string extractScenarioLogoFile(const GameSettings *settings, string &result, bool &loadingImageUsed, Logger *logger=NULL, string factionLogoFilter="loading_screen.*");
-	static string extractFactionLogoFile(bool &loadingImageUsed, string factionName, string scenarioDir, string techName, Logger *logger=NULL, string factionLogoFilter="loading_screen.*");
-	static string extractTechLogoFile(string scenarioDir, string techName, bool &loadingImageUsed, Logger *logger=NULL,string factionLogoFilter="loading_screen.*");
+	static Texture2D * findFactionLogoTexture(const GameSettings *settings, Logger *logger=NULL,string factionLogoFilter=GameConstants::LOADING_SCREEN_FILE_FILTER, bool useTechDefaultIfFilterNotFound=true);
+	static string findFactionLogoFile(const GameSettings *settings, Logger *logger=NULL, string factionLogoFilter=GameConstants::LOADING_SCREEN_FILE_FILTER);
+	static string extractScenarioLogoFile(const GameSettings *settings, string &result, bool &loadingImageUsed, Logger *logger=NULL, string factionLogoFilter=GameConstants::LOADING_SCREEN_FILE_FILTER);
+	static string extractFactionLogoFile(bool &loadingImageUsed, string factionName, string scenarioDir, string techName, Logger *logger=NULL, string factionLogoFilter=GameConstants::LOADING_SCREEN_FILE_FILTER);
+	static string extractTechLogoFile(string scenarioDir, string techName, bool &loadingImageUsed, Logger *logger=NULL,string factionLogoFilter=GameConstants::LOADING_SCREEN_FILE_FILTER);
 
 	void loadHudTexture(const GameSettings *settings);
 
@@ -300,8 +322,8 @@ public:
 	void stopStreamingVideo(const string &playVideo);
 	void stopAllVideo();
 
-	string saveGame(string name);
-	static void loadGame(string name,Program *programPtr,bool isMasterserverMode);
+	string saveGame(string name, string path="saved/");
+	static void loadGame(string name,Program *programPtr,bool isMasterserverMode, const GameSettings *joinGameSettings=NULL);
 
 	void addNetworkCommandToReplayList(NetworkCommand* networkCommand,int worldFrameCount);
 
@@ -314,6 +336,17 @@ public:
 	void highlightUnit(int unitId,float radius, float thickness, Vec4f color);
 	void unhighlightUnit(int unitId);
 
+	bool showTranslatedTechTree() const;
+
+	void DumpCRCWorldLogIfRequired(string fileSuffix="");
+
+	bool getDisableSpeedChange() const { return disableSpeedChange; }
+	void setDisableSpeedChange(bool value) { disableSpeedChange = value; }
+
+	string getGamePerformanceCounts(bool displayWarnings) const;
+	virtual void addPerformanceCount(string key,int64 value);
+	bool getRenderInGamePerformance() const { return renderInGamePerformance; }
+
 private:
 	//render
     void render3d();
@@ -360,6 +393,16 @@ private:
 
 	void startMarkCell();
 	void startCameraFollowUnit();
+
+	bool switchSetupForSlots(ServerInterface *& serverInterface,
+			int startIndex, int endIndex, bool onlyNetworkUnassigned);
+	void processNetworkSynchChecksIfRequired();
+	Stats getEndGameStats();
+	void checkWinnerStandardHeadlessOrObserver();
+	void checkWinnerStandardPlayer();
+	std::map<int, int> getTeamsAlive();
+
+	virtual bool clientLagHandler(int slotIndex,bool networkPauseGameForLaggedClients);
 };
 
 }}//end namespace
