@@ -46,18 +46,19 @@ const float GameCamera::centerOffsetZ= 8.0f;
 
 GameCamera::GameCamera() : pos(0.f, defaultHeight, 0.f),
 		destPos(0.f, defaultHeight, 0.f), destAng(startingVAng, startingHAng) {
-	Config &config = Config::getInstance();
+	//Config &config = Config::getInstance();
 	calculatedDefault=defaultHeight;
     state= sGame;
 
     cacheVisibleQuad.clear();
-    MaxVisibleQuadItemCache = config.getInt("MaxVisibleQuadItemCache",intToStr(-1).c_str());
-    if(Config::getInstance().getBool("DisableCaching","false") == true) {
-    	MaxVisibleQuadItemCache = 0;
-    }
+    //MaxVisibleQuadItemCache = config.getInt("MaxVisibleQuadItemCache",intToStr(-1).c_str());
+    MaxVisibleQuadItemCache = -1;
+    //if(Config::getInstance().getBool("DisableCaching","false") == true) {
+    //	MaxVisibleQuadItemCache = 0;
+    //}
 
 	//config
-	speed= 15.f / GameConstants::cameraFps;
+	speed= Config::getInstance().getFloat("CameraMoveSpeed","15") / GameConstants::cameraFps;
 	clampBounds= !Config::getInstance().getBool("PhotoMode");
 	clampDisable = false;
 
@@ -144,11 +145,7 @@ void GameCamera::update(){
 
 	//free state
 	if(state==sFree ){
-#ifdef USE_STREFLOP
-		if(streflop::fabs(static_cast<streflop::Simple>(rotate)) == 1){
-#else
-		if(fabs(rotate) == 1){
-#endif
+		if(std::fabs(rotate) == 1){
 			rotateHV(speed*5*rotate, 0);
 		}
 		if(move.y>0){
@@ -213,27 +210,14 @@ Quad2i GameCamera::computeVisibleQuad() {
 		}
 	}
 
-	//const float nearDist= 20.f;
-	//const float farDist= 90.f;
-	//const float dist= 20.f;
-
-//	float nearDist = 20.f;
-//	float dist = pos.y > 20.f ? pos.y * 1.2f : 20.f;
-//	float farDist = 90.f * (pos.y > 20.f ? pos.y / 15.f : 1.f);
 	float nearDist = 15.f;
 	float dist = pos.y > nearDist ? pos.y * 1.2f : nearDist;
 	float farDist = 90.f * (pos.y > nearDist ? pos.y / 15.f : 1.f);
 	const float viewDegree = 180.f;
 
-#ifdef USE_STREFLOP
-	Vec2f v(streflop::sinf(static_cast<streflop::Simple>(degToRad(viewDegree - hAng))), streflop::cosf(static_cast<streflop::Simple>(degToRad(viewDegree - hAng))));
-	Vec2f v1(streflop::sinf(static_cast<streflop::Simple>(degToRad(viewDegree - hAng - fov))), streflop::cosf(static_cast<streflop::Simple>(degToRad(viewDegree - hAng - fov))));
-	Vec2f v2(streflop::sinf(static_cast<streflop::Simple>(degToRad(viewDegree - hAng + fov))), streflop::cosf(static_cast<streflop::Simple>(degToRad(viewDegree - hAng + fov))));
-#else
-	Vec2f v(sinf(degToRad(viewDegree - hAng)), cosf(degToRad(viewDegree - hAng)));
-	Vec2f v1(sinf(degToRad(viewDegree - hAng - fov)), cosf(degToRad(viewDegree - hAng - fov)));
-	Vec2f v2(sinf(degToRad(viewDegree - hAng + fov)), cosf(degToRad(viewDegree - hAng + fov)));
-#endif
+	Vec2f v(std::sin(degToRad(viewDegree - hAng)), std::cos(degToRad(viewDegree - hAng)));
+	Vec2f v1(std::sin(degToRad(viewDegree - hAng - fov)), std::cos(degToRad(viewDegree - hAng - fov)));
+	Vec2f v2(std::sin(degToRad(viewDegree - hAng + fov)), std::cos(degToRad(viewDegree - hAng + fov)));
 
 	v.normalize();
 	v1.normalize();
@@ -253,7 +237,7 @@ Quad2i GameCamera::computeVisibleQuad() {
 
 		result = Quad2i(p1, p2, p3, p4);
 		if(MaxVisibleQuadItemCache != 0 &&
-		   (MaxVisibleQuadItemCache < 0 || cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
+		   (MaxVisibleQuadItemCache < 0 || (int)cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
 			cacheVisibleQuad[fov][hAng][pos] = result;
 		}
 	}
@@ -262,7 +246,7 @@ Quad2i GameCamera::computeVisibleQuad() {
 
 		result = Quad2i(p3, p1, p4, p2);
 		if(MaxVisibleQuadItemCache != 0 &&
-		   (MaxVisibleQuadItemCache < 0 || cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
+		   (MaxVisibleQuadItemCache < 0 || (int)cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
 			cacheVisibleQuad[fov][hAng][pos] = result;
 		}
 	}
@@ -271,7 +255,7 @@ Quad2i GameCamera::computeVisibleQuad() {
 
 		result = Quad2i(p2, p4, p1, p3);
 		if(MaxVisibleQuadItemCache != 0 &&
-		   (MaxVisibleQuadItemCache < 0 || cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
+		   (MaxVisibleQuadItemCache < 0 || (int)cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
 			cacheVisibleQuad[fov][hAng][pos] = result;
 		}
 	}
@@ -280,7 +264,7 @@ Quad2i GameCamera::computeVisibleQuad() {
 
 		result = Quad2i(p4, p3, p2, p1);
 		if(MaxVisibleQuadItemCache != 0 &&
-		   (MaxVisibleQuadItemCache < 0 || cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
+		   (MaxVisibleQuadItemCache < 0 || (int)cacheVisibleQuad[fov][hAng].size() <= MaxVisibleQuadItemCache)) {
 			cacheVisibleQuad[fov][hAng][pos] = Quad2i(p4, p3, p2, p1);
 		}
 	}
@@ -340,13 +324,8 @@ void GameCamera::rotateToVH(float v, float h) {
 }
 
 void GameCamera::zoom(float dist) {
-#ifdef USE_STREFLOP
-	float flatDist = dist * streflop::cosf(static_cast<streflop::Simple>(degToRad(vAng)));
-	Vec3f offset(flatDist * streflop::sinf(static_cast<streflop::Simple>(degToRad(hAng))), dist * streflop::sinf(static_cast<streflop::Simple>(degToRad(vAng))), flatDist  * -streflop::cosf(static_cast<streflop::Simple>(degToRad(hAng))));
-#else
-	float flatDist = dist * cosf(degToRad(vAng));
-	Vec3f offset(flatDist * sinf(degToRad(hAng)), dist * sinf(degToRad(vAng)), flatDist  * -cosf(degToRad(hAng)));
-#endif
+	float flatDist = dist * std::cos(degToRad(vAng));
+	Vec3f offset(flatDist * std::sin(degToRad(hAng)), dist * std::sin(degToRad(vAng)), flatDist  * -std::cos(degToRad(hAng)));
 
 	destPos += offset;
 }
@@ -420,11 +399,7 @@ void GameCamera::clampAng() {
 
 //move camera forwad but never change heightFactor
 void GameCamera::moveForwardH(float d, float response) {
-#ifdef USE_STREFLOP
-	Vec3f offset(streflop::sinf(static_cast<streflop::Simple>(degToRad(hAng))) * d, 0.f, -streflop::cosf(static_cast<streflop::Simple>(degToRad(hAng))) * d);
-#else
-	Vec3f offset(sinf(degToRad(hAng)) * d, 0.f, -cosf(degToRad(hAng)) * d);
-#endif
+	Vec3f offset(std::sin(degToRad(hAng)) * d, 0.f, -std::cos(degToRad(hAng)) * d);
 	destPos += offset;
 	pos.x += offset.x * response;
 	pos.z += offset.z * response;
@@ -432,11 +407,7 @@ void GameCamera::moveForwardH(float d, float response) {
 
 //move camera to a side but never change heightFactor
 void GameCamera::moveSideH(float d, float response){
-#ifdef USE_STREFLOP
-	Vec3f offset(streflop::sinf(static_cast<streflop::Simple>(degToRad(hAng+90))) * d, 0.f, -streflop::cosf(static_cast<streflop::Simple>(degToRad(hAng+90))) * d);
-#else
-	Vec3f offset(sinf(degToRad(hAng+90)) * d, 0.f, -cosf(degToRad(hAng+90)) * d);
-#endif
+	Vec3f offset(std::sin(degToRad(hAng+90)) * d, 0.f, -std::cos(degToRad(hAng+90)) * d);
 	destPos += offset;
 	pos.x += (destPos.x - pos.x) * response;
 	pos.z += (destPos.z - pos.z) * response;
@@ -457,18 +428,18 @@ void GameCamera::saveGame(XmlNode *rootNode) {
 	gamecameraNode->addAttribute("destPos",destPos.getString(), mapTagReplacements);
 //
 //    float hAng;	//YZ plane positive -Z axis
-	gamecameraNode->addAttribute("hAng",floatToStr(hAng), mapTagReplacements);
+	gamecameraNode->addAttribute("hAng",floatToStr(hAng,6), mapTagReplacements);
 //    float vAng;	//XZ plane positive +Z axis
-	gamecameraNode->addAttribute("vAng",floatToStr(vAng), mapTagReplacements);
+	gamecameraNode->addAttribute("vAng",floatToStr(vAng,6), mapTagReplacements);
 //	float lastHAng;
-	gamecameraNode->addAttribute("lastHAng",floatToStr(lastHAng), mapTagReplacements);
+	gamecameraNode->addAttribute("lastHAng",floatToStr(lastHAng,6), mapTagReplacements);
 
 //    float lastVAng;
-	gamecameraNode->addAttribute("lastVAng",floatToStr(lastVAng), mapTagReplacements);
+	gamecameraNode->addAttribute("lastVAng",floatToStr(lastVAng,6), mapTagReplacements);
 //	Vec2f destAng;
 	gamecameraNode->addAttribute("destAng",destAng.getString(), mapTagReplacements);
 //	float rotate;
-	gamecameraNode->addAttribute("rotate",floatToStr(rotate), mapTagReplacements);
+	gamecameraNode->addAttribute("rotate",floatToStr(rotate,6), mapTagReplacements);
 //	Vec3f move;
 	gamecameraNode->addAttribute("move",move.getString(), mapTagReplacements);
 //	State state;
@@ -479,24 +450,24 @@ void GameCamera::saveGame(XmlNode *rootNode) {
 	gamecameraNode->addAttribute("limitY",intToStr(limitY), mapTagReplacements);
 //	//config
 //	float speed;
-	gamecameraNode->addAttribute("speed",floatToStr(speed), mapTagReplacements);
+	gamecameraNode->addAttribute("speed",floatToStr(speed,6), mapTagReplacements);
 //	bool clampBounds;
 	gamecameraNode->addAttribute("clampBounds",intToStr(clampBounds), mapTagReplacements);
 //	//float maxRenderDistance;
 //	float maxHeight;
-	gamecameraNode->addAttribute("maxHeight",floatToStr(maxHeight), mapTagReplacements);
+	gamecameraNode->addAttribute("maxHeight",floatToStr(maxHeight,6), mapTagReplacements);
 //	float minHeight;
-	gamecameraNode->addAttribute("minHeight",floatToStr(minHeight), mapTagReplacements);
+	gamecameraNode->addAttribute("minHeight",floatToStr(minHeight,6), mapTagReplacements);
 //	//float maxCameraDist;
 //	//float minCameraDist;
 //	float minVAng;
-	gamecameraNode->addAttribute("minVAng",floatToStr(minVAng), mapTagReplacements);
+	gamecameraNode->addAttribute("minVAng",floatToStr(minVAng,6), mapTagReplacements);
 //	float maxVAng;
-	gamecameraNode->addAttribute("maxVAng",floatToStr(maxVAng), mapTagReplacements);
+	gamecameraNode->addAttribute("maxVAng",floatToStr(maxVAng,6), mapTagReplacements);
 //	float fov;
-	gamecameraNode->addAttribute("fov",floatToStr(fov), mapTagReplacements);
+	gamecameraNode->addAttribute("fov",floatToStr(fov,6), mapTagReplacements);
 //	float calculatedDefault;
-	gamecameraNode->addAttribute("calculatedDefault",floatToStr(calculatedDefault), mapTagReplacements);
+	gamecameraNode->addAttribute("calculatedDefault",floatToStr(calculatedDefault,6), mapTagReplacements);
 //	std::map<float, std::map<float, std::map<Vec3f, Quad2i> > > cacheVisibleQuad;
 //	int MaxVisibleQuadItemCache;
 	gamecameraNode->addAttribute("MaxVisibleQuadItemCache",intToStr(MaxVisibleQuadItemCache), mapTagReplacements);
