@@ -133,6 +133,7 @@ public:
 
 	static int networkFramePeriod;
 	static const int networkPingInterval = 5;
+	static const int networkSmoothInterval= 30;
 	//static const int networkExtraLatency= 200;
 	static const int maxClientConnectHandshakeSecs= 10;
 
@@ -152,6 +153,7 @@ public:
 	static const char *OBSERVER_SLOTNAME;
 	static const char *RANDOMFACTION_SLOTNAME;
 
+	static const char *preCacheThreadCacheLookupKey;
 	static const char *playerTextureCacheLookupKey;
 	static const char *ircClientCacheLookupKey;
 	static const char *factionPreviewTextureCacheLookupKey;
@@ -163,6 +165,10 @@ public:
 
 	static const char *application_name;
 	
+	static const char *saveNetworkGameFileServerCompressed;
+	static const char *saveNetworkGameFileServer;
+	static const char *saveNetworkGameFileClientCompressed;
+	static const char *saveNetworkGameFileClient;
 	static const char *saveGameFileDefault;
 	static const char *saveGameFileAutoTestDefault;
 	static const char *saveGameFilePattern;
@@ -174,6 +180,13 @@ public:
 	static const float megaMultiplier;
 	//
 	
+	static const char * LOADING_SCREEN_FILE;
+	static const char * LOADING_SCREEN_FILE_FILTER;
+	static const char * PREVIEW_SCREEN_FILE;
+	static const char * PREVIEW_SCREEN_FILE_FILTER;
+	static const char * HUD_SCREEN_FILE;
+	static const char * HUD_SCREEN_FILE_FILTER;
+
 };
 
 enum PathType {
