@@ -61,14 +61,20 @@ public:
 // =====================================================
 
 enum FlagTypes1 {
-    ft1_none                = 0x00,
-    ft1_show_map_resources  = 0x01,
-    ft1_allow_team_switching  = 0x02
-    //ft1_xx                  = 0x04,
-    //ft1_xx                  = 0x08,
-    //ft1_xx                  = 0x10,
+    ft1_none                			= 0x00,
+    ft1_show_map_resources  			= 0x01,
+    ft1_allow_team_switching  			= 0x02,
+    ft1_allow_in_game_joining 			= 0x04,
+    ft1_network_synch_checks_verbose 	= 0x08,
+    ft1_network_synch_checks 			= 0x10
+    //ft1_xx                  = 0x20,
+    //ft1_xx                  = 0x40,
 };
 
+inline static bool isFlagType1BitEnabled(uint32 flagValue,FlagTypes1 type) {
+	return ((flagValue & (uint32)type) == (uint32)type);
+}
+
 enum NetworkPlayerStatusType {
 	npst_None					= 0,
 	npst_PickSettings			= 1,
@@ -87,12 +93,14 @@ private:
 	string scenarioDir;
 	string factionTypeNames[GameConstants::maxPlayers]; //faction names
 	string networkPlayerNames[GameConstants::maxPlayers];
+	string networkPlayerPlatform[GameConstants::maxPlayers];
 	int    networkPlayerStatuses[GameConstants::maxPlayers];
 	string networkPlayerLanguages[GameConstants::maxPlayers];
 	int    networkPlayerGameStatus[GameConstants::maxPlayers];
 
 	ControlType factionControls[GameConstants::maxPlayers];
 	int resourceMultiplierIndex[GameConstants::maxPlayers];
+	string networkPlayerUUID[GameConstants::maxPlayers];
 
 
 	int thisFactionIndex;
@@ -126,6 +134,10 @@ private:
 
     int masterserver_admin_factionIndex;
 
+    bool networkAllowNativeLanguageTechtree;
+
+    string gameUUID;
+
 public:
 
     static string playerDisconnectedText;
@@ -149,6 +161,7 @@ public:
     	for(int i = 0; i < GameConstants::maxPlayers; ++i) {
     		factionTypeNames[i] = "";
     		networkPlayerNames[i] = "";
+    		networkPlayerPlatform[i] = "";
     		networkPlayerStatuses[i] = npst_None;
     		networkPlayerLanguages[i] = DEFAULT_LANG;
     		factionControls[i] = ctClosed;
@@ -156,6 +169,8 @@ public:
     		teams[i] = 0;
     		startLocationIndex[i] = i;
     		networkPlayerGameStatus[i] = 0;
+
+    		networkPlayerUUID[i] = "";
     	}
 
     	flagTypes1 = ft1_none;
@@ -168,6 +183,7 @@ public:
 		masterserver_admin = -1;
 		masterserver_admin_factionIndex = -1;
     	fallbackCpuMultiplier = 1.0f;
+    	networkAllowNativeLanguageTechtree = true;
     }
 
 	// default copy constructor will do fine, and will maintain itself ;)
@@ -180,6 +196,10 @@ public:
 	const string &getScenario() const								{return scenario;}
 	const string &getScenarioDir() const							{return scenarioDir;}
 	const string &getFactionTypeName(int factionIndex) const {
+		if(factionIndex == -1) {
+			static string HEADLESS_FACTION = "headless-server";
+			return HEADLESS_FACTION;
+		}
 		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
 			char szBuf[8096]="";
 			snprintf(szBuf,8096,"In [%s] Invalid factionIndex = %d\n",__FUNCTION__,factionIndex);
@@ -200,6 +220,17 @@ public:
 		}
 		return result;
 	}
+	string getNetworkPlayerPlatform(int factionIndex) const  {
+		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"In [%s] Invalid factionIndex = %d\n",__FUNCTION__,factionIndex);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		string result = networkPlayerPlatform[factionIndex];
+		return result;
+	}
+
 	const int    getNetworkPlayerStatuses(int factionIndex) const {
 		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
 			char szBuf[8096]="";
@@ -254,6 +285,17 @@ public:
 		}
 		return result;
 	}
+	const string getNetworkPlayerPlatformByPlayerIndex(int playerIndex) const  {
+		string result = "";
+		for(int i = 0; i < GameConstants::maxPlayers; ++i) {
+			if(startLocationIndex[i] == playerIndex) {
+				result = networkPlayerPlatform[i];
+				break;
+			}
+		}
+		return result;
+	}
+
 	ControlType getFactionControl(int factionIndex) const {
 		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
 			char szBuf[8096]="";
@@ -273,11 +315,20 @@ public:
 		return resourceMultiplierIndex[factionIndex];
 	}
 
+	const string &getNetworkPlayerUUID(int factionIndex) const {
+		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"In [%s] Invalid factionIndex = %d\n",__FUNCTION__,factionIndex);
+			throw megaglest_runtime_error(szBuf);
+		}
+		return networkPlayerUUID[factionIndex];
+	}
+
 	bool isNetworkGame() const {
 		bool result = false;
 		for(int idx = 0; idx < GameConstants::maxPlayers; ++idx) {
 			if(factionControls[idx] == ctNetwork || factionControls[idx] == ctNetworkUnassigned ||
-					networkPlayerStatuses[idx] == npst_Disconnected) {
+				networkPlayerStatuses[idx] == npst_Disconnected) {
 				result = true;
 				break;
 			}
@@ -305,6 +356,21 @@ public:
 
 		return startLocationIndex[factionIndex];
 	}
+	int getFactionIndexForStartLocation(int startIndex) const {
+		if(startIndex < 0 || startIndex >= GameConstants::maxPlayers) {
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"In [%s] Invalid startIndex = %d\n",__FUNCTION__,startIndex);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		for(int i =0; i < GameConstants::maxPlayers; ++i) {
+			if(startLocationIndex[i] == startIndex) {
+				return i;
+			}
+		}
+		return -1;
+	}
+
 	int getMapFilterIndex() const 						{return mapFilterIndex;}
 
 	bool getDefaultUnits() const				{return defaultUnits;}
@@ -325,6 +391,8 @@ public:
 	uint32 getTechCRC() const { return techCRC; }
 	vector<pair<string,uint32> > getFactionCRCList() const { return factionCRCList; }
 
+	const string &getGameUUID() const								{return gameUUID;}
+
 	//set
 	void setDescription(const string& description)						{this->description= description;}
 	void setMap(const string& map)										{this->map= map;}
@@ -351,6 +419,16 @@ public:
 
 		this->networkPlayerNames[factionIndex]= playername;
 	}
+	void setNetworkPlayerPlatform(int factionIndex,const string& platform) {
+		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"In [%s] Invalid factionIndex = %d\n",__FUNCTION__,factionIndex);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		this->networkPlayerPlatform[factionIndex]= platform;
+	}
+
 	void setNetworkPlayerStatuses(int factionIndex,int status) {
 		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
 			char szBuf[8096]="";
@@ -396,9 +474,23 @@ public:
 			throw megaglest_runtime_error(szBuf);
 		}
 
+		//if(multiplierIndex >= 45) {
+		//	printf("gamesettings Line: %d multiplier index: %d factionIndex: %d\n",__LINE__,multiplierIndex,factionIndex);
+		//}
+
 		this->resourceMultiplierIndex[factionIndex]= multiplierIndex;
 	}
 
+	void setNetworkPlayerUUID(int factionIndex, const string& uuid) {
+		if(factionIndex < 0 || factionIndex >= GameConstants::maxPlayers) {
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"In [%s] Invalid factionIndex = %d\n",__FUNCTION__,factionIndex);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		this->networkPlayerUUID[factionIndex]= uuid;
+	}
+
 	void setThisFactionIndex(int thisFactionIndex) {
 		this->thisFactionIndex= thisFactionIndex;
 	}
@@ -443,21 +535,27 @@ public:
 
 	void setFactionCRCList(vector<pair<string,uint32> > value) { factionCRCList = value; }
 
-	int getAiAcceptSwitchTeamPercentChance() const 					{ return aiAcceptSwitchTeamPercentChance;}
-	void setAiAcceptSwitchTeamPercentChance(int value)				{ aiAcceptSwitchTeamPercentChance = value; }
+	int getAiAcceptSwitchTeamPercentChance() const 				{ return aiAcceptSwitchTeamPercentChance;}
+	void setAiAcceptSwitchTeamPercentChance(int value)			{ aiAcceptSwitchTeamPercentChance = value; }
 
 	int getFallbackCpuMultiplier() const 					{ return fallbackCpuMultiplier;}
-	void setFallbackCpuMultiplier(int value)					{ fallbackCpuMultiplier = value; }
+	void setFallbackCpuMultiplier(int value)				{ fallbackCpuMultiplier = value; }
 
-	int getMasterserver_admin() const 								{ return masterserver_admin;}
-	void setMasterserver_admin(int value)							{ masterserver_admin = value; }
+	int getMasterserver_admin() const 						{ return masterserver_admin;}
+	void setMasterserver_admin(int value)					{ masterserver_admin = value; }
 
-	int getMasterserver_admin_faction_index() const 								{ return masterserver_admin_factionIndex;}
-	void setMasterserver_admin_faction_index(int value)							{ masterserver_admin_factionIndex = value; }
+	int getMasterserver_admin_faction_index() const 		{ return masterserver_admin_factionIndex;}
+	void setMasterserver_admin_faction_index(int value)		{ masterserver_admin_factionIndex = value; }
+
+	bool getNetworkAllowNativeLanguageTechtree() const 		{ return networkAllowNativeLanguageTechtree;}
+	void setNetworkAllowNativeLanguageTechtree(bool value)	{ networkAllowNativeLanguageTechtree = value; }
+
+	void setGameUUID(const string& gameUUID)				{this->gameUUID= gameUUID;}
 
 	string toString() const {
 		string result = "";
 
+		result += "Game ID = " + gameUUID + "\n";
 		result += "description = " + description + "\n";
 		result += "mapFilterIndex = " + intToStr(mapFilterIndex) + "\n";
 		result += "map = " + map + "\n";
@@ -470,12 +568,14 @@ public:
 			result += "player index = " + intToStr(idx) + "\n";
 			result += "factionTypeName = " + factionTypeNames[idx] + "\n";
 			result += "networkPlayerName = " + networkPlayerNames[idx] + "\n";
+			result += "networkPlayerPlatform = " + networkPlayerPlatform[idx] + "\n";
 			result += "networkPlayerLanguage = " + networkPlayerLanguages[idx] + "\n";
 
 			result += "factionControl = " + intToStr(factionControls[idx]) + "\n";
 			result += "resourceMultiplierIndex = " + intToStr(resourceMultiplierIndex[idx]) + "\n";
 			result += "team = " + intToStr(teams[idx]) + "\n";
 			result += "startLocationIndex = " + intToStr(startLocationIndex[idx]) + "\n";
+			result += "networkPlayerUUID = " + networkPlayerUUID[idx] + "\n";
 		}
 
 		result += "thisFactionIndex = " + intToStr(thisFactionIndex) + "\n";
@@ -503,6 +603,7 @@ public:
 		result += "masterserver_admin = " + intToStr(masterserver_admin) + "\n";
 		result += "masterserver_admin_factionIndex = " + intToStr(masterserver_admin_factionIndex) + "\n";
 
+		result += "networkAllowNativeLanguageTechtree = " + intToStr(networkAllowNativeLanguageTechtree) + "\n";
 		return result;
 	}
 
@@ -510,6 +611,8 @@ public:
 		std::map<string,string> mapTagReplacements;
 		XmlNode *gameSettingsNode = rootNode->addChild("GameSettings");
 
+		gameSettingsNode->addAttribute("gameUUID",gameUUID, mapTagReplacements);
+
 //		string description;
 		gameSettingsNode->addAttribute("description",description, mapTagReplacements);
 //		string map;
@@ -534,6 +637,11 @@ public:
 			networkPlayerNamesNode->addAttribute("name",networkPlayerNames[idx], mapTagReplacements);
 		}
 
+		for(int idx =0; idx < GameConstants::maxPlayers; idx++) {
+			XmlNode *networkPlayerNamesNode = gameSettingsNode->addChild("networkPlayerPlatform");
+			networkPlayerNamesNode->addAttribute("name",networkPlayerPlatform[idx], mapTagReplacements);
+		}
+
 //		int    networkPlayerStatuses[GameConstants::maxPlayers];
 		for(int idx =0; idx < GameConstants::maxPlayers; idx++) {
 			XmlNode *networkPlayerStatusesNode = gameSettingsNode->addChild("networkPlayerStatuses");
@@ -546,8 +654,6 @@ public:
 			networkPlayerStatusesNode->addAttribute("game_status",intToStr(networkPlayerGameStatus[idx]), mapTagReplacements);
 		}
 
-
-
 //		string networkPlayerLanguages[GameConstants::maxPlayers];
 		for(int idx =0; idx < GameConstants::maxPlayers; idx++) {
 			XmlNode *networkPlayerLanguagesNode = gameSettingsNode->addChild("networkPlayerLanguages");
@@ -582,6 +688,11 @@ public:
 			startLocationIndexNode->addAttribute("location",intToStr(startLocationIndex[idx]), mapTagReplacements);
 		}
 
+		for(int idx =0; idx < GameConstants::maxPlayers; idx++) {
+			XmlNode *networkPlayerUUIDNode = gameSettingsNode->addChild("networkPlayerUUID");
+			networkPlayerUUIDNode->addAttribute("value",networkPlayerUUID[idx], mapTagReplacements);
+		}
+
 //		int mapFilterIndex;
 		gameSettingsNode->addAttribute("mapFilterIndex",intToStr(mapFilterIndex), mapTagReplacements);
 //
@@ -628,11 +739,17 @@ public:
 		gameSettingsNode->addAttribute("masterserver_admin",intToStr(masterserver_admin), mapTagReplacements);
 
 		gameSettingsNode->addAttribute("masterserver_admin_factionIndex",intToStr(masterserver_admin_factionIndex), mapTagReplacements);
+
+		gameSettingsNode->addAttribute("networkAllowNativeLanguageTechtree",intToStr(networkAllowNativeLanguageTechtree), mapTagReplacements);
 	}
 
 	void loadGame(const XmlNode *rootNode) {
 		const XmlNode *gameSettingsNode = rootNode->getChild("GameSettings");
 
+		if(gameSettingsNode->hasAttribute("gameUUID") == true) {
+			gameUUID = gameSettingsNode->getAttribute("gameUUID")->getValue();
+		}
+
 //		string description;
 		description = gameSettingsNode->getAttribute("description")->getValue();
 //		string map;
@@ -693,6 +810,13 @@ public:
 			resourceMultiplierIndex[idx] = resourceMultiplierIndexNode->getAttribute("multiplier")->getIntValue();
 		}
 
+		for(int idx =0; idx < GameConstants::maxPlayers; idx++) {
+			if(gameSettingsNode->hasChildAtIndex("networkPlayerUUID",idx) == true) {
+				const XmlNode *networkPlayerUUIDNode = gameSettingsNode->getChild("networkPlayerUUID",idx);
+				networkPlayerUUID[idx] = networkPlayerUUIDNode->getAttribute("value")->getValue();
+			}
+		}
+
 //		int thisFactionIndex;
 		thisFactionIndex = gameSettingsNode->getAttribute("thisFactionIndex")->getIntValue();
 //		int factionCount;
@@ -757,6 +881,10 @@ public:
 		if(gameSettingsNode->hasAttribute("masterserver_admin_factionIndex") == true) {
 			masterserver_admin_factionIndex = gameSettingsNode->getAttribute("masterserver_admin_factionIndex")->getIntValue();
 		}
+
+		if(gameSettingsNode->hasAttribute("networkAllowNativeLanguageTechtree") == true) {
+			networkAllowNativeLanguageTechtree = gameSettingsNode->getAttribute("networkAllowNativeLanguageTechtree")->getIntValue() != 0;
+		}
 	}
 
 };
