@@ -28,7 +28,6 @@
 #include "leak_dumper.h"
 
 using std::string;
-//using std::queue;
 using std::list;
 using Shared::Graphics::Vec2i;
 using Shared::Lua::LuaScript;
@@ -52,14 +51,16 @@ private:
 	string header;
 	int factionIndex;
 	int teamIndex;
+	bool messageNotTranslated;
 
 public:
 	ScriptManagerMessage();
-	ScriptManagerMessage(string text, string header, int factionIndex=-1,int teamIndex=-1);
+	ScriptManagerMessage(string text, string header, int factionIndex=-1,int teamIndex=-1,bool messageNotTranslated=false);
 	const string &getText() const	{return text;}
 	const string &getHeader() const	{return header;}
 	int getFactionIndex() const	{return factionIndex;}
 	int getTeamIndex() const	{return teamIndex;}
+	bool getMessageNotTranslated() const { return messageNotTranslated; }
 
 	void saveGame(XmlNode *rootNode);
 	void loadGame(const XmlNode *rootNode);
@@ -96,6 +97,15 @@ private:
 //	class ScriptManager
 // =====================================================
 
+enum UnitTriggerEventType {
+	utet_None = 0,
+	utet_HPChanged = 1,
+	utet_EPChanged,
+	utet_LevelChanged,
+	utet_FieldChanged,
+	utet_SkillChanged
+};
+
 enum CellTriggerEventType {
 	ctet_Unit,
 	ctet_UnitPos,
@@ -198,9 +208,18 @@ private:
 	bool inCellTriggerEvent;
 	std::vector<int> unRegisterCellTriggerEventList;
 
+	bool registeredDayNightEvent;
+	int lastDayNightTriggerStatus;
+
+	std::map<int,UnitTriggerEventType> UnitTriggerEventList;
+	int lastUnitTriggerEventUnitId;
+	UnitTriggerEventType lastUnitTriggerEventType;
+
 	RandomGen random;
 	const XmlNode *rootNode;
 
+	std::map<string, string> luaSavedGameData;
+
 private:
 	static ScriptManager* thisScriptManager;
 
@@ -215,6 +234,7 @@ public:
 	void init(World* world, GameCamera *gameCamera,const XmlNode *rootNode);
 
 	//message box functions
+	void addMessageToQueue(ScriptManagerMessage msg);
 	bool getMessageBoxEnabled() const									{return !messageQueue.empty();}
 	GraphicMessageBox* getMessageBox()									{return &messageBox;}
 	string getDisplayText() const										{return displayText;}
@@ -228,8 +248,11 @@ public:
 	void onUnitAttacked(const Unit* unit);
 	void onUnitAttacking(const Unit* unit);
 	void onGameOver(bool won);
+
 	void onCellTriggerEvent(Unit *movingUnit);
 	void onTimerTriggerEvent();
+	void onDayNightTriggerEvent();
+	void onUnitTriggerEvent(const Unit *unit, UnitTriggerEventType event);
 
 	bool getGameWon() const;
 	bool getIsGameOver() const;
@@ -381,6 +404,28 @@ private:
 	void setAttackWarningsEnabled(bool enabled);
 	bool getAttackWarningsEnabled();
 
+	int getIsDayTime();
+	int getIsNightTime();
+	float getTimeOfDay();
+	void registerDayNightEvent();
+	void unregisterDayNightEvent();
+
+	void registerUnitTriggerEvent(int unitId);
+	void unregisterUnitTriggerEvent(int unitId);
+	int getLastUnitTriggerEventUnitId();
+	UnitTriggerEventType getLastUnitTriggerEventType();
+	int getUnitProperty(int unitId, UnitTriggerEventType type);
+	const string getUnitPropertyName(int unitId, UnitTriggerEventType type);
+
+	void disableSpeedChange();
+	void enableSpeedChange();
+	bool getSpeedChangeEnabled();
+	void storeSaveGameData(string name, string value);
+	string loadSaveGameData(string name);
+
+	ControlType getFactionPlayerType(int factionIndex);
+	// -----------------------------------------------------------------------
+
 	//callbacks, commands
 	static int networkShowMessageForFaction(LuaHandle* luaHandle);
 	static int networkShowMessageForTeam(LuaHandle* luaHandle);
@@ -528,6 +573,27 @@ private:
 	static int setAttackWarningsEnabled(LuaHandle* luaHandle);
 	static int getAttackWarningsEnabled(LuaHandle* luaHandle);
 
+	static int getIsDayTime(LuaHandle* luaHandle);
+	static int getIsNightTime(LuaHandle* luaHandle);
+	static int getTimeOfDay(LuaHandle* luaHandle);
+	static int registerDayNightEvent(LuaHandle* luaHandle);
+	static int unregisterDayNightEvent(LuaHandle* luaHandle);
+
+	static int registerUnitTriggerEvent(LuaHandle* luaHandle);
+	static int unregisterUnitTriggerEvent(LuaHandle* luaHandle);
+	static int getLastUnitTriggerEventUnitId(LuaHandle* luaHandle);
+	static int getLastUnitTriggerEventType(LuaHandle* luaHandle);
+	static int getUnitProperty(LuaHandle* luaHandle);
+	static int getUnitPropertyName(LuaHandle* luaHandle);
+
+	static int disableSpeedChange(LuaHandle* luaHandle);
+	static int enableSpeedChange(LuaHandle* luaHandle);
+	static int getSpeedChangeEnabled(LuaHandle* luaHandle);
+
+	static int storeSaveGameData(LuaHandle* luaHandle);
+	static int loadSaveGameData(LuaHandle* luaHandle);
+
+	static int getFactionPlayerType(LuaHandle* luaHandle);
 };
 
 }}//end namespace
