@@ -50,18 +50,32 @@ const char *GameConstants::folder_path_screenshots	= "screens/";
 const char *GameConstants::OBSERVER_SLOTNAME		= "*Observer*";
 const char *GameConstants::RANDOMFACTION_SLOTNAME	= "*Random*";
 
+const char *GameConstants::preCacheThreadCacheLookupKey  		= "preCacheThreadCache";
 const char *GameConstants::ircClientCacheLookupKey  			= "ircClientCache";
 const char *GameConstants::playerTextureCacheLookupKey  		= "playerTextureCache";
 const char *GameConstants::factionPreviewTextureCacheLookupKey  = "factionPreviewTextureCache";
 const char *GameConstants::characterMenuScreenPositionListCacheLookupKey  = "characterMenuScreenPositionListCache";
 const char *GameConstants::application_name				= "MegaGlest";
 
+const char * GameConstants::LOADING_SCREEN_FILE 	   = "loading_screen";
+const char * GameConstants::LOADING_SCREEN_FILE_FILTER = "loading_screen*.*";
+const char * GameConstants::PREVIEW_SCREEN_FILE 	   = "preview_screen";
+const char * GameConstants::PREVIEW_SCREEN_FILE_FILTER = "preview_screen*.*";
+const char * GameConstants::HUD_SCREEN_FILE 	   	   = "hud";
+const char * GameConstants::HUD_SCREEN_FILE_FILTER     = "hud*.*";
+
 
 const char *GameConstants::pathCacheLookupKey           = "pathCache_";
 const char *GameConstants::path_data_CacheLookupKey     = "data";
 const char *GameConstants::path_ini_CacheLookupKey      = "ini";
 const char *GameConstants::path_logs_CacheLookupKey     = "logs";
 
+const char *GameConstants::saveNetworkGameFileServer			= "megaglest-saved-server.xml";
+const char *GameConstants::saveNetworkGameFileServerCompressed 	= "megaglest-saved-server.zip";
+
+const char *GameConstants::saveNetworkGameFileClient			= "megaglest-saved-client.xml";
+const char *GameConstants::saveNetworkGameFileClientCompressed 	= "megaglest-saved-client.zip";
+
 const char *GameConstants::saveGameFileDefault 			= "megaglest-saved.xml";
 const char *GameConstants::saveGameFileAutoTestDefault 	= "megaglest-auto-saved_%s.xml";
 const char *GameConstants::saveGameFilePattern 			= "megaglest-saved_%s.xml";
@@ -73,6 +87,11 @@ const char *Config::glestkeys_ini_filename              = "glestkeys.ini";
 const char *Config::glestuserkeys_ini_filename          = "glestuserkeys.ini";
 
 const char *Config::ACTIVE_MOD_PROPERTY_NAME			= "current_mod_name";
+
+ const char *Config::colorPicking = "color";
+ const char *Config::selectBufPicking = "selectbuf";
+ const char *Config::frustumPicking = "frustum";
+
 map<string,string> Config::customRuntimeProperties;
 
 // =====================================================
@@ -92,8 +111,6 @@ Config::Config() {
 	fileName.second 			= "";
 	fileNameParameter.first 	= "";
 	fileNameParameter.second 	= "";
-	fileLoaded.first 			= false;
-	fileLoaded.second 			= false;
 }
 
 bool Config::tryCustomPath(std::pair<ConfigType,ConfigType> &type, std::pair<string,string> &file, string custom_path) {
@@ -172,10 +189,20 @@ Config::Config(std::pair<ConfigType,ConfigType> type, std::pair<string,string> f
     }
 #endif
 
+    if(SystemFlags::VERBOSE_MODE_ENABLED) printf("foundPath = [%d]\n",foundPath);
+
     if(fileMustExist.first == true && fileExists(fileName.first) == false) {
     	//string currentpath = extractDirectoryPathFromFile(Properties::getApplicationPath());
     	fileName.first = currentpath + fileName.first;
     }
+
+#if defined(WIN32)
+	//string test = "C:\\Code\\megaglest\\mk\\windoze\\.\\..\\..\\data\\glest_game\\glest.ini";
+	//updatePathClimbingParts(test);
+
+	updatePathClimbingParts(fileName.first);
+#endif
+
     if(SystemFlags::VERBOSE_MODE_ENABLED) printf("-=-=-=-=-=-=-= About to load fileName.first = [%s]\n",fileName.first.c_str());
 
     if(fileMustExist.first == true ||
@@ -199,6 +226,11 @@ Config::Config(std::pair<ConfigType,ConfigType> type, std::pair<string,string> f
         	}
         	fileName.second = userData + fileNameParameter.second;
         }
+
+#if defined(WIN32)
+		updatePathClimbingParts(fileName.second);
+#endif
+
     }
     else if(cfgType.first == cfgMainKeys) {
         Config &mainCfg = Config::getInstance();
@@ -216,6 +248,11 @@ Config::Config(std::pair<ConfigType,ConfigType> type, std::pair<string,string> f
         	}
         	fileName.second = userData + fileNameParameter.second;
         }
+
+#if defined(WIN32)
+	updatePathClimbingParts(fileName.second);
+#endif
+
     }
 
     if(SystemFlags::VERBOSE_MODE_ENABLED) printf("-=-=-=-=-=-=-= About to load fileName.second = [%s]\n",fileName.second.c_str());
@@ -711,6 +748,10 @@ vector<string> Config::getPathListForType(PathType type, string scenarioDir) {
         }
     }
     if(scenarioDir != "") {
+		if(EndsWith(scenarioDir, ".xml") == true) {
+			scenarioDir = extractDirectoryPathFromFile(scenarioDir);
+		}
+
     	//string scenarioLocation = data_path + scenarioDir;
     	string scenarioLocation = scenarioDir;
     	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Scenario path [%s]\n",scenarioLocation.c_str());
@@ -780,7 +821,7 @@ string Config::findValidLocalFileFromPath(string fileName) {
 		vector<string> dirList = config.getPathListForType(ptMaps);
 		replaceFileWithLocalFile(dirList, fileNamePart, result);
 
-		printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
 	}
 	else if(fileName.find("tilesets/") != fileName.npos ) {
 		size_t pos = fileName.find("tilesets/");
@@ -789,7 +830,7 @@ string Config::findValidLocalFileFromPath(string fileName) {
 		vector<string> dirList = config.getPathListForType(ptTilesets);
 		replaceFileWithLocalFile(dirList, fileNamePart, result);
 
-		printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
 	}
 	else if(fileName.find("techs/") != fileName.npos ) {
 		size_t pos = fileName.find("techs/");
@@ -798,7 +839,7 @@ string Config::findValidLocalFileFromPath(string fileName) {
 		vector<string> dirList = config.getPathListForType(ptTechs);
 		replaceFileWithLocalFile(dirList, fileNamePart, result);
 
-		printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
 	}
 	else if(fileName.find("scenarios/") != fileName.npos) {
 		size_t pos = fileName.find("scenarios/");
@@ -807,7 +848,7 @@ string Config::findValidLocalFileFromPath(string fileName) {
 		vector<string> dirList = config.getPathListForType(ptScenarios);
 		replaceFileWithLocalFile(dirList, fileNamePart, result);
 
-		printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
 	}
 	else if(fileName.find("tutorials/") != fileName.npos) {
 		size_t pos = fileName.find("tutorials/");
@@ -816,11 +857,39 @@ string Config::findValidLocalFileFromPath(string fileName) {
 		vector<string> dirList = config.getPathListForType(ptTutorials);
 		replaceFileWithLocalFile(dirList, fileNamePart, result);
 
-		printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Found file [%s] @ " MG_SIZE_T_SPECIFIER " [%s]\nNew File [%s]\n",fileName.c_str(),pos,fileNamePart.c_str(),result.c_str());
 	}
 
 	return result;
 }
 
 
+// static
+string Config::getMapPath(const string &mapName, string scenarioDir, bool errorOnNotFound) {
+
+    Config &config = Config::getInstance();
+    vector<string> pathList = config.getPathListForType(ptMaps,scenarioDir);
+
+    for(int idx = 0; idx < (int)pathList.size(); idx++) {
+        string map_path = pathList[idx];
+    	endPathWithSlash(map_path);
+
+        const string mega = map_path + mapName + ".mgm";
+        const string glest = map_path + mapName + ".gbm";
+        if (fileExists(mega)) {
+            return mega;
+        }
+        else if (fileExists(glest)) {
+            return glest;
+        }
+    }
+
+	if(errorOnNotFound == true) {
+		//abort();
+		throw megaglest_runtime_error("Map not found [" + mapName + "]\nScenario [" + scenarioDir + "]");
+	}
+
+	return "";
+}
+
 }}// end namespace
