@@ -66,6 +66,10 @@ public:
 
     static const char *ACTIVE_MOD_PROPERTY_NAME;
 
+    static const char *colorPicking;
+    static const char *selectBufPicking;
+    static const char *frustumPicking;
+
 protected:
 
 	Config();
@@ -117,6 +121,8 @@ public:
 	static void setCustomRuntimeProperty(string key, string value) 	{ customRuntimeProperties[key] = value; }
 
 	static string findValidLocalFileFromPath(string fileName);
+
+	static string getMapPath(const string &mapName, string scenarioDir="", bool errorOnNotFound=true);
 };
 
 }}//end namespace
