@@ -52,6 +52,16 @@ ParticleSystemType::ParticleSystemType() {
     minHp=0;
     maxHp=0;
     minmaxIsPercent=false;
+
+    modelCycle=0;
+    size=0;
+    sizeNoEnergy=0;
+    speed=0;
+	gravity=0;
+	emissionRate=0;
+	energyMax=0;
+	energyVar=0;
+
 }
 
 ParticleSystemType::ParticleSystemType(const ParticleSystemType &src) {
@@ -79,8 +89,10 @@ ParticleSystemType::~ParticleSystemType() {
 		memoryObjectList[this]--;
 		assert(memoryObjectList[this] == 0);
 	}
-	for(Children::iterator it = children.begin(); it != children.end(); ++it)
+	for(Children::iterator it = children.begin(); it != children.end(); ++it) {
 		delete *it;
+	}
+	children.clear();
 }
 
 void ParticleSystemType::copyAll(const ParticleSystemType &src) {
@@ -158,10 +170,7 @@ void ParticleSystemType::load(const XmlNode *particleSystemNode, const string &d
 			endPathWithSlash(currentPath);
 
 			string path= modelNode->getAttribute("path")->getRestrictedValue(currentPath);
-			model= renderer->newModel(rsGame);
-			if(model) {
-				model->load(path, false, &loadedFileList, &parentLoader);
-			}
+			model= renderer->newModel(rsGame,path, false, &loadedFileList, &parentLoader);
 			loadedFileList[path].push_back(make_pair(parentLoader,modelNode->getAttribute("path")->getRestrictedValue()));
 			
 			if(modelNode->hasChild("cycles")) {
@@ -280,6 +289,7 @@ void ParticleSystemType::setValues(AttackParticleSystem *ats){
 	// add instances of all children; some settings will cascade to all children
 	for(Children::iterator i=children.begin(); i!=children.end(); ++i){
 		UnitParticleSystem *child = new UnitParticleSystem();
+		child->setParticleOwner(ats->getParticleOwner());
 		(*i)->setValues(child);
 		ats->addChild(child);
 		child->setState(ParticleSystem::sPlay);
@@ -305,6 +315,46 @@ void ParticleSystemType::setValues(AttackParticleSystem *ats){
 	ats->setBlendMode(ParticleSystem::strToBlendMode(mode));
 }
 
+void ParticleSystemType::loadGame(const XmlNode *rootNode) {
+	const XmlNode *particleSystemTypeNode = rootNode->getChild("ParticleSystemType");
+
+	type = particleSystemTypeNode->getAttribute("type")->getIntValue();
+
+	modelCycle = particleSystemTypeNode->getAttribute("modelCycle")->getFloatValue();
+	primitive = particleSystemTypeNode->getAttribute("primitive")->getValue();
+	offset = Vec3f::strToVec3(particleSystemTypeNode->getAttribute("offset")->getValue());
+	color = Vec4f::strToVec4(particleSystemTypeNode->getAttribute("color")->getValue());
+	colorNoEnergy = Vec4f::strToVec4(particleSystemTypeNode->getAttribute("colorNoEnergy")->getValue());
+	size = particleSystemTypeNode->getAttribute("size")->getFloatValue();
+	sizeNoEnergy = particleSystemTypeNode->getAttribute("sizeNoEnergy")->getFloatValue();
+	speed = particleSystemTypeNode->getAttribute("speed")->getFloatValue();
+	gravity = particleSystemTypeNode->getAttribute("gravity")->getFloatValue();
+	emissionRate = particleSystemTypeNode->getAttribute("emissionRate")->getFloatValue();
+	energyMax = particleSystemTypeNode->getAttribute("energyMax")->getIntValue();
+	energyVar = particleSystemTypeNode->getAttribute("energyVar")->getIntValue();
+	mode = particleSystemTypeNode->getAttribute("mode")->getValue();
+	teamcolorNoEnergy = (particleSystemTypeNode->getAttribute("teamcolorNoEnergy")->getIntValue() != 0);
+	teamcolorEnergy = (particleSystemTypeNode->getAttribute("teamcolorEnergy")->getIntValue() != 0);
+	alternations = particleSystemTypeNode->getAttribute("alternations")->getIntValue();
+	particleSystemStartDelay = particleSystemTypeNode->getAttribute("particleSystemStartDelay")->getIntValue();
+
+	if(particleSystemTypeNode->hasChild("UnitParticleSystemType")) {
+		vector<XmlNode *> particleSystemTypeNodeList = particleSystemTypeNode->getChildList("UnitParticleSystemType");
+		for(unsigned int i = 0; i < particleSystemTypeNodeList.size(); ++i) {
+			XmlNode *node = particleSystemTypeNodeList[i];
+
+			UnitParticleSystemType *child = new UnitParticleSystemType();
+			child->loadGame(node);
+			children.push_back(child);
+		}
+	}
+
+	minmaxEnabled = (particleSystemTypeNode->getAttribute("minmaxEnabled")->getIntValue() != 0);
+	minHp = particleSystemTypeNode->getAttribute("minHp")->getIntValue();
+	maxHp = particleSystemTypeNode->getAttribute("maxHp")->getIntValue();
+	minmaxIsPercent = (particleSystemTypeNode->getAttribute("minmaxIsPercent")->getIntValue() != 0);
+}
+
 void ParticleSystemType::saveGame(XmlNode *rootNode) {
 	std::map<string,string> mapTagReplacements;
 	XmlNode *particleSystemTypeNode = rootNode->addChild("ParticleSystemType");
@@ -314,7 +364,7 @@ void ParticleSystemType::saveGame(XmlNode *rootNode) {
 //	Texture2D *texture;
 //	Model *model;
 //	float modelCycle;
-	particleSystemTypeNode->addAttribute("modelCycle",floatToStr(modelCycle), mapTagReplacements);
+	particleSystemTypeNode->addAttribute("modelCycle",floatToStr(modelCycle,6), mapTagReplacements);
 //	string primitive;
 	particleSystemTypeNode->addAttribute("primitive",primitive, mapTagReplacements);
 //	Vec3f offset;
@@ -324,15 +374,15 @@ void ParticleSystemType::saveGame(XmlNode *rootNode) {
 //	Vec4f colorNoEnergy;
 	particleSystemTypeNode->addAttribute("colorNoEnergy",colorNoEnergy.getString(), mapTagReplacements);
 //	float size;
-	particleSystemTypeNode->addAttribute("size",floatToStr(size), mapTagReplacements);
+	particleSystemTypeNode->addAttribute("size",floatToStr(size,6), mapTagReplacements);
 //	float sizeNoEnergy;
-	particleSystemTypeNode->addAttribute("sizeNoEnergy",floatToStr(sizeNoEnergy), mapTagReplacements);
+	particleSystemTypeNode->addAttribute("sizeNoEnergy",floatToStr(sizeNoEnergy,6), mapTagReplacements);
 //	float speed;
-	particleSystemTypeNode->addAttribute("speed",floatToStr(speed), mapTagReplacements);
+	particleSystemTypeNode->addAttribute("speed",floatToStr(speed,6), mapTagReplacements);
 //	float gravity;
-	particleSystemTypeNode->addAttribute("gravity",floatToStr(gravity), mapTagReplacements);
+	particleSystemTypeNode->addAttribute("gravity",floatToStr(gravity,6), mapTagReplacements);
 //	float emissionRate;
-	particleSystemTypeNode->addAttribute("emissionRate",floatToStr(emissionRate), mapTagReplacements);
+	particleSystemTypeNode->addAttribute("emissionRate",floatToStr(emissionRate,6), mapTagReplacements);
 //	int energyMax;
 	particleSystemTypeNode->addAttribute("energyMax",intToStr(energyMax), mapTagReplacements);
 //	int energyVar;
@@ -427,9 +477,9 @@ void ParticleSystemTypeProjectile::load(const XmlNode* particleFileNode, const s
 	}
 }
 
-ProjectileParticleSystem *ParticleSystemTypeProjectile::create() {
+ProjectileParticleSystem *ParticleSystemTypeProjectile::create(ParticleOwner *owner) {
 	ProjectileParticleSystem *ps=  new ProjectileParticleSystem();
-
+	ps->setParticleOwner(owner);
 	ParticleSystemType::setValues(ps);
 
 	ps->setTrajectory(ProjectileParticleSystem::strToTrajectory(trajectory));
@@ -453,11 +503,11 @@ void ParticleSystemTypeProjectile::saveGame(XmlNode *rootNode) {
 //	string trajectory;
 	particleSystemTypeProjectileNode->addAttribute("trajectory",trajectory, mapTagReplacements);
 //	float trajectorySpeed;
-	particleSystemTypeProjectileNode->addAttribute("trajectorySpeed",floatToStr(trajectorySpeed), mapTagReplacements);
+	particleSystemTypeProjectileNode->addAttribute("trajectorySpeed",floatToStr(trajectorySpeed,6), mapTagReplacements);
 //	float trajectoryScale;
-	particleSystemTypeProjectileNode->addAttribute("trajectoryScale",floatToStr(trajectoryScale), mapTagReplacements);
+	particleSystemTypeProjectileNode->addAttribute("trajectoryScale",floatToStr(trajectoryScale,6), mapTagReplacements);
 //	float trajectoryFrequency;
-	particleSystemTypeProjectileNode->addAttribute("trajectoryFrequency",floatToStr(trajectoryFrequency), mapTagReplacements);
+	particleSystemTypeProjectileNode->addAttribute("trajectoryFrequency",floatToStr(trajectoryFrequency,6), mapTagReplacements);
 }
 
 // ===========================================================
@@ -513,9 +563,9 @@ void ParticleSystemTypeSplash::load(const XmlNode* particleFileNode, const strin
 	}
 }
 
-SplashParticleSystem *ParticleSystemTypeSplash::create(){
+SplashParticleSystem *ParticleSystemTypeSplash::create(ParticleOwner *owner) {
 	SplashParticleSystem *ps=  new SplashParticleSystem();
-
+	ps->setParticleOwner(owner);
 	ParticleSystemType::setValues(ps);
 
 	ps->setEmissionRateFade(emissionRateFade);
@@ -536,15 +586,15 @@ void ParticleSystemTypeSplash::saveGame(XmlNode *rootNode) {
 	XmlNode *particleSystemTypeSplashNode = rootNode->addChild("ParticleSystemTypeSplash");
 
 //	float emissionRateFade;
-	particleSystemTypeSplashNode->addAttribute("emissionRateFade",floatToStr(emissionRateFade), mapTagReplacements);
+	particleSystemTypeSplashNode->addAttribute("emissionRateFade",floatToStr(emissionRateFade,6), mapTagReplacements);
 //	float verticalSpreadA;
-	particleSystemTypeSplashNode->addAttribute("verticalSpreadA",floatToStr(verticalSpreadA), mapTagReplacements);
+	particleSystemTypeSplashNode->addAttribute("verticalSpreadA",floatToStr(verticalSpreadA,6), mapTagReplacements);
 //	float verticalSpreadB;
-	particleSystemTypeSplashNode->addAttribute("verticalSpreadB",floatToStr(verticalSpreadB), mapTagReplacements);
+	particleSystemTypeSplashNode->addAttribute("verticalSpreadB",floatToStr(verticalSpreadB,6), mapTagReplacements);
 //	float horizontalSpreadA;
-	particleSystemTypeSplashNode->addAttribute("horizontalSpreadA",floatToStr(horizontalSpreadA), mapTagReplacements);
+	particleSystemTypeSplashNode->addAttribute("horizontalSpreadA",floatToStr(horizontalSpreadA,6), mapTagReplacements);
 //	float horizontalSpreadB;
-	particleSystemTypeSplashNode->addAttribute("horizontalSpreadB",floatToStr(horizontalSpreadB), mapTagReplacements);
+	particleSystemTypeSplashNode->addAttribute("horizontalSpreadB",floatToStr(horizontalSpreadB,6), mapTagReplacements);
 }
 
 }}//end mamespace
