@@ -55,8 +55,8 @@ enum DebugUILevelType {
 
 namespace Glest{ namespace Game{
 
-using namespace Shared::Graphics;
-using namespace Shared::PlatformCommon;
+using namespace ::Shared::Graphics;
+using namespace ::Shared::PlatformCommon;
 
 //non shared classes
 class Config;
@@ -116,7 +116,7 @@ public:
 
 	inline VisibleQuadContainerCache() {
 		cacheFrame = 0;
-		clearFrustrumData();
+		clearFrustumData();
 		clearCacheData();
 	}
 	inline VisibleQuadContainerCache(const VisibleQuadContainerCache &obj) {
@@ -149,7 +149,7 @@ public:
 		visibleObjectList.reserve(500);
 		visibleScaledCellList.reserve(500);
 	}
-	inline void clearFrustrumData() {
+	inline void clearFrustumData() {
 		frustumData = vector<vector<float> >(6,vector<float>(4,0));
 		proj = vector<float>(16,0);
 		modl = vector<float>(16,0);
@@ -248,7 +248,7 @@ private:
     bool photoMode;
 	int shadowTextureSize;
 	int shadowFrameSkip;
-	float shadowAlpha;
+	float shadowIntensity;
 	bool focusArrows;
 	bool textures3D;
 	Shadows shadows;
@@ -318,7 +318,7 @@ private:
 	std::vector<std::pair<ParticleSystem *, ResourceScope> > deferredParticleSystems;
 
 	SimpleTaskThread *saveScreenShotThread;
-	Mutex saveScreenShotThreadAccessor;
+	Mutex *saveScreenShotThreadAccessor;
 	std::list<std::pair<string,Pixmap2D *> > saveScreenQueue;
 
 	std::map<Vec3f,Vec3f> worldToScreenPosCache;
@@ -367,6 +367,28 @@ private:
 				vbo_fowTexCoords(0), vbo_surfTexCoords(0),
 				vbo_indices(0), indexCount(0),
 				textureHandle(th),textureCRC(0) {}
+
+			inline Layer & operator=(Layer &obj) {
+				this->vertices = obj.vertices;
+				this->normals = obj.normals;
+				this->fowTexCoords = obj.fowTexCoords; 
+				this->surfTexCoords = obj.surfTexCoords;
+				this->indices = obj.indices;
+				this->cellToIndicesMap = obj.cellToIndicesMap;
+				this->rowsToRenderCache = obj.rowsToRenderCache;
+				this->vbo_vertices = obj.vbo_vertices;
+				this->vbo_normals = obj.vbo_normals;
+				this->vbo_fowTexCoords = obj.vbo_fowTexCoords;
+				this->vbo_surfTexCoords = obj.vbo_surfTexCoords;
+				this->vbo_indices = obj.vbo_indices;
+				this->indexCount = obj.indexCount;
+				this->textureHandle = obj.textureHandle;
+				this->texturePath = obj.texturePath;
+				this->textureCRC = obj.textureCRC;
+
+				return *this;
+			}
+
 			~Layer();
 			void load_vbos(bool vboEnabled);
 			void render(VisibleQuadContainerCache &qCache);
@@ -382,7 +404,7 @@ private:
 				vbo_fowTexCoords, vbo_surfTexCoords,
 				vbo_indices;
 			int indexCount;
-			const int textureHandle;
+			int textureHandle;
 			string texturePath;
 			uint32 textureCRC;
 		};
@@ -436,7 +458,7 @@ public:
 	void endTexture(ResourceScope rs, Texture *texture,bool mustExistInList=false);
 	void endLastTexture(ResourceScope rs, bool mustExistInList=false);
 
-	Model *newModel(ResourceScope rs);
+	Model *newModel(ResourceScope rs,const string &path,bool deletePixMapAfterLoad=false,std::map<string,vector<pair<string, string> > > *loadedFileList=NULL, string *sourceLoader=NULL);
 	void endModel(ResourceScope rs, Model *model, bool mustExistInList=false);
 	void endLastModel(ResourceScope rs, bool mustExistInList=false);
 
@@ -444,7 +466,7 @@ public:
 	Texture3D *newTexture3D(ResourceScope rs);
 	Font2D *newFont(ResourceScope rs);
 	Font3D *newFont3D(ResourceScope rs);
-	void endFont(Shared::Graphics::Font *font, ResourceScope rs, bool mustExistInList=false);
+	void endFont(::Shared::Graphics::Font *font, ResourceScope rs, bool mustExistInList=false);
 	void resetFontManager(ResourceScope rs);
 
 	inline TextRenderer2D *getTextRenderer() const	{return textRenderer;}
@@ -454,6 +476,7 @@ public:
 	void cleanupParticleSystems(vector<ParticleSystem *> &particleSystems,ResourceScope rs);
 	void cleanupUnitParticleSystems(vector<UnitParticleSystem *> &particleSystems,ResourceScope rs);
 	bool validateParticleSystemStillExists(ParticleSystem * particleSystem,ResourceScope rs) const;
+	void removeParticleSystemsForParticleOwner(ParticleOwner * particleOwner,ResourceScope rs);
 	void updateParticleManager(ResourceScope rs,int renderFps=-1);
 	void renderParticleManager(ResourceScope rs);
 	void swapBuffers();
@@ -479,6 +502,7 @@ public:
 
 	void renderChatManager(const ChatManager *chatManager);
 	void renderClock();
+	void renderPerformanceStats();
 	void renderResourceStatus();
 	void renderSelectionQuad();
 	void renderText(const string &text, Font2D *font, float alpha, int x, int y, bool centered= false);
@@ -521,7 +545,7 @@ public:
 	void renderObjects(const int renderFps);
 
 	void renderWater();
-    void renderUnits(const int renderFps);
+    void renderUnits(bool airUnits, const int renderFps);
     void renderUnitsToBuild(const int renderFps);
 
 	void renderSelectionEffects();
@@ -603,7 +627,7 @@ public:
 	void removeObjectFromQuadCache(const Object *o);
 	void removeUnitFromQuadCache(const Unit *unit);
 
-	uint64 getCurrentPixelByteCount(ResourceScope rs=rsGame) const;
+	std::size_t getCurrentPixelByteCount(ResourceScope rs=rsGame) const;
 	unsigned int getSaveScreenQueueSize();
 
 	Texture2D *saveScreenToTexture(int x, int y, int width, int height);
@@ -612,7 +636,7 @@ public:
 
 	static Texture2D * findTexture(string logoFilename);
 	static Texture2D * preloadTexture(string logoFilename);
-	inline int getCachedSurfaceDataSize() const { return mapSurfaceData.size(); }
+	inline int getCachedSurfaceDataSize() const { return (int)mapSurfaceData.size(); }
 
 	//void setCustom3dMenuList(GLuint *customlist3dMenu) { this->customlist3dMenu = customlist3dMenu; }
 	//inline GLuint * getCustom3dMenuList() const { return this->customlist3dMenu; }
@@ -661,7 +685,7 @@ private:
 	void renderTile(const Vec2i &pos);
 	void renderQuad(int x, int y, int w, int h, const Texture2D *texture);
 
-	void simpleTask(BaseThread *callingThread);
+	void simpleTask(BaseThread *callingThread,void *userdata);
 
 	//static
     static Texture2D::Filter strToTextureFilter(const string &s);
