@@ -345,16 +345,19 @@ void Gui::groupKey(int groupIndex) {
 	else{
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] groupIndex = %d\n",__FILE__,__FUNCTION__,__LINE__,groupIndex);
 
-		Config &config = Config::getInstance();
-		int recallGroupCenterCameraTimeout = config.getInt("RecallGroupCenterCameraTimeoutMilliseconds","1500");
+		int recallGroupCenterCameraTimeout = Config::getInstance().getInt("RecallGroupCenterCameraTimeoutMilliseconds","1500");
+
 		if(lastGroupRecall == groupIndex &&
 			lastGroupRecallTime.getMillis() > 0 &&
 			lastGroupRecallTime.getMillis() <= recallGroupCenterCameraTimeout) {
+
+			selection.recallGroup(groupIndex);
 			centerCameraOnSelection();
 		}
 		else {
 			selection.recallGroup(groupIndex);
 		}
+
 		lastGroupRecallTime.start();
 		lastGroupRecall = groupIndex;
 	}
@@ -437,15 +440,18 @@ void Gui::giveOneClickOrders(){
 }
 
 void Gui::giveDefaultOrders(int x, int y) {
-
 	//compute target
+	//printf("In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
 	const Unit *targetUnit= NULL;
 	Vec2i targetPos;
 	if(computeTarget(Vec2i(x, y), targetPos, targetUnit) == false) {
 		console->addStdMessage("InvalidPosition");
 		return;
 	}
+	//printf("In [%s::%s Line: %d] targetUnit = %p\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,targetUnit);
 	giveDefaultOrders(targetPos.x,targetPos.y,targetUnit,true);
+	//printf("In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 }
 
 void Gui::givePreparedDefaultOrders(int x, int y){
@@ -457,9 +463,13 @@ void Gui::giveDefaultOrders(int x, int y,const Unit *targetUnit, bool paintMouse
 	//printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 	bool queueKeyDown = isKeyDown(queueCommandKey);
 	Vec2i targetPos=Vec2i(x, y);
+
 	//give order
+	//printf("In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 	std::pair<CommandResult,string> result= commander->tryGiveCommand(&selection, targetPos, targetUnit, queueKeyDown);
 
+	//printf("In [%s::%s Line: %d] selected units = %d result.first = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,selection.getCount(),result.first);
+
 	//graphical result
 	addOrdersResultToConsole(activeCommandClass, result);
 	if(result.first == crSuccess || result.first == crSomeFailed) {
@@ -605,6 +615,29 @@ void Gui::mouseDownDisplayUnitSkills(int posDisplay) {
 				//uniform selection
 				if(selection.isUniform()) {
 				    const CommandType *ct = display.getCommandType(posDisplay);
+					if(activeCommandClass == ccAttack)
+					{// try to switch to next attack type
+						int maxI=unit->getType()->getCommandTypeCount();
+						int i=activeCommandType->getId();
+						int k=i+1;
+						while(k!=i)
+						{
+							if(k>=maxI)
+							{
+								k=0;
+							}
+							const CommandType *ctype=display.getCommandType(k);
+							if(ctype != NULL && ctype->getClass() == ccAttack)
+							{
+								if(ctype != NULL && unit->getFaction()->reqsOk(ctype)) {
+									posDisplay=k;
+									break;
+								}
+							}
+							k++;
+						}
+					}
+
 					if(ct != NULL && unit->getFaction()->reqsOk(ct)) {
 						activeCommandType= display.getCommandType(posDisplay);
 						activeCommandClass= activeCommandType->getClass();
@@ -682,7 +715,7 @@ string Gui::computeDefaultInfoString() {
 	if(selection.isUniform()) {
 		if(selection.isObserver() || selection.isCommandable()) {
 			// default is the description extension
-			result = selection.getFrontUnit()->getDescExtension();
+			result = selection.getFrontUnit()->getDescExtension(game->showTranslatedTechTree());
 		}
 	}
 	return result;
@@ -700,10 +733,10 @@ void Gui::computeInfoString(int posDisplay){
 	if(posDisplay!=invalidPos && selection.isCommandable()){
 		if(!selectingBuilding){
 			if(posDisplay==cancelPos){
-				display.setInfoText(lang.get("Cancel"));
+				display.setInfoText(lang.getString("Cancel"));
 			}
 			else if(posDisplay==meetingPointPos){
-				display.setInfoText(lang.get("MeetingPoint"));
+				display.setInfoText(lang.getString("MeetingPoint"));
 			}
 			else{
 				//uniform selection
@@ -713,22 +746,22 @@ void Gui::computeInfoString(int posDisplay){
 
 					if(ct!=NULL){
 						if(unit->getFaction()->reqsOk(ct)){
-							display.setInfoText(ct->getDesc(unit->getTotalUpgrade()));
+							display.setInfoText(ct->getDesc(unit->getTotalUpgrade(),game->showTranslatedTechTree()));
 						}
 						else{
 							if(ct->getClass()==ccUpgrade){
 								string text="";
 								const UpgradeCommandType *uct= static_cast<const UpgradeCommandType*>(ct);
 								if(unit->getFaction()->getUpgradeManager()->isUpgrading(uct->getProducedUpgrade())){
-									text=lang.get("Upgrading")+"\n\n";
+									text=lang.getString("Upgrading")+"\n\n";
 								}
 								else if(unit->getFaction()->getUpgradeManager()->isUpgraded(uct->getProducedUpgrade())){
-									text=lang.get("AlreadyUpgraded")+"\n\n";
+									text=lang.getString("AlreadyUpgraded")+"\n\n";
 								}
-								display.setInfoText(text+ct->getReqDesc());
+								display.setInfoText(text+ct->getReqDesc(game->showTranslatedTechTree()));
 							}
 							else{
-								display.setInfoText(ct->getReqDesc());
+								display.setInfoText(ct->getReqDesc(game->showTranslatedTechTree()));
 							}
 						}
 					}
@@ -739,19 +772,19 @@ void Gui::computeInfoString(int posDisplay){
 					const UnitType *ut= selection.getFrontUnit()->getType();
 					CommandClass cc= display.getCommandClass(posDisplay);
 					if(cc!=ccNull){
-						display.setInfoText(lang.get("CommonCommand") + ": " + ut->getFirstCtOfClass(cc)->toString());
+						display.setInfoText(lang.getString("CommonCommand") + ": " + ut->getFirstCtOfClass(cc)->toString(true));
 					}
 				}
 			}
 		}
 		else{
 			if(posDisplay==cancelPos){
-				display.setInfoText(lang.get("Return"));
+				display.setInfoText(lang.getString("Return"));
 			}
 			else{
 				if(activeCommandType!=NULL && activeCommandType->getClass()==ccBuild){
 					const BuildCommandType *bct= static_cast<const BuildCommandType*>(activeCommandType);
-					display.setInfoText(bct->getBuilding(posDisplay)->getReqDesc());
+					display.setInfoText(bct->getBuilding(posDisplay)->getReqDesc(game->showTranslatedTechTree()));
 				}
 			}
 		}
@@ -769,16 +802,16 @@ void Gui::computeDisplay(){
 	const Object *selectedResourceObject =getSelectedResourceObject();
 	if(selection.isEmpty() && selectedResourceObject != NULL && selectedResourceObject->getResource() != NULL) {
 		Resource *r = selectedResourceObject->getResource();
-		display.setTitle(r->getType()->getName(true));
-		display.setText(lang.get("Amount")+ ": "+intToStr(r->getAmount())+" / "+intToStr(r->getType()->getDefResPerPatch()));
+		display.setTitle(r->getType()->getName(game->showTranslatedTechTree()));
+		display.setText(lang.getString("Amount")+ ": "+intToStr(r->getAmount())+" / "+intToStr(r->getType()->getDefResPerPatch()));
 		//display.setProgressBar(r->);
 		display.setUpImage(0, r->getType()->getImage());
 	}
 	else {
 		//title, text and progress bar
 		if(selection.getCount() == 1){
-			display.setTitle(selection.getFrontUnit()->getFullName());
-			display.setText(selection.getFrontUnit()->getDesc());
+			display.setTitle(selection.getFrontUnit()->getFullName(game->showTranslatedTechTree()));
+			display.setText(selection.getFrontUnit()->getDesc(game->showTranslatedTechTree()));
 			display.setProgressBar(selection.getFrontUnit()->getProductionPercent());
 		}
 
@@ -1020,6 +1053,7 @@ void Gui::computeSelected(bool doubleClick, bool force){
 		const Object *selectedResourceObject=NULL;
 		selectedResourceObjectPos=Vec2i(-1,-1);
 		if(selectionQuad.isEnabled() && selectionQuad.getPosUp().dist(selectionQuad.getPosDown())<minQuadSize){
+			//Renderer::getInstance().computeSelected(units, selectedResourceObject, true, selectionQuad.getPosDown(), selectionQuad.getPosDown());
 			Renderer::getInstance().computeSelected(units, selectedResourceObject, true, selectionQuad.getPosDown(), selectionQuad.getPosDown());
 			if(selectedResourceObject!=NULL){
 				selectedResourceObjectPos=Map::toSurfCoords(selectedResourceObject->getMapPos());
@@ -1037,7 +1071,7 @@ void Gui::computeSelected(bool doubleClick, bool force){
 			int factionIndex= refUnit->getFactionIndex();
 			for(int i=0; i<world->getFaction(factionIndex)->getUnitCount(); ++i){
 				Unit *unit= world->getFaction(factionIndex)->getUnit(i);
-				if(unit->getPos().dist(refUnit->getPos())<doubleClickSelectionRadius &&
+				if(unit->getPos().dist(refUnit->getPosNotThreadSafe())<doubleClickSelectionRadius &&
 					unit->getType()==refUnit->getType())
 				{
 					units.push_back(unit);
@@ -1076,7 +1110,7 @@ bool Gui::computeTarget(const Vec2i &screenPos, Vec2i &targetPos, const Unit *&t
 
 	if(uc.empty() == false){
 		targetUnit= getRelevantObjectFromSelection(&uc);
-		targetPos= targetUnit->getPos();
+		targetPos= targetUnit->getPosNotThreadSafe();
 		highlightedUnitId=targetUnit->getId();
 		getHighlightedUnit()->resetHighlight();
 		return true;
@@ -1084,94 +1118,100 @@ bool Gui::computeTarget(const Vec2i &screenPos, Vec2i &targetPos, const Unit *&t
 	else if(obj != NULL){
 		targetUnit= NULL;
 		highlightedResourceObjectPos=Map::toSurfCoords(obj->getMapPos());
-		getHighlightedResourceObject()->resetHighlight();
-		// get real click pos
-		renderer.computePosition(screenPos, targetPos);
 
-		//validPosObjWorld= true;
-		//posObjWorld = targetPos;
+		Object *selObj = getHighlightedResourceObject();
+		if(selObj != NULL) {
+			selObj->resetHighlight();
+			// get real click pos
+			renderer.computePosition(screenPos, targetPos);
 
-		int tx= targetPos.x;
-		int ty= targetPos.y;
+			//validPosObjWorld= true;
+			//posObjWorld = targetPos;
 
-		int ox= obj->getMapPos().x;
-		int oy= obj->getMapPos().y;
+			int tx= targetPos.x;
+			int ty= targetPos.y;
 
-		Resource* clickedRecource= world->getMap()->getSurfaceCell(Map::toSurfCoords(obj->getMapPos()))->getResource();
+			int ox= obj->getMapPos().x;
+			int oy= obj->getMapPos().y;
 
-		// lets see if the click had the same Resource
-		if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(targetPos))->getResource()){
-			// same ressource is meant, so use the user selected position
+			Resource* clickedRecource= world->getMap()->getSurfaceCell(Map::toSurfCoords(obj->getMapPos()))->getResource();
+
+			// lets see if the click had the same Resource
+			if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(targetPos))->getResource()){
+				// same ressource is meant, so use the user selected position
+				return true;
+			}
+			else{// calculate a valid resource position which is as near as possible to the selected position
+				Vec2i testIt= Vec2i(obj->getMapPos());
+				///////////////
+				// test both //
+				///////////////
+				if(ty < oy){
+					testIt.y--;
+				}
+				else if(ty > oy){
+					testIt.y++;
+				}
+				if(tx < ox){
+					testIt.x--;
+				}
+				else if(tx > ox){
+					testIt.x++;
+				}
+
+				if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(testIt))->getResource()){
+					// same ressource is meant, so use this position
+					targetPos= testIt;
+					//posObjWorld= targetPos;
+					return true;
+				}
+				else{
+					testIt= Vec2i(obj->getMapPos());
+				}
+
+				/////////////////
+				// test y-only //
+				/////////////////
+				if(ty < oy){
+					testIt.y--;
+				}
+				else if(ty > oy){
+					testIt.y++;
+				}
+				if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(testIt))->getResource()){
+					// same ressource is meant, so use this position
+					targetPos= testIt;
+					//posObjWorld= targetPos;
+					return true;
+				}
+				else{
+					testIt= Vec2i(obj->getMapPos());
+				}
+
+				/////////////////
+				// test x-only //
+				/////////////////
+				if(tx < ox){
+					testIt.x--;
+				}
+				else if(tx > ox){
+					testIt.x++;
+				}
+				if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(testIt))->getResource()){
+					// same ressource is meant, so use this position
+					targetPos= testIt;
+					//posObjWorld= targetPos;
+					return true;
+				}
+			}
+			// give up and use the object position;
+			targetPos= obj->getMapPos();
+			posObjWorld= targetPos;
 			return true;
 		}
-		else{// calculate a valid resource position which is as near as possible to the selected position
-			Vec2i testIt= Vec2i(obj->getMapPos());
-			///////////////
-			// test both //
-			///////////////
-			if(ty < oy){
-				testIt.y--;
-			}
-			else if(ty > oy){
-				testIt.y++;
-			}
-			if(tx < ox){
-				testIt.x--;
-			}
-			else if(tx > ox){
-				testIt.x++;
-			}
-
-			if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(testIt))->getResource()){
-				// same ressource is meant, so use this position
-				targetPos= testIt;
-				//posObjWorld= targetPos;
-				return true;
-			}
-			else{
-				testIt= Vec2i(obj->getMapPos());
-			}
-
-			/////////////////
-			// test y-only //
-			/////////////////
-			if(ty < oy){
-				testIt.y--;
-			}
-			else if(ty > oy){
-				testIt.y++;
-			}
-			if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(testIt))->getResource()){
-				// same ressource is meant, so use this position
-				targetPos= testIt;
-				//posObjWorld= targetPos;
-				return true;
-			}
-			else{
-				testIt= Vec2i(obj->getMapPos());
-			}
-
-			/////////////////
-			// test x-only //
-			/////////////////
-			if(tx < ox){
-				testIt.x--;
-			}
-			else if(tx > ox){
-				testIt.x++;
-			}
-			if(clickedRecource == world->getMap()->getSurfaceCell(Map::toSurfCoords(testIt))->getResource()){
-				// same ressource is meant, so use this position
-				targetPos= testIt;
-				//posObjWorld= targetPos;
-				return true;
-			}
+		else {
+			return false;
 		}
-		// give up and use the object position;
-		targetPos= obj->getMapPos();
-		posObjWorld= targetPos;
-		return true;
-
 	}
 	else{
 		targetUnit= NULL;
@@ -1179,10 +1219,13 @@ bool Gui::computeTarget(const Vec2i &screenPos, Vec2i &targetPos, const Unit *&t
 			validPosObjWorld= true;
 			posObjWorld= targetPos;
 
-			if(world->getMap()->getSurfaceCell(Map::toSurfCoords(targetPos))->getResource()!=NULL)
-			{
+			if(world->getMap()->getSurfaceCell(Map::toSurfCoords(targetPos))->getResource() != NULL) {
 				highlightedResourceObjectPos=Map::toSurfCoords(targetPos);
-				getHighlightedResourceObject()->resetHighlight();
+
+				Object *selObj = getHighlightedResourceObject();
+				if(selObj != NULL) {
+					selObj->resetHighlight();
+				}
 			}
 
 			return true;
@@ -1195,7 +1238,7 @@ bool Gui::computeTarget(const Vec2i &screenPos, Vec2i &targetPos, const Unit *&t
 
 Unit* Gui::getRelevantObjectFromSelection(Selection::UnitContainer *uc){
 	Unit *resultUnit=NULL;
-	for(int i= 0; i < uc->size(); ++i){
+	for(int i= 0; i < (int)uc->size(); ++i) {
 		resultUnit= uc->at(i);
 		if(resultUnit->getType()->hasSkillClass(scMove)){// moving units are more relevant than non moving ones
 			break;
@@ -1226,12 +1269,12 @@ void Gui::saveGame(XmlNode *rootNode) const {
 //	const UnitType *choosenBuildingType;
 	if(choosenBuildingType != NULL) {
 		const Faction* thisFaction= world->getThisFaction();
-		guiNode->addAttribute("choosenBuildingType",choosenBuildingType->getName(), mapTagReplacements);
+		guiNode->addAttribute("choosenBuildingType",choosenBuildingType->getName(false), mapTagReplacements);
 		guiNode->addAttribute("choosenBuildingTypeFactionIndex",intToStr(thisFaction->getIndex()), mapTagReplacements);
 	}
 //	const CommandType *activeCommandType;
 	if(activeCommandType != NULL) {
-		guiNode->addAttribute("activeCommandType",activeCommandType->getName(), mapTagReplacements);
+		guiNode->addAttribute("activeCommandType",activeCommandType->getName(false), mapTagReplacements);
 	}
 
 //	CommandClass activeCommandClass;
