@@ -41,49 +41,70 @@ bool Selection::select(Unit *unit) {
 	bool result = false;
 	//check size
 	//if(selectedUnits.size() >= maxUnits){
-	if(selectedUnits.size() >= Config::getInstance().getInt("MaxUnitSelectCount",intToStr(maxUnits).c_str())) {
+	if((int)selectedUnits.size() >= Config::getInstance().getInt("MaxUnitSelectCount",intToStr(maxUnits).c_str())) {
 		return result;
 	}
 
-	//check if already selected
-	for(int i=0; i < selectedUnits.size(); ++i) {
-		if(selectedUnits[i ]== unit) {
-			return true;
+	// Fix Bug reported on sourceforge.net: Glest::Game::Selection::select crash with NULL pointer - ID: 3608835
+	if(unit != NULL) {
+		//check if already selected
+		for(int i=0; i < (int)selectedUnits.size(); ++i) {
+			if(selectedUnits[i ]== unit) {
+				return true;
+			}
 		}
+
+		//check if dead
+		if(unit->isDead()) {
+			return false;
+		}
+
+		//check if multisel
+		if(!unit->getType()->getMultiSelect() && !isEmpty()) {
+			return false;
+		}
+
+		//check if enemy
+		if(unit->getFactionIndex() != factionIndex && !isEmpty()) {
+			return false;
+		}
+
+		//check existing enemy
+		if(selectedUnits.size()==1 && selectedUnits.front()->getFactionIndex() != factionIndex) {
+			clear();
+		}
+
+		//check existing multisel
+		if(selectedUnits.size()==1 && !selectedUnits.front()->getType()->getMultiSelect()){
+			clear();
+		}
+
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] unit selected [%s]\n",__FILE__,__FUNCTION__,__LINE__,unit->toString().c_str());
+
+		unit->addObserver(this);
+
+		int unitTypeId=unit->getType()->getId();
+		bool inserted=false;
+		for(int i=0; i < (int)selectedUnits.size(); ++i) {
+			int currentTypeId=selectedUnits[i]->getType()->getId();
+
+			if(unitTypeId<=currentTypeId) {
+				//place unit here
+				selectedUnits.insert(selectedUnits.begin()+i,unit);
+				inserted=true;
+				break;
+			}
+			else if(unitTypeId>currentTypeId) {
+				//do nothing
+			}
+		}
+		if( !inserted ){
+			selectedUnits.push_back(unit);
+		}
+		result = true;
+		gui->onSelectionChanged();
 	}
 
-	//check if dead
-	if(unit->isDead()) {
-		return false;
-	}
-
-	//check if multisel
-	if(!unit->getType()->getMultiSelect() && !isEmpty()) {
-		return false;
-	}
-
-	//check if enemy
-	if(unit->getFactionIndex() != factionIndex && !isEmpty()) {
-		return false;
-	}
-
-	//check existing enemy
-	if(selectedUnits.size()==1 && selectedUnits.front()->getFactionIndex() != factionIndex) {
-		clear();
-	}
-
-	//check existing multisel
-	if(selectedUnits.size()==1 && !selectedUnits.front()->getType()->getMultiSelect()){
-		clear();
-	}
-
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] unit selected [%s]\n",__FILE__,__FUNCTION__,__LINE__,unit->toString().c_str());
-
-	unit->addObserver(this);
-	selectedUnits.push_back(unit);
-	result = true;
-	gui->onSelectionChanged();
-
 	return result;
 }
 
@@ -99,8 +120,8 @@ void Selection::unSelect(const UnitContainer &units){
 
 	//add units to gui
 	for(UnitIterator it= units.begin(); it!=units.end(); ++it){
-		for(int i=0; i<selectedUnits.size(); ++i){
-			if(selectedUnits[i]==*it){
+		for(int i = 0; i < (int)selectedUnits.size(); ++i) {
+			if(selectedUnits[i] == *it) {
 				unSelect(i);
 			}
 		}
@@ -125,7 +146,7 @@ bool Selection::isUniform() const{
 
 	const UnitType *ut= selectedUnits.front()->getType();
 
-	for(int i=0; i<selectedUnits.size(); ++i){
+	for(int i = 0; i < (int)selectedUnits.size(); ++i) {
 		if(selectedUnits[i]->getType()!=ut){
             return false;
 		}
@@ -228,7 +249,7 @@ void Selection::recallGroup(int groupIndex){
 	}
 
 	clear();
-	for(int i=0; i<groups[groupIndex].size(); ++i) {
+	for(int i = 0; i < (int)groups[groupIndex].size(); ++i) {
 		select(groups[groupIndex][i]);
 	}
 }
@@ -238,7 +259,7 @@ void Selection::unitEvent(UnitObserver::Event event, const Unit *unit) {
 	if(event==UnitObserver::eKill){
 
 		//remove from selection
-		for(int i=0; i<selectedUnits.size(); ++i){
+		for(int i = 0; i < (int)selectedUnits.size(); ++i) {
 			if(selectedUnits[i]==unit){
 				selectedUnits.erase(selectedUnits.begin()+i);
 				break;
@@ -247,7 +268,7 @@ void Selection::unitEvent(UnitObserver::Event event, const Unit *unit) {
 
 		//remove from groups
 		for(int i=0; i<maxGroups; ++i){
-			for(int j=0; j<groups[i].size(); ++j){
+			for(int j = 0; j < (int)groups[i].size(); ++j) {
 				if(groups[i][j]==unit){
 					groups[i].erase(groups[i].begin()+j);
 					break;
@@ -257,7 +278,7 @@ void Selection::unitEvent(UnitObserver::Event event, const Unit *unit) {
 
 		//notify gui only if no more units to execute the command
 		//of course the selection changed, but this doesn't matter in this case.
-		if( selectedUnits.size()<1 ){
+		if( selectedUnits.empty() == true ){
 			gui->onSelectionChanged();
 		}
 
@@ -280,9 +301,9 @@ void Selection::saveGame(XmlNode *rootNode) const {
 		selectedUnitsNode->addAttribute("unitId",intToStr(unit->getId()), mapTagReplacements);
 	}
 //	UnitContainer groups[maxGroups];
-	for(unsigned int x = 0; x < maxGroups; ++x) {
+	for(unsigned int x = 0; x < (unsigned int)maxGroups; ++x) {
 		XmlNode *groupsNode = selectionNode->addChild("groups");
-		for(unsigned int i = 0; i < groups[x].size(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)groups[x].size(); ++i) {
 			Unit *unit = groups[x][i];
 
 			XmlNode *selectedUnitsNode = groupsNode->addChild("selectedUnits");
