@@ -96,7 +96,7 @@ Intro::Intro(Program *program):
 	SoundRenderer &soundRenderer= SoundRenderer::getInstance();
 
 	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false &&
-		(Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == false ||
+		(::Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == false ||
 		CoreData::getInstance().hasIntroVideoFilename() == false)) {
 		soundRenderer.playMusic(CoreData::getInstance().getIntroMusic());
 	}
@@ -206,12 +206,11 @@ Intro::Intro(Program *program):
 		string introPath = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/main_model/intro*.g3d";
 		vector<string> introModels;
 		findAll(introPath, introModels, false, false);
-		for(int i = 0; i < introModels.size(); ++i) {
+		for(int i = 0; i < (int)introModels.size(); ++i) {
 			string logo = introModels[i];
-			Model *model= renderer.newModel(rsMenu);
-			if(model) {
-				model->load(getGameCustomCoreDataPath(data_path, "") + "data/core/menu/main_model/" + logo);
-				models.push_back(model);
+			Model *model= renderer.newModel(rsMenu, getGameCustomCoreDataPath(data_path, "") + "data/core/menu/main_model/" + logo);
+            if(model) {
+                models.push_back(model);
 				//printf("#1 Intro model [%s]\n",model->getFileName().c_str());
 			}
 		}
@@ -220,11 +219,10 @@ Intro::Intro(Program *program):
 			introPath = data_path + "data/core/menu/main_model/intro*.g3d";
 			//vector<string> introModels;
 			findAll(introPath, introModels, false, false);
-			for(int i = 0; i < introModels.size(); ++i) {
+			for(int i = 0; i < (int)introModels.size(); ++i) {
 				string logo = introModels[i];
-				Model *model= renderer.newModel(rsMenu);
-				if(model) {
-					model->load(data_path + "data/core/menu/main_model/" + logo);
+				Model *model= renderer.newModel(rsMenu, data_path + "data/core/menu/main_model/" + logo);
+                if(model) {
 					models.push_back(model);
 					//printf("#2 Intro model [%s]\n",model->getFileName().c_str());
 				}
@@ -249,7 +247,7 @@ Intro::Intro(Program *program):
 				//printf("picIndex = %d list count = %d\n",picIndex,coreData.getMiscTextureList().size());
 				modelList.push_back(models[index]);
 				usedIndex[index]=true;
-				srand((unsigned int)seed.getCurTicks() / modelList.size());
+				srand((unsigned int)seed.getCurTicks() / (unsigned int)modelList.size());
 			}
 			models = modelList;
 		}
@@ -259,7 +257,7 @@ Intro::Intro(Program *program):
 	int appear= Intro::appearTime;
 	int disappear= Intro::showTime+Intro::appearTime+(Intro::disapearTime * 2);
 
-	const int maxIntroLines = 100;
+	const unsigned int maxIntroLines = 100;
 	Lang &lang= Lang::getInstance();
 	for(unsigned int i = 1; i < maxIntroLines; ++i) {
 		string introTagName 		= "IntroText" + intToStr(i);
@@ -270,14 +268,14 @@ Intro::Intro(Program *program):
 			string lineText = "";
 
 			if(lang.hasString(introTagName,"",true) == true) {
-				lineText = lang.get(introTagName,"",true);
+				lineText = lang.getString(introTagName,"",true);
 			}
 
 			string showStartTime = "IntroStartMilliseconds" + intToStr(i);
 
 			int displayTime = appear;
 			if(lang.hasString(showStartTime,"",true) == true) {
-				displayTime = strToInt(lang.get(showStartTime,"",true));
+				displayTime = strToInt(lang.getString(showStartTime,"",true));
 			}
 			else {
 				if(i == 1) {
@@ -298,7 +296,7 @@ Intro::Intro(Program *program):
 				string introTagTextureWidthName 	= "IntroTextureWidth" + intToStr(i);
 				string introTagTextureHeightName 	= "IntroTextureHeight" + intToStr(i);
 
-				lineText = lang.get(introTagTextureName,"",true);
+				lineText = lang.getString(introTagTextureName,"",true);
 				Texture2D *logoTexture= renderer.newTexture2D(rsGlobal);
 				if(logoTexture) {
 					logoTexture->setMipmap(false);
@@ -313,7 +311,7 @@ Intro::Intro(Program *program):
 					textureWidth = logoTexture->getTextureWidth();
 				}
 				if(lang.hasString(introTagTextureWidthName,"",true) == true) {
-					textureWidth = strToInt(lang.get(introTagTextureWidthName,"",true));
+					textureWidth = strToInt(lang.getString(introTagTextureWidthName,"",true));
 				}
 
 				int textureHeight = 128;
@@ -321,7 +319,7 @@ Intro::Intro(Program *program):
 					textureHeight = logoTexture->getTextureHeight();
 				}
 				if(lang.hasString(introTagTextureHeightName,"",true) == true) {
-					textureHeight = strToInt(lang.get(introTagTextureHeightName,"",true));
+					textureHeight = strToInt(lang.getString(introTagTextureHeightName,"",true));
 				}
 
 				texts.push_back(new Text(logoTexture, Vec2i(w/2-(textureWidth/2), h/2-(textureHeight/2)), Vec2i(textureWidth, textureHeight), displayTime));
@@ -334,7 +332,7 @@ Intro::Intro(Program *program):
 
 				int textX = -1;
 				if(lang.hasString(introTagTextXName,"",true) == true) {
-					string value = lang.get(introTagTextXName,"",true);
+					string value = lang.getString(introTagTextXName,"",true);
 					if(value.length() > 0 &&
 							(value[0] == '+' || value[0] == '-')) {
 						textX = w/2 + strToInt(value);
@@ -346,7 +344,7 @@ Intro::Intro(Program *program):
 
 				int textY = -1;
 				if(lang.hasString(introTagTextYName,"",true) == true) {
-					string value = lang.get(introTagTextYName,"",true);
+					string value = lang.getString(introTagTextYName,"",true);
 					if(value.length() > 0 &&
 							(value[0] == '+' || value[0] == '-')) {
 						textY = h/2 + strToInt(value);
@@ -360,7 +358,7 @@ Intro::Intro(Program *program):
 				Font3D *font3d = coreData.getMenuFontVeryBig3D();
 
 				if(lang.hasString(introTagTextFontTypeName,"",true) == true) {
-					string value =lang.get(introTagTextFontTypeName,"",true);
+					string value =lang.getString(introTagTextFontTypeName,"",true);
 					if(value == "displaynormal") {
 						font = coreData.getDisplayFont();
 						font3d = coreData.getDisplayFont3D();
@@ -396,7 +394,7 @@ Intro::Intro(Program *program):
 	}
 	modelShowTime = disappear *(displayItemNumber);
 	if(lang.hasString("IntroModelStartMilliseconds","",true) == true) {
-		modelShowTime = strToInt(lang.get("IntroModelStartMilliseconds","",true));
+		modelShowTime = strToInt(lang.getString("IntroModelStartMilliseconds","",true));
 	}
 	else {
 		modelShowTime = disappear *(displayItemNumber);
@@ -429,7 +427,7 @@ Intro::Intro(Program *program):
 			srand((unsigned int)seed.getCurTicks());
 			int failedLookups=0;
 			std::map<int,bool> usedIndex;
-			for(;intoTexList.size() < showIntroPics;) {
+			for(;(int)intoTexList.size() < showIntroPics;) {
 				int picIndex = rand() % coreData.getMiscTextureList().size();
 				if(usedIndex.find(picIndex) != usedIndex.end()) {
 					failedLookups++;
@@ -451,13 +449,13 @@ Intro::Intro(Program *program):
 				//printf("picIndex = %d list count = %d\n",picIndex,coreData.getMiscTextureList().size());
 				intoTexList.push_back(coreData.getMiscTextureList()[picIndex]);
 				usedIndex[picIndex]=true;
-				srand((unsigned int)seed.getCurTicks() / intoTexList.size());
+				srand((unsigned int)seed.getCurTicks() / (unsigned int)intoTexList.size());
 			}
 		}
 		else {
 			for(unsigned int i = 0;
-					i < coreData.getMiscTextureList().size() &&
-					i < showIntroPics; ++i) {
+					i < (unsigned int)coreData.getMiscTextureList().size() &&
+					i < (unsigned int)showIntroPics; ++i) {
 				Texture2D *tex = coreData.getMiscTextureList()[i];
 				intoTexList.push_back(tex);
 			}
@@ -495,7 +493,7 @@ Intro::Intro(Program *program):
 
 			int textureStartTime = disappear * displayItemNumber;
 			if(lang.hasString("IntroTextureStartMilliseconds","",true) == true) {
-				textureStartTime = strToInt(lang.get("IntroTextureStartMilliseconds","",true));
+				textureStartTime = strToInt(lang.getString("IntroTextureStartMilliseconds","",true));
 			}
 
 			texts.push_back(new Text(tex, texPlacement, Vec2i(tex->getTextureWidth(), tex->getTextureHeight()), textureStartTime +(showMiscTime*(i+1))));
@@ -509,7 +507,7 @@ Intro::Intro(Program *program):
 	SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false &&
-		Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == true &&
+		::Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == true &&
 		CoreData::getInstance().hasIntroVideoFilename() == true) {
 		string introVideoFile = CoreData::getInstance().getIntroVideoFilename();
 		string introVideoFileFallback = CoreData::getInstance().getIntroVideoFilenameFallback();
@@ -521,7 +519,7 @@ Intro::Intro(Program *program):
 
 		string vlcPluginsPath = Config::getInstance().getString("VideoPlayerPluginsPath","");
 		//printf("screen->w = %d screen->h = %d screen->format->BitsPerPixel = %d\n",screen->w,screen->h,screen->format->BitsPerPixel);
-		Shared::Graphics::VideoPlayer player(
+		::Shared::Graphics::VideoPlayer player(
 				&Renderer::getInstance(),
 				introVideoFile,
 				introVideoFileFallback,
@@ -571,8 +569,6 @@ void Intro::update() {
 
 		AutoTest::getInstance().updateIntro(program);
 		return;
-
-		SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 	}
 
 	mouse2d= (mouse2d+1) % Renderer::maxMouse2dAnim;
@@ -622,7 +618,7 @@ void Intro::renderModelBackground() {
 	if(models.empty() == false) {
 		int difTime= 1000 * timer / GameConstants::updateFps - modelShowTime;
 		int totalModelShowTime = Intro::introTime - modelShowTime;
-		int individualModelShowTime = totalModelShowTime / models.size();
+		int individualModelShowTime = totalModelShowTime / (int)models.size();
 
 		//printf("difTime = %d individualModelShowTime = %d modelIndex = %d\n",difTime,individualModelShowTime,modelIndex);
 
@@ -630,7 +626,7 @@ void Intro::renderModelBackground() {
 		if(difTime > 0) {
 			if(difTime > ((modelIndex+1) * individualModelShowTime)) {
 				//int oldmodelIndex = modelIndex;
-				if(modelIndex+1 < models.size()) {
+				if(modelIndex + 1 < (int)models.size()) {
 					modelIndex++;
 
 					//position
@@ -669,8 +665,6 @@ void Intro::render() {
 		return;
 	}
 
-	int difTime=0;
-
 	canRender();
 	incrementFps();
 
@@ -685,10 +679,10 @@ void Intro::render() {
 
 	renderer.reset2d();
 
-	for(int i = 0; i < texts.size(); ++i) {
+	for(int i = 0; i < (int)texts.size(); ++i) {
 		Text *text= texts[i];
 
-		difTime= 1000 * timer / GameConstants::updateFps - text->getTime();
+		int difTime= 1000 * timer / GameConstants::updateFps - text->getTime();
 
 		if(difTime > 0 && difTime < appearTime + showTime + disapearTime) {
 			float alpha= 1.f;
