@@ -45,6 +45,7 @@ Program *Program::singleton = NULL;
 const int SOUND_THREAD_UPDATE_MILLISECONDS = 25;
 
 bool Program::wantShutdownApplicationAfterGame = false;
+const char *ProgramState::MAIN_PROGRAM_RENDER_KEY = "MEGAGLEST.RENDER";
 
 // =====================================================
 // 	class Program::CrashProgramState
@@ -172,6 +173,9 @@ void Program::ShowMessageProgramState::update() {
 
 // ===================== PUBLIC ========================
 
+bool Program::rendererInitOk = false;
+bool Program::tryingRendererInit = false;
+
 Program::Program() {
 	//this->masterserverMode = false;
 	this->window = NULL;
@@ -185,7 +189,7 @@ Program::Program() {
 
 	//mesage box
 	Lang &lang= Lang::getInstance();
-	msgBox.init(lang.get("Ok"));
+	msgBox.init(lang.getString("Ok"));
 	msgBox.setEnabled(false);
 }
 
@@ -361,7 +365,7 @@ void Program::eventMouseMove(int x, int y, const MouseState *ms) {
 	}
 }
 
-void Program::simpleTask(BaseThread *callingThread) {
+void Program::simpleTask(BaseThread *callingThread,void *userdata) {
 	loopWorker();
 }
 
@@ -383,6 +387,14 @@ void Program::loopWorker() {
 		}
 	}
 
+	Chrono chronoPerformanceCounts;
+
+	bool showPerfStats = Config::getInstance().getBool("ShowPerfStats","false");
+	Chrono chronoPerf;
+	char perfBuf[8096]="";
+	std::vector<string> perfList;
+	if(showPerfStats) chronoPerf.start();
+
 	Chrono chronoLoop;
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled) chronoLoop.start();
 
@@ -407,18 +419,43 @@ void Program::loopWorker() {
     }
     ProgramState *prevState = this->programState;
 
+	if(showPerfStats) {
+		sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis());
+		perfList.push_back(perfBuf);
+	}
+
     assert(programState != NULL);
+
+    chronoPerformanceCounts.start();
+
     programState->render();
 
+    programState->addPerformanceCount(ProgramState::MAIN_PROGRAM_RENDER_KEY,chronoPerformanceCounts.getMillis());
+
+	if(showPerfStats) {
+		sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis());
+		perfList.push_back(perfBuf);
+	}
+
     if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chrono.getMillis() > 0) SystemFlags::OutputDebug(SystemFlags::debugPerformance,"In [%s::%s] Line: %d programState->render took msecs: %lld ==============> MAIN LOOP RENDERING\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
     if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chrono.getMillis() > 0) chrono.start();
 
 	//update camera
     if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled) chrono.start();
-	while(updateCameraTimer.isTime()){
+
+    chronoPerformanceCounts.start();
+
+    while(updateCameraTimer.isTime()) {
 		programState->updateCamera();
 	}
 
+    programState->addPerformanceCount("programState->updateCamera()",chronoPerformanceCounts.getMillis());
+
+	if(showPerfStats) {
+		sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis());
+		perfList.push_back(perfBuf);
+	}
+
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chrono.getMillis() > 0) SystemFlags::OutputDebug(SystemFlags::debugPerformance,"In [%s::%s] Line: %d programState->render took msecs: %lld ==============> MAIN LOOP CAMERA UPDATING\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chrono.getMillis() > 0) chrono.start();
 
@@ -428,13 +465,25 @@ void Program::loopWorker() {
 	while(prevState == this->programState && updateTimer.isTime()) {
 		Chrono chronoUpdateLoop;
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled) chronoUpdateLoop.start();
+		if(showPerfStats) {
+			sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER " updateCount: %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis(),updateCount);
+			perfList.push_back(perfBuf);
+		}
 
 		GraphicComponent::update();
 		programState->update();
+
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chronoUpdateLoop.getMillis() > 0) SystemFlags::OutputDebug(SystemFlags::debugPerformance,"In [%s::%s Line: %d] programState->update took msecs: %lld, updateCount = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoUpdateLoop.getMillis(),updateCount);
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chronoUpdateLoop.getMillis() > 0) chronoUpdateLoop.start();
 
+		if(showPerfStats) {
+			sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER " updateCount: %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis(),updateCount);
+			perfList.push_back(perfBuf);
+		}
+
 		if(prevState == this->programState) {
+			chronoPerformanceCounts.start();
+
 			if(soundThreadManager == NULL || soundThreadManager->isThreadExecutionLagging()) {
 				if(soundThreadManager != NULL) {
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s %d] ERROR / WARNING soundThreadManager->isThreadExecutionLagging is TRUE\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
@@ -445,9 +494,27 @@ void Program::loopWorker() {
 				if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chronoUpdateLoop.getMillis() > 0) chronoUpdateLoop.start();
 			}
 
+			programState->addPerformanceCount("SoundRenderer::getInstance().update()",chronoPerformanceCounts.getMillis());
+
+			if(showPerfStats) {
+				sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER " updateCount: %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis(),updateCount);
+				perfList.push_back(perfBuf);
+			}
+
+			chronoPerformanceCounts.start();
+
 			NetworkManager::getInstance().update();
+
+			programState->addPerformanceCount("NetworkManager::getInstance().update()",chronoPerformanceCounts.getMillis());
+
 			if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chronoUpdateLoop.getMillis() > 0) SystemFlags::OutputDebug(SystemFlags::debugPerformance,"In [%s::%s Line: %d] NetworkManager::getInstance().update() took msecs: %lld, updateCount = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoUpdateLoop.getMillis(),updateCount);
 			if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chronoUpdateLoop.getMillis() > 0) chronoUpdateLoop.start();
+
+			if(showPerfStats) {
+				sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER " updateCount: %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis(),updateCount);
+				perfList.push_back(perfBuf);
+			}
+
 		}
 		updateCount++;
 	}
@@ -456,13 +523,27 @@ void Program::loopWorker() {
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chrono.getMillis() > 0) chrono.start();
 
+	if(showPerfStats) {
+		sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis());
+		perfList.push_back(perfBuf);
+	}
+
 	if(prevState == this->programState) {
 		//fps timer
+		chronoPerformanceCounts.start();
+
 		chrono.start();
 		while(fpsTimer.isTime()) {
 			programState->tick();
 		}
 
+		programState->addPerformanceCount("programState->tick()",chronoPerformanceCounts.getMillis());
+
+		if(showPerfStats) {
+			sprintf(perfBuf,"In [%s::%s] Line: %d took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoPerf.getMillis());
+			perfList.push_back(perfBuf);
+		}
+
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chrono.getMillis() > 0) SystemFlags::OutputDebug(SystemFlags::debugPerformance,"In [%s::%s] Line: %d programState->render took msecs: %lld ==============> MAIN LOOP TICKING\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled && chrono.getMillis() > 0) chrono.start();
 
@@ -471,6 +552,12 @@ void Program::loopWorker() {
 
 	}
 
+	if(showPerfStats && chronoPerf.getMillis() >= 100) {
+		for(unsigned int x = 0; x < perfList.size(); ++x) {
+			printf("%s",perfList[x].c_str());
+		}
+	}
+
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugPerformance).enabled) SystemFlags::OutputDebug(SystemFlags::debugPerformance,"In [%s::%s Line: %d] ------------------------------- MAIN LOOP END, stats: loop took msecs: %lld -------------------------------\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chronoLoop.getMillis());
 }
 
@@ -551,7 +638,7 @@ void Program::setState(ProgramState *programStateNew, bool cleanupOldState) {
 
 		//mesage box
 		Lang &lang= Lang::getInstance();
-		msgBox.init(lang.get("Ok"));
+		msgBox.init(lang.getString("Ok"));
 		msgBox.setEnabled(msgBoxEnabled);
 
 		fpsTimer.init(1, maxTimes);
@@ -560,6 +647,9 @@ void Program::setState(ProgramState *programStateNew, bool cleanupOldState) {
 
 		this->programState= programStateNew;
 		assert(programStateNew != NULL);
+		if(programStateNew == NULL) {
+			throw megaglest_runtime_error("programStateNew == NULL");
+		}
 		programStateNew->load();
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
@@ -725,7 +815,10 @@ void Program::init(WindowGl *window, bool initSound, bool toggleFullScreen){
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	//init renderer (load global textures)
+    tryingRendererInit = true;
 	renderer.init();
+	tryingRendererInit = false;
+	rendererInitOk = true;
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
@@ -741,7 +834,7 @@ void Program::init(WindowGl *window, bool initSound, bool toggleFullScreen){
         // END
 
         if(initOk == false) {
-        	string sError = "Sound System could not be initialzed!";
+        	string sError = "Sound System could not be initialized!";
         	this->showMessage(sError.c_str());
         }
 
@@ -750,8 +843,9 @@ void Program::init(WindowGl *window, bool initSound, bool toggleFullScreen){
 			if(BaseThread::shutdownAndWait(soundThreadManager) == true) {
 				delete soundThreadManager;
 			}
+			static string mutexOwnerId = string(extractFileFromDirectoryPath(__FILE__).c_str()) + string("_") + intToStr(__LINE__);
 			soundThreadManager = new SimpleTaskThread(&SoundRenderer::getInstance(),0,SOUND_THREAD_UPDATE_MILLISECONDS);
-			soundThreadManager->setUniqueID(extractFileFromDirectoryPath(__FILE__).c_str());
+			soundThreadManager->setUniqueID(mutexOwnerId);
 			soundThreadManager->start();
 		}
 	}
@@ -774,7 +868,7 @@ void Program::setDisplaySettings(){
 
 	if(!config.getBool("Windowed")) {
 
-		int freq= config.getInt("RefreshFrequency");
+		//int freq= config.getInt("RefreshFrequency");
 		int colorBits= config.getInt("ColorBits");
 		int screenWidth= config.getInt("ScreenWidth");
 		int screenHeight= config.getInt("ScreenHeight");
@@ -785,15 +879,8 @@ void Program::setDisplaySettings(){
             config.setInt("ScreenWidth",screenWidth);
             config.setInt("ScreenHeight",screenHeight);
         }
-
-		if(!(changeVideoMode(screenWidth, screenHeight, colorBits, freq) ||
-			changeVideoMode(screenWidth, screenHeight, colorBits, 0)))
-		{
-			throw megaglest_runtime_error(
-				"Error setting video mode: " +
-				intToStr(screenWidth) + "x" + intToStr(screenHeight) + "x" + intToStr(colorBits));
-		}
 	}
+	changeVideoModeFullScreen(!config.getBool("Windowed"));
 }
 
 void Program::restoreDisplaySettings(){
@@ -836,8 +923,9 @@ void Program::stopSoundSystem() {
 void Program::startSoundSystem() {
 	stopSoundSystem();
 	if(SoundRenderer::getInstance().runningThreaded() == true) {
+		static string mutexOwnerId = string(extractFileFromDirectoryPath(__FILE__).c_str()) + string("_") + intToStr(__LINE__);
 		soundThreadManager = new SimpleTaskThread(&SoundRenderer::getInstance(),0,SOUND_THREAD_UPDATE_MILLISECONDS);
-		soundThreadManager->setUniqueID(extractFileFromDirectoryPath(__FILE__).c_str());
+		soundThreadManager->setUniqueID(mutexOwnerId);
 		soundThreadManager->start();
 	}
 }
