@@ -65,6 +65,9 @@ protected:
     int lastFps;
 
 public:
+
+    static const char *MAIN_PROGRAM_RENDER_KEY;
+
 	ProgramState(Program *program);
 	virtual ~ProgramState(){};
 
@@ -105,6 +108,7 @@ public:
 	virtual void consoleAddLine(string line) { };
 
 	virtual void reloadUI() {};
+	virtual void addPerformanceCount(string key,int64 value) {};
 
 protected:
 	virtual void incrementFps();
@@ -159,11 +163,17 @@ private:
     bool shutdownApplicationEnabled;
     static bool wantShutdownApplicationAfterGame;
 
+    static bool tryingRendererInit;
+    static bool rendererInitOk;
+
 public:
     Program();
     virtual ~Program();
 
-    static Program *getInstance()	{return singleton;}
+    static bool getTryingRendererInit() { return tryingRendererInit; }
+    static bool getRendererInitOk() { return rendererInitOk; }
+
+    static Program *getInstance()	{ return singleton;      }
 
     static void setWantShutdownApplicationAfterGame(bool value) { wantShutdownApplicationAfterGame = value; }
     static bool getWantShutdownApplicationAfterGame() { return wantShutdownApplicationAfterGame; }
@@ -200,7 +210,7 @@ public:
 	void init(WindowGl *window, bool initSound=true, bool toggleFullScreen=false);
 	void exit();
 
-	virtual void simpleTask(BaseThread *callingThread);
+	virtual void simpleTask(BaseThread *callingThread,void *userdata);
 
 	void mouseDownLeft(int x, int y);
 	void eventMouseMove(int x, int y, const MouseState *ms);
