@@ -49,6 +49,8 @@ MenuState * MainMenu::oldstate=NULL;
 MainMenu::MainMenu(Program *program) : ProgramState(program), menuBackgroundVideo(NULL) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
+	//printf("In MainMenu::MainMenu()\n");
+
 	mouseX=100;
 	mouseY=100;
 
@@ -78,6 +80,8 @@ void MainMenu::reloadUI() {
 MainMenu::~MainMenu() {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
+	//printf("In MainMenu::~MainMenu()\n");
+
 	if(menuBackgroundVideo != NULL) {
 		menuBackgroundVideo->closePlayer();
 		delete menuBackgroundVideo;
@@ -109,7 +113,7 @@ void MainMenu::init() {
 
 void MainMenu::initBackgroundVideo() {
 	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false &&
-		Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == true &&
+		::Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == true &&
 		CoreData::getInstance().hasMainMenuVideoFilename() == true) {
 		string introVideoFile = CoreData::getInstance().getMainMenuVideoFilename();
 		string introVideoFileFallback = CoreData::getInstance().getMainMenuVideoFilenameFallback();
@@ -242,31 +246,38 @@ void MainMenu::keyPress(SDL_KeyboardEvent c) {
 void MainMenu::setState(MenuState *newstate) {
     //printf("In [%s::%s Line: %d] oldstate [%p] newstate [%p] this->state [%p]\n",__FILE__,__FUNCTION__,__LINE__,oldstate,newstate,this->state);
 
-    //delete this->state;
-    //this->state = newstate;
+	//printf("In MainMenu::setState() #1\n");
 
 	if(oldstate != NULL && oldstate != newstate) {
+		MenuState *oldstatePtr = oldstate;
 		delete oldstate;
 
+		//printf("In MainMenu::setState() #2\n");
+
         //printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
-		if(oldstate != this->state) {
+		if(oldstatePtr != this->state) {
 			oldstate=this->state;
 			//printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+			//printf("In MainMenu::setState() #3\n");
 		}
 		else {
 			oldstate = NULL;
 			//printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+			//printf("In MainMenu::setState() #4\n");
 		}
 	}
 	else {
 		oldstate=this->state;
 		//printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		//printf("In MainMenu::setState() #5\n");
 	}
 	//printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 	this->state= newstate;
 
 	GraphicComponent::resetFade();
-	menuBackground.setTargetCamera(newstate->getCamera());
+	if(newstate) {
+		menuBackground.setTargetCamera(newstate->getCamera());
+	}
 }
 
 bool MainMenu::isInSpecialKeyCaptureEvent() {
@@ -405,7 +416,7 @@ bool MenuState::keyPressEditLabel(SDL_KeyboardEvent c, GraphicLabel **activeInpu
 		//if((c>='0' && c<='9') || (c>='a' && c<='z') || (c>='A' && c<='Z') ||
 		//   (c=='-') || (c=='(') || (c==')')) {
 		if(isAllowedInputTextKey(key)) {
-			if(activeInputLabel->getText().size() < maxTextSize) {
+			if((int)activeInputLabel->getText().size() < maxTextSize) {
 				string text= activeInputLabel->getText();
 
 				wchar_t keyW = extractKeyPressedUnicode(c);
@@ -529,7 +540,7 @@ bool MenuState::keyDownEditLabel(SDL_KeyboardEvent c, GraphicLabel **activeInput
 						//}
 
 						//for(unsigned int i = 0; i < textCharLength[textCharLength.size()-2]; ++i) {
-						for(unsigned int i = 0; i < activeInputLabel->getTextCharLengthList()[activeInputLabel->getTextCharLengthList().size()-2]; ++i) {
+						for(unsigned int i = 0; i < (unsigned int)activeInputLabel->getTextCharLengthList()[activeInputLabel->getTextCharLengthList().size()-2]; ++i) {
 							//printf("erase A1 i = %d [%s]\n",i,text.c_str());
 							text.erase(text.end() -2);
 							//printf("erase A2 i = %d [%s]\n",i,text.c_str());
@@ -545,7 +556,7 @@ bool MenuState::keyDownEditLabel(SDL_KeyboardEvent c, GraphicLabel **activeInput
 				}
 				else {
 					//for(unsigned int i = 0; i < textCharLength[textCharLength.size()-1]; ++i) {
-					for(unsigned int i = 0; i < activeInputLabel->getTextCharLengthList()[activeInputLabel->getTextCharLengthList().size()-1]; ++i) {
+					for(unsigned int i = 0; i < (unsigned int)activeInputLabel->getTextCharLengthList()[activeInputLabel->getTextCharLengthList().size()-1]; ++i) {
 						//printf("erase B1 i = %d [%s]\n",i,text.c_str());
 						text.erase(text.end() -1);
 						//printf("erase B2 i = %d [%s]\n",i,text.c_str());
