@@ -29,6 +29,8 @@ namespace Shared { namespace Graphics {
 
 namespace Glest { namespace Game {
 
+class TechTree;
+
 enum JoinMenu {
 	jmSimple,
 	jmMasterserver,
@@ -87,11 +89,14 @@ private:
 
 	GraphicListBox listBoxPlayerStatus;
 	GraphicLabel labelPlayerStatus[GameConstants::maxPlayers];
+	int nonAdminPlayerStatusX;
 
 	GraphicLabel labelAllowObservers;
-	//GraphicListBox listBoxAllowObservers;
 	GraphicCheckBox checkBoxAllowObservers;
 
+	GraphicLabel labelAllowNativeLanguageTechtree;
+	GraphicCheckBox checkBoxAllowNativeLanguageTechtree;
+
 	GraphicLabel *activeInputLabel;
 
     time_t timerLabelFlash;
@@ -99,6 +104,11 @@ private:
 
 	MapInfo mapInfo;
 	Texture2D *mapPreviewTexture;
+	bool zoomedMap;
+	int render_mapPreviewTexture_X;
+	int render_mapPreviewTexture_Y;
+	int render_mapPreviewTexture_W;
+	int render_mapPreviewTexture_H;
 
 	bool needToSetChangedGameSettings;
 	time_t lastSetChangedGameSettings;
@@ -132,7 +142,7 @@ private:
 	string currentFactionName_factionPreview;
 	string currentFactionLogo;
 	Texture2D *factionTexture;
-	Shared::Graphics::VideoPlayer *factionVideo;
+	::Shared::Graphics::VideoPlayer *factionVideo;
 	bool factionVideoSwitchedOffVolume;
 
 	MapPreview mapPreview;
@@ -170,12 +180,19 @@ private:
 
     string getMissingMapFromFTPServer;
     bool getMissingMapFromFTPServerInProgress;
+    time_t getMissingMapFromFTPServerLastPrompted;
 
     string getMissingTilesetFromFTPServer;
     bool getMissingTilesetFromFTPServerInProgress;
+    time_t getMissingTilesetFromFTPServerLastPrompted;
 
     string getMissingTechtreeFromFTPServer;
     bool getMissingTechtreeFromFTPServerInProgress;
+    time_t getMissingTechtreeFromFTPServerLastPrompted;
+
+    string getInProgressSavedGameFromFTPServer;
+    bool getInProgressSavedGameFromFTPServerInProgress;
+    bool readyToJoinInProgressGame;
 
     string lastCheckedCRCTilesetName;
     string lastCheckedCRCTechtreeName;
@@ -212,11 +229,13 @@ private:
 	bool enableScenarioTexturePreview;
 	Texture2D *scenarioLogoTexture;
 
-
 	bool needToBroadcastServerSettings;
 	time_t broadcastServerSettingsDelayTimer;
 	int lastGameSettingsReceivedCount;
 
+	bool launchingNewGame;
+	std::auto_ptr<TechTree> techTree;
+
 public:
 
 	MenuStateConnectedGame(Program *program, MainMenu *mainMenu, JoinMenu joinMenuInfo=jmSimple, bool openNetworkSlots= false);
@@ -269,17 +288,19 @@ private:
     void setupUIFromGameSettings(GameSettings *gameSettings, bool errorOnMissingData);
 
 	int setupMapList(string scenario);
-	int setupTechList(string scenario);
+	int setupTechList(string scenario, bool forceLoad=false);
 	void setupTilesetList(string scenario);
 
 	void loadScenarioInfo(string file, ScenarioInfo *scenarioInfo);
 	void initFactionPreview(const GameSettings *gameSettings);
 
-	virtual void simpleTask(BaseThread *callingThread);
+	virtual void simpleTask(BaseThread *callingThread,void *userdata);
 	string refreshTilesetModInfo(string tilesetInfo);
 	string refreshTechModInfo(string techInfo);
 	string refreshMapModInfo(string mapInfo);
 	string getMapCRC(string mapName);
+
+	void disconnectFromServer();
 };
 
 }}//end namespace
