@@ -28,6 +28,7 @@ namespace Glest { namespace Game {
 
 class SwitchSetupRequest;
 class ServerInterface;
+class TechTree;
 
 enum ParentMenuState {
 	pNewGame,
@@ -58,7 +59,6 @@ private:
 	GraphicLabel labelLocalGameVersion;
 	GraphicLabel labelLocalIP;
 	GraphicLabel labelGameName;
-	GraphicLabel labelGameNameLabel;
 
 	GraphicListBox listBoxMap;
 	GraphicListBox listBoxFogOfWar;
@@ -127,6 +127,12 @@ private:
 	GraphicLabel labelFallbackCpuMultiplier;
 	GraphicListBox listBoxFallbackCpuMultiplier;
 
+	GraphicLabel labelAllowInGameJoinPlayer;
+	GraphicCheckBox checkBoxAllowInGameJoinPlayer;
+
+	GraphicLabel labelAllowNativeLanguageTechtree;
+	GraphicCheckBox checkBoxAllowNativeLanguageTechtree;
+
 	GraphicCheckBox checkBoxScenario;
 	GraphicLabel labelScenario;
 	GraphicListBox listBoxScenario;
@@ -150,6 +156,7 @@ private:
 	bool needToBroadcastServerSettings;
 	std::map<string,string> publishToServerInfo;
 	SimpleTaskThread *publishToMasterserverThread;
+	SimpleTaskThread *publishToClientsThread;
 
 	ParentMenuState parentMenuState;
 	int soundConnectionCount;
@@ -180,11 +187,16 @@ private:
 	string currentFactionName_factionPreview;
 	string currentFactionLogo;
 	Texture2D *factionTexture;
-	Shared::Graphics::VideoPlayer *factionVideo;
+	::Shared::Graphics::VideoPlayer *factionVideo;
 	bool factionVideoSwitchedOffVolume;
 
 	MapPreview mapPreview;
 	Texture2D *mapPreviewTexture;
+	bool zoomedMap;
+	int render_mapPreviewTexture_X;
+	int render_mapPreviewTexture_Y;
+	int render_mapPreviewTexture_W;
+	int render_mapPreviewTexture_H;
 
 	bool autostart;
 	GameSettings *autoStartSettings;
@@ -196,6 +208,11 @@ private:
     string lastCheckedCRCTilesetName;
     string lastCheckedCRCTechtreeName;
     string lastCheckedCRCMapName;
+
+    string last_Forced_CheckedCRCTilesetName;
+    string last_Forced_CheckedCRCTechtreeName;
+    string last_Forced_CheckedCRCMapName;
+
     uint32 lastCheckedCRCTilesetValue;
     uint32 lastCheckedCRCTechtreeValue;
     uint32 lastCheckedCRCMapValue;
@@ -206,6 +223,9 @@ private:
     bool masterserverModeMinimalResources;
     int lastMasterServerSettingsUpdateCount;
 
+    std::auto_ptr<TechTree> techTree;
+
+    string gameUUID;
 public:
 	MenuStateCustomGame(Program *program, MainMenu *mainMenu ,
 			bool openNetworkSlots= false, ParentMenuState parentMenuState=pNewGame,
@@ -223,9 +243,9 @@ public:
     virtual void keyUp(SDL_KeyboardEvent key);
 
 
-    virtual void simpleTask(BaseThread *callingThread);
-	virtual void setupTask(BaseThread *callingThread);
-	virtual void shutdownTask(BaseThread *callingThread);
+    virtual void simpleTask(BaseThread *callingThread,void *userdata);
+	virtual void setupTask(BaseThread *callingThread,void *userdata);
+	virtual void shutdownTask(BaseThread *callingThread,void *userdata);
 	static void setupTaskStatic(BaseThread *callingThread);
 	static void shutdownTaskStatic(BaseThread *callingThread);
 
@@ -277,11 +297,18 @@ private:
 	void processScenario();
 	void SetupUIForScenarios();
 	int setupMapList(string scenario);
-	int setupTechList(string scenario);
+	int setupTechList(string scenario, bool forceLoad=false);
 	void reloadFactions(bool keepExistingSelectedItem, string scenario);
 	void setupTilesetList(string scenario);
+	void setSlotHuman(int i);
 
 	void initFactionPreview(const GameSettings *gameSettings);
+
+	bool checkNetworkPlayerDataSynch(bool checkMapCRC,bool checkTileSetCRC, bool checkTechTreeCRC);
+
+	void cleanupThread(SimpleTaskThread **thread);
+	void simpleTaskForMasterServer(BaseThread *callingThread);
+	void simpleTaskForClients(BaseThread *callingThread);
 };
 
 }}//end namespace
