@@ -31,7 +31,7 @@
 
 namespace Glest{ namespace Game{
 
-using namespace Shared::Util;
+using namespace ::Shared::Util;
 
 // ===============================
 // 	class MenuStateJoinGame
@@ -90,37 +90,37 @@ void MenuStateJoinGame::CommonInit(bool connect, Ip serverIp,int portNumberOverr
 	//buttons
 	buttonReturn.registerGraphicComponent(containerName,"buttonReturn");
 	buttonReturn.init(300, 300, 125);
-	buttonReturn.setText(lang.get("Return"));
+	buttonReturn.setText(lang.getString("Return"));
 
 	buttonConnect.registerGraphicComponent(containerName,"buttonConnect");
 	buttonConnect.init(450, 300, 125);
-	buttonConnect.setText(lang.get("Connect"));
+	buttonConnect.setText(lang.getString("Connect"));
 
 	buttonCreateGame.registerGraphicComponent(containerName,"buttonCreateGame");
 	buttonCreateGame.init(450, 250, 125);
-	buttonCreateGame.setText(lang.get("HostGame"));
+	buttonCreateGame.setText(lang.getString("HostGame"));
 
 	buttonAutoFindServers.registerGraphicComponent(containerName,"buttonAutoFindServers");
 	buttonAutoFindServers.init(595, 300, 225);
-	buttonAutoFindServers.setText(lang.get("FindLANGames"));
+	buttonAutoFindServers.setText(lang.getString("FindLANGames"));
 	buttonAutoFindServers.setEnabled(true);
 
 	//server type label
 	labelServerType.registerGraphicComponent(containerName,"labelServerType");
 	labelServerType.init(330, 490);
-	labelServerType.setText(lang.get("ServerType") + ":");
+	labelServerType.setText(lang.getString("ServerType") + ":");
 
 	//server type list box
 	listBoxServerType.registerGraphicComponent(containerName,"listBoxServerType");
 	listBoxServerType.init(465, 490);
-	listBoxServerType.pushBackItem(lang.get("ServerTypeNew"));
-	listBoxServerType.pushBackItem(lang.get("ServerTypePrevious"));
-	listBoxServerType.pushBackItem(lang.get("ServerTypeFound"));
+	listBoxServerType.pushBackItem(lang.getString("ServerTypeNew"));
+	listBoxServerType.pushBackItem(lang.getString("ServerTypePrevious"));
+	listBoxServerType.pushBackItem(lang.getString("ServerTypeFound"));
 
 	//server label
 	labelServer.registerGraphicComponent(containerName,"labelServer");
 	labelServer.init(330, 460);
-	labelServer.setText(lang.get("Server") + ": ");
+	labelServer.setText(lang.getString("Server") + ": ");
 
 	//server listbox
 	listBoxServers.registerGraphicComponent(containerName,"listBoxServers");
@@ -135,12 +135,15 @@ void MenuStateJoinGame::CommonInit(bool connect, Ip serverIp,int portNumberOverr
 
 	//server ip
 	labelServerIp.registerGraphicComponent(containerName,"labelServerIp");
+	labelServerIp.setEditable(true);
+	labelServerIp.setMaxEditWidth(15);
+	labelServerIp.setMaxEditRenderWidth(220);
 	labelServerIp.init(465, 460);
 
 	// server port
 	labelServerPortLabel.registerGraphicComponent(containerName,"labelServerPortLabel");
 	labelServerPortLabel.init(330,430);
-	labelServerPortLabel.setText(lang.get("ServerPort"));
+	labelServerPortLabel.setText(lang.getString("ServerPort"));
 
 	labelServerPort.registerGraphicComponent(containerName,"labelServerPort");
 	labelServerPort.init(465,430);
@@ -213,21 +216,21 @@ void MenuStateJoinGame::reloadUI() {
 
 	console.resetFonts();
 
-	buttonReturn.setText(lang.get("Return"));
-	buttonConnect.setText(lang.get("Connect"));
-	buttonCreateGame.setText(lang.get("HostGame"));
-	buttonAutoFindServers.setText(lang.get("FindLANGames"));
-	labelServerType.setText(lang.get("ServerType") + ":");
+	buttonReturn.setText(lang.getString("Return"));
+	buttonConnect.setText(lang.getString("Connect"));
+	buttonCreateGame.setText(lang.getString("HostGame"));
+	buttonAutoFindServers.setText(lang.getString("FindLANGames"));
+	labelServerType.setText(lang.getString("ServerType") + ":");
 
 	std::vector<string> listboxData;
-	listboxData.push_back(lang.get("ServerTypeNew"));
-	listboxData.push_back(lang.get("ServerTypePrevious"));
-	listboxData.push_back(lang.get("ServerTypeFound"));
+	listboxData.push_back(lang.getString("ServerTypeNew"));
+	listboxData.push_back(lang.getString("ServerTypePrevious"));
+	listboxData.push_back(lang.getString("ServerTypeFound"));
 	listBoxServerType.setItems(listboxData);
 
-	labelServer.setText(lang.get("Server") + ": ");
+	labelServer.setText(lang.getString("Server") + ": ");
 
-	labelServerPortLabel.setText(lang.get("ServerPort"));
+	labelServerPortLabel.setText(lang.getString("ServerPort"));
 
 	string host = labelServerIp.getText();
 	int portNumber = config.getInt("PortServer",intToStr(GameConstants::serverPort).c_str());
@@ -249,10 +252,12 @@ void MenuStateJoinGame::reloadUI() {
 
 MenuStateJoinGame::~MenuStateJoinGame() {
 	abortAutoFind = true;
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 }
 
 void MenuStateJoinGame::DiscoveredServers(std::vector<string> serverList) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	if(abortAutoFind == true) {
 		return;
@@ -262,6 +267,8 @@ void MenuStateJoinGame::DiscoveredServers(std::vector<string> serverList) {
 	//serverList.push_back("test2");
 	//
 
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+
 	autoConnectToServer = false;
 	buttonAutoFindServers.setEnabled(true);
 	buttonConnect.setEnabled(true);
@@ -271,7 +278,9 @@ void MenuStateJoinGame::DiscoveredServers(std::vector<string> serverList) {
 		int serverGamePort = config.getInt("PortServer",intToStr(GameConstants::serverPort).c_str());
 		std::vector<std::string> localIPList = Socket::getLocalIPAddressList();
 
-		for(int idx = 0; idx < serverList.size(); idx++) {
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+
+		for(int idx = 0; idx < (int)serverList.size(); idx++) {
 
 			vector<string> paramPartPortsTokens;
 			Tokenize(serverList[idx],paramPartPortsTokens,":");
@@ -288,11 +297,15 @@ void MenuStateJoinGame::DiscoveredServers(std::vector<string> serverList) {
 			}
 		}
 
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+
 		if(bestIPMatch != "") {
 			bestIPMatch += ":" + intToStr(serverGamePort);
 		}
 		labelServerIp.setText(bestIPMatch);
 
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+
 		if(serverList.size() > 1) {
 			listBoxServerType.setSelectedItemIndex(MenuStateJoinGame::foundServersIndex);
 			listBoxFoundServers.setItems(serverList);
@@ -302,6 +315,7 @@ void MenuStateJoinGame::DiscoveredServers(std::vector<string> serverList) {
 		}
 	}
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 }
 
 void MenuStateJoinGame::mouseClick(int x, int y, MouseButton mouseButton) {
@@ -362,41 +376,43 @@ void MenuStateJoinGame::mouseClick(int x, int y, MouseButton mouseButton) {
 		}
 		abortAutoFind = true;
 		mainMenu->setState(new MenuStateNewGame(program, mainMenu));
+		return;
     }
 
 	//connect
 	else if(buttonConnect.mouseClick(x, y) && buttonConnect.getEnabled() == true) {
 		ClientInterface* clientInterface= networkManager.getClientInterface();
 
-		soundRenderer.playFx(coreData.getClickSoundA());
+		soundRenderer.playFx(coreData.getClickSoundB());
 		labelInfo.setText("");
 
-		if(clientInterface->isConnected())
-		{
+		if(clientInterface->isConnected()) {
 			clientInterface->reset();
 		}
-		else
-		{
-			connectToServer();
+		else {
+			if(connectToServer() == true) {
+				return;
+			}
 		}
 	}
     else if(buttonCreateGame.mouseClick(x, y)){
     	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 		soundRenderer.playFx(coreData.getClickSoundB());
+
 		clientInterface->stopServerDiscovery();
 		if(clientInterface->getSocket() != NULL) {
 		    clientInterface->close();
 		}
 		abortAutoFind = true;
 		mainMenu->setState(new MenuStateCustomGame(program, mainMenu,true,pLanGame));
-		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		return;
     }
 
 	else if(buttonAutoFindServers.mouseClick(x, y) && buttonAutoFindServers.getEnabled() == true) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 		ClientInterface* clientInterface= networkManager.getClientInterface();
-		soundRenderer.playFx(coreData.getClickSoundA());
+		soundRenderer.playFx(coreData.getClickSoundB());
 
 		// Triggers a thread which calls back into MenuStateJoinGame::DiscoveredServers
 		// with the results
@@ -472,11 +488,11 @@ void MenuStateJoinGame::update()
 	//update status label
 	if(clientInterface->isConnected())
 	{
-		buttonConnect.setText(lang.get("Disconnect"));
+		buttonConnect.setText(lang.getString("Disconnect"));
 
 		if(clientInterface->getAllowDownloadDataSynch() == false)
 		{
-		    string label = lang.get("ConnectedToServer");
+		    string label = lang.getString("ConnectedToServer");
 
             if(!clientInterface->getServerName().empty())
             {
@@ -508,7 +524,7 @@ void MenuStateJoinGame::update()
 		}
 		else
 		{
-		    string label = lang.get("ConnectedToServer");
+		    string label = lang.getString("ConnectedToServer");
 
             if(!clientInterface->getServerName().empty())
             {
@@ -542,8 +558,8 @@ void MenuStateJoinGame::update()
 	}
 	else
 	{
-		buttonConnect.setText(lang.get("Connect"));
-		string connectedStatus = lang.get("NotConnected");
+		buttonConnect.setText(lang.getString("Connect"));
+		string connectedStatus = lang.getString("NotConnected");
 		if(buttonAutoFindServers.getEnabled() == false) {
 			connectedStatus += " - searching for servers, please wait...";
 		}
@@ -566,7 +582,7 @@ void MenuStateJoinGame::update()
 
 			//intro
 			if(clientInterface->getIntroDone()) {
-				labelInfo.setText(lang.get("WaitingHost"));
+				labelInfo.setText(lang.getString("WaitingHost"));
 
 				string host = labelServerIp.getText();
 				std::vector<std::string> hostPartsList;
@@ -595,14 +611,15 @@ void MenuStateJoinGame::update()
 				abortAutoFind = true;
 				clientInterface->stopServerDiscovery();
 				program->setState(new Game(program, clientInterface->getGameSettings(),false));
-
-				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] clientInterface->getLaunchGame() - C\n",__FILE__,__FUNCTION__);
+				return;
 			}
 		}
 	}
 	else if(autoConnectToServer == true) {
 		autoConnectToServer = false;
-		connectToServer();
+		if(connectToServer() == true) {
+			return;
+		}
 	}
 
     if(clientInterface != NULL && clientInterface->getLaunchGame()) if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] clientInterface->getLaunchGame() - D\n",__FILE__,__FUNCTION__);
@@ -636,7 +653,7 @@ void MenuStateJoinGame::keyDown(SDL_KeyboardEvent key) {
 		else if(isKeyPressed(configKeys.getSDLKey("SaveGUILayout"),key) == true) {
 			bool saved = GraphicComponent::saveAllCustomProperties(containerName);
 			Lang &lang= Lang::getInstance();
-			console.addLine(lang.get("GUILayoutSaved") + " [" + (saved ? lang.get("Yes") : lang.get("No"))+ "]");
+			console.addLine(lang.getString("GUILayoutSaved") + " [" + (saved ? lang.getString("Yes") : lang.getString("No"))+ "]");
 		}
 	}
 	else {
@@ -651,7 +668,7 @@ void MenuStateJoinGame::keyDown(SDL_KeyboardEvent key) {
         	if(isKeyPressed(configKeys.getSDLKey("SaveGUILayout"),key) == true) {
 				bool saved = GraphicComponent::saveAllCustomProperties(containerName);
 				Lang &lang= Lang::getInstance();
-				console.addLine(lang.get("GUILayoutSaved") + " [" + (saved ? lang.get("Yes") : lang.get("No"))+ "]");
+				console.addLine(lang.getString("GUILayoutSaved") + " [" + (saved ? lang.getString("Yes") : lang.getString("No"))+ "]");
 			}
         }
 	}
@@ -670,7 +687,7 @@ void MenuStateJoinGame::keyPress(SDL_KeyboardEvent c) {
 		//if(c>='0' && c<='9') {
 		if( (key >= SDLK_0 && key <= SDLK_9) ||
 			(key >= SDLK_KP0 && key <= SDLK_KP9)) {
-			if(labelServerIp.getText().size() < maxTextSize) {
+			if((int)labelServerIp.getText().size() < maxTextSize) {
 				string text= labelServerIp.getText();
 				//text.insert(text.end()-1, key);
 				char szCharText[20]="";
@@ -690,7 +707,7 @@ void MenuStateJoinGame::keyPress(SDL_KeyboardEvent c) {
 		}
 		//else if (c=='.') {
 		else if (key == SDLK_PERIOD) {
-			if(labelServerIp.getText().size() < maxTextSize) {
+			if((int)labelServerIp.getText().size() < maxTextSize) {
 				string text= labelServerIp.getText();
 				if(text.size() > 0) {
 					text.insert(text.end() -1, '.');
@@ -708,7 +725,7 @@ void MenuStateJoinGame::keyPress(SDL_KeyboardEvent c) {
 	}
 }
 
-void MenuStateJoinGame::connectToServer() {
+bool MenuStateJoinGame::connectToServer() {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] START\n",__FILE__,__FUNCTION__);
 
 	Config& config= Config::getInstance();
@@ -740,7 +757,7 @@ void MenuStateJoinGame::connectToServer() {
 	for(time_t elapsedWait = time(NULL);
 		clientInterface->getIntroDone() == false &&
 		clientInterface->isConnected() &&
-		difftime(time(NULL),elapsedWait) <= 8;) {
+		difftime(time(NULL),elapsedWait) <= 10;) {
 		if(clientInterface->isConnected()) {
 			//update lobby
 			clientInterface->updateLobby();
@@ -762,8 +779,10 @@ void MenuStateJoinGame::connectToServer() {
 		abortAutoFind = true;
 		clientInterface->stopServerDiscovery();
 		mainMenu->setState(new MenuStateConnectedGame(program, mainMenu));
+		return true;
 	}
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] END\n",__FILE__,__FUNCTION__);
+	return false;
 }
 
 }}//end namespace
