@@ -40,8 +40,8 @@ static string IRC_CHANNEL  = "#megaglest-lobby";
 // 	class MenuStateMasterserver
 // =====================================================
 
-MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMenu):
-	MenuState(program, mainMenu, "masterserver")
+MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMenu) :
+	MenuState(program, mainMenu, "masterserver"), mutexIRCClient(new Mutex(CODE_AT_LINE))
 {
 	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("\n\n\n\n******************** ENTERING MASTERSERVER MENU\n\n\n\n\n");
 
@@ -95,7 +95,7 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
 	announcementLoaded=false;
 
 	mainMessageBox.registerGraphicComponent(containerName,"mainMessageBox");
-	mainMessageBox.init(lang.get("Ok"));
+	mainMessageBox.init(lang.getString("Ok"));
 	mainMessageBox.setEnabled(false);
 	mainMessageBoxState=0;
 
@@ -120,10 +120,10 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
 	labelTitle.init(330, serverLinesYBase+40);
 	labelTitle.setFont(CoreData::getInstance().getMenuFontBig());
 	labelTitle.setFont3D(CoreData::getInstance().getMenuFontBig3D());
-	labelTitle.setText(lang.get("AvailableServers"));
+	labelTitle.setText(lang.getString("AvailableServers"));
 
 	if(Config::getInstance().getString("Masterserver","") == "") {
-		labelTitle.setText("*** " + lang.get("AvailableServers"));
+		labelTitle.setText("*** " + lang.getString("AvailableServers"));
 	}
 
 	// bottom
@@ -139,70 +139,70 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
 	//i+=10;
 	glestVersionLabel.registerGraphicComponent(containerName,"glestVersionLabel");
 	glestVersionLabel.init(i,startOffset-lineOffset);
-	glestVersionLabel.setText(lang.get("MGVersion"));
+	glestVersionLabel.setText(lang.getString("MGVersion"));
 
 	i+=70;
 	platformLabel.registerGraphicComponent(containerName,"platformLabel");
 	platformLabel.init(i,startOffset-lineOffset);
-	platformLabel.setText(lang.get("MGPlatform"));
+	platformLabel.setText(lang.getString("MGPlatform"));
 
 //	i+=50;
 //	binaryCompileDateLabel.registerGraphicComponent(containerName,"binaryCompileDateLabel");
 //	binaryCompileDateLabel.init(i,startOffset-lineOffset);
-//	binaryCompileDateLabel.setText(lang.get("MGBuildDateTime"));
+//	binaryCompileDateLabel.setText(lang.getString("MGBuildDateTime"));
 
 	//game info:
 	i+=130;
 	serverTitleLabel.registerGraphicComponent(containerName,"serverTitleLabel");
 	serverTitleLabel.init(i,startOffset-lineOffset);
-	serverTitleLabel.setText(lang.get("MGGameTitle"));
+	serverTitleLabel.setText(lang.getString("MGGameTitle"));
 
 	i+=150;
 	countryLabel.registerGraphicComponent(containerName,"countryLabel");
 	countryLabel.init(i,startOffset-lineOffset);
-	countryLabel.setText(lang.get("MGGameCountry"));
+	countryLabel.setText(lang.getString("MGGameCountry"));
 
 	i+=65;
 
 //	ipAddressLabel.registerGraphicComponent(containerName,"ipAddressLabel");
 //	ipAddressLabel.init(i,startOffset-lineOffset);
-//	ipAddressLabel.setText(lang.get("MGGameIP"));
+//	ipAddressLabel.setText(lang.getString("MGGameIP"));
 //	i+=100;
 
 	//game setup info:
 	techLabel.registerGraphicComponent(containerName,"techLabel");
 	techLabel.init(i,startOffset-lineOffset);
-	techLabel.setText(lang.get("TechTree"));
+	techLabel.setText(lang.getString("TechTree"));
 
 	i+=120;
 	mapLabel.registerGraphicComponent(containerName,"mapLabel");
 	mapLabel.init(i,startOffset-lineOffset);
-	mapLabel.setText(lang.get("Map"));
+	mapLabel.setText(lang.getString("Map"));
 
 	i+=120;
 //	tilesetLabel.registerGraphicComponent(containerName,"tilesetLabel");
 //	tilesetLabel.init(i,startOffset-lineOffset);
-//	tilesetLabel.setText(lang.get("Tileset"));
+//	tilesetLabel.setText(lang.getString("Tileset"));
 //	i+=100;
 
 	activeSlotsLabel.registerGraphicComponent(containerName,"activeSlotsLabel");
 	activeSlotsLabel.init(i,startOffset-lineOffset);
-	activeSlotsLabel.setText(lang.get("MGGameSlots"));
+	activeSlotsLabel.setText(lang.getString("MGGameSlots"));
 
 	i+=50;
 	externalConnectPort.registerGraphicComponent(containerName,"externalConnectPort");
 	externalConnectPort.init(i,startOffset-lineOffset);
-	externalConnectPort.setText(lang.get("Port"));
+	externalConnectPort.setText(lang.getString("Port"));
 
 	i+=60;
 	statusLabel.registerGraphicComponent(containerName,"statusLabel");
 	statusLabel.init(i,startOffset-lineOffset);
-	statusLabel.setText(lang.get("MGGameStatus"));
+	statusLabel.setText(lang.getString("MGGameStatus"));
 
 	i+=130;
 	selectButton.registerGraphicComponent(containerName,"selectButton");
 	selectButton.init(i, startOffset-lineOffset);
-	selectButton.setText(lang.get("MGJoinGameSlots"));
+	selectButton.setText(lang.getString("MGJoinGameSlots"));
 
 	// Titles for current games - END
 
@@ -215,17 +215,17 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
     buttonRefresh.registerGraphicComponent(containerName,"buttonRefresh");
     buttonRefresh.init(550, buttonPos, 150);
 
-	buttonRefresh.setText(lang.get("RefreshList"));
-	buttonReturn.setText(lang.get("Return"));
-	buttonCreateGame.setText(lang.get("HostGame"));
-	labelAutoRefresh.setText(lang.get("AutoRefreshRate"));
+	buttonRefresh.setText(lang.getString("RefreshList"));
+	buttonReturn.setText(lang.getString("Return"));
+	buttonCreateGame.setText(lang.getString("HostGame"));
+	labelAutoRefresh.setText(lang.getString("AutoRefreshRate"));
 
 	labelAutoRefresh.registerGraphicComponent(containerName,"labelAutoRefresh");
 	labelAutoRefresh.init(800,buttonPos+30);
 
 	listBoxAutoRefresh.registerGraphicComponent(containerName,"listBoxAutoRefresh");
 	listBoxAutoRefresh.init(800,buttonPos);
-	listBoxAutoRefresh.pushBackItem(lang.get("Off"));
+	listBoxAutoRefresh.pushBackItem(lang.getString("Off"));
 	listBoxAutoRefresh.pushBackItem("10 s");
 	listBoxAutoRefresh.pushBackItem("20 s");
 	listBoxAutoRefresh.pushBackItem("30 s");
@@ -234,7 +234,7 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
 
 	ircOnlinePeopleLabel.registerGraphicComponent(containerName,"ircOnlinePeopleLabel");
 	ircOnlinePeopleLabel.init(userButtonsXBase,userButtonsYBase+userButtonsLineHeight);
-	ircOnlinePeopleLabel.setText(lang.get("IRCPeopleOnline"));
+	ircOnlinePeopleLabel.setText(lang.getString("IRCPeopleOnline"));
 
 	ircOnlinePeopleStatusLabel.registerGraphicComponent(containerName,"ircOnlinePeopleStatusLabel");
 	ircOnlinePeopleStatusLabel.init(userButtonsXBase,userButtonsYBase+userButtonsLineHeight-20);
@@ -245,7 +245,7 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
 	NetworkManager::getInstance().end();
 	NetworkManager::getInstance().init(nrClient);
 
-	//console.addLine(lang.get("ToSwitchOffMusicPress")+" - \""+configKeys.getCharKey("ToggleMusic")+"\"");
+	//console.addLine(lang.getString("ToSwitchOffMusicPress")+" - \""+configKeys.getCharKey("ToggleMusic")+"\"");
 
 	GraphicComponent::applyAllCustomProperties(containerName);
 
@@ -270,8 +270,10 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
     chatManager.setFont3D(CoreData::getInstance().getMenuFontNormal3D());
 
 	needUpdateFromServer = true;
+
+	static string mutexOwnerId = string(extractFileFromDirectoryPath(__FILE__).c_str()) + string("_") + intToStr(__LINE__);
 	updateFromMasterserverThread = new SimpleTaskThread(this,0,100);
-	updateFromMasterserverThread->setUniqueID(extractFileFromDirectoryPath(__FILE__).c_str());
+	updateFromMasterserverThread->setUniqueID(mutexOwnerId);
 	updateFromMasterserverThread->start();
 
     if(Config::getInstance().getString("IRCServer","") != "") {
@@ -304,7 +306,7 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
     	ircArgs.push_back("");
     }
 
-    MutexSafeWrapper safeMutexIRCPtr(&mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
+    MutexSafeWrapper safeMutexIRCPtr(mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
 
     if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 IRCCLient Cache check\n");
     IRCThread * &ircThread = CacheManager::getCachedItem< IRCThread * >(GameConstants::ircClientCacheLookupKey);
@@ -321,9 +323,11 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
 
     if(ircThread == NULL) {
     	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 IRCCLient Cache check\n");
+
+    	static string mutexOwnerId = string(extractFileFromDirectoryPath(__FILE__).c_str()) + string("_") + intToStr(__LINE__);
     	ircThread = new IRCThread(ircArgs,this);
     	ircClient = ircThread;
-    	ircClient->setUniqueID(extractFileFromDirectoryPath(__FILE__).c_str());
+    	ircClient->setUniqueID(mutexOwnerId);
     	ircClient->setPlayerName(netPlayerName);
 		ircClient->start();
     }
@@ -334,6 +338,11 @@ MenuStateMasterserver::MenuStateMasterserver(Program *program, MainMenu *mainMen
     	ircClient->joinChannel();
     }
 
+    if(netPlayerName=="newbie"){
+    	showMessageBox(lang.getString("PlayerNameNotSetPrompt"),lang.getString("PlayerNameNotSetTitle"),false);
+    }
+    //showMessageBox("Go back and set your name in the game options!\n\nAt the moment you are just called >>newbie<< !","Player name not set!",false);
+
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 }
 
@@ -344,7 +353,7 @@ void MenuStateMasterserver::reloadUI() {
 	consoleIRC.setFont(CoreData::getInstance().getMenuFontNormal());
 	consoleIRC.setFont3D(CoreData::getInstance().getMenuFontNormal3D());
 
-	mainMessageBox.init(lang.get("Ok"));
+	mainMessageBox.init(lang.getString("Ok"));
 
     announcementLabel.setFont(CoreData::getInstance().getMenuFontBig());
     announcementLabel.setFont3D(CoreData::getInstance().getMenuFontBig3D());
@@ -354,40 +363,40 @@ void MenuStateMasterserver::reloadUI() {
 
 	labelTitle.setFont(CoreData::getInstance().getMenuFontBig());
 	labelTitle.setFont3D(CoreData::getInstance().getMenuFontBig3D());
-	labelTitle.setText(lang.get("AvailableServers"));
+	labelTitle.setText(lang.getString("AvailableServers"));
 
 	if(Config::getInstance().getString("Masterserver","") == "") {
-		labelTitle.setText("*** " + lang.get("AvailableServers"));
+		labelTitle.setText("*** " + lang.getString("AvailableServers"));
 	}
 
-	glestVersionLabel.setText(lang.get("MGVersion"));
+	glestVersionLabel.setText(lang.getString("MGVersion"));
 
-	platformLabel.setText(lang.get("MGPlatform"));
+	platformLabel.setText(lang.getString("MGPlatform"));
 
-	serverTitleLabel.setText(lang.get("MGGameTitle"));
+	serverTitleLabel.setText(lang.getString("MGGameTitle"));
 
-	countryLabel.setText(lang.get("MGGameCountry"));
+	countryLabel.setText(lang.getString("MGGameCountry"));
 
-	techLabel.setText(lang.get("TechTree"));
+	techLabel.setText(lang.getString("TechTree"));
 
-	mapLabel.setText(lang.get("Map"));
+	mapLabel.setText(lang.getString("Map"));
 
-	activeSlotsLabel.setText(lang.get("MGGameSlots"));
+	activeSlotsLabel.setText(lang.getString("MGGameSlots"));
 
-	externalConnectPort.setText(lang.get("Port"));
+	externalConnectPort.setText(lang.getString("Port"));
 
-	statusLabel.setText(lang.get("MGGameStatus"));
+	statusLabel.setText(lang.getString("MGGameStatus"));
 
-	selectButton.setText(lang.get("MGJoinGameSlots"));
+	selectButton.setText(lang.getString("MGJoinGameSlots"));
 
 	// Titles for current games - END
 
-	buttonRefresh.setText(lang.get("RefreshList"));
-	buttonReturn.setText(lang.get("Return"));
-	buttonCreateGame.setText(lang.get("HostGame"));
-	labelAutoRefresh.setText(lang.get("AutoRefreshRate"));
+	buttonRefresh.setText(lang.getString("RefreshList"));
+	buttonReturn.setText(lang.getString("Return"));
+	buttonCreateGame.setText(lang.getString("HostGame"));
+	labelAutoRefresh.setText(lang.getString("AutoRefreshRate"));
 
-	ircOnlinePeopleLabel.setText(lang.get("IRCPeopleOnline"));
+	ircOnlinePeopleLabel.setText(lang.getString("IRCPeopleOnline"));
 
     chatManager.setFont(CoreData::getInstance().getMenuFontNormal());
     chatManager.setFont3D(CoreData::getInstance().getMenuFontNormal3D());
@@ -410,7 +419,7 @@ void MenuStateMasterserver::setButtonLinePosition(int pos){
 }
 
 void MenuStateMasterserver::IRC_CallbackEvent(IRCEventType evt, const char* origin, const char **params, unsigned int count) {
-    MutexSafeWrapper safeMutexIRCPtr(&mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
+    MutexSafeWrapper safeMutexIRCPtr(mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
     if(ircClient != NULL) {
         if(evt == IRC_evt_exitThread) {
         	ircClient->leaveChannel();
@@ -444,20 +453,46 @@ void MenuStateMasterserver::cleanup() {
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
-    if(updateFromMasterserverThread != NULL &&
-        updateFromMasterserverThread->canShutdown(true) == true) {
-        if(updateFromMasterserverThread->shutdownAndWait() == true) {
-            delete updateFromMasterserverThread;
-        }
-    }
-    updateFromMasterserverThread = NULL;
+//    if(updateFromMasterserverThread != NULL &&
+//        updateFromMasterserverThread->canShutdown(true) == true) {
+//        if(updateFromMasterserverThread->shutdownAndWait() == true) {
+//            delete updateFromMasterserverThread;
+//        }
+//    }
+//    updateFromMasterserverThread = NULL;
+	if(updateFromMasterserverThread != NULL) {
+		updateFromMasterserverThread->signalQuit();
+	}
+	if(updateFromMasterserverThread != NULL && updateFromMasterserverThread->canShutdown(false) == true  &&
+			updateFromMasterserverThread->getRunningStatus() == false) {
+		//printf("#2 Ending client SLOT: %d\n",playerIndex);
+
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+        delete updateFromMasterserverThread;
+        if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	}
+	//else if(BaseThread::shutdownAndWait(slotThreadWorker) == true) {
+	else if(updateFromMasterserverThread != NULL && updateFromMasterserverThread->canShutdown(true) == true) {
+		if(updateFromMasterserverThread->getRunningStatus() == false) {
+			//printf("#3 Ending client SLOT: %d\n",playerIndex);
+			if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+			delete updateFromMasterserverThread;
+			if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		}
+		else {
+			updateFromMasterserverThread->setDeleteSelfOnExecutionDone(true);
+			updateFromMasterserverThread->setDeleteAfterExecute(true);
+		}
+	}
+	//printf("#4 Ending client SLOT: %d\n",playerIndex);
+	updateFromMasterserverThread = NULL;
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	clearServerLines();
 	clearUserButtons();
 
-    MutexSafeWrapper safeMutexIRCPtr(&mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
+    MutexSafeWrapper safeMutexIRCPtr(mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
     if(ircClient != NULL) {
     	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
@@ -473,7 +508,12 @@ void MenuStateMasterserver::cleanup() {
 
 MenuStateMasterserver::~MenuStateMasterserver() {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
 	cleanup();
+
+	delete mutexIRCClient;
+	mutexIRCClient = NULL;
+
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] END\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 }
 
@@ -499,23 +539,21 @@ void MenuStateMasterserver::mouseClick(int x, int y, MouseButton mouseButton){
 
 	if(mainMessageBox.getEnabled()){
 		int button= 0;
-		if(mainMessageBox.mouseClick(x, y, button))
-		{
+		if(mainMessageBox.mouseClick(x, y, button)) {
 			soundRenderer.playFx(coreData.getClickSoundA());
-			if(button==0)
-			{
+			if(button == 0) {
 				mainMessageBox.setEnabled(false);
 			}
 		}
 	}
 	else if(userScrollBar.mouseClick(x, y)){
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
-			soundRenderer.playFx(coreData.getClickSoundB());
+			soundRenderer.playFx(coreData.getClickSoundA());
 			if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 	    }
 	else if(serverScrollBar.mouseClick(x, y)){
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
-			soundRenderer.playFx(coreData.getClickSoundB());
+			soundRenderer.playFx(coreData.getClickSoundA());
 			if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 	    }
 	else if(buttonRefresh.mouseClick(x, y)){
@@ -530,7 +568,7 @@ void MenuStateMasterserver::mouseClick(int x, int y, MouseButton mouseButton){
     else if(buttonReturn.mouseClick(x, y)){
     	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
-		soundRenderer.playFx(coreData.getClickSoundB());
+		soundRenderer.playFx(coreData.getClickSoundA());
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 		if (ircClient != NULL && ircClient->isConnected() == true
@@ -587,6 +625,8 @@ void MenuStateMasterserver::mouseClick(int x, int y, MouseButton mouseButton){
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 					safeMutex.ReleaseLock();
 					if(connected){
+						soundRenderer.playFx(coreData.getClickSoundB());
+
 						if (ircClient != NULL && ircClient->isConnected() == true
 									&& ircClient->getHasJoinedChannel() == true) {
 							ircClient->SendIRCCmdMessage(IRC_CHANNEL, "connecting to '"+serverLines[i]->getMasterServerInfo()->getServerTitle()+"'");
@@ -603,7 +643,7 @@ void MenuStateMasterserver::mouseClick(int x, int y, MouseButton mouseButton){
 	    if(!clicked && userScrollBar.getElementCount()!=0){
 	    	for(int i = userScrollBar.getVisibleStart(); i <= userScrollBar.getVisibleEnd(); ++i) {
 	       		if(userButtons[i]->getEnabled() == true && userButtons[i]->mouseClick(x, y)) {
-	       			clicked=true;
+	       			//clicked=true;
 	     			if(!chatManager.getEditEnabled())
 	     			{
 	     				chatManager.switchOnEdit();
@@ -696,7 +736,7 @@ void MenuStateMasterserver::render(){
 		renderer.renderLabel(&selectButton,&titleLabelColor);
 
 		Lang &lang= Lang::getInstance();
-		MutexSafeWrapper safeMutexIRCPtr(&mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
+		MutexSafeWrapper safeMutexIRCPtr(mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
         if(ircClient != NULL &&
            ircClient->isConnected() == true &&
            ircClient->getHasJoinedChannel() == true) {
@@ -712,8 +752,8 @@ void MenuStateMasterserver::render(){
         else {
             const Vec4f titleLabelColor = RED;
 
-            if(ircOnlinePeopleStatusLabel.getText() != lang.get("Connecting")) {
-            	ircOnlinePeopleStatusLabel.setText(lang.get("Connecting"));
+            if(ircOnlinePeopleStatusLabel.getText() != lang.getString("Connecting")) {
+            	ircOnlinePeopleStatusLabel.setText(lang.getString("Connecting"));
             }
 
             renderer.renderLabel(&ircOnlinePeopleLabel,&titleLabelColor);
@@ -721,7 +761,7 @@ void MenuStateMasterserver::render(){
         }
         safeMutexIRCPtr.ReleaseLock();
 
-        const Vec4f titleLabelColorList = YELLOW;
+        //const Vec4f titleLabelColorList = YELLOW;
 
 		if(serverScrollBar.getElementCount()!=0 ) {
 			for(int i = serverScrollBar.getVisibleStart(); i <= serverScrollBar.getVisibleEnd(); ++i) {
@@ -730,7 +770,7 @@ void MenuStateMasterserver::render(){
 		}
 		renderer.renderScrollBar(&serverScrollBar);
 
-	    for(int i=0; i<sizeof(lines) / sizeof(lines[0]); ++i){
+	    for(int i = 0; i < (int)sizeof(lines) / (int)sizeof(lines[0]); ++i){
 	    	renderer.renderLine(&lines[i]);
 	    }
 	    renderer.renderButton(&buttonRefresh);
@@ -785,7 +825,7 @@ void MenuStateMasterserver::update() {
     //console
     consoleIRC.update();
 
-    MutexSafeWrapper safeMutexIRCPtr(&mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
+    MutexSafeWrapper safeMutexIRCPtr(mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
     if(ircClient != NULL) {
         std::vector<string> nickList = ircClient->getNickList();
 
@@ -810,7 +850,7 @@ void MenuStateMasterserver::update() {
 
         if(isNew) {
 	        clearUserButtons();
-	        for(int i = 0; i < nickList.size(); ++i) {
+	        for(int i = 0; i < (int)nickList.size(); ++i) {
 	                GraphicButton *button=new GraphicButton();
 	                button->init(userButtonsXBase,userButtonsYBase,userButtonsWidth,userButtonsHeight);
 	                //button->init(userButtonsXBase,userButtonsYBase-userButtonsLineHeight*i,userButtonsWidth,userButtonsHeight);
@@ -826,7 +866,7 @@ void MenuStateMasterserver::update() {
 
 	                userButtons.push_back(button);
 	        }
-	        userScrollBar.setElementCount(userButtons.size());
+	        userScrollBar.setElementCount((int)userButtons.size());
 	        oldNickList = nickList;
 	        chatManager.setAutoCompleteTextList(oldNickList);
         }
@@ -843,7 +883,7 @@ void MenuStateMasterserver::update() {
     	serverInfoString="empty";
     }
 
-    serverScrollBar.setElementCount(serverLines.size());
+    serverScrollBar.setElementCount((int)serverLines.size());
 	if(serverScrollBar.getElementCount()!=0 ) {
 		for(int i = serverScrollBar.getVisibleStart(); i <= serverScrollBar.getVisibleEnd(); ++i) {
 			serverLines[i]->setY(serverLinesYBase-serverLinesLineHeight*(i-serverScrollBar.getVisibleStart()));
@@ -863,7 +903,7 @@ void MenuStateMasterserver::update() {
 	}
 }
 
-void MenuStateMasterserver::simpleTask(BaseThread *callingThread) {
+void MenuStateMasterserver::simpleTask(BaseThread *callingThread,void *userdata) {
 	if(callingThread->getQuitStatus() == true) {
 		return;
 	}
@@ -952,7 +992,7 @@ void MenuStateMasterserver::simpleTask(BaseThread *callingThread) {
                 consoleIRC.addLine("---------------------------------------------");
                 // write hint to console:
                 Config &configKeys = Config::getInstance(std::pair<ConfigType,ConfigType>(cfgMainKeys,cfgUserKeys));
-                consoleIRC.addLine(Lang::getInstance().get("ToSwitchOffMusicPress")+" - \""+configKeys.getString("ToggleMusic")+"\"");
+                consoleIRC.addLine(Lang::getInstance().getString("ToSwitchOffMusicPress")+" - \""+configKeys.getString("ToggleMusic")+"\"");
 
                 announcementLoaded=true;
             }
@@ -963,7 +1003,10 @@ void MenuStateMasterserver::simpleTask(BaseThread *callingThread) {
 
                     safeMutex.ReleaseLock(true);
 					CURL *handle = SystemFlags::initHTTP();
-                    std::string localServerInfoString = SystemFlags::getHTTP(Config::getInstance().getString("Masterserver") + "showServersForGlest.php",handle);
+
+					string playerUUID = "?uuid=" + SystemFlags::escapeURL(Config::getInstance().getString("PlayerId",""));
+                    std::string localServerInfoString = SystemFlags::getHTTP(
+                    		Config::getInstance().getString("Masterserver") + "showServersForGlest.php" + playerUUID,handle);
 					SystemFlags::cleanupHTTP(&handle);
                     if(callingThread->getQuitStatus() == true) {
                         return;
@@ -987,7 +1030,7 @@ void MenuStateMasterserver::simpleTask(BaseThread *callingThread) {
 }
 
 void MenuStateMasterserver::rebuildServerLines(const string &serverInfo) {
-	int numberOfOldServerLines=serverLines.size();
+	int numberOfOldServerLines = (int)serverLines.size();
 	clearServerLines();
     Lang &lang= Lang::getInstance();
     try {
@@ -997,7 +1040,7 @@ void MenuStateMasterserver::rebuildServerLines(const string &serverInfo) {
 
 			std::vector<std::string> serverList;
 			Tokenize(serverInfo,serverList,"\n");
-			for(int i=0; i < serverList.size(); i++) {
+			for(int i = 0; i < (int)serverList.size(); i++) {
 				string &server = serverList[i];
 				if(trim(server) == "") {
 					continue;
@@ -1008,11 +1051,11 @@ void MenuStateMasterserver::rebuildServerLines(const string &serverInfo) {
 
 				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("--------------> server [%s] serverEntities.size() = " MG_SIZE_T_SPECIFIER " MIN_FIELDS_EXPECTED = %d\n",server.c_str(),serverEntities.size(),MIN_FIELDS_EXPECTED);
 
-				if(serverEntities.size() >= MIN_FIELDS_EXPECTED) {
-					labelTitle.setText(lang.get("AvailableServers"));
+				if((int)serverEntities.size() >= MIN_FIELDS_EXPECTED) {
+					labelTitle.setText(lang.getString("AvailableServers"));
 
 					if(Config::getInstance().getString("Masterserver","") == "") {
-						labelTitle.setText("*** " + lang.get("AvailableServers"));
+						labelTitle.setText("*** " + lang.getString("AvailableServers"));
 					}
 
 					MasterServerInfo *masterServerInfo=new MasterServerInfo();
@@ -1049,12 +1092,13 @@ void MenuStateMasterserver::rebuildServerLines(const string &serverInfo) {
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 					serverLines.push_back(new ServerLine( masterServerInfo, i, serverLinesYBase, serverLinesLineHeight, containerName));
+					delete masterServerInfo;
 				}
 				else {
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 					Lang &lang= Lang::getInstance();
-					labelTitle.setText("*** " + lang.get("AvailableServers") + "[" + intToStr(serverEntities.size()) + "][" + intToStr(MIN_FIELDS_EXPECTED) + "] [" + serverInfo + "]");
+					labelTitle.setText("*** " + lang.getString("AvailableServers") + "[" + intToStr(serverEntities.size()) + "][" + intToStr(MIN_FIELDS_EXPECTED) + "] [" + serverInfo + "]");
 
 					if(masterserverParseErrorShown == false) {
 						masterserverParseErrorShown = true;
@@ -1070,12 +1114,12 @@ void MenuStateMasterserver::rebuildServerLines(const string &serverInfo) {
 
     }
 	catch(const exception &ex) {
-	    labelTitle.setText("*** " + lang.get("AvailableServers") + " [" + ex.what() + "]");
+	    labelTitle.setText("*** " + lang.getString("AvailableServers") + " [" + ex.what() + "]");
 	    SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line %d] error during Internet game status update: [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
 	}
 
-	if(serverLines.size()>numberOfOldServerLines) {
-		playServerFoundSound=true;
+	if((int)serverLines.size() > numberOfOldServerLines) {
+		playServerFoundSound = true;
 	}
 }
 
@@ -1097,7 +1141,7 @@ bool MenuStateMasterserver::connectToServer(string ipString, int port) {
 
 		mainMessageBoxState=1;
 		Lang &lang= Lang::getInstance();
-		showMessageBox(lang.get("CouldNotConnect"), lang.get("ConnectionFailed"), false);
+		showMessageBox(lang.getString("CouldNotConnect"), lang.getString("ConnectionFailed"), false);
 		return false;
 
 		//if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] connection failed\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__);
@@ -1153,7 +1197,7 @@ void MenuStateMasterserver::keyDown(SDL_KeyboardEvent key) {
 		//chatmanger only if connected to irc!
 		if (chatManager.getEditEnabled() == true) {
 			//printf("keyDown key [%d] chatManager.getText() [%s]\n",key,chatManager.getText().c_str());
-			MutexSafeWrapper safeMutexIRCPtr(&mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
+			MutexSafeWrapper safeMutexIRCPtr(mutexIRCClient,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
 			//if (key == vkReturn && ircClient != NULL) {
 			if(isKeyPressed(SDLK_RETURN,key) == true && ircClient != NULL) {
 				ircClient->SendIRCCmdMessage(IRC_CHANNEL, chatManager.getText());
@@ -1172,20 +1216,20 @@ void MenuStateMasterserver::keyDown(SDL_KeyboardEvent key) {
             float currentVolume = CoreData::getInstance().getMenuMusic()->getVolume();
             if(currentVolume > 0) {
                 CoreData::getInstance().getMenuMusic()->setVolume(0.f);
-                consoleIRC.addLine(lang.get("GameMusic") + " " + lang.get("Off"));
+                consoleIRC.addLine(lang.getString("GameMusic") + " " + lang.getString("Off"));
             }
             else {
                 CoreData::getInstance().getMenuMusic()->setVolume(configVolume);
                 //If the config says zero, use the default music volume
                 //gameMusic->setVolume(configVolume ? configVolume : 0.9);
-                consoleIRC.addLine(lang.get("GameMusic"));
+                consoleIRC.addLine(lang.getString("GameMusic"));
             }
         }
         //else if(key == configKeys.getCharKey("SaveGUILayout")) {
     	else if(isKeyPressed(configKeys.getSDLKey("SaveGUILayout"),key) == true) {
             bool saved = GraphicComponent::saveAllCustomProperties(containerName);
             Lang &lang= Lang::getInstance();
-            consoleIRC.addLine(lang.get("GUILayoutSaved") + " [" + (saved ? lang.get("Yes") : lang.get("No"))+ "]");
+            consoleIRC.addLine(lang.getString("GUILayoutSaved") + " [" + (saved ? lang.getString("Yes") : lang.getString("No"))+ "]");
         }
     }
 }
