@@ -177,7 +177,7 @@ private:
 	void loadMapPreview(string name);
 	void showRemoteDesription(ModInfo *modInfo);
 
-	std::map<string,uint32> mapCRCUpdateList;
+	std::map<string,bool> mapCRCUpdateList;
 public:
 
 	MenuStateMods(Program *program, MainMenu *mainMenu);
@@ -193,7 +193,7 @@ public:
     virtual void keyPress(SDL_KeyboardEvent c);
     virtual void keyUp(SDL_KeyboardEvent key);
 
-    virtual void simpleTask(BaseThread *callingThread);
+    virtual void simpleTask(BaseThread *callingThread,void *userdata);
 
     virtual void reloadUI();
 
