@@ -25,7 +25,7 @@
 
 namespace Glest{ namespace Game{
 
-using namespace	Shared::Xml;
+using namespace	::Shared::Xml;
 
 // =====================================================
 // 	class MenuStateScenario
@@ -50,7 +50,7 @@ MenuStateScenario::MenuStateScenario(Program *program, MainMenu *mainMenu,
     }
 	catch(const std::exception &ex) {
 		char szBuf[8096]="";
-		snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
+		snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
 		SystemFlags::OutputDebug(SystemFlags::debugError,szBuf);
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s",szBuf);
 
@@ -59,7 +59,7 @@ MenuStateScenario::MenuStateScenario(Program *program, MainMenu *mainMenu,
 	}
 
 	mainMessageBox.registerGraphicComponent(containerName,"mainMessageBox");
-	mainMessageBox.init(lang.get("Ok"));
+	mainMessageBox.init(lang.getString("Ok"));
 	mainMessageBox.setEnabled(false);
 	mainMessageBoxState=0;
 
@@ -68,34 +68,41 @@ MenuStateScenario::MenuStateScenario(Program *program, MainMenu *mainMenu,
 
 	this->dirList = dirList;
 
-	int startY=100;
-	int startX=350;
+	int buttonStartY=50;
+	int buttonStartX=70;
+
+	buttonReturn.registerGraphicComponent(containerName,"buttonReturn");
+    buttonReturn.init(buttonStartX, buttonStartY, 125);
+	buttonReturn.setText(lang.getString("Return"));
+
+    buttonPlayNow.registerGraphicComponent(containerName,"buttonPlayNow");
+	buttonPlayNow.init(buttonStartX+150, buttonStartY, 125);
+	buttonPlayNow.setText(lang.getString("PlayNow"));
+
+	int startY=700;
+	int startX=50;
+
+    labelScenario.registerGraphicComponent(containerName,"labelScenario");
+	labelScenario.init(startX, startY);
+
+	listBoxScenario.registerGraphicComponent(containerName,"listBoxScenario");
+    listBoxScenario.init(startX, startY-30, 290);
+
+	labelScenarioName.registerGraphicComponent(containerName,"labelScenarioName");
+	labelScenarioName.init(startX, startY-80);
+	labelScenarioName.setFont(CoreData::getInstance().getMenuFontBig());
+	labelScenarioName.setFont3D(CoreData::getInstance().getMenuFontBig3D());
 
 	labelInfo.registerGraphicComponent(containerName,"labelInfo");
-    labelInfo.init(startX, startY+130);
+    labelInfo.init(startX, startY-110);
 	labelInfo.setFont(CoreData::getInstance().getMenuFontNormal());
 	labelInfo.setFont3D(CoreData::getInstance().getMenuFontNormal3D());
 
-	buttonReturn.registerGraphicComponent(containerName,"buttonReturn");
-    buttonReturn.init(startX, startY, 125);
-
-    buttonPlayNow.registerGraphicComponent(containerName,"buttonPlayNow");
-	buttonPlayNow.init(startX+175, startY, 125);
-
-	listBoxScenario.registerGraphicComponent(containerName,"listBoxScenario");
-    listBoxScenario.init(startX, startY+160, 190);
-
-    labelScenario.registerGraphicComponent(containerName,"labelScenario");
-	labelScenario.init(startX, startY+190);
-
-	buttonReturn.setText(lang.get("Return"));
-	buttonPlayNow.setText(lang.get("PlayNow"));
-
 	if(this->isTutorialMode == true) {
-		labelScenario.setText(lang.get("Tutorial"));
+		labelScenario.setText(lang.getString("Tutorial"));
 	}
 	else {
-		labelScenario.setText(lang.get("Scenario"));
+		labelScenario.setText(lang.getString("Scenario"));
 	}
 
     //scenario listbox
@@ -113,43 +120,69 @@ MenuStateScenario::MenuStateScenario(Program *program, MainMenu *mainMenu,
         	showMessageBox( "Error: There are no scenarios found to load", "Error detected", false);
         }
 	}
-	for(int i= 0; i<results.size(); ++i){
-		results[i] = formatString(results[i]);
+
+	std::map<string,string> scenarioErrors;
+	for(int i= 0; i < (int)results.size(); ++i){
+			results[i] = formatString(results[i]);
 	}
     listBoxScenario.setItems(results);
 
     try {
     	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] listBoxScenario.getSelectedItemIndex() = %d scenarioFiles.size() = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,listBoxScenario.getSelectedItemIndex(),(int)scenarioFiles.size());
-    	if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 && listBoxScenario.getSelectedItemIndex() < scenarioFiles.size()) {
-    		loadScenarioInfo(Scenario::getScenarioPath(dirList, scenarioFiles[listBoxScenario.getSelectedItemIndex()]), &scenarioInfo );
+
+    	if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 && listBoxScenario.getSelectedItemIndex() < (int)scenarioFiles.size()) {
+    		string scenarioPath = Scenario::getScenarioPath(dirList, scenarioFiles[listBoxScenario.getSelectedItemIndex()]);
+    		//printf("scenarioPath [%s]\n",scenarioPath.c_str());
+
+    		loadScenarioInfo(scenarioPath, &scenarioInfo );
     		labelInfo.setText(scenarioInfo.desc);
+    		if(scenarioInfo.namei18n != "") {
+    			labelScenarioName.setText(scenarioInfo.namei18n);
+    		}
+    		else {
+    			labelScenarioName.setText(listBoxScenario.getSelectedItem());
+    		}
     	}
 
         GraphicComponent::applyAllCustomProperties(containerName);
     }
 	catch(const std::exception &ex) {
 		char szBuf[8096]="";
-		snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
+		snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
 		SystemFlags::OutputDebug(SystemFlags::debugError,szBuf);
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s",szBuf);
 
         mainMessageBoxState=1;
         showMessageBox( "Error: " + string(ex.what()), "Error detected", false);
 	}
+
+	if(scenarioErrors.empty() == false) {
+        mainMessageBoxState=1;
+
+        string errorMsg = "";
+        for(std::map<string,string>::iterator iterMap = scenarioErrors.begin();
+        		iterMap != scenarioErrors.end(); ++iterMap) {
+        	errorMsg += "scenario: " + iterMap->first + " error text: " + iterMap->second.substr(0,400) + "\n";
+        }
+        showMessageBox( "Error loading scenario(s): " + errorMsg, "Error detected", false);
+	}
 }
 
 void MenuStateScenario::reloadUI() {
 	Lang &lang= Lang::getInstance();
 
 	console.resetFonts();
-	mainMessageBox.init(lang.get("Ok"));
+	mainMessageBox.init(lang.getString("Ok"));
 	labelInfo.setFont(CoreData::getInstance().getMenuFontNormal());
 	labelInfo.setFont3D(CoreData::getInstance().getMenuFontNormal3D());
 
-	buttonReturn.setText(lang.get("Return"));
-	buttonPlayNow.setText(lang.get("PlayNow"));
+	labelScenarioName.setFont(CoreData::getInstance().getMenuFontNormal());
+	labelScenarioName.setFont3D(CoreData::getInstance().getMenuFontNormal3D());
 
-    labelScenario.setText(lang.get("Scenario"));
+	buttonReturn.setText(lang.getString("Return"));
+	buttonPlayNow.setText(lang.getString("PlayNow"));
+
+    labelScenario.setText(lang.getString("Scenario"));
 
 	GraphicComponent::reloadFontsForRegisterGraphicComponents(containerName);
 }
@@ -159,7 +192,7 @@ MenuStateScenario::~MenuStateScenario() {
 }
 
 void MenuStateScenario::cleanupPreviewTexture() {
-	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] scenarioLogoTexture [%p]\n",__FILE__,__FUNCTION__,__LINE__,scenarioLogoTexture);
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] scenarioLogoTexture [%p]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,scenarioLogoTexture);
 
 	if(scenarioLogoTexture != NULL) {
 		Renderer::getInstance().endTexture(rsGlobal, scenarioLogoTexture, false);
@@ -170,6 +203,7 @@ void MenuStateScenario::cleanupPreviewTexture() {
 void MenuStateScenario::mouseClick(int x, int y, MouseButton mouseButton) {
 	CoreData &coreData= CoreData::getInstance();
 	SoundRenderer &soundRenderer= SoundRenderer::getInstance();
+	string advanceToItemStartingWith = "";
 
 	if(mainMessageBox.getEnabled()){
 		int button= 0;
@@ -184,8 +218,19 @@ void MenuStateScenario::mouseClick(int x, int y, MouseButton mouseButton) {
 				}
 			}
 		}
+		return;
 	}
-	else if(buttonReturn.mouseClick(x,y)){
+    else {
+    	if(::Shared::Platform::Window::isKeyStateModPressed(KMOD_SHIFT) == true) {
+    		const wchar_t lastKey = ::Shared::Platform::Window::extractLastKeyPressed();
+//        		xxx:
+//        		string hehe=lastKey;
+//        		printf("lastKey = %d [%c] '%s'\n",lastKey,lastKey,hehe);
+    		advanceToItemStartingWith =  lastKey;
+    	}
+    }
+
+	if(buttonReturn.mouseClick(x,y)){
 		soundRenderer.playFx(coreData.getClickSoundA());
 		mainMenu->setState(new MenuStateNewGame(program, mainMenu));
 		return;
@@ -195,18 +240,24 @@ void MenuStateScenario::mouseClick(int x, int y, MouseButton mouseButton) {
 		launchGame();
 		return;
 	}
-    else if(listBoxScenario.mouseClick(x, y)){
+    else if(listBoxScenario.mouseClick(x, y,advanceToItemStartingWith)){
         try {
         	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] listBoxScenario.getSelectedItemIndex() = %d scenarioFiles.size() = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,listBoxScenario.getSelectedItemIndex(),(int)scenarioFiles.size());
 
-        	if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 && listBoxScenario.getSelectedItemIndex() < scenarioFiles.size()) {
+        	if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 && listBoxScenario.getSelectedItemIndex() < (int)scenarioFiles.size()) {
         		loadScenarioInfo(Scenario::getScenarioPath(dirList, scenarioFiles[listBoxScenario.getSelectedItemIndex()]), &scenarioInfo);
         		labelInfo.setText(scenarioInfo.desc);
+        		if(scenarioInfo.namei18n != "") {
+        			labelScenarioName.setText(scenarioInfo.namei18n);
+        		}
+        		else {
+        			labelScenarioName.setText(listBoxScenario.getSelectedItem());
+        		}
         	}
         }
         catch(const std::exception &ex) {
             char szBuf[8096]="";
-            snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
+            snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
             SystemFlags::OutputDebug(SystemFlags::debugError,szBuf);
             if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s",szBuf);
 
@@ -233,7 +284,7 @@ void MenuStateScenario::render(){
 	Renderer &renderer= Renderer::getInstance();
 
 	if(scenarioLogoTexture != NULL) {
-		renderer.renderTextureQuad(300,350,400,300,scenarioLogoTexture,1.0f);
+		renderer.renderTextureQuad(450,200,533,400,scenarioLogoTexture,1.0f);
 		//renderer.renderBackground(scenarioLogoTexture);
 	}
 
@@ -242,6 +293,8 @@ void MenuStateScenario::render(){
 	}
 	else {
 		renderer.renderLabel(&labelInfo);
+		renderer.renderLabel(&labelScenarioName);
+
 		renderer.renderLabel(&labelScenario);
 		renderer.renderListBox(&listBoxScenario);
 
@@ -258,23 +311,64 @@ void MenuStateScenario::update() {
 		return;
 	}
 	if(this->autoloadScenarioName != "") {
-		listBoxScenario.setSelectedItem(formatString(this->autoloadScenarioName),false);
+		string scenarioPath = Scenario::getScenarioPath(dirList, this->autoloadScenarioName);
 
-		if(listBoxScenario.getSelectedItem() != formatString(this->autoloadScenarioName)) {
+		//if(SystemFlags::VERBOSE_MODE_ENABLED) printf("[%s:%s] Line: %d this->autoloadScenarioName [%s] scenarioPath [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,this->autoloadScenarioName.c_str(),scenarioPath.c_str());
+		printf("[%s:%s] Line: %d this->autoloadScenarioName [%s] scenarioPath [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,this->autoloadScenarioName.c_str(),scenarioPath.c_str());
+
+		loadScenarioInfo(scenarioPath, &scenarioInfo );
+		//if(scenarioInfo.namei18n != "") {
+		//	this->autoloadScenarioName = scenarioInfo.namei18n;
+		//}
+		//else {
+		this->autoloadScenarioName = formatString(this->autoloadScenarioName);
+		//}
+
+		//if(SystemFlags::VERBOSE_MODE_ENABLED) printf("[%s:%s] Line: %d this->autoloadScenarioName [%s] scenarioPath [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,this->autoloadScenarioName.c_str(),scenarioPath.c_str());
+		printf("[%s:%s] Line: %d this->autoloadScenarioName [%s] scenarioPath [%s] file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,this->autoloadScenarioName.c_str(),scenarioPath.c_str(),scenarioInfo.file.c_str());
+
+		listBoxScenario.setSelectedItem(this->autoloadScenarioName,false);
+
+		if(listBoxScenario.getSelectedItem() != this->autoloadScenarioName) {
 			mainMessageBoxState=1;
-			showMessageBox( "Could not find scenario name: " + formatString(this->autoloadScenarioName), "Scenario Missing", false);
+			showMessageBox( "Could not find scenario name: " + this->autoloadScenarioName, "Scenario Missing", false);
 			this->autoloadScenarioName = "";
 		}
 		else {
-			if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 && listBoxScenario.getSelectedItemIndex() < scenarioFiles.size()) {
-				loadScenarioInfo(Scenario::getScenarioPath(dirList, scenarioFiles[listBoxScenario.getSelectedItemIndex()]), &scenarioInfo);
-				labelInfo.setText(scenarioInfo.desc);
+			try {
+				this->autoloadScenarioName = "";
+				if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 &&
+					listBoxScenario.getSelectedItemIndex() < (int)scenarioFiles.size()) {
 
-				SoundRenderer &soundRenderer= SoundRenderer::getInstance();
-				CoreData &coreData= CoreData::getInstance();
-				soundRenderer.playFx(coreData.getClickSoundC());
-				launchGame();
-				return;
+					printf("[%s:%s] Line: %d scenarioFiles[listBoxScenario.getSelectedItemIndex()] [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,scenarioFiles[listBoxScenario.getSelectedItemIndex()].c_str());
+
+					loadScenarioInfo(Scenario::getScenarioPath(dirList, scenarioFiles[listBoxScenario.getSelectedItemIndex()]), &scenarioInfo);
+
+					printf("[%s:%s] Line: %d scenarioInfo.file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,scenarioInfo.file.c_str());
+
+					labelInfo.setText(scenarioInfo.desc);
+	        		if(scenarioInfo.namei18n != "") {
+	        			labelScenarioName.setText(scenarioInfo.namei18n);
+	        		}
+	        		else {
+	        			labelScenarioName.setText(listBoxScenario.getSelectedItem());
+	        		}
+
+					SoundRenderer &soundRenderer= SoundRenderer::getInstance();
+					CoreData &coreData= CoreData::getInstance();
+					soundRenderer.playFx(coreData.getClickSoundC());
+					launchGame();
+					return;
+				}
+			}
+			catch(const std::exception &ex) {
+				char szBuf[8096]="";
+				snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
+				SystemFlags::OutputDebug(SystemFlags::debugError,szBuf);
+				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s",szBuf);
+
+				mainMessageBoxState=1;
+				showMessageBox( "Error: " + string(ex.what()), "Error detected", false);
 			}
 		}
 	}
@@ -295,7 +389,22 @@ void MenuStateScenario::launchGame() {
 	if(scenarioInfo.file != "" && scenarioInfo.tilesetName != "" && scenarioInfo.mapName != "" && scenarioInfo.techTreeName != "") {
 		GameSettings gameSettings;
 		loadGameSettings(&scenarioInfo, &gameSettings);
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] gameSettings.getScenarioDir() [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,gameSettings.getScenarioDir().c_str());
+
+		const vector<string> pathTechList = Config::getInstance().getPathListForType(ptTechs,gameSettings.getScenarioDir());
+		if(TechTree::exists(gameSettings.getTech(), pathTechList) == false) {
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"Line ref: %d Error: cannot find techtree [%s]\n",__LINE__,scenarioInfo.techTreeName.c_str());
+			SystemFlags::OutputDebug(SystemFlags::debugError,szBuf);
+
+	        mainMessageBoxState=1;
+	        showMessageBox( szBuf, "Error detected", false);
+
+			return;
+		}
 		program->setState(new Game(program, &gameSettings, false));
+		return;
 	}
 }
 
@@ -305,7 +414,11 @@ void MenuStateScenario::setScenario(int i) {
 }
 
 void MenuStateScenario::loadScenarioInfo(string file, ScenarioInfo *scenarioInfo) {
-	Scenario::loadScenarioInfo(file, scenarioInfo);
+	bool isTutorial = Scenario::isGameTutorial(file);
+
+	//printf("[%s:%s] Line: %d file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,file.c_str());
+
+	Scenario::loadScenarioInfo(file, scenarioInfo, isTutorial);
 
 	cleanupPreviewTexture();
 	previewLoadDelayTimer=time(NULL);
@@ -315,7 +428,7 @@ void MenuStateScenario::loadScenarioInfo(string file, ScenarioInfo *scenarioInfo
 void MenuStateScenario::loadScenarioPreviewTexture(){
 	if(enableScenarioTexturePreview == true) {
 		//if(listBoxScenario.getSelectedItemIndex() >= 0) {
-		if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 && listBoxScenario.getSelectedItemIndex() < scenarioFiles.size()) {
+		if(listBoxScenario.getItemCount() > 0 && listBoxScenario.getSelectedItemIndex() >= 0 && listBoxScenario.getSelectedItemIndex() < (int)scenarioFiles.size()) {
 			GameSettings gameSettings;
 			loadGameSettings(&scenarioInfo, &gameSettings);
 
@@ -324,7 +437,7 @@ void MenuStateScenario::loadScenarioPreviewTexture(){
 
 			Game::extractScenarioLogoFile(&gameSettings, scenarioLogo, loadingImageUsed);
 
-			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] scenarioLogo [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioLogo.c_str());
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] scenarioLogo [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,scenarioLogo.c_str());
 
 			if(scenarioLogo != "") {
 				cleanupPreviewTexture();
@@ -344,7 +457,7 @@ void MenuStateScenario::loadGameSettings(const ScenarioInfo *scenarioInfo, GameS
 		snprintf(szBuf,8096,"listBoxScenario.getSelectedItemIndex() < 0, = %d",listBoxScenario.getSelectedItemIndex());
 		throw megaglest_runtime_error(szBuf);
 	}
-	else if(listBoxScenario.getSelectedItemIndex() >= scenarioFiles.size()) {
+	else if(listBoxScenario.getSelectedItemIndex() >= (int)scenarioFiles.size()) {
 		char szBuf[8096]="";
 		snprintf(szBuf,8096,"listBoxScenario.getSelectedItemIndex() >= scenarioFiles.size(), = [%d][%d]",listBoxScenario.getSelectedItemIndex(),(int)scenarioFiles.size());
 		throw megaglest_runtime_error(szBuf);
