@@ -28,12 +28,31 @@ using std::vector;
 
 namespace Glest{ namespace Game{
 
+class ClientInterface;
+
+class ClientInterfaceThread : public BaseThread, public SlaveThreadControllerInterface {
+protected:
+
+	ClientInterface *clientInterface;
+
+	virtual void setQuitStatus(bool value);
+
+public:
+	ClientInterfaceThread(ClientInterface *client);
+	virtual ~ClientInterfaceThread();
+    virtual void execute();
+
+    virtual void setMasterController(MasterSlaveThreadController *master) { }
+	virtual void signalSlave(void *userdata) { }
+
+	virtual bool canShutdown(bool deleteSelfIfShutdownDelayed=false);
+};
+
 // =====================================================
 //	class ClientInterface
 // =====================================================
 
-class ClientInterface: public GameNetworkInterface,
-						public SimpleTaskCallbackInterface {
+class ClientInterface: public GameNetworkInterface {
 private:
 	static const int messageWaitTimeout;
 	static const int waitSleepTime;
@@ -54,6 +73,7 @@ private:
 	int port;
 
 	int currentFrameCount;
+	int lastSentFrameCount;
 	time_t lastNetworkCommandListSendTime;
 
 	time_t clientSimulationLagStartTime;
@@ -61,19 +81,50 @@ private:
 	int sessionKey;
 	int serverFTPPort;
 
-	SimpleTaskThread *networkCommandListThread;
+	string serverUUID;
+	string serverPlatform;
+
+	ClientInterfaceThread *networkCommandListThread;
+
 	Mutex *networkCommandListThreadAccessor;
 	std::map<int,Commands> cachedPendingCommands;	//commands ready to be given
+	std::map<int,vector<uint32> > cachedPendingCommandCRCs;	//commands ready to be given
 	uint64 cachedPendingCommandsIndex;
+	uint64 cachedLastPendingFrameCount;
+	int64 timeClientWaitedForLastMessage;
+
+	Mutex *flagAccessor;
+	bool joinGameInProgress;
+	bool joinGameInProgressLaunch;
+	bool readyForInGameJoin;
+	bool resumeInGameJoin;
+
+	Mutex *quitThreadAccessor;
+	bool quitThread;
+
+	bool getQuitThread();
+	void setQuitThread(bool value);
+	bool getQuit();
+	void setQuit(bool value);
 
 public:
 	ClientInterface();
 	virtual ~ClientInterface();
 
 	virtual Socket* getSocket(bool mutexLock=true)					{return clientSocket;}
-	//virtual const Socket* getSocket() const		{return clientSocket;}
 	virtual void close();
 
+	bool getJoinGameInProgress();
+	bool getJoinGameInProgressLaunch();
+
+	bool getReadyForInGameJoin();
+
+	bool getResumeInGameJoin();
+	void sendResumeGameMessage();
+
+	uint64 getCachedLastPendingFrameCount();
+	int64 getTimeClientWaitedForLastMessage();
+
 	//message processing
 	virtual void update();
 	virtual void updateLobby();
@@ -102,7 +153,6 @@ public:
 	int getGameSettingsReceivedCount() const { return gameSettingsReceivedCount; }
 
 	int getPlayerIndex() const				{return playerIndex;}
-	//const GameSettings *getGameSettings()	{return &gameSettings;}
 
 	void connect(const Ip &ip, int port);
 	void reset();
@@ -110,8 +160,8 @@ public:
 	void discoverServers(DiscoveredServersInterface *cb);
 	void stopServerDiscovery();
 	
-	void sendSwitchSetupRequest(string selectedFactionName, int8 currentFactionIndex,
-								int8 toFactionIndex, int8 toTeam,string networkPlayerName,
+	void sendSwitchSetupRequest(string selectedFactionName, int8 currentSlotIndex,
+								int8 toSlotIndex, int8 toTeam,string networkPlayerName,
 								int8 networkPlayerStatus, int8 flags,
 								string language);
 	virtual bool getConnectHasHandshaked() const { return gotIntro; }
@@ -133,19 +183,21 @@ public:
     void broadcastGameSetup(const GameSettings *gameSettings);
     void broadcastGameStart(const GameSettings *gameSettings);
 
-    virtual void simpleTask(BaseThread *callingThread);
+    void updateNetworkFrame();
 
     virtual void saveGame(XmlNode *rootNode) {};
 
 protected:
 
 	Mutex * getServerSynchAccessor() { return NULL; }
-	NetworkMessageType waitForMessage();
+	NetworkMessageType waitForMessage(int waitMicroseconds=0);
 	bool shouldDiscardNetworkMessage(NetworkMessageType networkMessageType);
 
 	void updateFrame(int *checkFrame);
-	void shutdownNetworkCommandListThread();
+	void shutdownNetworkCommandListThread(MutexSafeWrapper &safeMutexWrapper);
 	bool getNetworkCommand(int frameCount, int currentCachedPendingCommandsIndex);
+
+	void close(bool lockMutex);
 };
 
 }}//end namespace
