@@ -12,11 +12,12 @@
 #ifndef _GLEST_GAME_CONNECTIONSLOT_H_
 #define _GLEST_GAME_CONNECTIONSLOT_H_
 
-#include <vector>
 #include "socket.h"
 #include "network_interface.h"
-#include <time.h>
 #include "base_thread.h"
+#include <time.h>
+#include <vector>
+
 #include "leak_dumper.h"
 
 using Shared::Platform::ServerSocket;
@@ -66,11 +67,16 @@ public:
 //
 class ConnectionSlotCallbackInterface {
 public:
+	virtual bool isClientConnected(int index) = 0;
+	virtual bool getAllowInGameConnections() const = 0;
+	virtual ConnectionSlot *getSlot(int index, bool lockMutex) = 0;
+	virtual Mutex *getSlotMutex(int index) = 0;
+
 	virtual void slotUpdateTask(ConnectionSlotEvent *event) = 0;
 	virtual ~ConnectionSlotCallbackInterface() {}
 };
 
-class ConnectionSlotThread : public BaseThread
+class ConnectionSlotThread : public BaseThread, public SlaveThreadControllerInterface
 {
 protected:
 
@@ -79,10 +85,13 @@ protected:
 	Mutex *triggerIdMutex;
 	vector<ConnectionSlotEvent> eventList;
 	int slotIndex;
+	MasterSlaveThreadController *masterController;
+
+	Mutex *triggerGameStarted;
+	bool gameStarted;
 
 	virtual void setQuitStatus(bool value);
 	virtual void setTaskCompleted(int eventId);
-	virtual bool canShutdown(bool deleteSelfIfShutdownDelayed=false);
 
 	void slotUpdateTask(ConnectionSlotEvent *event);
 
@@ -91,14 +100,24 @@ public:
 	ConnectionSlotThread(ConnectionSlotCallbackInterface *slotInterface,int slotIndex);
 	virtual ~ConnectionSlotThread();
 
+	bool getGameStarted();
+	void setGameStarted(bool value);
+
+	virtual void setMasterController(MasterSlaveThreadController *master) { masterController = master; }
+	virtual void signalSlave(void *userdata);
+
     virtual void execute();
     void signalUpdate(ConnectionSlotEvent *event);
     bool isSignalCompleted(ConnectionSlotEvent *event);
+
     int getSlotIndex() const {return slotIndex; }
+    void setSlotIndex(int index) { this->slotIndex = index; }
 
     void purgeCompletedEvents();
     void purgeAllEvents();
     void setAllEventsCompleted();
+
+    virtual bool canShutdown(bool deleteSelfIfShutdownDelayed=false);
 };
 
 // =====================================================
@@ -133,21 +152,56 @@ private:
 	uint32 connectedRemoteIPAddress;
 	int playerStatus;
 	string playerLanguage;
+	string playerUUID;
+	string platform;
 
+	bool skipLagCheck;
+	bool joinGameInProgress;
 	bool canAcceptConnections;
+	bool startInGameConnectionLaunch;
+	bool pauseForInGameConnection;
+	bool unPauseForInGameConnection;
+	bool sentSavedGameInfo;
+
+	int autoPauseGameCountForLag;
 
 public:
 	ConnectionSlot(ServerInterface* serverInterface, int playerIndex);
 	~ConnectionSlot();
 
+	int getAutoPauseGameCountForLag();
+	void incrementAutoPauseGameCountForLag();
+
+	bool getGameStarted();
+	void setGameStarted(bool value);
+
+	bool getStartInGameConnectionLaunch() const { return startInGameConnectionLaunch; }
+	void setStartInGameConnectionLaunch(bool value) { startInGameConnectionLaunch = value; }
+
+	bool getPauseForInGameConnection() const { return pauseForInGameConnection; }
+	void setPauseForInGameConnection(bool value) { pauseForInGameConnection = value; }
+
+	bool getUnPauseForInGameConnection() const { return unPauseForInGameConnection; }
+	void setUnPauseForInGameConnection(bool value) { unPauseForInGameConnection = value; }
+
+	bool getSkipLagCheck() const { return skipLagCheck; }
+	bool getJoinGameInProgress() const { return joinGameInProgress; }
+
+	bool getSentSavedGameInfo() const { return sentSavedGameInfo; }
+	void setSentSavedGameInfo(bool value) { sentSavedGameInfo = value; }
+
+	ConnectionSlotThread *getWorkerThread() { return slotThreadWorker; }
+
     void update(bool checkForNewClients,int lockedSlotIndex);
-	void setPlayerIndex(int value) { playerIndex = value; }
+	void setPlayerIndex(int value);
 	int getPlayerIndex() const {return playerIndex;}
 
 	uint32 getConnectedRemoteIPAddress() const { return connectedRemoteIPAddress; }
 
-	void setReady()					{ready= true;}
+	void setReady();
 	const string &getName() const	{return name;}
+	const string &getUUID() const	{return playerUUID;}
+	const string &getPlatform() const { return platform; }
 	void setName(string value)      {name = value;}
 	bool isReady() const			{return ready;}
 
@@ -205,6 +259,9 @@ public:
 
 	virtual void saveGame(XmlNode *rootNode) {};
 
+	void resetJoinGameInProgressFlags();
+	void setJoinGameInProgressFlags();
+
 protected:
 
 	Mutex * getServerSynchAccessor();
