@@ -40,22 +40,94 @@ DisplayMessageFunction NetworkInterface::pCB_DisplayMessage = NULL;
 
 Vec3f MarkedCell::static_system_marker_color(MAGENTA.x,MAGENTA.y,MAGENTA.z);
 
+NetworkInterface::NetworkInterface() {
+	networkAccessMutex = new Mutex(CODE_AT_LINE);
+
+	networkGameDataSynchCheckOkMap=false;
+	networkGameDataSynchCheckOkTile=false;
+	networkGameDataSynchCheckOkTech=false;
+	receivedDataSynchCheck=false;
+
+	networkPlayerFactionCRCMutex = new Mutex(CODE_AT_LINE);
+	for(unsigned int index = 0; index < (unsigned int)GameConstants::maxPlayers; ++index) {
+		networkPlayerFactionCRC[index] = 0;
+	}
+}
+
+void NetworkInterface::init() {
+	networkAccessMutex = NULL;
+
+	networkGameDataSynchCheckOkMap=false;
+	networkGameDataSynchCheckOkTile=false;
+	networkGameDataSynchCheckOkTech=false;
+	receivedDataSynchCheck=false;
+
+	gameSettings = GameSettings();
+
+	networkPlayerFactionCRCMutex = NULL;
+	for(unsigned int index = 0; index < (unsigned int)GameConstants::maxPlayers; ++index) {
+		networkPlayerFactionCRC[index] = 0;
+	}
+}
+
+NetworkInterface::~NetworkInterface() {
+	delete networkAccessMutex;
+	networkAccessMutex = NULL;
+
+	delete networkPlayerFactionCRCMutex;
+	networkPlayerFactionCRCMutex = NULL;
+}
+
+uint32 NetworkInterface::getNetworkPlayerFactionCRC(int index) {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkPlayerFactionCRCMutex,mutexOwnerId);
+
+	return networkPlayerFactionCRC[index];
+}
+void NetworkInterface::setNetworkPlayerFactionCRC(int index, uint32 crc) {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkPlayerFactionCRCMutex,mutexOwnerId);
+
+	networkPlayerFactionCRC[index]=crc;
+}
+
+void NetworkInterface::addChatInfo(const ChatMsgInfo &msg) {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	chatTextList.push_back(msg);
+}
+
+void NetworkInterface::addMarkedCell(const MarkedCell &msg) {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	markedCellList.push_back(msg);
+}
+void NetworkInterface::addUnMarkedCell(const UnMarkedCell &msg) {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	unmarkedCellList.push_back(msg);
+}
+
 void NetworkInterface::sendMessage(NetworkMessage* networkMessage){
 	Socket* socket= getSocket(false);
 
 	networkMessage->send(socket);
 }
 
-NetworkMessageType NetworkInterface::getNextMessageType()
+NetworkMessageType NetworkInterface::getNextMessageType(int waitMilliseconds)
 {
 	Socket* socket= getSocket(false);
 	int8 messageType= nmtInvalid;
 
     if(socket != NULL &&
-        socket->hasDataToRead() == true) {
+        ((waitMilliseconds <= 0 && socket->hasDataToRead() == true) ||
+         (waitMilliseconds > 0 && socket->hasDataToReadWithWait(waitMilliseconds) == true))) {
         //peek message type
 		int dataSize = socket->getDataToRead();
-		if(dataSize >= sizeof(messageType)) {
+		if(dataSize >= (int)sizeof(messageType)) {
 				if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] socket->getDataToRead() dataSize = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,dataSize);
 
 			int iPeek = socket->peek(&messageType, sizeof(messageType));
@@ -94,6 +166,33 @@ bool NetworkInterface::isConnected(){
 	return result;
 }
 
+void NetworkInterface::setLastPingInfo(const NetworkMessagePing &ping) {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	this->lastPingInfo = ping;
+}
+
+void NetworkInterface::setLastPingInfoToNow() {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	this->lastPingInfo.setPingReceivedLocalTime(time(NULL));
+}
+
+NetworkMessagePing NetworkInterface::getLastPingInfo() {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	return lastPingInfo;
+}
+double NetworkInterface::getLastPingLag() {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	return difftime((long int)time(NULL),lastPingInfo.getPingReceivedLocalTime());
+}
+
 void NetworkInterface::DisplayErrorMessage(string sErr, bool closeSocket) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] sErr [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,sErr.c_str());
 	SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] sErr [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,sErr.c_str());
@@ -112,6 +211,10 @@ void NetworkInterface::DisplayErrorMessage(string sErr, bool closeSocket) {
 
 std::vector<ChatMsgInfo> NetworkInterface::getChatTextList(bool clearList) {
 	std::vector<ChatMsgInfo> result;
+
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(chatTextList.empty() == false) {
 		result = chatTextList;
 
@@ -123,6 +226,9 @@ std::vector<ChatMsgInfo> NetworkInterface::getChatTextList(bool clearList) {
 }
 
 void NetworkInterface::clearChatInfo() {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(chatTextList.empty() == false) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] chatTextList.size() = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chatTextList.size());
 		chatTextList.clear();
@@ -131,6 +237,10 @@ void NetworkInterface::clearChatInfo() {
 
 std::vector<MarkedCell> NetworkInterface::getMarkedCellList(bool clearList) {
 	std::vector<MarkedCell> result;
+
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(markedCellList.empty() == false) {
 		result = markedCellList;
 
@@ -142,6 +252,9 @@ std::vector<MarkedCell> NetworkInterface::getMarkedCellList(bool clearList) {
 }
 
 void NetworkInterface::clearMarkedCellList() {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(markedCellList.empty() == false) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] markedCellList.size() = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,markedCellList.size());
 		markedCellList.clear();
@@ -150,6 +263,10 @@ void NetworkInterface::clearMarkedCellList() {
 
 std::vector<UnMarkedCell> NetworkInterface::getUnMarkedCellList(bool clearList) {
 	std::vector<UnMarkedCell> result;
+
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(unmarkedCellList.empty() == false) {
 		result = unmarkedCellList;
 
@@ -161,6 +278,9 @@ std::vector<UnMarkedCell> NetworkInterface::getUnMarkedCellList(bool clearList)
 }
 
 void NetworkInterface::clearUnMarkedCellList() {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(unmarkedCellList.empty() == false) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] unmarkedCellList.size() = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,unmarkedCellList.size());
 		unmarkedCellList.clear();
@@ -169,6 +289,10 @@ void NetworkInterface::clearUnMarkedCellList() {
 
 std::vector<MarkedCell> NetworkInterface::getHighlightedCellList(bool clearList) {
 	std::vector<MarkedCell> result;
+
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(highlightedCellList.empty() == false) {
 		result = highlightedCellList;
 
@@ -180,6 +304,9 @@ std::vector<MarkedCell> NetworkInterface::getHighlightedCellList(bool clearList)
 }
 
 void NetworkInterface::clearHighlightedCellList() {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
 	if(highlightedCellList.empty() == false) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] markedCellList.size() = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,markedCellList.size());
 		highlightedCellList.clear();
@@ -187,7 +314,10 @@ void NetworkInterface::clearHighlightedCellList() {
 }
 
 void NetworkInterface::setHighlightedCell(const MarkedCell &msg){
-	for(int idx = 0; idx < highlightedCellList.size(); idx++) {
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+	MutexSafeWrapper safeMutex(networkAccessMutex,mutexOwnerId);
+
+	for(int idx = 0; idx < (int)highlightedCellList.size(); idx++) {
 		MarkedCell mc = highlightedCellList[idx];
 		if(mc.getFactionIndex()==msg.getFactionIndex()){
 			highlightedCellList.erase(highlightedCellList.begin()+ idx);
@@ -226,17 +356,15 @@ GameNetworkInterface::GameNetworkInterface(){
 
 void GameNetworkInterface::requestCommand(const NetworkCommand *networkCommand, bool insertAtStart) {
 	assert(networkCommand != NULL);
-	//Mutex *mutex = getServerSynchAccessor();
+	Mutex *mutex = getServerSynchAccessor();
 
     if(insertAtStart == false) {
-    	//if(mutex != NULL) mutex->p();
+    	MutexSafeWrapper safeMutex(mutex,string(__FILE__) + "_" + intToStr(__LINE__));
         requestedCommands.push_back(*networkCommand);
-        //if(mutex != NULL) mutex->v();
     }
     else {
-    	//if(mutex != NULL) mutex->p();
+    	MutexSafeWrapper safeMutex(mutex,string(__FILE__) + "_" + intToStr(__LINE__));
         requestedCommands.insert(requestedCommands.begin(),*networkCommand);
-        //if(mutex != NULL) mutex->v();
     }
 }
 
@@ -256,7 +384,6 @@ void FileTransferSocketThread::execute()
     if(info.hostType == eServer)
     {
         ServerSocket serverSocket;
-        //serverSocket.setBlock(false);
         serverSocket.bind(this->info.serverPort);
         serverSocket.listen(1);
         Socket *clientSocket = serverSocket.accept();
@@ -333,7 +460,6 @@ void FileTransferSocketThread::execute()
     {
         Ip ip(this->info.serverIP);
         ClientSocket clientSocket;
-        //clientSocket.setBlock(false);
         clientSocket.connect(this->info.serverIP, this->info.serverPort);
 
         if(clientSocket.isConnected() == true)
