@@ -36,12 +36,21 @@ using namespace Shared::Platform;
 
 namespace Glest{ namespace Game{
 
-//class GameSettings;
-
 // =====================================================
 //	class NetworkInterface
 // =====================================================
 
+//
+// This interface describes the methods to notify when a client is lagging
+//
+class ClientLagCallbackInterface {
+public:
+	virtual bool clientLagHandler(int slotIndex, bool networkPauseGameForLaggedClients) = 0;
+
+	virtual ~ClientLagCallbackInterface() {}
+};
+
+
 class ChatMsgInfo {
 
 protected:
@@ -173,8 +182,9 @@ protected:
 	string networkGameDataSynchCheckTechMismatchReport;
 	bool receivedDataSynchCheck;
 
-	std::vector<ChatMsgInfo> chatTextList;
 	NetworkMessagePing lastPingInfo;
+
+	std::vector<ChatMsgInfo> chatTextList;
 	std::vector<MarkedCell> markedCellList;
 	std::vector<UnMarkedCell> unmarkedCellList;
 
@@ -185,12 +195,32 @@ protected:
 
 	std::vector<MarkedCell> highlightedCellList;
 
+	Mutex *networkAccessMutex;
+
+	void init();
+
+	Mutex *networkPlayerFactionCRCMutex;
+	uint32 networkPlayerFactionCRC[GameConstants::maxPlayers];
+
 public:
 	static const int readyWaitTimeout;
 	GameSettings gameSettings;
 
 public:
-	virtual ~NetworkInterface(){}
+	NetworkInterface();
+	virtual ~NetworkInterface();
+
+	NetworkInterface(const NetworkInterface& obj) {
+		init();
+		throw megaglest_runtime_error("class NetworkInterface is NOT safe to copy!");
+	}
+	NetworkInterface & operator=(const NetworkInterface& obj) {
+		init();
+		throw megaglest_runtime_error("class NetworkInterface is NOT safe to assign!");
+	}
+
+	uint32 getNetworkPlayerFactionCRC(int index);
+	void setNetworkPlayerFactionCRC(int index, uint32 crc);
 
 	virtual Socket* getSocket(bool mutexLock=true)= 0;
 	virtual void close()= 0;
@@ -204,7 +234,7 @@ public:
 	string getHostName() const	{return Socket::getHostName();}
 
 	virtual void sendMessage(NetworkMessage* networkMessage);
-	NetworkMessageType getNextMessageType();
+	NetworkMessageType getNextMessageType(int waitMilliseconds=0);
 	bool receiveMessage(NetworkMessage* networkMessage);
 
 	virtual bool isConnected();
@@ -228,15 +258,15 @@ public:
 
     std::vector<ChatMsgInfo> getChatTextList(bool clearList);
 	void clearChatInfo();
-	void addChatInfo(const ChatMsgInfo &msg) { chatTextList.push_back(msg); }
+	void addChatInfo(const ChatMsgInfo &msg);
 
     std::vector<MarkedCell> getMarkedCellList(bool clearList);
 	void clearMarkedCellList();
-	void addMarkedCell(const MarkedCell &msg) { markedCellList.push_back(msg); }
+	void addMarkedCell(const MarkedCell &msg);
 
     std::vector<UnMarkedCell> getUnMarkedCellList(bool clearList);
 	void clearUnMarkedCellList();
-	void addUnMarkedCell(const UnMarkedCell &msg) { unmarkedCellList.push_back(msg); }
+	void addUnMarkedCell(const UnMarkedCell &msg);
 
     std::vector<MarkedCell> getHighlightedCellList(bool clearList);
 	void clearHighlightedCellList();
@@ -244,10 +274,10 @@ public:
 
 	virtual bool getConnectHasHandshaked() const= 0;
 
-	NetworkMessagePing getLastPingInfo() const { return lastPingInfo; }
-	double getLastPingLag() const {
-		return difftime((long int)time(NULL),lastPingInfo.getPingReceivedLocalTime());
-	}
+	void setLastPingInfo(const NetworkMessagePing &ping);
+	void setLastPingInfoToNow();
+	NetworkMessagePing getLastPingInfo();
+	double getLastPingLag();
 
 	std::string getIpAddress();
 	float getThreadedPingMS(std::string host);
@@ -281,6 +311,7 @@ protected:
 
 public:
 	GameNetworkInterface();
+	virtual ~GameNetworkInterface(){}
 
 	//message processimg
 	virtual void update()= 0;
@@ -304,7 +335,7 @@ public:
 
 	//access functions
 	void requestCommand(const NetworkCommand *networkCommand, bool insertAtStart=false);
-	int getPendingCommandCount() const							{return pendingCommands.size();}
+	int getPendingCommandCount() const							{return (int)pendingCommands.size();}
 	NetworkCommand* getPendingCommand(int i) 		            {return &pendingCommands[i];}
 	void clearPendingCommands()									{pendingCommands.clear();}
 	bool getQuit() const										{return quit;}
