@@ -50,6 +50,26 @@ class ScriptManager;
 class World;
 class Faction;
 class GameSettings;
+class SurfaceCell;
+
+class FowAlphaCellsLookupItem {
+public:
+
+	std::map<Vec2i,float> surfPosAlphaList;
+};
+
+class ExploredCellsLookupItem {
+public:
+
+	ExploredCellsLookupItem() {
+		ExploredCellsLookupItemCacheTimerCountIndex = 0;
+	}
+	int ExploredCellsLookupItemCacheTimerCountIndex;
+	std::vector<SurfaceCell *> exploredCellList;
+	std::vector<SurfaceCell *> visibleCellList;
+
+	static time_t lastDebug;
+};
 
 // =====================================================
 // 	class Faction
@@ -67,13 +87,14 @@ struct CommandGroupUnitSorterId {
 	bool operator()(const int l, const int r);
 };
 
-class FactionThread : public BaseThread {
+class FactionThread : public BaseThread, public SlaveThreadControllerInterface {
 protected:
 
 	Faction *faction;
 	Semaphore semTaskSignalled;
 	Mutex *triggerIdMutex;
 	std::pair<int,bool> frameIndex;
+	MasterSlaveThreadController *masterController;
 
 	virtual void setQuitStatus(bool value);
 	virtual void setTaskCompleted(int frameIndex);
@@ -83,6 +104,10 @@ public:
 	FactionThread(Faction *faction);
 	virtual ~FactionThread();
     virtual void execute();
+
+	virtual void setMasterController(MasterSlaveThreadController *master) { masterController = master; }
+	virtual void signalSlave(void *userdata) { signalPathfinder(*((int *)(userdata))); }
+
     void signalPathfinder(int frameIndex);
     bool isSignalPathfinderCompleted(int frameIndex);
 };
@@ -151,7 +176,13 @@ private:
 	TechTree *techTree;
 	const XmlNode *loadWorldNode;
 
-	std::map<int,std::map<Field, std::map<Vec2i,bool> > > mapSharedPathFinderCache;
+	std::vector<string> worldSynchThreadedLogList;
+
+	std::map<int,string> crcWorldFrameDetails;
+
+	std::map<int,const Unit *> aliveUnitListCache;
+	std::map<int,const Unit *> mobileUnitListCache;
+	std::map<int,const Unit *> beingBuiltUnitListCache;
 
 public:
 	Faction();
@@ -166,25 +197,30 @@ public:
 		throw megaglest_runtime_error("class Faction is NOT safe to assign!");
 	}
 
-	inline const bool * aproxCanMoveSoonCached(int size, Field field, const Vec2i &pos1, const Vec2i &pos2) const {
-		const bool *result = NULL;
-//		std::map<int,std::map<Field, std::map<Vec2i,bool> > >::const_iterator iterFind1 = mapSharedPathFinderCache.find(size);
-//		if(iterFind1 != mapSharedPathFinderCache.end()) {
-//			std::map<Field, std::map<Vec2i,bool> >::const_iterator iterFind2 = iterFind1->second.find(field);
-//			if(iterFind2 != iterFind1->second.end()) {
-//				std::map<Vec2i,bool>::const_iterator iterFind3 = iterFind2->second.find(pos2);
-//				if(iterFind3 != iterFind2->second.end()) {
-//					result = &iterFind3->second;
-//				}
-//			}
-//		}
-		return result;
+	void notifyUnitAliveStatusChange(const Unit *unit);
+	void notifyUnitTypeChange(const Unit *unit, const UnitType *newType);
+	void notifyUnitSkillTypeChange(const Unit *unit, const SkillType *newType);
+	bool hasAliveUnits(bool filterMobileUnits, bool filterBuiltUnits) const;
+
+	inline void addWorldSynchThreadedLogList(const string &data) {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugWorldSynch).enabled == true) {
+			worldSynchThreadedLogList.push_back(data);
+		}
 	}
-	inline void addAproxCanMoveSoonCached(int size, Field field, const Vec2i &pos1, const Vec2i &pos2,bool result) {
-		//mapSharedPathFinderCache[size][field][pos2]=result;
+	inline void clearWorldSynchThreadedLogList() {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugWorldSynch).enabled == true) {
+			worldSynchThreadedLogList.clear();
+		}
 	}
-	inline void clearAproxCanMoveSoonCached() {
-		//mapSharedPathFinderCache.clear();
+	inline void dumpWorldSynchThreadedLogList() {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugWorldSynch).enabled == true) {
+			if(worldSynchThreadedLogList.empty() == false) {
+				for(unsigned int index = 0; index < worldSynchThreadedLogList.size(); ++index) {
+					SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,worldSynchThreadedLogList[index].c_str());
+				}
+				worldSynchThreadedLogList.clear();
+			}
+		}
 	}
 
 	inline void addLivingUnits(int id) { livingUnits.insert(id); }
@@ -247,7 +283,7 @@ public:
 		return result;
 	}
 	inline int getUnitCount() const {
-		int result = units.size();
+		int result = (int)units.size();
 		return result;
 	}
 	inline Mutex * getUnitMutex() {return unitsMutex;}
@@ -285,7 +321,7 @@ public:
 	Unit *findUnit(int id) const;
 	void addUnit(Unit *unit);
 	void removeUnit(Unit *unit);
-	void addStore(const UnitType *unitType);
+	void addStore(const UnitType *unitType, bool replaceStorage);
 	void removeStore(const UnitType *unitType);
 
 	//resources
@@ -298,10 +334,10 @@ public:
 	void addResourceTargetToCache(const Vec2i &pos,bool incrementUseCounter=true);
 	void removeResourceTargetFromCache(const Vec2i &pos);
 	void addCloseResourceTargetToCache(const Vec2i &pos);
-	Vec2i getClosestResourceTypeTargetFromCache(Unit *unit, const ResourceType *type);
+	Vec2i getClosestResourceTypeTargetFromCache(Unit *unit, const ResourceType *type,int frameIndex);
 	Vec2i getClosestResourceTypeTargetFromCache(const Vec2i &pos, const ResourceType *type);
-	void cleanupResourceTypeTargetCache(std::vector<Vec2i> *deleteListPtr);
-	inline int getCacheResourceTargetListSize() const { return cacheResourceTargetList.size(); }
+	void cleanupResourceTypeTargetCache(std::vector<Vec2i> *deleteListPtr,int frameIndex);
+	inline int getCacheResourceTargetListSize() const { return (int)cacheResourceTargetList.size(); }
 
 	Unit * findClosestUnitWithSkillClass(const Vec2i &pos,const CommandClass &cmdClass,
 										const std::vector<SkillClass> &skillClassList,
@@ -311,8 +347,11 @@ public:
 
 	inline World * getWorld() { return world; }
 	int getFrameCount();
+
 	void signalWorkerThread(int frameIndex);
 	bool isWorkerThreadSignalCompleted(int frameIndex);
+	FactionThread *getWorkerThread() { return workerThread; }
+
 	void limitResourcesToStore();
 
 	void sortUnitsByCommandGroups();
@@ -320,13 +359,22 @@ public:
 	bool canCreateUnit(const UnitType *ut, bool checkBuild, bool checkProduce, bool checkMorph) const;
 
 	string getCacheStats();
-	uint64 getCacheKBytes(uint64 *cache1Size, uint64 *cache2Size);
+	uint64 getCacheKBytes(uint64 *cache1Size, uint64 *cache2Size, uint64 *cache3Size, uint64 *cache4Size, uint64 *cache5Size);
 
-	std::string toString() const;
+	std::string toString(bool crcMode=false) const;
 
 	void saveGame(XmlNode *rootNode);
 	void loadGame(const XmlNode *rootNode, int factionIndex,GameSettings *settings,World *world);
 
+	void clearCaches();
+
+	Checksum getCRC();
+	void addCRC_DetailsForWorldFrame(int worldFrameCount,bool isNetworkServer);
+	string getCRC_DetailsForWorldFrame(int worldFrameCount);
+	std::pair<int,string> getCRC_DetailsForWorldFrameIndex(int worldFrameIndex) const;
+	string getCRC_DetailsForWorldFrames() const;
+	uint64 getCRC_DetailsForWorldFrameCount() const;
+
 private:
 	void init();
 	void resetResourceAmount(const ResourceType *rt);
