@@ -37,20 +37,28 @@ ObjectStateInterface *Object::stateCallback=NULL;
 Object::Object(ObjectType *objectType, const Vec3f &pos, const Vec2i &mapPos) : BaseColorPickEntity() {
 	RandomGen random;
 
-	random.init(static_cast<int>(pos.x*pos.z));
+	random.init(static_cast<int>(pos.x * pos.z));
 	this->lastRenderFrame = 0;
 	this->objectType= objectType;
 	resource= NULL;
 	highlight= 0.f;
+	animated= false;
 	this->mapPos = mapPos;
 	this->pos= pos + Vec3f(random.randRange(-0.6f, 0.6f), 0.0f, random.randRange(-0.6f, 0.6f));
 	rotation= random.randRange(0.f, 360.f);
 	if(objectType!=NULL){
 		variation = random.randRange(0, objectType->getModelCount()-1);
 		TilesetModelType *tmt=objectType->getTilesetModelType(variation);
-		if(tmt->getRotationAllowed()!=true){
+		if(tmt->getRotationAllowed() != true) {
 			rotation=0;
 		}
+		if(tmt->getRandomPositionEnabled() != true) {
+			this->pos = pos;
+		}
+		animated=tmt->getAnimSpeed()>0;
+	}
+	else {
+		variation=0;
 	}
 	visible=false;
 	animProgress=0.0f;
@@ -66,12 +74,14 @@ Object::~Object() {
 		}
 		unitParticleSystems.pop_back();
 	}
+	Renderer::getInstance().removeParticleSystemsForParticleOwner(this,rsGame);
 	renderer.removeObjectFromQuadCache(this);
 	if(stateCallback) {
 		stateCallback->removingObjectEvent(this);
 	}
 	delete resource;
 	resource = NULL;
+
 }
 
 void Object::end() {
@@ -102,6 +112,7 @@ void Object::initParticlesFromTypes(const ModelParticleSystemTypes *particleType
 			particleTypes->empty() == false && unitParticleSystems.empty() == true) {
 		for(ObjectParticleSystemTypes::const_iterator it= particleTypes->begin(); it != particleTypes->end(); ++it){
 			UnitParticleSystem *ups= new UnitParticleSystem(200);
+			ups->setParticleOwner(this);
 			(*it)->setValues(ups);
 			ups->setPos(this->pos);
 			ups->setRotation(this->rotation);
@@ -113,8 +124,15 @@ void Object::initParticlesFromTypes(const ModelParticleSystemTypes *particleType
 	}
 }
 
+void Object::end(ParticleSystem *particleSystem) {
+	vector<UnitParticleSystem *>::iterator iterFind = find(unitParticleSystems.begin(),unitParticleSystems.end(),particleSystem);
+	if(iterFind != unitParticleSystems.end()) {
+		unitParticleSystems.erase(iterFind);
+	}
+}
+
 void Object::setHeight(float height) {
-	pos.y=height;
+	pos.y = height;
 
 	for(UnitParticleSystems::iterator it= unitParticleSystems.begin(); it != unitParticleSystems.end(); ++it) {
 		bool particleValid = Renderer::getInstance().validateParticleSystemStillExists((*it),rsGame);
@@ -124,34 +142,38 @@ void Object::setHeight(float height) {
 	}
 }
 
-void Object::update() {
+void Object::updateHighlight() {
 	//highlight
 	if(highlight > 0.f) {
-		const Game *game = Renderer::getInstance().getGame();
-		highlight -= 1.f / (Game::highlightTime * game->getWorld()->getUpdateFps(-1));
+		//const Game *game = Renderer::getInstance().getGame();
+		//highlight -= 1.f / (Game::highlightTime * game->getWorld()->getUpdateFps(-1));
+		highlight -= 1.f / (Game::highlightTime * GameConstants::updateFps);
 	}
+}
 
-	if(objectType != NULL && objectType->getTilesetModelType(variation) != NULL &&
-			objectType->getTilesetModelType(variation)->getAnimSpeed() != 0.0) {
+void Object::update() {
+	//if(objectType != NULL && objectType->getTilesetModelType(variation) != NULL &&
+	//		objectType->getTilesetModelType(variation)->getAnimSpeed() != 0.0) {
+	if(animated == true) {
 //		printf("#1 Object updating [%s] Speed [%d] animProgress [%f]\n",this->objectType->getTilesetModelType(variation)->getModel()->getFileName().c_str(),objectType->getTilesetModelType(variation)->getAnimSpeed(),animProgress);
 
-		float heightFactor   = 1.f;
-		const float speedDivider= 100.f;
-		float speedDenominator = (speedDivider * GameConstants::updateFps);
-		float newAnimProgress = animProgress + (((float)objectType->getTilesetModelType(variation)->getAnimSpeed() * heightFactor) / speedDenominator);
+		if(objectType != NULL && objectType->getTilesetModelType(variation) != NULL) {
+			float heightFactor   = 1.f;
+			const float speedDivider= 100.f;
+			float speedDenominator = (speedDivider * GameConstants::updateFps);
 
-//		printf("A [%f] B [%f] C [%f] D [%f] E [%f] F [%f]\n",
-//				((float)objectType->getTilesetModelType(variation)->getAnimSpeed() * heightFactor),
-//				speedDenominator,
-//				((objectType->getTilesetModelType(variation)->getAnimSpeed() * heightFactor) / speedDenominator),
-//				(animProgress + ((objectType->getTilesetModelType(variation)->getAnimSpeed() * heightFactor) / speedDenominator)),
-//				animProgress,newAnimProgress);
+			// smooth TwoFrameanimations
+			float f=1.0f;
+			if(objectType->getTilesetModelType(variation)->getSmoothTwoFrameAnim()==true){
+				f=abs(std::sin(animProgress*2*3.16))+0.4f;
+			}
 
-		animProgress = newAnimProgress;
-//		printf("#2 new animProgress [%f]\n",animProgress);
+			float newAnimProgress = animProgress + f*(((float)objectType->getTilesetModelType(variation)->getAnimSpeed() * heightFactor) / speedDenominator);
 
-		if(animProgress > 1.f) {
-			animProgress = 0.f;
+			animProgress = newAnimProgress;
+			if(animProgress > 1.f) {
+				animProgress = 0.f;
+			}
 		}
 	}
 }
@@ -211,7 +233,7 @@ void Object::setVisible( bool visible)
 string Object::getUniquePickName() const {
 	string result = "";
 	if(resource != NULL) {
-		result += resource->getDescription() + " : ";
+		result += resource->getDescription(false) + " : ";
 	}
 	result += mapPos.getString();
 	return result;
@@ -239,7 +261,7 @@ void Object::saveGame(XmlNode *rootNode) {
 //	Vec3f pos;
 	objectNode->addAttribute("pos",pos.getString(), mapTagReplacements);
 //	float rotation;
-	objectNode->addAttribute("rotation",floatToStr(rotation), mapTagReplacements);
+	objectNode->addAttribute("rotation",floatToStr(rotation,6), mapTagReplacements);
 //	int variation;
 	objectNode->addAttribute("variation",intToStr(variation), mapTagReplacements);
 //	int lastRenderFrame;
