@@ -161,6 +161,11 @@ public:
 	virtual Map * getMap() = 0;
 
 	virtual void saveGame(XmlNode *rootNode) = 0;
+	virtual void loadGame(const XmlNode *rootNode) = 0;
+
+	virtual void clearCaches() = 0;
+
+	virtual Checksum getCRC() = 0;
 };
 
 class UnitPathBasic : public UnitPathInterface {
@@ -175,7 +180,6 @@ private:
 private:
 	int blockCount;
 	vector<Vec2i> pathQueue;
-	vector<Vec2i> lastPathCacheQueue;
 
 public:
 	UnitPathBasic();
@@ -193,13 +197,9 @@ public:
 	virtual void clearBlockCount() { blockCount = 0; }
 	virtual void incBlockCount();
 	virtual void add(const Vec2i &path);
-	void addToLastPathCache(const Vec2i &path);
 	Vec2i pop(bool removeFrontPos=true);
 	virtual int getBlockCount() const { return blockCount; }
-	virtual int getQueueCount() const { return pathQueue.size(); }
-
-	int getLastPathCacheQueueCount() const { return lastPathCacheQueue.size(); }
-	vector<Vec2i> getLastPathCacheQueue() const { return lastPathCacheQueue; }
+	virtual int getQueueCount() const { return (int)pathQueue.size(); }
 
 	virtual vector<Vec2i> getQueue() const { return pathQueue; }
 
@@ -209,6 +209,10 @@ public:
 	virtual std::string toString() const;
 
 	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
+	virtual void clearCaches();
+
+	virtual Checksum getCRC();
 };
 
 // =====================================================
@@ -227,11 +231,12 @@ private:
 
 public:
 	UnitPath() : UnitPathInterface(), blockCount(0), map(NULL) {} /**< Construct path object */
+
 	virtual bool isBlocked() const	{return blockCount >= maxBlockCount;} /**< is this path blocked	   */
 	virtual bool isEmpty() const	{return list<Vec2i>::empty();}	/**< is path empty				  */
 	virtual bool isStuck() const	{return false; }
 
-	int  size() const		{return list<Vec2i>::size();}	/**< size of path				 */
+	int  size() const		{return (int)list<Vec2i>::size();}	/**< size of path				 */
 	virtual void clear()			{list<Vec2i>::clear(); blockCount = 0;} /**< clear the path		*/
 	virtual void clearBlockCount() { blockCount = 0; }
 	virtual void incBlockCount()	{++blockCount;}		   /**< increment block counter			   */
@@ -267,6 +272,10 @@ public:
 	virtual std::string toString() const;
 
 	virtual void saveGame(XmlNode *rootNode) {};
+	virtual void loadGame(const XmlNode *rootNode) {};
+	virtual void clearCaches() {};
+
+	virtual Checksum getCRC() { return Checksum(); };
 };
 
 class WaypointPath : public list<Vec2i> {
@@ -286,17 +295,27 @@ public:
 // ===============================
 
 class UnitAttackBoostEffect {
+private:
+	int unitId;
+	const Unit *unitPtr;
+
+	const Unit *source;
+
+	void applyLoadedAttackBoostParticles(UnitParticleSystemType *upstPtr,const XmlNode* node, Unit* unit);
 public:
 
 	UnitAttackBoostEffect();
 	virtual ~UnitAttackBoostEffect();
 
 	const AttackBoost *boost;
-	const Unit *source;
+	//const Unit *source;
+	const Unit * getSource();
+	void setSource(const Unit *unit);
 	UnitParticleSystem *ups;
 	UnitParticleSystemType *upst;
 
 	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode, Unit *unit, World *world, bool applyToOriginator);
 };
 
 class UnitAttackBoostEffectOriginator {
@@ -310,9 +329,10 @@ public:
 	UnitAttackBoostEffect *currentAppliedEffect;
 
 	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode, Unit *unit, World *world);
 };
 
-class Unit : public BaseColorPickEntity, ValueCheckerVault {
+class Unit : public BaseColorPickEntity, ValueCheckerVault, public ParticleOwner {
 private:
     typedef list<Command*> Commands;
 	typedef list<UnitObserver*> Observers;
@@ -322,8 +342,11 @@ private:
 	static std::map<Unit *,bool> mapMemoryList;
 #endif
 
+	static const float ANIMATION_SPEED_MULTIPLIER;
+	static const int64 PROGRESS_SPEED_MULTIPLIER;
+
 public:
-	static const float speedDivider;
+	static const int speedDivider;
 	static const int maxDeadCount;
 	static const int invalidId;
 
@@ -333,18 +356,19 @@ public:
 #endif
 
 private:
-	const int id;
-    int hp;
-    int ep;
-    int loadCount;
-    int deadCount;
-    float progress;			//between 0 and 1
-	float lastAnimProgress;	//between 0 and 1
-	float animProgress;		//between 0 and 1
+	const int32 id;
+	int32 hp;
+	int32 ep;
+	int32 loadCount;
+	int32 deadCount;
+    //float progress;			//between 0 and 1
+    int64 progress;			//between 0 and 1
+	int64 lastAnimProgress;	//between 0 and 1
+	int64 animProgress;		//between 0 and 1
 	float highlight;
-	int progress2;
-	int kills;
-	int enemyKills;
+	int32 progress2;
+	int32 kills;
+	int32 enemyKills;
 	bool morphFieldsBlocked;
 
 	UnitReference targetRef;
@@ -367,11 +391,12 @@ private:
 	float rotationZ;
 	float rotationX;
 
+	const UnitType *preMorph_type;
     const UnitType *type;
     const ResourceType *loadType;
     const SkillType *currSkill;
-    int lastModelIndexForCurrSkillType;
-    int animationRandomCycleCount;
+    int32 lastModelIndexForCurrSkillType;
+    int32 animationRandomCycleCount;
 
     bool toBeUndertaken;
 	bool alive;
@@ -395,14 +420,15 @@ private:
 
 	vector<ParticleSystem*> fireParticleSystems;
 	vector<UnitParticleSystem*> smokeParticleSystems;
+	vector<ParticleSystem*> attackParticleSystems;
 
 	CardinalDir modelFacing;
 
 	std::string lastSynchDataString;
 	std::string lastFile;
-	int lastLine;
+	int32 lastLine;
 	std::string lastSource;
-	int lastRenderFrame;
+	int32 lastRenderFrame;
 	bool visible;
 
 	int retryCurrCommandCount;
@@ -433,7 +459,7 @@ private:
 	uint32 lastPathfindFailedFrame;
 	Vec2i lastPathfindFailedPos;
 	bool usePathfinderExtendedMaxNodes;
-	int maxQueuedCommandDisplayCount;
+	int32 maxQueuedCommandDisplayCount;
 
 	UnitAttackBoostEffectOriginator currentAttackBoostOriginatorEffect;
 
@@ -446,16 +472,28 @@ private:
 
 	bool changedActiveCommand;
 
-	int lastAttackerUnitId;
-	int lastAttackedUnitId;
+	int32 lastAttackerUnitId;
+	int32 lastAttackedUnitId;
 	CauseOfDeathType causeOfDeath;
 
 	uint32 pathfindFailedConsecutiveFrameCount;
 	Vec2i currentPathFinderDesiredFinalPos;
 
 	RandomGen random;
-	int pathFindRefreshCellCount;
+	int32 pathFindRefreshCellCount;
 
+	FowAlphaCellsLookupItem cachedFow;
+	Vec2i cachedFowPos;
+
+	ExploredCellsLookupItem cacheExploredCells;
+	std::pair<Vec2i, int> cacheExploredCellsKey;
+
+	Vec2i lastHarvestedResourcePos;
+
+	string networkCRCLogInfo;
+	string networkCRCParticleLogInfo;
+	vector<string> networkCRCDecHpList;
+	vector<string> networkCRCParticleInfoList;
 
 public:
     Unit(int id, UnitPathInterface *path, const Vec2i &pos, const UnitType *type, Faction *faction, Map *map, CardinalDir placeFacing);
@@ -470,14 +508,17 @@ public:
     void setCurrentPathFinderDesiredFinalPos(const Vec2i &finalPos) { currentPathFinderDesiredFinalPos = finalPos; }
     Vec2i getCurrentPathFinderDesiredFinalPos() const { return currentPathFinderDesiredFinalPos; }
 
-    //const std::pair<const SkillType *,std::vector<Unit *> > & getCurrentAttackBoostUnits() const { return currentAttackBoostUnits; }
     const UnitAttackBoostEffectOriginator & getAttackBoostOriginatorEffect() const { return currentAttackBoostOriginatorEffect; }
-    bool unitHasAttackBoost(const AttackBoost *boost, const Unit *source) const;
+    bool unitHasAttackBoost(const AttackBoost *boost, const Unit *source);
 
     inline uint32 getPathfindFailedConsecutiveFrameCount() const { return pathfindFailedConsecutiveFrameCount; }
     inline void incrementPathfindFailedConsecutiveFrameCount() { pathfindFailedConsecutiveFrameCount++; }
     inline void resetPathfindFailedConsecutiveFrameCount() { pathfindFailedConsecutiveFrameCount=0; }
 
+    const FowAlphaCellsLookupItem & getCachedFow() const { return cachedFow; }
+    FowAlphaCellsLookupItem getFogOfWarRadius(bool useCache) const;
+    void calculateFogOfWarRadius();
+
     //queries
     Command *getCurrrentCommandThreadSafe();
     void setIgnoreCheckCommand(bool value)      { ignoreCheckCommand=value;}
@@ -485,9 +526,12 @@ public:
     inline int getId() const							{return id;}
     inline Field getCurrField() const					{return currField;}
     inline int getLoadCount() const					{return loadCount;}
-    inline float getLastAnimProgress() const			{return lastAnimProgress;}
-	//float getProgress() const					{return progress;}
-    inline float getAnimProgress() const				{return animProgress;}
+
+    //inline int getLastAnimProgress() const			{return lastAnimProgress;}
+    //inline int getAnimProgress() const				{return animProgress;}
+    inline float getLastAnimProgressAsFloat() const	{return static_cast<float>(lastAnimProgress) / ANIMATION_SPEED_MULTIPLIER;}
+    inline float getAnimProgressAsFloat() const		{return static_cast<float>(animProgress) / ANIMATION_SPEED_MULTIPLIER;}
+
     inline float getHightlight() const					{return highlight;}
     inline int getProgress2() const					{return progress2;}
 	inline int getFactionIndex() const {
@@ -509,18 +553,22 @@ public:
 	inline Vec2i getMeetingPos() const					{return meetingPos;}
 	inline Faction *getFaction() const					{return faction;}
 	inline const ResourceType *getLoadType() const		{return loadType;}
+
 	inline const UnitType *getType() const				{return type;}
+	void setType(const UnitType *newType);
+	inline const UnitType *getPreMorphType() const		{return preMorph_type;}
+
 	inline const SkillType *getCurrSkill() const		{return currSkill;}
 	inline const TotalUpgrade *getTotalUpgrade() const	{return &totalUpgrade;}
 	inline float getRotation() const					{return rotation;}
 	float getRotationX() const;
 	float getRotationZ() const;
-	inline ParticleSystem *getFire() const				{return fire;}
+	ParticleSystem *getFire() const;
 	inline int getKills() const						{return kills;}
 	inline int getEnemyKills() const					{return enemyKills;}
 	inline const Level *getLevel() const				{return level;}
 	const Level *getNextLevel() const;
-	string getFullName() const;
+	string getFullName(bool translatedValue) const;
 	inline const UnitPathInterface *getPath() const	{return unitPath;}
 	inline UnitPathInterface *getPath()				{return unitPath;}
 	inline WaypointPath *getWaypointPath()				{return &waypointPath;}
@@ -535,7 +583,8 @@ public:
 	inline void setCauseOfDeath(CauseOfDeathType cause) { causeOfDeath = cause; }
 
     //pos
-	inline Vec2i getPos() const				{return pos;}
+	inline Vec2i getPosNotThreadSafe() const				{return pos;}
+	Vec2i getPos();
 	Vec2i getPosWithCellMapSet() const;
 	inline Vec2i getLastPos() const			{return lastPos;}
 	Vec2i getCenteredPos() const;
@@ -561,17 +610,21 @@ public:
 	bool isInteresting(InterestingUnitType iut) const;
 
     //set
-	inline void setCurrField(Field currField)					{this->currField= currField;}
+	void setCurrField(Field currField);
     void setCurrSkill(const SkillType *currSkill);
     void setCurrSkill(SkillClass sc);
 
     void setMorphFieldsBlocked ( bool value ) {this->morphFieldsBlocked=value;}
 	bool getMorphFieldsBlocked() const { return morphFieldsBlocked; }
 
+	inline void setLastHarvestedResourcePos(Vec2i pos)		{ this->lastHarvestedResourcePos = pos; }
+	inline Vec2i getLastHarvestedResourcePos() const	{ return this->lastHarvestedResourcePos; }
+
     inline void setLoadCount(int loadCount)					{this->loadCount= loadCount;}
     inline void setLoadType(const ResourceType *loadType)		{this->loadType= loadType;}
     inline void setProgress2(int progress2)					{this->progress2= progress2;}
 	void setPos(const Vec2i &pos,bool clearPathFinder=false);
+	void refreshPos();
 	void setTargetPos(const Vec2i &targetPos);
 	void setTarget(const Unit *unit);
 	void setTargetVec(const Vec3f &targetVec);
@@ -615,8 +668,8 @@ public:
     //other
 	void resetHighlight();
 	const CommandType *computeCommandType(const Vec2i &pos, const Unit *targetUnit= NULL) const;
-	string getDesc() const;
-	string getDescExtension() const;
+	string getDesc(bool translatedValue) const;
+	string getDescExtension(bool translatedValue) const;
     bool computeEp();
     //bool computeHp();
     bool repair();
@@ -699,12 +752,20 @@ public:
 	//void addCurrentTargetPathTakenCell(const Vec2i &target,const Vec2i &cell);
 
 	void logSynchData(string file,int line,string source="");
-	std::string toString() const;
-	bool needToUpdate();
+	void logSynchDataThreaded(string file,int line,string source="");
 
-	bool isLastStuckFrameWithinCurrentFrameTolerance() const;
+	std::string toString(bool crcMode=false) const;
+	bool needToUpdate();
+	float getProgressAsFloat() const;
+	int64 getUpdateProgress();
+	int64 getDiagonalFactor();
+	int64 getHeightFactor(int64 speedMultiplier=PROGRESS_SPEED_MULTIPLIER);
+	int64 getSpeedDenominator(int64 updateFPS);
+	bool isChangedActiveCommand() const { return changedActiveCommand; }
+
+	bool isLastStuckFrameWithinCurrentFrameTolerance(bool evalMode);
 	inline uint32 getLastStuckFrame() const { return lastStuckFrame; }
-	inline void setLastStuckFrame(uint32 value) { lastStuckFrame = value; }
+	//inline void setLastStuckFrame(uint32 value) { lastStuckFrame = value; }
 	void setLastStuckFrameToCurrentFrame();
 
 	inline Vec2i getLastStuckPos() const { return lastStuckPos; }
@@ -727,7 +788,26 @@ public:
 	void saveGame(XmlNode *rootNode);
 	static Unit * loadGame(const XmlNode *rootNode,GameSettings *settings,Faction *faction, World *world);
 
+	void clearCaches();
+	bool showTranslatedTechTree() const;
+
+	void addAttackParticleSystem(ParticleSystem *ps);
+
+	Checksum getCRC();
+
+	virtual void end(ParticleSystem *particleSystem);
+	virtual void logParticleInfo(string info);
+	void setNetworkCRCParticleLogInfo(string networkCRCParticleLogInfo) { this->networkCRCParticleLogInfo = networkCRCParticleLogInfo; }
+	void clearParticleInfo();
+	void addNetworkCRCDecHp(string info);
+	void clearNetworkCRCDecHpList();
+
 private:
+
+	bool isNetworkCRCEnabled();
+	string getNetworkCRCDecHpList() const;
+	string getParticleInfo() const;
+
 	float computeHeight(const Vec2i &pos) const;
 	void calculateXZRotation();
 	void updateTarget();
@@ -737,12 +817,19 @@ private:
 	void stopDamageParticles(bool force);
 	void startDamageParticles();
 
-	int getFrameCount() const;
+	uint32 getFrameCount() const;
 	void checkCustomizedParticleTriggers(bool force);
 	bool checkModelStateInfoForNewHpValue();
 	void checkUnitLevel();
 
 	void morphAttackBoosts(Unit *unit);
+
+	int64 getUpdatedProgress(int64 currentProgress, int64 updateFPS, int64 speed, int64 diagonalFactor, int64 heightFactor);
+
+	void logSynchDataCommon(string file,int line,string source="",bool threadedMode=false);
+	void updateAttackBoostProgress(const Game* game);
+
+	void setAlive(bool value);
 };
 
 }}// end namespace
