@@ -48,6 +48,7 @@ enum CommandClass {
 	ccUpgrade,
 	ccMorph,
 	ccSwitchTeam,
+	ccHarvestEmergencyReturn,
 
 	ccCount,
 	ccNull
@@ -76,6 +77,9 @@ protected:
     Clicks clicks;
 	int id;
 
+	std::map<string,bool> fogOfWarSkillAttachments;
+	const FogOfWarSkillType* fogOfWarSkillType;
+
 public:
 	static const int invalidId= -1;
     CommandClass commandTypeClass;
@@ -85,13 +89,15 @@ public:
 	    commandTypeClass 	= ccNull;
 	    clicks 				= cOne;
 		id 					= -1;
+		fogOfWarSkillType	= NULL;
+		fogOfWarSkillAttachments.clear();
 	}
     virtual void update(UnitUpdater *unitUpdater, Unit *unit, int frameIndex) const= 0;
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const= 0;
-	virtual string toString() const= 0;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const= 0;
+	virtual string toString(bool translatedValue) const= 0;
 	virtual const ProducibleType *getProduced() const	{return NULL;}
 	virtual Queueability isQueuable() const						{return qOnRequest;}
 	bool isQueuable(bool tryQueue) const {
@@ -110,6 +116,10 @@ public:
     CommandClass getClass() const;
 	Clicks getClicks() const		{return clicks;}
 	int getId() const				{return id;}
+
+	const FogOfWarSkillType *getFogOfWarSkillType() const	{return fogOfWarSkillType;};
+
+	bool hasFogOfWarSkillType(string name) const;
 };
 
 // ===============================
@@ -126,8 +136,8 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir, const TechTree *tt,
     		const FactionType *ft, const UnitType &ut, std::map<string,vector<pair<string, string> > > &loadedFileList,
     		string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 	virtual Queueability isQueuable() const						{return qNever;}
 	virtual int getTypePriority() const							{return 100000;}
     //get
@@ -151,8 +161,8 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 
     //get
 	const MoveSkillType *getMoveSkillType() const	{return moveSkillType;};
@@ -176,8 +186,8 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 
 
     //get
@@ -202,8 +212,8 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 
     //get
 	const StopSkillType * getStopSkillType() const		{return stopSkillType;}
@@ -232,13 +242,13 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 
     //get
 	const MoveSkillType *getMoveSkillType() const	{return moveSkillType;}
 	const BuildSkillType *getBuildSkillType() const	{return buildSkillType;}
-	int getBuildingCount() const					{return buildings.size();}
+	int getBuildingCount() const					{return (int)buildings.size();}
 	const UnitType * getBuilding(int i) const		{return buildings[i];}
 	StaticSound *getStartSound() const				{return startSounds.getRandSound();}
 	StaticSound *getBuiltSound() const				{return builtSounds.getRandSound();}
@@ -267,8 +277,8 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 	virtual Queueability isQueuable() const						{return qOnRequest;}
 
     //get
@@ -278,13 +288,34 @@ public:
 	const StopSkillType *getStopLoadedSkillType() const		{return stopLoadedSkillType;}
 	int getMaxLoad() const									{return maxLoad;}
 	int getHitsPerUnit() const								{return hitsPerUnit;}
-	int getHarvestedResourceCount() const					{return harvestedResources.size();}
+	int getHarvestedResourceCount() const					{return (int)harvestedResources.size();}
 	const ResourceType* getHarvestedResource(int i) const	{return harvestedResources[i];}
 	bool canHarvest(const ResourceType *resourceType) const;
 
 	virtual bool usesPathfinder() const { return true; }
 };
 
+// ===============================
+// 	class HarvestEmergencyReturnCommandType
+// ===============================
+
+class HarvestEmergencyReturnCommandType: public CommandType {
+private:
+
+public:
+    HarvestEmergencyReturnCommandType();
+	virtual void update(UnitUpdater *unitUpdater, Unit *unit, int frameIndex) const;
+    virtual void load(int id, const XmlNode *n, const string &dir,
+    		const TechTree *tt, const FactionType *ft, const UnitType &ut,
+    		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
+	virtual Queueability isQueuable() const						{return qOnRequest;}
+
+    //get
+	virtual bool usesPathfinder() const { return true; }
+};
+
 
 // ===============================
 // 	class RepairCommandType
@@ -303,15 +334,15 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 
     //get
 	const MoveSkillType *getMoveSkillType() const			{return moveSkillType;};
 	const RepairSkillType *getRepairSkillType() const		{return repairSkillType;};
     bool isRepairableUnitType(const UnitType *unitType) const;
 
-	int getRepairCount() const					{return repairableUnits.size();}
+	int getRepairCount() const					{return (int)repairableUnits.size();}
 	const UnitType * getRepair(int i) const		{return repairableUnits[i];}
 
 	virtual bool usesPathfinder() const { return true; }
@@ -333,9 +364,9 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-    virtual string getReqDesc() const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+    virtual string getReqDesc(bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 	virtual const ProducibleType *getProduced() const;
 	virtual Queueability isQueuable() const						{return qAlways;}
 	virtual int getTypePriority() const {return 15;}
@@ -363,9 +394,9 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
-	virtual string getReqDesc() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
+	virtual string getReqDesc(bool translatedValue) const;
 	virtual const ProducibleType *getProduced() const;
 	virtual Queueability isQueuable() const						{return qAlways;}
 	virtual int getTypePriority() const {return 15;}
@@ -387,6 +418,7 @@ private:
     const UnitType* morphUnit;
 	int discount;
 	bool ignoreResourceRequirements;
+	bool replaceStorage;
 
 public:
     MorphCommandType();
@@ -394,9 +426,9 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
-	virtual string getReqDesc() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
+	virtual string getReqDesc(bool translatedValue) const;
 	virtual const ProducibleType *getProduced() const;
 	Queueability isQueuable() const						{return qOnlyLast;} //After morph anything can happen
 
@@ -405,6 +437,7 @@ public:
 	const UnitType *getMorphUnit() const				{return morphUnit;}
 	int getDiscount() const								{return discount;}
 	bool getIgnoreResourceRequirements() const			{return ignoreResourceRequirements;}
+	bool getReplaceStorage() const						{return replaceStorage;}
 
 	virtual bool usesPathfinder() const { return false; }
 };
@@ -423,8 +456,8 @@ public:
     virtual void load(int id, const XmlNode *n, const string &dir,
     		const TechTree *tt, const FactionType *ft, const UnitType &ut,
     		std::map<string,vector<pair<string, string> > > &loadedFileList, string parentLoader);
-    virtual string getDesc(const TotalUpgrade *totalUpgrade) const;
-	virtual string toString() const;
+    virtual string getDesc(const TotalUpgrade *totalUpgrade, bool translatedValue) const;
+	virtual string toString(bool translatedValue) const;
 
 	virtual bool usesPathfinder() const { return false; }
 };
