@@ -38,12 +38,14 @@ FactionType::FactionType() {
 
 //load a faction, given a directory
 void FactionType::load(const string &factionName, const TechTree *techTree, Checksum* checksum,
-		Checksum *techtreeChecksum, std::map<string,vector<pair<string, string> > > &loadedFileList) {
+		Checksum *techtreeChecksum,
+		std::map<string,vector<pair<string, string> > > &loadedFileList,
+		bool validationMode) {
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	string techTreePath = techTree->getPath();
-	string techTreeName=techTree->getName();
+	string techTreeName=techTree->getNameUntranslated();
 	string currentPath = "";
 
 	//open xml file
@@ -116,7 +118,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 	}
 
 	char szBuf[8096]="";
-	snprintf(szBuf,8096,Lang::getInstance().get("LogScreenGameLoadingFactionType","",true).c_str(),formatString(name).c_str());
+	snprintf(szBuf,8096,Lang::getInstance().getString("LogScreenGameLoadingFactionType","",true).c_str(),formatString(this->getName()).c_str());
 	Logger::getInstance().add(szBuf, true);
 
 	if(personalityType == fpt_Normal) {
@@ -134,7 +136,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 
 		unitTypes.resize(unitFilenames.size());
 
-		for(int i = 0; i < unitTypes.size(); ++i) {
+		for(int i = 0; i < (int)unitTypes.size(); ++i) {
 			string str= currentPath + "units/" + unitFilenames[i];
 			unitTypes[i].preLoad(str);
 
@@ -149,7 +151,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 		findDirs(upgradesPath, upgradeFilenames,false,false);
 
 		upgradeTypes.resize(upgradeFilenames.size());
-		for(int i = 0; i < upgradeTypes.size(); ++i) {
+		for(int i = 0; i < (int)upgradeTypes.size(); ++i) {
 			string str= currentPath + "upgrades/" + upgradeFilenames[i];
 			upgradeTypes[i].preLoad(str);
 
@@ -160,31 +162,51 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 		try {
 			Logger &logger= Logger::getInstance();
 			int progressBaseValue=logger.getProgress();
-			for(int i = 0; i < unitTypes.size(); ++i) {
+			for(int i = 0; i < (int)unitTypes.size(); ++i) {
 				string str= currentPath + "units/" + unitTypes[i].getName();
-				unitTypes[i].loaddd(i, str, techTree,techTreePath, this, checksum,techtreeChecksum,
-						loadedFileList);
-				logger.setProgress(progressBaseValue+(int)((((double)i + 1.0) / (double)unitTypes.size()) * 100.0/techTree->getTypeCount()));
-				SDL_PumpEvents();
+
+				try {
+					unitTypes[i].loaddd(i, str, techTree,techTreePath, this, checksum,techtreeChecksum,
+						loadedFileList,validationMode);
+					logger.setProgress(progressBaseValue+(int)((((double)i + 1.0) / (double)unitTypes.size()) * 100.0/techTree->getTypeCount()));
+					SDL_PumpEvents();
+				}
+				catch(megaglest_runtime_error& ex) {
+					if(validationMode == false) {
+						throw;
+					}
+					else {
+						SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
+					}
+				}
 			}
 		}
 	    catch(megaglest_runtime_error& ex) {
-	    	//printf("1111111b ex.wantStackTrace() = %d\n",ex.wantStackTrace());
 			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
-			//printf("222222b\n");
-			throw megaglest_runtime_error("Error loading units: "+ currentPath + "\n" + ex.what(),!ex.wantStackTrace());
+			throw megaglest_runtime_error("Error loading units: "+ currentPath + "\nMessage: " + ex.what(),!ex.wantStackTrace());
 	    }
 		catch(const exception &e) {
 			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
-			throw megaglest_runtime_error("Error loading units: "+ currentPath + "\n" + e.what());
+			throw megaglest_runtime_error("Error loading units: "+ currentPath + "\nMessage: " + e.what());
 		}
 
 		// b2) load upgrades
 		try{
-			for(int i = 0; i < upgradeTypes.size(); ++i) {
+			for(int i = 0; i < (int)upgradeTypes.size(); ++i) {
 				string str= currentPath + "upgrades/" + upgradeTypes[i].getName();
-				upgradeTypes[i].load(str, techTree, this, checksum,techtreeChecksum,
-						loadedFileList);
+
+				try {
+					upgradeTypes[i].load(str, techTree, this, checksum,
+							techtreeChecksum,loadedFileList,validationMode);
+				}
+				catch(megaglest_runtime_error& ex) {
+					if(validationMode == false) {
+						throw;
+					}
+					else {
+						SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
+					}
+				}
 
 				SDL_PumpEvents();
 			}
@@ -210,18 +232,29 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 		const XmlNode *startingResourcesNode= factionNode->getChild("starting-resources");
 
 		startingResources.resize(startingResourcesNode->getChildCount());
-		for(int i = 0; i < startingResources.size(); ++i) {
+		for(int i = 0; i < (int)startingResources.size(); ++i) {
 			const XmlNode *resourceNode= startingResourcesNode->getChild("resource", i);
 			string name= resourceNode->getAttribute("name")->getRestrictedValue();
 			int amount= resourceNode->getAttribute("amount")->getIntValue();
-			startingResources[i].init(techTree->getResourceType(name), amount);
+
+			try {
+				startingResources[i].init(techTree->getResourceType(name), amount);
+			}
+			catch(megaglest_runtime_error& ex) {
+				if(validationMode == false) {
+					throw;
+				}
+				else {
+					SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\nFor FactionType: %s for StartResource: %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what(),this->name.c_str(),name.c_str());
+				}
+			}
 
 			SDL_PumpEvents();
 		}
 
 		//read starting units
 		const XmlNode *startingUnitsNode= factionNode->getChild("starting-units");
-		for(int i = 0; i < startingUnitsNode->getChildCount(); ++i) {
+		for(int i = 0; i < (int)startingUnitsNode->getChildCount(); ++i) {
 			const XmlNode *unitNode= startingUnitsNode->getChild("unit", i);
 			string name= unitNode->getAttribute("name")->getRestrictedValue();
 			int amount= unitNode->getAttribute("amount")->getIntValue();
@@ -248,7 +281,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 
 			if(aiNode->hasChild("static-values") == true) {
 				const XmlNode *aiNodeUnits= aiNode->getChild("static-values");
-				for(int i = 0; i < aiNodeUnits->getChildCount(); ++i) {
+				for(int i = 0; i < (int)aiNodeUnits->getChildCount(); ++i) {
 					const XmlNode *unitNode= aiNodeUnits->getChild("static", i);
 					AIBehaviorStaticValueCategory type  = aibsvcMaxBuildRadius;
 					if(unitNode->hasAttribute("type") == true) {
@@ -269,7 +302,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 
 			if(aiNode->hasChild("worker-units") == true) {
 				const XmlNode *aiNodeUnits= aiNode->getChild("worker-units");
-				for(int i = 0; i < aiNodeUnits->getChildCount(); ++i) {
+				for(int i = 0; i < (int)aiNodeUnits->getChildCount(); ++i) {
 					const XmlNode *unitNode= aiNodeUnits->getChild("unit", i);
 					string name= unitNode->getAttribute("name")->getRestrictedValue();
 					int minimum= unitNode->getAttribute("minimum")->getIntValue();
@@ -279,7 +312,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 			}
 			if(aiNode->hasChild("warrior-units") == true) {
 				const XmlNode *aiNodeUnits= aiNode->getChild("warrior-units");
-				for(int i = 0; i < aiNodeUnits->getChildCount(); ++i) {
+				for(int i = 0; i < (int)aiNodeUnits->getChildCount(); ++i) {
 					const XmlNode *unitNode= aiNodeUnits->getChild("unit", i);
 					string name= unitNode->getAttribute("name")->getRestrictedValue();
 					int minimum= unitNode->getAttribute("minimum")->getIntValue();
@@ -289,7 +322,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 			}
 			if(aiNode->hasChild("resource-producer-units") == true) {
 				const XmlNode *aiNodeUnits= aiNode->getChild("resource-producer-units");
-				for(int i = 0; i < aiNodeUnits->getChildCount(); ++i) {
+				for(int i = 0; i < (int)aiNodeUnits->getChildCount(); ++i) {
 					const XmlNode *unitNode= aiNodeUnits->getChild("unit", i);
 					string name= unitNode->getAttribute("name")->getRestrictedValue();
 					int minimum= unitNode->getAttribute("minimum")->getIntValue();
@@ -299,7 +332,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 			}
 			if(aiNode->hasChild("building-units") == true) {
 				const XmlNode *aiNodeUnits= aiNode->getChild("building-units");
-				for(int i = 0; i < aiNodeUnits->getChildCount(); ++i) {
+				for(int i = 0; i < (int)aiNodeUnits->getChildCount(); ++i) {
 					const XmlNode *unitNode= aiNodeUnits->getChild("unit", i);
 					string name= unitNode->getAttribute("name")->getRestrictedValue();
 					int minimum= unitNode->getAttribute("minimum")->getIntValue();
@@ -310,7 +343,7 @@ void FactionType::load(const string &factionName, const TechTree *techTree, Chec
 
 			if(aiNode->hasChild("upgrades") == true) {
 				const XmlNode *aiNodeUpgrades= aiNode->getChild("upgrades");
-				for(int i = 0; i < aiNodeUpgrades->getChildCount(); ++i) {
+				for(int i = 0; i < (int)aiNodeUpgrades->getChildCount(); ++i) {
 					const XmlNode *upgradeNode= aiNodeUpgrades->getChild("upgrade", i);
 					string name= upgradeNode->getAttribute("name")->getRestrictedValue();
 
@@ -356,10 +389,10 @@ std::vector<std::string> FactionType::validateFactionType() {
 		results.push_back(szBuf);
 	}
 
-    for(int i=0; i<unitTypes.size(); ++i){
+    for(int i=0; i < (int)unitTypes.size(); ++i){
     	UnitType &unitType = unitTypes[i];
 
-    	for(int i = 0; i < unitType.getSelectionSounds().getSounds().size(); ++i) {
+    	for(int i = 0; i < (int)unitType.getSelectionSounds().getSounds().size(); ++i) {
     		StaticSound *sound = unitType.getSelectionSounds().getSounds()[i];
     		if(sound != NULL && sound->getInfo()->getBitRate() > MAX_BITRATE_WARNING) {
 				char szBuf[8096]="";
@@ -367,7 +400,7 @@ std::vector<std::string> FactionType::validateFactionType() {
 				results.push_back(szBuf);
     		}
     	}
-    	for(int i = 0; i < unitType.getCommandSounds().getSounds().size(); ++i) {
+    	for(int i = 0; i < (int)unitType.getCommandSounds().getSounds().size(); ++i) {
     		StaticSound *sound = unitType.getCommandSounds().getSounds()[i];
     		if(sound != NULL && sound->getInfo()->getBitRate() > MAX_BITRATE_WARNING) {
 				char szBuf[8096]="";
@@ -377,7 +410,7 @@ std::vector<std::string> FactionType::validateFactionType() {
     	}
 
     	int morphCommandCount = 0;
-    	for(int j = 0; j < unitType.getCommandTypeCount(); ++j) {
+    	for(int j = 0; j < (int)unitType.getCommandTypeCount(); ++j) {
     		const CommandType *cmdType = unitType.getCommandType(j);
     		if(cmdType != NULL) {
     	    	// Check every unit's commands to validate that for every upgrade-requirements
@@ -388,16 +421,18 @@ std::vector<std::string> FactionType::validateFactionType() {
 					if(upgradeType != NULL) {
 						// Now lets find a unit that can produced-upgrade this upgrade
 						bool foundUpgraderUnit = false;
-						for(int l=0; l<unitTypes.size() && foundUpgraderUnit == false; ++l){
+						for(int l=0; l < (int)unitTypes.size() && foundUpgraderUnit == false; ++l){
 							UnitType &unitType2 = unitTypes[l];
 							for(int m = 0; m < unitType2.getCommandTypeCount() && foundUpgraderUnit == false; ++m) {
 								const CommandType *cmdType2 = unitType2.getCommandType(m);
 								if(cmdType2 != NULL && dynamic_cast<const UpgradeCommandType *>(cmdType2) != NULL) {
 									const UpgradeCommandType *uct = dynamic_cast<const UpgradeCommandType *>(cmdType2);
-									const UpgradeType *upgradeType2 = uct->getProducedUpgrade();
-									if(upgradeType2 != NULL && upgradeType2->getName() == upgradeType->getName()) {
-										 foundUpgraderUnit = true;
-										 break;
+									if(uct != NULL) {
+										const UpgradeType *upgradeType2 = uct->getProducedUpgrade();
+										if(upgradeType2 != NULL && upgradeType2->getName() == upgradeType->getName()) {
+											 foundUpgraderUnit = true;
+											 break;
+										}
 									}
 								}
 							}
@@ -415,12 +450,15 @@ std::vector<std::string> FactionType::validateFactionType() {
 				// exist in this faction
 				if(cmdType->getClass() == ccBuild) {
 					const BuildCommandType *build = dynamic_cast<const BuildCommandType *>(cmdType);
+					if(build == NULL) {
+						throw megaglest_runtime_error("build == NULL");
+					}
 					for(int k = 0; k < build->getBuildingCount(); ++k) {
 						const UnitType *buildUnit = build->getBuilding(k);
 
 						// Now lets find the unit that we should be able to build
 						bool foundUnit = false;
-						for(int l=0; l<unitTypes.size() && foundUnit == false; ++l){
+						for(int l=0; l < (int)unitTypes.size() && foundUnit == false; ++l){
 							UnitType &unitType2 = unitTypes[l];
 							if(unitType2.getName() == buildUnit->getName()) {
 								foundUnit = true;
@@ -448,12 +486,15 @@ std::vector<std::string> FactionType::validateFactionType() {
 				// exist in this faction
 				if(cmdType->getClass() == ccRepair) {
 					const RepairCommandType *repair = dynamic_cast<const RepairCommandType *>(cmdType);
+					if(repair == NULL) {
+						throw megaglest_runtime_error("repair == NULL");
+					}
 					for(int k = 0; k < repair->getRepairCount(); ++k) {
 						const UnitType *repairUnit = repair->getRepair(k);
 
 						// Now lets find the unit that we should be able to repair
 						bool foundUnit = false;
-						for(int l=0; l<unitTypes.size() && foundUnit == false; ++l){
+						for(int l=0; l < (int)unitTypes.size() && foundUnit == false; ++l){
 							UnitType &unitType2 = unitTypes[l];
 							if(unitType2.getName() == repairUnit->getName()) {
 								 foundUnit = true;
@@ -479,7 +520,7 @@ std::vector<std::string> FactionType::validateFactionType() {
 						// Now lets find the unit that we should be able to morph
 						// to
 						bool foundUnit = false;
-						for(int l=0; l<unitTypes.size() && foundUnit == false; ++l){
+						for(int l=0; l < (int)unitTypes.size() && foundUnit == false; ++l){
 							UnitType &unitType2 = unitTypes[l];
 							if(unitType2.getName() == morphUnit->getName()) {
 								 foundUnit = true;
@@ -511,7 +552,7 @@ std::vector<std::string> FactionType::validateFactionType() {
 			if(unitType2 != NULL) {
 				// Now lets find the required unit
 				bool foundUnit = false;
-				for(int l=0; l<unitTypes.size() && foundUnit == false; ++l){
+				for(int l=0; l < (int)unitTypes.size() && foundUnit == false; ++l){
 					UnitType &unitType3 = unitTypes[l];
 
 					if(unitType2->getName() == unitType3.getName()) {
@@ -530,7 +571,7 @@ std::vector<std::string> FactionType::validateFactionType() {
 
 		// Now check that at least 1 other unit can produce, build or morph this unit
 		bool foundUnit = false;
-		for(int l=0; l<unitTypes.size() && foundUnit == false; ++l){
+		for(int l=0; l < (int)unitTypes.size() && foundUnit == false; ++l){
 			UnitType &unitType2 = unitTypes[l];
 
 	    	for(int j = 0; j < unitType2.getCommandTypeCount() && foundUnit == false; ++j) {
@@ -553,6 +594,9 @@ std::vector<std::string> FactionType::validateFactionType() {
 	    			// Check if this is a build command
 					if(cmdType->getClass() == ccBuild) {
 						const BuildCommandType *build = dynamic_cast<const BuildCommandType *>(cmdType);
+						if(build == NULL) {
+							throw megaglest_runtime_error("build == NULL");
+						}
 						for(int k = 0; k < build->getBuildingCount() && foundUnit == false; ++k) {
 							const UnitType *buildUnit = build->getBuilding(k);
 
@@ -570,6 +614,9 @@ std::vector<std::string> FactionType::validateFactionType() {
 					// Check if this is a morph command
 					if(cmdType->getClass() == ccMorph) {
 							const MorphCommandType *morph = dynamic_cast<const MorphCommandType *>(cmdType);
+							if(morph == NULL) {
+								throw megaglest_runtime_error("morph == NULL");
+							}
 							const UnitType *morphUnit = morph->getMorphUnit();
 
 							if( morphUnit != NULL &&
@@ -611,7 +658,7 @@ std::vector<std::string> FactionType::validateFactionType() {
                 const SkillType *st = unitType.getSkillType(j);
                 if(st != NULL && dynamic_cast<const AttackSkillType *>(st) != NULL) {
                     const AttackSkillType *ast = dynamic_cast<const AttackSkillType *>(st);
-                    if(ast->getAttackVar() < 0) {
+                    if(ast != NULL && ast->getAttackVar() < 0) {
                         char szBuf[8096]="";
                         snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the skill [%s] with an INVALID attack var value which is < 0 [%d]!",unitType.getName().c_str(),this->getName().c_str(),ast->getName().c_str(),ast->getAttackVar());
                         results.push_back(szBuf);
@@ -647,7 +694,7 @@ std::vector<std::string> FactionType::validateFactionType() {
 std::vector<std::string> FactionType::validateFactionTypeResourceTypes(vector<ResourceType> &resourceTypes) {
 	std::vector<std::string> results;
 
-    for(int i=0; i<unitTypes.size(); ++i) {
+    for(int i=0; i < (int)unitTypes.size(); ++i) {
     	UnitType &unitType = unitTypes[i];
 
 		// Check every unit's required resources to validate that for every resource-requirements
@@ -657,7 +704,7 @@ std::vector<std::string> FactionType::validateFactionTypeResourceTypes(vector<Re
     		if(r != NULL && r->getType() != NULL) {
     			bool foundResourceType = false;
     			// Now lets find a matching faction resource type for the unit
-    			for(int k=0; k<resourceTypes.size(); ++k){
+    			for(int k=0; k < (int)resourceTypes.size(); ++k){
     				ResourceType &rt = resourceTypes[k];
 
 					if(r->getType()->getName() == rt.getName()) {
@@ -681,7 +728,7 @@ std::vector<std::string> FactionType::validateFactionTypeResourceTypes(vector<Re
 			if(r != NULL && r->getType() != NULL) {
 				bool foundResourceType = false;
 				// Now lets find a matching faction resource type for the unit
-				for(int k=0; k<resourceTypes.size(); ++k){
+				for(int k=0; k < (int)resourceTypes.size(); ++k){
 					ResourceType &rt = resourceTypes[k];
 
 					if(r->getType()->getName() == rt.getName()) {
@@ -705,12 +752,15 @@ std::vector<std::string> FactionType::validateFactionTypeResourceTypes(vector<Re
 				// exist in this faction
 				if(cmdType->getClass() == ccHarvest) {
 					const HarvestCommandType *harvest = dynamic_cast<const HarvestCommandType *>(cmdType);
+					if(harvest == NULL) {
+						throw megaglest_runtime_error("harvest == NULL");
+					}
 					for(int k = 0; k < harvest->getHarvestedResourceCount(); ++k) {
 						const ResourceType *harvestResource = harvest->getHarvestedResource(k);
 
 						bool foundResourceType = false;
 						// Now lets find a matching faction resource type for the unit
-						for(int k=0; k<resourceTypes.size(); ++k){
+						for(int k=0; k < (int)resourceTypes.size(); ++k){
 							ResourceType &rt = resourceTypes[k];
 
 							if(harvestResource->getName() == rt.getName()) {
@@ -738,12 +788,12 @@ std::vector<std::string> FactionType::validateFactionTypeUpgradeTypes() {
 
 	// For each upgrade type make sure there is at least 1 unit that can produce
 	// the upgrade
-	for(int i = 0; i < upgradeTypes.size(); ++i) {
+	for(int i = 0; i < (int)upgradeTypes.size(); ++i) {
 		const UpgradeType &upgradeType = upgradeTypes[i];
 
 		// First find a unit with a command type to upgrade to this Upgrade type
 		bool foundUnit = false;
-	    for(int j=0; j<unitTypes.size() && foundUnit == false; ++j){
+	    for(int j=0; j < (int)unitTypes.size() && foundUnit == false; ++j){
 	    	UnitType &unitType = unitTypes[j];
 	    	for(int k = 0; k < unitType.getCommandTypeCount() && foundUnit == false; ++k) {
 				const CommandType *cmdType = unitType.getCommandType(k);
@@ -778,59 +828,59 @@ std::vector<std::string> FactionType::validateFactionTypeUpgradeTypes() {
 // ==================== get ====================
 
 const UnitType *FactionType::getUnitType(const string &name) const{
-    for(int i=0; i<unitTypes.size();i++){
-		if(unitTypes[i].getName()==name){
+    for(int i=0; i < (int)unitTypes.size();i++){
+		if(unitTypes[i].getName(false)==name){
             return &unitTypes[i];
 		}
     }
 
     printf("In [%s::%s Line: %d] scanning [%s] size = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
-    for(int i=0; i<unitTypes.size();i++){
-    	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str());
+    for(int i=0; i < (int)unitTypes.size();i++){
+    	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName(false).c_str());
     }
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
-    for(int i=0; i<unitTypes.size();i++){
-    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str());
+    for(int i=0; i < (int)unitTypes.size();i++){
+    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName(false).c_str());
     }
 
 	throw megaglest_runtime_error("Unit type not found: [" + name + "] in faction type [" + this->name + "]");
 }
 
 const UnitType *FactionType::getUnitTypeById(int id) const{
-    for(int i=0; i<unitTypes.size();i++){
+    for(int i=0; i < (int)unitTypes.size();i++){
 		if(unitTypes[i].getId() == id) {
             return &unitTypes[i];
 		}
     }
 
     printf("In [%s::%s Line: %d] scanning [%d] size = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,id,unitTypes.size());
-    for(int i=0; i<unitTypes.size();i++){
-    	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s][%d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str(),unitTypes[i].getId());
+    for(int i=0; i < (int)unitTypes.size();i++){
+    	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s][%d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName(false).c_str(),unitTypes[i].getId());
     }
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
-    for(int i=0; i<unitTypes.size();i++){
-    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str());
+    for(int i=0; i < (int)unitTypes.size();i++){
+    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName(false).c_str());
     }
 
 	throw megaglest_runtime_error("Unit type not found: [" + intToStr(id) + "] in faction type [" + this->name + "]");
 }
 
 const UpgradeType *FactionType::getUpgradeType(const string &name) const{
-    for(int i=0; i<upgradeTypes.size();i++){
+    for(int i=0; i < (int)upgradeTypes.size();i++){
 		if(upgradeTypes[i].getName()==name){
             return &upgradeTypes[i];
 		}
     }
 
     printf("In [%s::%s Line: %d] scanning [%s] size = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
-    for(int i=0; i<upgradeTypes.size();i++){
+    for(int i=0; i < (int)upgradeTypes.size();i++){
     	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,upgradeTypes[i].getName().c_str());
     }
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
-    for(int i=0; i<upgradeTypes.size();i++){
+    for(int i=0; i < (int)upgradeTypes.size();i++){
     	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,upgradeTypes[i].getName().c_str());
     }
 
@@ -838,7 +888,7 @@ const UpgradeType *FactionType::getUpgradeType(const string &name) const{
 }
 
 int FactionType::getStartingResourceAmount(const ResourceType *resourceType) const{
-	for(int i=0; i<startingResources.size(); ++i){
+	for(int i=0; i < (int)startingResources.size(); ++i){
 		if(startingResources[i].getType()==resourceType){
 			return startingResources[i].getAmount();
 		}
@@ -847,7 +897,7 @@ int FactionType::getStartingResourceAmount(const ResourceType *resourceType) con
 }
 
 void FactionType::deletePixels() {
-	for(int i = 0; i <unitTypes.size(); ++i) {
+	for(int i = 0; i < (int)unitTypes.size(); ++i) {
 		UnitType &unitType = unitTypes[i];
 		Texture2D *texture = unitType.getMeetingPointImage();
 		if(texture != NULL) {
@@ -858,29 +908,32 @@ void FactionType::deletePixels() {
 
 bool FactionType::factionUsesResourceType(const ResourceType *rt) const {
 	bool factionUsesResourceType = false;
-	for(unsigned int j = 0; factionUsesResourceType == false && j < this->getUnitTypeCount(); ++j) {
-		const UnitType *ut= this->getUnitType(j);
-		for(int k = 0; factionUsesResourceType == false && k < ut->getCostCount(); ++k) {
-			const Resource *costResource = ut->getCost(k);
-			//printf("#1 factionUsesResourceType, unit [%s] resource [%s] cost [%s]\n",ut->getName().c_str(),rt->getName().c_str(),costResource->getType()->getName().c_str());
+	if(rt != NULL) {
+		for(unsigned int j = 0; factionUsesResourceType == false && j < (unsigned int)this->getUnitTypeCount(); ++j) {
+			const UnitType *ut= this->getUnitType(j);
+			for(int k = 0; factionUsesResourceType == false && k < ut->getCostCount(); ++k) {
+				const Resource *costResource = ut->getCost(k);
+				//printf("#1 factionUsesResourceType, unit [%s] resource [%s] cost [%s]\n",ut->getName().c_str(),rt->getName().c_str(),costResource->getType()->getName().c_str());
 
-			if(costResource != NULL && costResource->getType()->getName() == rt->getName()) {
-				factionUsesResourceType = true;
-				break;
+				if(costResource != NULL && costResource->getType() != NULL &&
+						costResource->getType()->getName() == rt->getName()) {
+					factionUsesResourceType = true;
+					break;
+				}
 			}
-		}
-		if(factionUsesResourceType == false) {
-			for(unsigned int k = 0; factionUsesResourceType == false && k < ut->getCommandTypeCount(); ++k) {
-				const CommandType *commandType = ut->getCommandType(k);
-				if(commandType != NULL && commandType->getClass() == ccHarvest) {
-					const HarvestCommandType *hct = dynamic_cast<const HarvestCommandType *>(commandType);
-					if(hct != NULL && hct->getHarvestedResourceCount() > 0) {
-						for(unsigned int l = 0; factionUsesResourceType == false && l < hct->getHarvestedResourceCount(); ++l) {
-							//printf("#2 factionUsesResourceType, unit [%s] resource [%s] harvest [%s]\n",ut->getName().c_str(),rt->getName().c_str(),hct->getHarvestedResource(l)->getName().c_str());
+			if(factionUsesResourceType == false) {
+				for(unsigned int k = 0; factionUsesResourceType == false && k < (unsigned int)ut->getCommandTypeCount(); ++k) {
+					const CommandType *commandType = ut->getCommandType(k);
+					if(commandType != NULL && commandType->getClass() == ccHarvest) {
+						const HarvestCommandType *hct = dynamic_cast<const HarvestCommandType *>(commandType);
+						if(hct != NULL && hct->getHarvestedResourceCount() > 0) {
+							for(unsigned int l = 0; factionUsesResourceType == false && l < (unsigned int)hct->getHarvestedResourceCount(); ++l) {
+								//printf("#2 factionUsesResourceType, unit [%s] resource [%s] harvest [%s]\n",ut->getName().c_str(),rt->getName().c_str(),hct->getHarvestedResource(l)->getName().c_str());
 
-							if(hct->getHarvestedResource(l)->getName() == rt->getName()) {
-								factionUsesResourceType = true;
-								break;
+								if(hct->getHarvestedResource(l)->getName() == rt->getName()) {
+									factionUsesResourceType = true;
+									break;
+								}
 							}
 						}
 					}
@@ -888,7 +941,6 @@ bool FactionType::factionUsesResourceType(const ResourceType *rt) const {
 			}
 		}
 	}
-
 	return factionUsesResourceType;
 }
 
@@ -896,18 +948,25 @@ std::string FactionType::toString() const {
 	std::string result = "Faction Name: " + name + "\n";
 
 	result += "Unit Type List count = " + intToStr(this->getUnitTypeCount()) + "\n";
-    for(int i=0; i<unitTypes.size();i++) {
+    for(int i=0; i < (int)unitTypes.size();i++) {
 		result += unitTypes[i].toString() + "\n";
     }
 
 	result += "Upgrade Type List count = " + intToStr(this->getUpgradeTypeCount()) + "\n";
-    for(int i=0; i<upgradeTypes.size();i++) {
-		result += "index: " + intToStr(i) + " " + upgradeTypes[i].getReqDesc() + "\n";
+    for(int i=0; i < (int)upgradeTypes.size();i++) {
+		result += "index: " + intToStr(i) + " " + upgradeTypes[i].getReqDesc(false) + "\n";
     }
 
 	return result;
 }
 
+string FactionType::getName(bool translatedValue) const {
+	if(translatedValue == false) return name;
+
+	Lang &lang = Lang::getInstance();
+	return lang.getTechTreeString("FactionName_" + name,name.c_str());
+}
+
 void FactionType::saveGame(XmlNode *rootNode) {
 	std::map<string,string> mapTagReplacements;
 	XmlNode *factionTypeNode = rootNode->addChild("FactionType");
@@ -917,18 +976,18 @@ void FactionType::saveGame(XmlNode *rootNode) {
 //    UnitTypes unitTypes;
 	for(unsigned int i = 0; i < unitTypes.size(); ++i) {
 		XmlNode *unitTypesNode = factionTypeNode->addChild("unitTypes");
-		unitTypesNode->addAttribute("name",unitTypes[i].getName(), mapTagReplacements);
+		unitTypesNode->addAttribute("name",unitTypes[i].getName(false), mapTagReplacements);
 	}
 //    UpgradeTypes upgradeTypes;
 	for(unsigned int i = 0; i < upgradeTypes.size(); ++i) {
 		XmlNode *upgradeTypesNode = factionTypeNode->addChild("upgradeTypes");
-		upgradeTypesNode->addAttribute("name",upgradeTypes[i].getName(), mapTagReplacements);
+		upgradeTypesNode->addAttribute("name",upgradeTypes[i].getName(false), mapTagReplacements);
 	}
 
 //	StartingUnits startingUnits;
 	for(unsigned int i = 0; i < startingUnits.size(); ++i) {
 		XmlNode *startingUnitsNode = factionTypeNode->addChild("startingUnits");
-		startingUnitsNode->addAttribute("name",startingUnits[i].first->getName(), mapTagReplacements);
+		startingUnitsNode->addAttribute("name",startingUnits[i].first->getName(false), mapTagReplacements);
 		startingUnitsNode->addAttribute("count",intToStr(startingUnits[i].second), mapTagReplacements);
 	}
 
@@ -950,7 +1009,7 @@ void FactionType::saveGame(XmlNode *rootNode) {
 
 			XmlNode *mapAIBehaviorUnitCategoriesNode = factionTypeNode->addChild("mapAIBehaviorUnitCategories");
 			mapAIBehaviorUnitCategoriesNode->addAttribute("key",intToStr(iterMap->first), mapTagReplacements);
-			mapAIBehaviorUnitCategoriesNode->addAttribute("unitType",item.first->getName(), mapTagReplacements);
+			mapAIBehaviorUnitCategoriesNode->addAttribute("unitType",item.first->getName(false), mapTagReplacements);
 			mapAIBehaviorUnitCategoriesNode->addAttribute("count",intToStr(item.second), mapTagReplacements);
 		}
 	}
