@@ -100,8 +100,10 @@ public:
 	//init
 	FactionType();
     void load(const string &factionName, const TechTree *techTree, Checksum* checksum,
-    		Checksum *techtreeChecksum, std::map<string,vector<pair<string, string> > > &loadedFileList);
-	~FactionType();
+    		Checksum *techtreeChecksum,
+    		std::map<string,vector<pair<string, string> > > &loadedFileList,
+    		bool validationMode=false);
+	virtual ~FactionType();
 
 	const std::vector<FactionType::PairPUnitTypeInt> getAIBehaviorUnits(AIBehaviorUnitCategory category) const;
 	const std::vector<const UpgradeType*> getAIBehaviorUpgrades() const { return vctAIBehaviorUpgrades; };
@@ -109,13 +111,13 @@ public:
 
     //get
 	bool getIsLinked() const { return isLinked; }
-	int getUnitTypeCount() const						{return unitTypes.size();}
-	int getUpgradeTypeCount() const						{return upgradeTypes.size();}
-	string getName() const								{return name;}
+	int getUnitTypeCount() const						{return (int)unitTypes.size();}
+	int getUpgradeTypeCount() const						{return (int)upgradeTypes.size();}
+	virtual string getName(bool translatedValue=false) const;
 	const UnitType *getUnitType(int i) const			{return &unitTypes[i];}
 	const UpgradeType *getUpgradeType(int i) const		{return &upgradeTypes[i];}
 	StrSound *getMusic() const							{return music;}
-	int getStartingUnitCount() const					{return startingUnits.size();}
+	int getStartingUnitCount() const					{return (int)startingUnits.size();}
 	const UnitType *getStartingUnit(int i) const		{return startingUnits[i].first;}
 	int getStartingUnitAmount(int i) const				{return startingUnits[i].second;}
 
