@@ -68,6 +68,7 @@ enum SkillClass{
     scUpgrade,
 	scMorph,
 	scDie,
+	scFogOfWar,
 
     scCount
 };
@@ -105,9 +106,10 @@ public:
 	string name;
 
 	bool isAffected(const Unit *source, const Unit *dest) const;
-	virtual string getDesc() const;
+	virtual string getDesc(bool translatedValue) const;
 
 	virtual void saveGame(XmlNode *rootNode) const;
+	virtual void loadGame(const XmlNode *rootNode, Faction *faction, const SkillType *skillType);
 };
 
 class AnimationAttributes {
@@ -165,7 +167,7 @@ public:
     static void resetNextAttackBoostId() { nextAttackBoostId=0; }
 
     const AnimationAttributes getAnimationAttribute(int index) const;
-    int getAnimationCount() const { return animations.size(); }
+    int getAnimationCount() const { return (int)animations.size(); }
 
     //get
 	const string &getName() const		{return name;}
@@ -183,11 +185,11 @@ public:
 	//virtual string getDesc(const TotalUpgrade *totalUpgrade) const= 0;
 
 	//other
-	virtual string toString() const= 0;	
+	virtual string toString(bool translatedValue) const= 0;
 	virtual int getTotalSpeed(const TotalUpgrade *) const	{return speed;}
 	static string skillClassToStr(SkillClass skillClass); 
 	static string fieldToStr(Field field);
-	virtual string getBoostDesc() const {return attackBoost.getDesc();}
+	virtual string getBoostDesc(bool translatedValue) const {return attackBoost.getDesc(translatedValue);}
 
 	virtual void saveGame(XmlNode *rootNode);
 };
@@ -199,7 +201,7 @@ public:
 class StopSkillType: public SkillType{
 public:
     StopSkillType();
-    virtual string toString() const;
+    virtual string toString(bool translatedValue) const;
 };
 
 // ===============================
@@ -209,7 +211,7 @@ public:
 class MoveSkillType: public SkillType{
 public:
     MoveSkillType();
-    virtual string toString() const;
+    virtual string toString(bool translatedValue) const;
 
 	virtual int getTotalSpeed(const TotalUpgrade *totalUpgrade) const;
 };
@@ -244,7 +246,7 @@ public:
     virtual void load(const XmlNode *sn, const XmlNode *attackBoostsNode, const string &dir, const TechTree *tt,
     		const FactionType *ft, std::map<string,vector<pair<string, string> > > &loadedFileList,
     		string parentLoader);
-	virtual string toString() const;
+	virtual string toString(bool translatedValue) const;
     
 	//get
 	inline int getAttackStrength() const				{return attackStrength;}
@@ -282,7 +284,7 @@ public:
 class BuildSkillType: public SkillType{
 public:
     BuildSkillType();
-    virtual string toString() const;
+    virtual string toString(bool translatedValue) const;
 };
 
 // ===============================
@@ -292,7 +294,7 @@ public:
 class HarvestSkillType: public SkillType{
 public:
     HarvestSkillType();
-	virtual string toString() const;
+	virtual string toString(bool translatedValue) const;
 };
 
 // ===============================
@@ -302,7 +304,7 @@ public:
 class RepairSkillType: public SkillType{
 public:
     RepairSkillType();
-    virtual string toString() const;
+    virtual string toString(bool translatedValue) const;
 };
 
 // ===============================
@@ -319,7 +321,7 @@ public:
     			const FactionType *ft, std::map<string,vector<pair<string, string> > > &loadedFileList,
     			string parentLoader);
 
-    virtual string toString() const;
+    virtual string toString(bool translatedValue) const;
 
 	virtual int getTotalSpeed(const TotalUpgrade *totalUpgrade) const;
 
@@ -340,7 +342,7 @@ public:
     			const FactionType *ft, std::map<string,vector<pair<string, string> > > &loadedFileList,
     			string parentLoader);
 
-	virtual string toString() const;
+	virtual string toString(bool translatedValue) const;
 
 	virtual int getTotalSpeed(const TotalUpgrade *totalUpgrade) const;
 
@@ -363,7 +365,7 @@ public:
     virtual void load(const XmlNode *sn, const XmlNode *attackBoostsNode, const string &dir, const TechTree *tt,
     			const FactionType *ft, std::map<string,vector<pair<string, string> > > &loadedFileList,
     			string parentLoader);
-    virtual string toString() const;
+    virtual string toString(bool translatedValue) const;
 
     virtual void saveGame(XmlNode *rootNode);
 };
@@ -384,7 +386,7 @@ public:
     			const FactionType *ft, std::map<string,vector<pair<string, string> > > &loadedFileList,
     			string parentLoader);
 
-    virtual string toString() const;
+    virtual string toString(bool translatedValue) const;
 	virtual int getTotalSpeed(const TotalUpgrade *totalUpgrade) const;
 
 	virtual void saveGame(XmlNode *rootNode);
@@ -394,7 +396,7 @@ public:
 // 	class DieSkillType  
 // ===============================
 
-class DieSkillType: public SkillType{
+class DieSkillType: public SkillType {
 private:
 	bool fade;
 
@@ -405,7 +407,31 @@ public:
 	virtual void load(const XmlNode *sn, const XmlNode *attackBoostsNode, const string &dir, const TechTree *tt,
 			const FactionType *ft, std::map<string,vector<pair<string, string> > > &loadedFileList,
 			string parentLoader);
-	virtual string toString() const;
+	virtual string toString(bool translatedValue) const;
+
+	virtual void saveGame(XmlNode *rootNode);
+};
+
+// ===============================
+// 	class FogOfWarSkillType
+// ===============================
+
+class FogOfWarSkillType: public SkillType {
+private:
+	bool fowEnable;
+	bool applyToTeam;
+	float durationTime;
+
+public:
+	FogOfWarSkillType();
+    bool getFowEnable() const	{return fowEnable;}
+    bool getApplyToTeam() const	{return applyToTeam;}
+    float getDurationTime() const	{return durationTime;}
+
+	virtual void load(const XmlNode *sn, const XmlNode *attackBoostsNode, const string &dir, const TechTree *tt,
+			const FactionType *ft, std::map<string,vector<pair<string, string> > > &loadedFileList,
+			string parentLoader);
+	virtual string toString(bool translatedValue) const;
 
 	virtual void saveGame(XmlNode *rootNode);
 };
@@ -414,7 +440,7 @@ public:
 // 	class SkillFactory  
 // ===============================
 
-class SkillTypeFactory: public MultiFactory<SkillType>{
+class SkillTypeFactory: public MultiFactory<SkillType> {
 private:
 	SkillTypeFactory();
 public:
