@@ -56,7 +56,7 @@ public:
 	}
 	void init(string name, int kills);
 
-	const string &getName() const	{return name;}
+	string getName(bool translatedValue=false) const;
 	int getKills() const			{return kills;}
 
 	void saveGame(XmlNode *rootNode) const ;
@@ -122,6 +122,7 @@ private:
     bool multiSelect;
     int sight;
     int size;							//size in cells
+    int renderSize;						//size to render in cells
     int height;
     float rotatedBuildPos;
     bool rotationAllowed;
@@ -156,14 +157,19 @@ private:
 
     UnitCountsInVictoryConditions countInVictoryConditions;
 
+    static auto_ptr<CommandType> ctHarvestEmergencyReturnCommandType;
+
 public:
 	//creation and loading
     UnitType();
     virtual ~UnitType();
 	void preLoad(const string &dir);
-    void loaddd(int id, const string &dir, const TechTree *techTree,const string &techTreePath,
+    void loaddd(int id, const string &dir, const TechTree *techTree,
+    		const string &techTreePath,
     		const FactionType *factionType, Checksum* checksum,
-    		Checksum* techtreeChecksum, std::map<string,vector<pair<string, string> > > &loadedFileList);
+    		Checksum* techtreeChecksum,
+    		std::map<string,vector<pair<string, string> > > &loadedFileList,
+    		bool validationMode=false);
 
     virtual string getName(bool translatedValue=false) const;
 
@@ -184,17 +190,18 @@ public:
 	const CommandType *getCommandType(int i) const;
 	inline const Level *getLevel(int i) const					{return &levels[i];}
 	const Level *getLevel(string name) const;
-	inline int getSkillTypeCount() const						{return skillTypes.size();}
-	inline int getCommandTypeCount() const						{return commandTypes.size();}
-	inline int getLevelCount() const							{return levels.size();}
+	inline int getSkillTypeCount() const						{return (int)skillTypes.size();}
+	inline int getCommandTypeCount() const						{return (int)commandTypes.size();}
+	inline int getLevelCount() const							{return (int)levels.size();}
 	inline bool getLight() const								{return light;}
 	inline bool getRotationAllowed() const						{return rotationAllowed;}
 	inline Vec3f getLightColor() const							{return lightColor;}
 	inline bool getMultiSelect() const							{return multiSelect;}
 	inline int getSight() const								{return sight;}
 	inline int getSize() const									{return size;}
+	inline int getRenderSize() const								{return renderSize;}
 	int getHeight() const								{return height;}
-	int getStoredResourceCount() const					{return storedResources.size();}
+	int getStoredResourceCount() const					{return (int)storedResources.size();}
 	inline const Resource *getStoredResource(int i) const		{return &storedResources[i];}
 	bool getCellMapCell(int x, int y, CardinalDir facing) const;
 	inline bool getMeetingPoint() const						{return meetingPoint;}
@@ -215,6 +222,7 @@ public:
 	const SkillType *getFirstStOfClass(SkillClass skillClass) const;
     const CommandType *getFirstCtOfClass(CommandClass commandClass) const;
     const HarvestCommandType *getFirstHarvestCommand(const ResourceType *resourceType,const Faction *faction) const;
+    const HarvestEmergencyReturnCommandType *getFirstHarvestEmergencyReturnCommand() const;
 	const AttackCommandType *getFirstAttackCommand(Field field) const;
 	const AttackStoppedCommandType *getFirstAttackStoppedCommand(Field field) const;
 	const RepairCommandType *getFirstRepairCommand(const UnitType *repaired) const;
@@ -250,7 +258,7 @@ public:
 	inline void setRotatedBuildPos(float value) { rotatedBuildPos = value; }
 
 	//other
-    virtual string getReqDesc() const;
+    virtual string getReqDesc(bool translatedValue) const;
 
     std::string toString() const;
 
