@@ -45,81 +45,129 @@ void UpgradeTypeBase::load(const XmlNode *upgradeNode, string upgradename) {
 	this->upgradename = upgradename;
 	//values
 	maxHpIsMultiplier = false;
-	maxHp= upgradeNode->getChild("max-hp")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("max-hp")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		maxHpIsMultiplier = upgradeNode->getChild("max-hp")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+	if(upgradeNode->hasChild("max-hp") == true) {
+		maxHp = upgradeNode->getChild("max-hp")->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChild("max-hp")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			maxHpIsMultiplier = upgradeNode->getChild("max-hp")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found maxHpIsMultiplier = %d\n",maxHpIsMultiplier);
+			//printf("Found maxHpIsMultiplier = %d\n",maxHpIsMultiplier);
+		}
+	}
+	else {
+		maxHp = 0;
 	}
 	maxHpRegeneration = 0;
 	//maxHpRegenerationIsMultiplier = false;
-	if(upgradeNode->getChild("max-hp")->getAttribute(VALUE_REGEN_KEY_NAME,false) != NULL) {
-		maxHpRegeneration = upgradeNode->getChild("max-hp")->getAttribute(VALUE_REGEN_KEY_NAME)->getIntValue();
+	if(upgradeNode->hasChild("max-hp") == true) {
+		if(upgradeNode->getChild("max-hp")->getAttribute(VALUE_REGEN_KEY_NAME,false) != NULL) {
+			maxHpRegeneration = upgradeNode->getChild("max-hp")->getAttribute(VALUE_REGEN_KEY_NAME)->getIntValue();
 
-		//printf("Found maxHpIsMultiplier = %d\n",maxHpIsMultiplier);
+			//printf("Found maxHpIsMultiplier = %d\n",maxHpIsMultiplier);
+		}
 	}
 
 	maxEpIsMultiplier = false;
-	maxEp= upgradeNode->getChild("max-ep")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("max-ep")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		maxEpIsMultiplier = upgradeNode->getChild("max-ep")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+	if(upgradeNode->hasChild("max-ep") == true) {
+		maxEp = upgradeNode->getChild("max-ep")->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChild("max-ep")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			maxEpIsMultiplier = upgradeNode->getChild("max-ep")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found maxEpIsMultiplier = %d\n",maxEpIsMultiplier);
+			//printf("Found maxEpIsMultiplier = %d\n",maxEpIsMultiplier);
+		}
+	}
+	else {
+		maxEp = 0;
 	}
 	maxEpRegeneration = 0;
 	//maxEpRegenerationIsMultiplier = false;
-	if(upgradeNode->getChild("max-ep")->getAttribute(VALUE_REGEN_KEY_NAME,false) != NULL) {
-		maxEpRegeneration = upgradeNode->getChild("max-ep")->getAttribute(VALUE_REGEN_KEY_NAME)->getIntValue();
+	if(upgradeNode->hasChild("max-ep") == true) {
+		if(upgradeNode->getChild("max-ep")->getAttribute(VALUE_REGEN_KEY_NAME,false) != NULL) {
+			maxEpRegeneration = upgradeNode->getChild("max-ep")->getAttribute(VALUE_REGEN_KEY_NAME)->getIntValue();
 
-		//printf("Found maxHpIsMultiplier = %d\n",maxHpIsMultiplier);
+			//printf("Found maxHpIsMultiplier = %d\n",maxHpIsMultiplier);
+		}
 	}
 
 	sightIsMultiplier = false;
-	sight= upgradeNode->getChild("sight")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("sight")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		sightIsMultiplier = upgradeNode->getChild("sight")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+	if(upgradeNode->hasChild("sight") == true) {
+		sight= upgradeNode->getChild("sight")->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChild("sight")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			sightIsMultiplier = upgradeNode->getChild("sight")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found sightIsMultiplier = %d\n",sightIsMultiplier);
+			//printf("Found sightIsMultiplier = %d\n",sightIsMultiplier);
+		}
+	}
+	else {
+		sight = 0;
 	}
 
 	attackStrengthIsMultiplier = false;
-	attackStrength= upgradeNode->getChild("attack-strenght")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("attack-strenght")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		attackStrengthIsMultiplier = upgradeNode->getChild("attack-strenght")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found attackStrengthIsMultiplier = %d\n",attackStrengthIsMultiplier);
+	std::vector<string> attackStrengthXMLTags;
+	attackStrengthXMLTags.push_back("attack-strenght");
+	attackStrengthXMLTags.push_back("attack-strength");
+	if(upgradeNode->hasChildWithAliases(attackStrengthXMLTags) == true) {
+		attackStrength= upgradeNode->getChildWithAliases(attackStrengthXMLTags)->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChildWithAliases(attackStrengthXMLTags)->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			attackStrengthIsMultiplier = upgradeNode->getChildWithAliases(attackStrengthXMLTags)->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+
+			//printf("Found attackStrengthIsMultiplier = %d\n",attackStrengthIsMultiplier);
+		}
+	}
+	else {
+		attackStrength = 0;
 	}
 
 	attackRangeIsMultiplier = false;
-	attackRange= upgradeNode->getChild("attack-range")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("attack-range")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		attackRangeIsMultiplier = upgradeNode->getChild("attack-range")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+	if(upgradeNode->hasChild("attack-range") == true) {
+		attackRange= upgradeNode->getChild("attack-range")->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChild("attack-range")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			attackRangeIsMultiplier = upgradeNode->getChild("attack-range")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found attackRangeIsMultiplier = %d\n",attackRangeIsMultiplier);
+			//printf("Found attackRangeIsMultiplier = %d\n",attackRangeIsMultiplier);
+		}
+	}
+	else {
+		attackRange = 0;
 	}
 
 	armorIsMultiplier = false;
-	armor= upgradeNode->getChild("armor")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("armor")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		armorIsMultiplier = upgradeNode->getChild("armor")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+	if(upgradeNode->hasChild("armor") == true) {
+		armor= upgradeNode->getChild("armor")->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChild("armor")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			armorIsMultiplier = upgradeNode->getChild("armor")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found armorIsMultiplier = %d\n",armorIsMultiplier);
+			//printf("Found armorIsMultiplier = %d\n",armorIsMultiplier);
+		}
+	}
+	else {
+		armor = 0;
 	}
 
 	moveSpeedIsMultiplier = false;
-	moveSpeed= upgradeNode->getChild("move-speed")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("move-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		moveSpeedIsMultiplier = upgradeNode->getChild("move-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+	if(upgradeNode->hasChild("move-speed") == true) {
+		moveSpeed= upgradeNode->getChild("move-speed")->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChild("move-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			moveSpeedIsMultiplier = upgradeNode->getChild("move-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found moveSpeedIsMultiplier = %d\n",moveSpeedIsMultiplier);
+			//printf("Found moveSpeedIsMultiplier = %d\n",moveSpeedIsMultiplier);
+		}
+	}
+	else {
+		moveSpeed= 0;
 	}
 
 	prodSpeedIsMultiplier = false;
-	prodSpeed= upgradeNode->getChild("production-speed")->getAttribute("value")->getIntValue();
-	if(upgradeNode->getChild("production-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
-		prodSpeedIsMultiplier = upgradeNode->getChild("production-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
+	if(upgradeNode->hasChild("production-speed") == true) {
+		prodSpeed= upgradeNode->getChild("production-speed")->getAttribute("value")->getIntValue();
+		if(upgradeNode->getChild("production-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME,false) != NULL) {
+			prodSpeedIsMultiplier = upgradeNode->getChild("production-speed")->getAttribute(VALUE_PERCENT_MULTIPLIER_KEY_NAME)->getBoolValue();
 
-		//printf("Found prodSpeedIsMultiplier = %d\n",prodSpeedIsMultiplier);
+			//printf("Found prodSpeedIsMultiplier = %d\n",prodSpeedIsMultiplier);
+		}
+	}
+	else {
+		prodSpeed = 0;
 	}
 }
 
@@ -193,7 +241,7 @@ int UpgradeTypeBase::getProdSpeed(const SkillType *st) const {
 	}
 }
 
-string UpgradeTypeBase::getDesc() const{
+string UpgradeTypeBase::getDesc(bool translatedValue) const{
 
     string str="";
     string indent="->";
@@ -202,75 +250,106 @@ string UpgradeTypeBase::getDesc() const{
 
 	if(maxHp != 0) {
 		if(maxHpIsMultiplier) {
-			str += indent+lang.get("Hp") + " *" + intToStr(maxHp);
+			str += indent+lang.getString("Hp",(translatedValue == true ? "" : "english")) + " *" + intToStr(maxHp);
 		}
 		else {
-			str += indent+lang.get("Hp") + " +" + intToStr(maxHp);
+			str += indent+lang.getString("Hp",(translatedValue == true ? "" : "english")) + " +" + intToStr(maxHp);
 		}
 
 		if(maxHpRegeneration != 0) {
 			str += " [" + intToStr(maxHpRegeneration) + "]";
 		}
 	}
+
 	if(sight != 0) {
+		if(str != "") {
+			str += "\n";
+		}
 		if(sightIsMultiplier) {
-			str+= indent+lang.get("Sight") + " *" + intToStr(sight);
+			str+= indent+lang.getString("Sight",(translatedValue == true ? "" : "english")) + " *" + intToStr(sight);
 		}
 		else {
-			str+= indent+lang.get("Sight") + " +" + intToStr(sight);
+			str+= indent+lang.getString("Sight",(translatedValue == true ? "" : "english")) + " +" + intToStr(sight);
 		}
 	}
 	if(maxEp != 0) {
+		if(str != "") {
+			str += "\n";
+		}
+
 		if(maxEpIsMultiplier) {
-			str+= indent+lang.get("Ep") + " *" + intToStr(maxEp)+"\n";
+			str+= indent+lang.getString("Ep",(translatedValue == true ? "" : "english")) + " *" + intToStr(maxEp);
 		}
 		else {
-			str+= indent+lang.get("Ep") + " +" + intToStr(maxEp)+"\n";
+			str+= indent+lang.getString("Ep",(translatedValue == true ? "" : "english")) + " +" + intToStr(maxEp);
 		}
 		if(maxEpRegeneration != 0) {
 			str += " [" + intToStr(maxEpRegeneration) + "]";
 		}
 	}
 	if(attackStrength != 0) {
+		if(str != "") {
+			str += "\n";
+		}
+
 		if(attackStrengthIsMultiplier) {
-			str+= indent+lang.get("AttackStrenght") + " *" + intToStr(attackStrength)+"\n";
+			str+= indent+lang.getString("AttackStrenght",(translatedValue == true ? "" : "english")) + " *" + intToStr(attackStrength);
 		}
 		else {
-			str+= indent+lang.get("AttackStrenght") + " +" + intToStr(attackStrength)+"\n";
+			str+= indent+lang.getString("AttackStrenght",(translatedValue == true ? "" : "english")) + " +" + intToStr(attackStrength);
 		}
 	}
 	if(attackRange != 0) {
+		if(str != "") {
+			str += "\n";
+		}
+
 		if(attackRangeIsMultiplier) {
-			str+= indent+lang.get("AttackDistance") + " *" + intToStr(attackRange)+"\n";
+			str+= indent+lang.getString("AttackDistance",(translatedValue == true ? "" : "english")) + " *" + intToStr(attackRange);
 		}
 		else {
-			str+= indent+lang.get("AttackDistance") + " +" + intToStr(attackRange)+"\n";
+			str+= indent+lang.getString("AttackDistance",(translatedValue == true ? "" : "english")) + " +" + intToStr(attackRange);
 		}
 	}
 	if(armor != 0) {
+		if(str != "") {
+			str += "\n";
+		}
+
 		if(armorIsMultiplier) {
-			str+= indent+lang.get("Armor") + " *" + intToStr(armor)+"\n";
+			str+= indent+lang.getString("Armor",(translatedValue == true ? "" : "english")) + " *" + intToStr(armor);
 		}
 		else {
-			str+= indent+lang.get("Armor") + " +" + intToStr(armor)+"\n";
+			str+= indent+lang.getString("Armor",(translatedValue == true ? "" : "english")) + " +" + intToStr(armor);
 		}
 	}
 	if(moveSpeed != 0) {
+		if(str != "") {
+			str += "\n";
+		}
+
 		if(moveSpeedIsMultiplier) {
-			str+= indent+lang.get("WalkSpeed") + " *" + intToStr(moveSpeed)+"\n";
+			str+= indent+lang.getString("WalkSpeed",(translatedValue == true ? "" : "english")) + " *" + intToStr(moveSpeed);
 		}
 		else {
-			str+= indent+lang.get("WalkSpeed") + " +" + intToStr(moveSpeed)+"\n";
+			str+= indent+lang.getString("WalkSpeed",(translatedValue == true ? "" : "english")) + " +" + intToStr(moveSpeed);
 		}
 	}
 	if(prodSpeed != 0) {
+		if(str != "") {
+			str += "\n";
+		}
+
 		if(prodSpeedIsMultiplier) {
-			str+= indent+lang.get("ProductionSpeed") + " *" + intToStr(prodSpeed)+"\n";
+			str+= indent+lang.getString("ProductionSpeed",(translatedValue == true ? "" : "english")) + " *" + intToStr(prodSpeed);
 		}
 		else {
-			str+= indent+lang.get("ProductionSpeed") + " +" + intToStr(prodSpeed)+"\n";
+			str+= indent+lang.getString("ProductionSpeed",(translatedValue == true ? "" : "english")) + " +" + intToStr(prodSpeed);
 		}
 	}
+	if(str != "") {
+		str += "\n";
+	}
 
     return str;
 }
@@ -483,18 +562,18 @@ string UpgradeType::getName(bool translatedValue) const {
 	return lang.getTechTreeString("UpgradeTypeName_" + name,name.c_str());
 }
 
-string UpgradeType::getReqDesc() const{
+string UpgradeType::getReqDesc(bool translatedValue) const{
 	Lang &lang= Lang::getInstance();
-    string str= ProducibleType::getReqDesc();
+    string str= ProducibleType::getReqDesc(translatedValue);
     string indent="  ";
 	if(getEffectCount()>0){
-		str+= "\n"+ lang.get("Upgrades")+"\n";
+		str+= "\n"+ lang.getString("Upgrades",(translatedValue == true ? "" : "english"))+"\n";
 	}
-	str+=UpgradeTypeBase::getDesc();
+	str+=UpgradeTypeBase::getDesc(translatedValue);
 	if(getEffectCount()>0){
-		str+= lang.get("AffectedUnits")+"\n";
+		str+= lang.getString("AffectedUnits",(translatedValue == true ? "" : "english"))+"\n";
 		for(int i=0; i<getEffectCount(); ++i){
-					str+= indent+getEffect(i)->getName(true)+"\n";
+			str+= indent+getEffect(i)->getName(translatedValue)+"\n";
 		}
 	}
 	return str;
@@ -506,11 +585,13 @@ void UpgradeType::preLoad(const string &dir){
 
 void UpgradeType::load(const string &dir, const TechTree *techTree,
 		const FactionType *factionType, Checksum* checksum,
-		Checksum* techtreeChecksum, std::map<string,vector<pair<string, string> > > &loadedFileList) {
+		Checksum* techtreeChecksum, std::map<string,
+		vector<pair<string, string> > > &loadedFileList,
+		bool validationMode) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	char szBuf[8096]="";
-	snprintf(szBuf,8096,Lang::getInstance().get("LogScreenGameLoadingUpgradeType","",true).c_str(),formatString(name).c_str());
+	snprintf(szBuf,8096,Lang::getInstance().getString("LogScreenGameLoadingUpgradeType","",true).c_str(),formatString(this->getName(true)).c_str());
 	Logger::getInstance().add(szBuf, true);
 
 	string currentPath = dir;
@@ -530,28 +611,10 @@ void UpgradeType::load(const string &dir, const TechTree *techTree,
 		const XmlNode *upgradeNode= xmlTree.getRootNode();
 
 		//image
-		const XmlNode *imageNode= upgradeNode->getChild("image");
-		image= Renderer::getInstance().newTexture2D(rsGame);
-		if(image) {
-			image->load(imageNode->getAttribute("path")->getRestrictedValue(currentPath,true));
-		}
-		loadedFileList[imageNode->getAttribute("path")->getRestrictedValue(currentPath,true)].push_back(make_pair(sourceXMLFile,imageNode->getAttribute("path")->getRestrictedValue()));
-
-		//if(fileExists(imageNode->getAttribute("path")->getRestrictedValue(currentPath,true)) == false) {
-		//	printf("\n***ERROR MISSING FILE [%s]\n",imageNode->getAttribute("path")->getRestrictedValue(currentPath,true).c_str());
-		//}
+		image = NULL; // Not used for upgrade types
 
 		//image cancel
-		const XmlNode *imageCancelNode= upgradeNode->getChild("image-cancel");
-		cancelImage= Renderer::getInstance().newTexture2D(rsGame);
-		if(cancelImage) {
-			cancelImage->load(imageCancelNode->getAttribute("path")->getRestrictedValue(currentPath,true));
-		}
-		loadedFileList[imageCancelNode->getAttribute("path")->getRestrictedValue(currentPath,true)].push_back(make_pair(sourceXMLFile,imageCancelNode->getAttribute("path")->getRestrictedValue()));
-
-		//if(fileExists(imageCancelNode->getAttribute("path")->getRestrictedValue(currentPath,true)) == false) {
-		//	printf("\n***ERROR MISSING FILE [%s]\n",imageCancelNode->getAttribute("path")->getRestrictedValue(currentPath,true).c_str());
-		//}
+		cancelImage = NULL; // Not used for upgrade types
 
 		//upgrade time
 		const XmlNode *upgradeTimeNode= upgradeNode->getChild("time");
@@ -562,7 +625,7 @@ void UpgradeType::load(const string &dir, const TechTree *techTree,
 		//unit requirements
 		bool hasDup = false;
 		const XmlNode *unitRequirementsNode= upgradeNode->getChild("unit-requirements");
-		for(int i = 0; i < unitRequirementsNode->getChildCount(); ++i) {
+		for(int i = 0; i < (int)unitRequirementsNode->getChildCount(); ++i) {
 			const XmlNode *unitNode= 	unitRequirementsNode->getChild("unit", i);
 			string name= unitNode->getAttribute("name")->getRestrictedValue();
 
@@ -586,7 +649,7 @@ void UpgradeType::load(const string &dir, const TechTree *techTree,
 
 		//upgrade requirements
 		const XmlNode *upgradeRequirementsNode= upgradeNode->getChild("upgrade-requirements");
-		for(int i = 0; i < upgradeRequirementsNode->getChildCount(); ++i) {
+		for(int i = 0; i < (int)upgradeRequirementsNode->getChildCount(); ++i) {
 			const XmlNode *upgradeReqNode= upgradeRequirementsNode->getChild("upgrade", i);
 			string name= upgradeReqNode->getAttribute("name")->getRestrictedValue();
 
@@ -613,7 +676,7 @@ void UpgradeType::load(const string &dir, const TechTree *techTree,
 		const XmlNode *resourceRequirementsNode= upgradeNode->getChild("resource-requirements");
 
 		costs.resize(resourceRequirementsNode->getChildCount());
-		for(int i = 0; i < costs.size(); ++i) {
+		for(int i = 0; i < (int)costs.size(); ++i) {
 			const XmlNode *resourceNode= 	resourceRequirementsNode->getChild("resource", i);
 			string name= resourceNode->getAttribute("name")->getRestrictedValue();
 			int amount= resourceNode->getAttribute("amount")->getIntValue();
@@ -638,15 +701,26 @@ void UpgradeType::load(const string &dir, const TechTree *techTree,
 		index = 0;
 		for(std::map<string,int>::iterator iterMap = sortedItems.begin();
 				iterMap != sortedItems.end(); ++iterMap) {
-			costs[index].init(techTree->getResourceType(iterMap->first), iterMap->second);
-			index++;
+			try {
+				costs[index].init(techTree->getResourceType(iterMap->first), iterMap->second);
+				index++;
+			}
+			catch(megaglest_runtime_error& ex) {
+				if(validationMode == false) {
+					throw;
+				}
+				else {
+					SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\nFor UpgradeType: %s Cost: %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what(),name.c_str(),iterMap->second);
+				}
+			}
+
 		}
 		sortedItems.clear();
 		hasDup = false;
 
 		//effects
 		const XmlNode *effectsNode= upgradeNode->getChild("effects");
-		for(int i = 0; i < effectsNode->getChildCount(); ++i) {
+		for(int i = 0; i < (int)effectsNode->getChildCount(); ++i) {
 			const XmlNode *unitNode= effectsNode->getChild("unit", i);
 			string name= unitNode->getAttribute("name")->getRestrictedValue();
 
@@ -777,6 +851,8 @@ void TotalUpgrade::sum(const UpgradeTypeBase *ut, const Unit *unit) {
 
 	if(ut->getMaxEpIsMultiplier() == true) {
 		maxEp += ((double)unit->getEp() * ((double)ut->getMaxEp() / (double)100));
+		// TODO: SoftCoder - Fix the bug below on next major release
+		// if(ut->getMaxEpRegeneration() != 0) {
 		if(ut->getMaxHpRegeneration() != 0) {
 			maxEpRegeneration += ((double)unit->getType()->getEpRegeneration() + ((double)max(maxEp,unit->getEp()) * ((double)ut->getMaxEpRegeneration() / (double)100)));
 		}
@@ -803,7 +879,7 @@ void TotalUpgrade::sum(const UpgradeTypeBase *ut, const Unit *unit) {
 	}
 
 	if(ut->getAttackStrengthIsMultiplier() == true) {
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const AttackSkillType *ast = dynamic_cast<const AttackSkillType *>(skillType);
 			if(ast != NULL) {
@@ -816,7 +892,7 @@ void TotalUpgrade::sum(const UpgradeTypeBase *ut, const Unit *unit) {
 	}
 
 	if(ut->getAttackRangeIsMultiplier() == true) {
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const AttackSkillType *ast = dynamic_cast<const AttackSkillType *>(skillType);
 			if(ast != NULL) {
@@ -831,7 +907,7 @@ void TotalUpgrade::sum(const UpgradeTypeBase *ut, const Unit *unit) {
 	if(ut->getMoveSpeedIsMultiplier() == true) {
 		//printf("BEFORE Applying moveSpeedIsMultiplier\n");
 
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const MoveSkillType *mst = dynamic_cast<const MoveSkillType *>(skillType);
 			if(mst != NULL) {
@@ -848,7 +924,7 @@ void TotalUpgrade::sum(const UpgradeTypeBase *ut, const Unit *unit) {
 	}
 
 	if(ut->getProdSpeedIsMultiplier() == true) {
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const ProduceSkillType *pst = dynamic_cast<const ProduceSkillType *>(skillType);
 			if(pst != NULL) {
@@ -884,10 +960,13 @@ void TotalUpgrade::deapply(const UpgradeTypeBase *ut,const Unit *unit) {
 	prodSpeedIsMultiplier		= ut->getProdSpeedIsMultiplier();
 
 	if(ut->getMaxHpIsMultiplier() == true) {
+		//printf("#1 Maxhp maxHp = %d, unit->getHp() = %d ut->getMaxHp() = %d\n",maxHp,unit->getHp(),ut->getMaxHp());
 		maxHp -= ((double)unit->getHp() * ((double)ut->getMaxHp() / (double)100));
+
 		if(ut->getMaxHpRegeneration() != 0) {
 			maxHpRegeneration -= ((double)unit->getType()->getHpRegeneration() + ((double)max(maxHp,unit->getHp()) * ((double)ut->getMaxHpRegeneration() / (double)100)));
 		}
+		//printf("#1.1 Maxhp maxHp = %d, unit->getHp() = %d ut->getMaxHp() = %d\n",maxHp,unit->getHp(),ut->getMaxHp());
 	}
 	else {
 		maxHp -= ut->getMaxHp();
@@ -895,11 +974,13 @@ void TotalUpgrade::deapply(const UpgradeTypeBase *ut,const Unit *unit) {
 			maxHpRegeneration -= ut->getMaxHpRegeneration();
 		}
 	}
+	enforceMinimumValue(0,maxHp);
+	enforceMinimumValue(0,maxHpRegeneration);
 
 	if(ut->getMaxEpIsMultiplier() == true) {
 		maxEp -= ((double)unit->getEp() * ((double)ut->getMaxEp() / (double)100));
 		if(ut->getMaxEpRegeneration() != 0) {
-			maxEpRegeneration += ((double)unit->getType()->getEpRegeneration() + ((double)max(maxEp,unit->getEp()) * ((double)ut->getMaxEpRegeneration() / (double)100)));
+			maxEpRegeneration -= ((double)unit->getType()->getEpRegeneration() + ((double)max(maxEp,unit->getEp()) * ((double)ut->getMaxEpRegeneration() / (double)100)));
 		}
 	}
 	else {
@@ -908,6 +989,8 @@ void TotalUpgrade::deapply(const UpgradeTypeBase *ut,const Unit *unit) {
 			maxEpRegeneration -= ut->getMaxEpRegeneration();
 		}
 	}
+	enforceMinimumValue(0,maxEp);
+	enforceMinimumValue(0,maxEpRegeneration);
 
 	if(ut->getSightIsMultiplier() == true) {
 		sight -= ((double)unit->getType()->getSight() * ((double)ut->getSight() / (double)100));
@@ -915,6 +998,7 @@ void TotalUpgrade::deapply(const UpgradeTypeBase *ut,const Unit *unit) {
 	else {
 		sight -= ut->getSight();
 	}
+	enforceMinimumValue(0,sight);
 
 	if(ut->getArmorIsMultiplier() == true) {
 		armor -= ((double)unit->getType()->getArmor() * ((double)ut->getArmor() / (double)100));
@@ -922,41 +1006,48 @@ void TotalUpgrade::deapply(const UpgradeTypeBase *ut,const Unit *unit) {
 	else {
 		armor -= ut->getArmor();
 	}
+	enforceMinimumValue(0,armor);
 
 	if(ut->getAttackStrengthIsMultiplier() == true) {
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const AttackSkillType *ast = dynamic_cast<const AttackSkillType *>(skillType);
 			if(ast != NULL) {
 				attackStrengthMultiplierValueList[ast->getName()] -= ((double)ast->getAttackStrength() * ((double)ut->getAttackStrength(NULL) / (double)100));
+				enforceMinimumValue(0,attackStrengthMultiplierValueList[ast->getName()]);
 			}
 		}
 	}
 	else {
 		attackStrength -= ut->getAttackStrength(NULL);
+		enforceMinimumValue(0,attackStrength);
 	}
 
+
 	if(ut->getAttackRangeIsMultiplier() == true) {
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const AttackSkillType *ast = dynamic_cast<const AttackSkillType *>(skillType);
 			if(ast != NULL) {
 				attackRangeMultiplierValueList[ast->getName()] -= ((double)ast->getAttackRange() * ((double)ut->getAttackRange(NULL) / (double)100));
+				enforceMinimumValue(0,attackRangeMultiplierValueList[ast->getName()]);
 			}
 		}
 	}
 	else {
 		attackRange -= ut->getAttackRange(NULL);
+		enforceMinimumValue(0,attackRange);
 	}
 
 	if(ut->getMoveSpeedIsMultiplier() == true) {
 		//printf("BEFORE Applying moveSpeedIsMultiplier, moveSpeed = %d, ut->getMoveSpeed() = %d\n",moveSpeed,ut->getMoveSpeed());
 
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const MoveSkillType *mst = dynamic_cast<const MoveSkillType *>(skillType);
 			if(mst != NULL) {
 				moveSpeedIsMultiplierValueList[mst->getName()] -= ((double)mst->getSpeed() * ((double)ut->getMoveSpeed(NULL) / (double)100));
+				enforceMinimumValue(0,moveSpeedIsMultiplierValueList[mst->getName()]);
 			}
 		}
 
@@ -964,27 +1055,32 @@ void TotalUpgrade::deapply(const UpgradeTypeBase *ut,const Unit *unit) {
 	}
 	else {
 		moveSpeed -= ut->getMoveSpeed(NULL);
+		enforceMinimumValue(0,moveSpeed);
 	}
 
 	if(ut->getProdSpeedIsMultiplier() == true) {
-		for(unsigned int i = 0; i < unit->getType()->getSkillTypeCount(); ++i) {
+		for(unsigned int i = 0; i < (unsigned int)unit->getType()->getSkillTypeCount(); ++i) {
 			const SkillType *skillType = unit->getType()->getSkillType(i);
 			const ProduceSkillType *pst = dynamic_cast<const ProduceSkillType *>(skillType);
 			if(pst != NULL) {
 				prodSpeedProduceIsMultiplierValueList[pst->getName()] -= ((double)pst->getSpeed() * ((double)ut->getProdSpeed(NULL) / (double)100));
+				enforceMinimumValue(0,prodSpeedProduceIsMultiplierValueList[pst->getName()]);
 			}
 			const UpgradeSkillType *ust = dynamic_cast<const UpgradeSkillType *>(skillType);
 			if(ust != NULL) {
 				prodSpeedUpgradeIsMultiplierValueList[ust->getName()] -= ((double)ust->getSpeed() * ((double)ut->getProdSpeed(NULL) / (double)100));
+				enforceMinimumValue(0,prodSpeedUpgradeIsMultiplierValueList[ust->getName()]);
 			}
 			const MorphSkillType *mst = dynamic_cast<const MorphSkillType *>(skillType);
 			if(mst != NULL) {
 				prodSpeedMorphIsMultiplierValueList[mst->getName()] -= ((double)mst->getSpeed() * ((double)ut->getProdSpeed(NULL) / (double)100));
+				enforceMinimumValue(0,prodSpeedMorphIsMultiplierValueList[mst->getName()]);
 			}
 		}
 	}
 	else {
 		prodSpeed -= ut->getProdSpeed(NULL);
+		enforceMinimumValue(0,prodSpeed);
 	}
 }
 
