@@ -81,6 +81,26 @@ protected:
     std::map<string,int> prodSpeedMorphIsMultiplierValueList;
 
 public:
+    UpgradeTypeBase() {
+        maxHp = 0;;
+        maxHpIsMultiplier = false;
+    	maxHpRegeneration = 0;
+        sight = 0;
+        sightIsMultiplier = false;
+        maxEp = 0;;
+        maxEpIsMultiplier = false;
+    	maxEpRegeneration = 0;
+        armor = 0;
+        armorIsMultiplier = false;
+        attackStrength = 0;
+        attackStrengthIsMultiplier = false;
+        attackRange = 0;
+        attackRangeIsMultiplier = false;
+        moveSpeed = 0;
+        moveSpeedIsMultiplier = false;
+        prodSpeed = 0;
+        prodSpeedIsMultiplier = false;
+    }
     virtual ~UpgradeTypeBase() {}
 
 	int getMaxHp() const			{return maxHp;}
@@ -110,7 +130,8 @@ public:
 
 	void load(const XmlNode *upgradeNode, string upgradename);
 
-	virtual string getDesc() const;
+	virtual string getDesc(bool translatedValue) const;
+
 	std::string toString() const {
 		std::string result = "";
 
@@ -124,26 +145,71 @@ public:
 		result += "sightIsMultiplier = " + intToStr(sightIsMultiplier);
 
 		result += " maxEp = " + intToStr(maxEp);
-		result += "maxEpIsMultiplier = " + intToStr(maxEpIsMultiplier);
+		result += " maxEpIsMultiplier = " + intToStr(maxEpIsMultiplier);
 		result += " maxEpRegeneration = " + intToStr(maxEpRegeneration);
 		//result += "maxEpRegenerationIsMultiplier = " + intToStr(maxEpRegenerationIsMultiplier);
 
 		result += " armor = " + intToStr(armor);
-		result += "armorIsMultiplier = " + intToStr(armorIsMultiplier);
+		result += " armorIsMultiplier = " + intToStr(armorIsMultiplier);
 		result += " attackStrength = " + intToStr(attackStrength);
-		result += "attackStrengthIsMultiplier = " + intToStr(attackStrengthIsMultiplier);
+		result += " attackStrengthIsMultiplier = " + intToStr(attackStrengthIsMultiplier);
 		result += " attackRange = " + intToStr(attackRange);
-		result += "attackRangeIsMultiplier = " + intToStr(attackRangeIsMultiplier);
+		result += " attackRangeIsMultiplier = " + intToStr(attackRangeIsMultiplier);
 		result += " moveSpeed = " + intToStr(moveSpeed);
-		result += "moveSpeedIsMultiplier = " + intToStr(moveSpeedIsMultiplier);
+		result += " moveSpeedIsMultiplier = " + intToStr(moveSpeedIsMultiplier);
 		result += " prodSpeed = " + intToStr(prodSpeed);
-		result += "prodSpeedIsMultiplier = " + intToStr(prodSpeedIsMultiplier);
+		result += " prodSpeedIsMultiplier = " + intToStr(prodSpeedIsMultiplier);
 
 		return result;
 	}
 
 	virtual void saveGame(XmlNode *rootNode) const;
 	static const UpgradeType * loadGame(const XmlNode *rootNode, Faction *faction);
+
+	Checksum getCRC() {
+		Checksum crcForUpgradeType;
+
+		crcForUpgradeType.addString(upgradename);
+		crcForUpgradeType.addInt(maxHp);
+		crcForUpgradeType.addInt(maxHpIsMultiplier);
+	    crcForUpgradeType.addInt(maxHpRegeneration);
+
+	    crcForUpgradeType.addInt(sight);
+	    crcForUpgradeType.addInt(sightIsMultiplier);
+
+	    crcForUpgradeType.addInt(maxEp);
+	    crcForUpgradeType.addInt(maxEpIsMultiplier);
+	    crcForUpgradeType.addInt(maxEpRegeneration);
+
+	    crcForUpgradeType.addInt(armor);
+	    crcForUpgradeType.addInt(armorIsMultiplier);
+
+	    crcForUpgradeType.addInt(attackStrength);
+	    crcForUpgradeType.addInt(attackStrengthIsMultiplier);
+	    //std::map<string,int> attackStrengthMultiplierValueList;
+	    crcForUpgradeType.addInt64((int64)attackStrengthMultiplierValueList.size());
+
+	    crcForUpgradeType.addInt(attackRange);
+	    crcForUpgradeType.addInt(attackRangeIsMultiplier);
+	    //std::map<string,int> attackRangeMultiplierValueList;
+	    crcForUpgradeType.addInt64((int64)attackRangeMultiplierValueList.size());
+
+	    crcForUpgradeType.addInt(moveSpeed);
+	    crcForUpgradeType.addInt(moveSpeedIsMultiplier);
+	    //std::map<string,int> moveSpeedIsMultiplierValueList;
+	    crcForUpgradeType.addInt64((int64)moveSpeedIsMultiplierValueList.size());
+
+	    crcForUpgradeType.addInt(prodSpeed);
+	    crcForUpgradeType.addInt(prodSpeedIsMultiplier);
+	    //std::map<string,int> prodSpeedProduceIsMultiplierValueList;
+	    crcForUpgradeType.addInt64((int64)prodSpeedProduceIsMultiplierValueList.size());
+	    //std::map<string,int> prodSpeedUpgradeIsMultiplierValueList;
+	    crcForUpgradeType.addInt64((int64)prodSpeedUpgradeIsMultiplierValueList.size());
+	    //std::map<string,int> prodSpeedMorphIsMultiplierValueList;
+	    crcForUpgradeType.addInt64((int64)prodSpeedMorphIsMultiplierValueList.size());
+
+		return crcForUpgradeType;
+	}
 };
 
 // ===============================
@@ -158,17 +224,19 @@ public:
 	void preLoad(const string &dir);
     void load(const string &dir, const TechTree *techTree,
     		const FactionType *factionType, Checksum* checksum,
-    		Checksum* techtreeChecksum, std::map<string,vector<pair<string, string> > > &loadedFileList);
+    		Checksum* techtreeChecksum,
+    		std::map<string,vector<pair<string, string> > > &loadedFileList,
+    		bool validationMode=false);
 
     virtual string getName(bool translatedValue=false) const;
 
     //get all
-	int getEffectCount() const				{return effects.size();}
+	int getEffectCount() const				{return (int)effects.size();}
 	const UnitType * getEffect(int i) const	{return effects[i];}
 	bool isAffected(const UnitType *unitType) const;
 
     //other methods
-	virtual string getReqDesc() const;
+	virtual string getReqDesc(bool translatedValue) const;
 
 	//virtual void saveGame(XmlNode *rootNode) const;
 	//virtual void loadGame(const XmlNode *rootNode);
