@@ -93,7 +93,7 @@ void Cell::saveGame(XmlNode *rootNode, int index) const {
 		}
 
 	//	float height;
-		cellNode->addAttribute("height",floatToStr(height), mapTagReplacements);
+		cellNode->addAttribute("height",floatToStr(getHeight(),6), mapTagReplacements);
 	}
 }
 
@@ -101,7 +101,7 @@ void Cell::loadGame(const XmlNode *rootNode, int index, World *world) {
 	if(rootNode->hasChild("Cell" + intToStr(index)) == true) {
 		const XmlNode *cellNode = rootNode->getChild("Cell" + intToStr(index));
 
-		int unitCount = cellNode->getChildCount();
+		unsigned int unitCount = (unsigned int)cellNode->getChildCount();
 		for(unsigned int i = 0; i < unitCount; ++i) {
 			if(cellNode->hasChildAtIndex("units",i) == true) {
 				const XmlNode *unitsNode = cellNode->getChild("units",i);
@@ -140,6 +140,7 @@ SurfaceCell::SurfaceCell() {
 
 SurfaceCell::~SurfaceCell() {
 	delete object;
+	object=NULL;
 }
 
 void SurfaceCell::end(){
@@ -156,6 +157,7 @@ void SurfaceCell::deleteResource() {
 }
 
 void SurfaceCell::setHeight(float height, bool cellChangedFromOriginalMapLoadValue) {
+	height = truncateDecimal<float>(height);
 	vertex.y= height;
 	if(cellChangedFromOriginalMapLoadValue == true) {
 		this->cellChangedFromOriginalMapLoad = true;
@@ -303,7 +305,7 @@ Map::Map() {
 }
 
 Map::~Map() {
-	Logger::getInstance().add(Lang::getInstance().get("LogScreenGameUnLoadingMapCells","",true), true);
+	Logger::getInstance().add(Lang::getInstance().getString("LogScreenGameUnLoadingMapCells","",true), true);
 
 	delete [] cells;
 	cells = NULL;
@@ -315,7 +317,7 @@ Map::~Map() {
 
 void Map::end(){
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
-    Logger::getInstance().add(Lang::getInstance().get("LogScreenGameUnLoadingMap","",true), true);
+    Logger::getInstance().add(Lang::getInstance().getString("LogScreenGameUnLoadingMap","",true), true);
 	//read heightmap
 	for(int j = 0; j < surfaceH; ++j) {
 		for(int i = 0; i < surfaceW; ++i) {
@@ -372,8 +374,10 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 			heightFactor= header.heightFactor;
 			if(heightFactor>100){
 				heightFactor=heightFactor/100;
+				heightFactor = truncateDecimal<float>(heightFactor,6);
 			}
 			waterLevel= static_cast<float>((header.waterLevel-0.01f)/heightFactor);
+			waterLevel = truncateDecimal<float>(waterLevel,6);
 			title= header.title;
 			maxPlayers= header.maxFactions;
 
@@ -392,9 +396,9 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 				//desc = header.version2.short_desc;
 				if(header.version2.cliffLevel > 0  && header.version2.cliffLevel < 5000){
 					cliffLevel=static_cast<float>((header.version2.cliffLevel-0.01f)/(heightFactor));
+					cliffLevel = truncateDecimal<float>(cliffLevel,6);
 				}
-				if(header.version2.cameraHeight > 0 && header.version2.cameraHeight < 5000)
-				{
+				if(header.version2.cameraHeight > 0 && header.version2.cameraHeight < 5000) {
 					cameraHeight = header.version2.cameraHeight;
 				}
 			}
@@ -409,7 +413,7 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 					snprintf(szBuf,8096,"fread returned wrong size = " MG_SIZE_T_SPECIFIER " on line: %d.",readBytes,__LINE__);
 					throw megaglest_runtime_error(szBuf);
 				}
-				x = Shared::PlatformByteOrder::fromCommonEndian(x);
+				x = ::Shared::PlatformByteOrder::fromCommonEndian(x);
 
 				readBytes = fread(&y, sizeof(int32), 1, f);
 				if(readBytes != 1) {
@@ -417,7 +421,7 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 					snprintf(szBuf,8096,"fread returned wrong size = " MG_SIZE_T_SPECIFIER " on line: %d.",readBytes,__LINE__);
 					throw megaglest_runtime_error(szBuf);
 				}
-				y = Shared::PlatformByteOrder::fromCommonEndian(y);
+				y = ::Shared::PlatformByteOrder::fromCommonEndian(y);
 
 				startLocations[i]= Vec2i(x, y)*cellScale;
 			}
@@ -436,7 +440,7 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 						snprintf(szBuf,8096,"fread returned wrong size = " MG_SIZE_T_SPECIFIER " on line: %d.",readBytes,__LINE__);
 						throw megaglest_runtime_error(szBuf);
 					}
-					alt = Shared::PlatformByteOrder::fromCommonEndian(alt);
+					alt = ::Shared::PlatformByteOrder::fromCommonEndian(alt);
 
 					SurfaceCell *sc= getSurfaceCell(i, j);
 					sc->setVertex(Vec3f(i*mapScale, alt / heightFactor, j*mapScale));
@@ -453,7 +457,7 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 						snprintf(szBuf,8096,"fread returned wrong size = " MG_SIZE_T_SPECIFIER " on line: %d.",readBytes,__LINE__);
 						throw megaglest_runtime_error(szBuf);
 					}
-					surf = Shared::PlatformByteOrder::fromCommonEndian(surf);
+					surf = ::Shared::PlatformByteOrder::fromCommonEndian(surf);
 
 					getSurfaceCell(i, j)->setSurfaceType(surf-1);
 				}
@@ -470,7 +474,7 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 						snprintf(szBuf,8096,"fread returned wrong size = " MG_SIZE_T_SPECIFIER " on line: %d.",readBytes,__LINE__);
 						throw megaglest_runtime_error(szBuf);
 					}
-					objNumber = Shared::PlatformByteOrder::fromCommonEndian(objNumber);
+					objNumber = ::Shared::PlatformByteOrder::fromCommonEndian(objNumber);
 
 					SurfaceCell *sc= getSurfaceCell(toSurfCoords(Vec2i(i, j)));
 					if(objNumber == 0) {
@@ -494,13 +498,11 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 					}
 				}
 			}
-		}
-		else{
 			if(f) fclose(f);
-
+		}
+		else {
 			throw megaglest_runtime_error("Can't open file");
 		}
-		fclose(f);
 	}
 	catch(const exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,e.what());
@@ -511,7 +513,7 @@ Checksum Map::load(const string &path, TechTree *techTree, Tileset *tileset) {
 }
 
 void Map::init(Tileset *tileset) {
-	Logger::getInstance().add(Lang::getInstance().get("LogScreenGameUnLoadingMap","",true), true);
+	Logger::getInstance().add(Lang::getInstance().getString("LogScreenGameUnLoadingMap","",true), true);
 	maxMapHeight=0.0f;
 	smoothSurface(tileset);
 	computeNormals();
@@ -531,7 +533,7 @@ public:
 };
 
 //returns if there is a resource next to a unit, in "resourcePos" is stored the relative position of the resource
-bool Map::isResourceNear(const Vec2i &pos, const ResourceType *rt, Vec2i &resourcePos,
+bool Map::isResourceNear(int frameIndex,const Vec2i &pos, const ResourceType *rt, Vec2i &resourcePos,
 		int size, Unit *unit, bool fallbackToPeersHarvestingSameResource,
 		Vec2i *resourceClickPos) const {
 
@@ -605,7 +607,7 @@ bool Map::isResourceNear(const Vec2i &pos, const ResourceType *rt, Vec2i &resour
 
 			// Check the faction cache for a known position where we can harvest
 			// this resource type
-			Vec2i result = unit->getFaction()->getClosestResourceTypeTargetFromCache(unit, rt);
+			Vec2i result = unit->getFaction()->getClosestResourceTypeTargetFromCache(unit, rt,frameIndex);
 			if(result.x >= 0) {
 				resourcePos = result;
 
@@ -613,7 +615,13 @@ bool Map::isResourceNear(const Vec2i &pos, const ResourceType *rt, Vec2i &resour
 					char szBuf[8096]="";
 					snprintf(szBuf,8096,"[found peer harvest pos] pos [%s] resourcePos [%s] unit->getFaction()->getCacheResourceTargetListSize() [%d]",
 										pos.getString().c_str(),resourcePos.getString().c_str(),unit->getFaction()->getCacheResourceTargetListSize());
-					unit->logSynchData(__FILE__,__LINE__,szBuf);
+
+					if(frameIndex < 0) {
+						unit->logSynchData(__FILE__,__LINE__,szBuf);
+					}
+					else {
+						unit->logSynchDataThreaded(__FILE__,__LINE__,szBuf);
+					}
 				}
 
 				if(unit->getPos().dist(resourcePos) <= size) {
@@ -649,13 +657,15 @@ bool Map::isResourceNear(const Vec2i &pos, const ResourceType *rt, Vec2i &resour
 								(distanceFromUnit < 0 || unit->getCenteredPos().dist(resPos) <= (distanceFromUnit + 2.0))) {
 
 								if(resourceClickPos->dist(resPos) <= 1.0) {
-									FindBestPos bestPosItem;
+									if(unit != NULL) {
+										FindBestPos bestPosItem;
 
-									bestPosItem.distanceFromUnitNoAdjustment = unit->getCenteredPos().dist(resPos);
-									bestPosItem.distanceFromClickNoAdjustment =distanceFromClick = resourceClickPos->dist(resPos);
-									bestPosItem.resourcePosNoAdjustment = resPos;
+										bestPosItem.distanceFromUnitNoAdjustment = unit->getCenteredPos().dist(resPos);
+										bestPosItem.distanceFromClickNoAdjustment =distanceFromClick = resourceClickPos->dist(resPos);
+										bestPosItem.resourcePosNoAdjustment = resPos;
 
-									bestPosList.push_back(bestPosItem);
+										bestPosList.push_back(bestPosItem);
+									}
 								}
 
 								//printf("!!!! unit [%s - %d] resPos = [%s] resourceClickPos->dist(resPos) [%f] distanceFromClick [%f] unit->getCenteredPos().dist(resPos) [%f] distanceFromUnit [%f]\n",unit->getFullName().c_str(),unit->getId(),resPos.getString().c_str(),resourceClickPos->dist(resPos),distanceFromClick,unit->getCenteredPos().dist(resPos),distanceFromUnit);
@@ -834,7 +844,7 @@ bool Map::canOccupy(const Vec2i &pos, Field field, const UnitType *ut, CardinalD
 					}
 				}
 				else {
-					false;
+					return false;
 				}
 			}
 		}
@@ -903,7 +913,7 @@ bool Map::canMove(const Unit *unit, const Vec2i &pos1, const Vec2i &pos2, std::m
 		}
 	//}
 
-	if(unit == NULL || isBadHarvestPos == true) {
+	if(isBadHarvestPos == true) {
 		if(lookupCache != NULL) {
 			(*lookupCache)[pos1][pos2][size][field]=false;
 		}
@@ -1074,11 +1084,16 @@ bool Map::aproxCanMove(const Unit *unit, const Vec2i &pos1, const Vec2i &pos2, s
 
 Vec2i Map::computeRefPos(const Selection *selection) const {
     Vec2i total= Vec2i(0);
+
+	if(selection == NULL) {
+		throw megaglest_runtime_error("selection == NULL");
+	}
+
     for(int i = 0; i < selection->getCount(); ++i) {
-    	if(selection == NULL || selection->getUnit(i) == NULL) {
+    	if(selection->getUnit(i) == NULL) {
     		throw megaglest_runtime_error("selection == NULL || selection->getUnit(i) == NULL");
     	}
-        total = total + selection->getUnit(i)->getPos();
+        total = total + selection->getUnit(i)->getPosNotThreadSafe();
     }
 
     return Vec2i(total.x / selection->getCount(), total.y / selection->getCount());
@@ -1110,7 +1125,7 @@ std::pair<float,Vec2i> Map::getUnitDistanceToPos(const Unit *unit,Vec2i pos,cons
 
 	std::pair<float,Vec2i> result(-1,Vec2i(0));
 	//int unitId= unit->getId();
-	Vec2i unitPos= computeDestPos(unit->getPos(), unit->getPos(), pos);
+	Vec2i unitPos= computeDestPos(unit->getPosNotThreadSafe(), unit->getPosNotThreadSafe(), pos);
 
 	Vec2i start = pos - Vec2i(1);
 	int unitTypeSize = 0;
@@ -1155,7 +1170,7 @@ const Unit * Map::findClosestUnitToPos(const Selection *selection, Vec2i origina
 	for(int i = 0; i < selection->getCount(); ++i) {
 		const Unit *unit = selection->getUnit(i);
 		//int unitId= unit->getId();
-		Vec2i unitBuilderPos= computeDestPos(refPos, unit->getPos(), pos);
+		Vec2i unitBuilderPos= computeDestPos(refPos, unit->getPosNotThreadSafe(), pos);
 
 		for(int i = start.x; i <= end.x; ++i) {
 			for(int j = start.y; j <= end.y; ++j){
@@ -1176,7 +1191,14 @@ const Unit * Map::findClosestUnitToPos(const Selection *selection, Vec2i origina
 }
 
 Vec2i Map::findBestBuildApproach(const Unit *unit, Vec2i originalBuildPos,const UnitType *ut) const {
-    Vec2i unitBuilderPos    = unit->getPos();
+	if(unit == NULL) {
+		throw megaglest_runtime_error("unit == NULL");
+	}
+	if(ut == NULL) {
+		throw megaglest_runtime_error("ut == NULL");
+	}
+
+    Vec2i unitBuilderPos    = unit->getPosNotThreadSafe();
 	Vec2i pos               = originalBuildPos;
 
 	float bestRange = -1;
@@ -1319,23 +1341,24 @@ void Map::putUnitCellsPrivate(Unit *unit, const Vec2i &pos, const UnitType *ut,
 //                              getCell(currPos)->getUnit(unit->getCurrField())->toString().c_str());
 //                    }
 				}
+
+
 				if(getCell(currPos)->getUnit(field) == NULL ||
-				                   getCell(currPos)->getUnit(field) == unit) {
-					if(isMorph){
-                    	// unit is beeing morphed to another unit with maybe other field.
-                    	getCell(currPos)->setUnit(field, unit);
-                    	canPutInCell = false;
+								   getCell(currPos)->getUnit(field) == unit) {
+					if(isMorph) {
+						// unit is beeing morphed to another unit with maybe other field.
+						getCell(currPos)->setUnit(field, unit);
+						canPutInCell = false;
+					}
+					if(canPutInCell == true) {
+						getCell(currPos)->setUnit(unit->getCurrField(), unit);
 					}
-	                if(canPutInCell == true) {
-	                    getCell(currPos)->setUnit(unit->getCurrField(), unit);
-	                }
 				}
-				else {
+				else if(canPutInCell == true) {
 					char szBuf[8096]="";
-					snprintf(szBuf,8096,"Trying to move unit [%d - %s] into occupied cell [%s] and field = %d, unit already in cell [%d - %s] ",unit->getId(),unit->getType()->getName().c_str(),pos.getString().c_str(),field,getCell(currPos)->getUnit(field)->getId(),getCell(currPos)->getUnit(field)->getType()->getName().c_str());
+					snprintf(szBuf,8096,"Trying to move unit [%d - %s] into occupied cell [%s] and field = %d, unit already in cell [%d - %s] ",unit->getId(),unit->getType()->getName(false).c_str(),pos.getString().c_str(),field,getCell(currPos)->getUnit(field)->getId(),getCell(currPos)->getUnit(field)->getType()->getName(false).c_str());
 					throw megaglest_runtime_error(szBuf);
 				}
-
 			}
 			else if(ut->hasCellMap() == true &&
 					ut->getAllowEmptyCellMap() == true &&
@@ -1441,7 +1464,7 @@ bool Map::isNextTo(const Vec2i &pos, const Unit *unit) const {
 
 //return if unit is next to pos
 bool Map::isNextTo(const Unit *unit1, const Unit *unit2) const {
-	Vec2i pos = unit1->getPos();
+	Vec2i pos = unit1->getPosNotThreadSafe();
 	const UnitType *ut = unit1->getType();
 	for (int y=-1; y < ut->getSize()+1; ++y) {
 		for (int x=-1; x < ut->getSize()+1; ++x) {
@@ -1515,7 +1538,7 @@ void Map::flatternTerrain(const Unit *unit){
 	float refHeight= getSurfaceCell(toSurfCoords(unit->getCenteredPos()))->getHeight();
 	for(int i=-1; i<=unit->getType()->getSize(); ++i){
         for(int j=-1; j<=unit->getType()->getSize(); ++j){
-            Vec2i pos= unit->getPos()+Vec2i(i, j);
+            Vec2i pos= unit->getPosNotThreadSafe()+Vec2i(i, j);
             if(isInside(pos) && isInsideSurface(toSurfCoords(pos))) {
 				Cell *c= getCell(pos);
 				SurfaceCell *sc= getSurfaceCell(toSurfCoords(pos));
@@ -1621,8 +1644,8 @@ void Map::smoothSurface(Tileset *tileset) {
 						}
 						if (formerObject == NULL) {
 							Object *o = new Object(tileset->getObjectType(9),
-									getSurfaceCell(i, j)->getVertex(), Vec2i(i,
-											j));
+									getSurfaceCell(i, j)->getVertex(),
+									Vec2i(i,j));
 							getSurfaceCell(i, j)->setObject(o);
 						}
 					}
@@ -1678,34 +1701,6 @@ void Map::computeCellColors(){
 	}
 }
 
-// static
-string Map::getMapPath(const string &mapName, string scenarioDir, bool errorOnNotFound) {
-
-    Config &config = Config::getInstance();
-    vector<string> pathList = config.getPathListForType(ptMaps,scenarioDir);
-
-    for(int idx = 0; idx < pathList.size(); idx++) {
-        string map_path = pathList[idx];
-    	endPathWithSlash(map_path);
-
-        const string mega = map_path + mapName + ".mgm";
-        const string glest = map_path + mapName + ".gbm";
-        if (fileExists(mega)) {
-            return mega;
-        }
-        else if (fileExists(glest)) {
-            return glest;
-        }
-    }
-
-	if(errorOnNotFound == true) {
-		//abort();
-		throw megaglest_runtime_error("Map not found [" + mapName + "]\nScenario [" + scenarioDir + "]");
-	}
-
-	return "";
-}
-
 void Map::saveGame(XmlNode *rootNode) const {
 	std::map<string,string> mapTagReplacements;
 	XmlNode *mapNode = rootNode->addChild("Map");
@@ -1713,11 +1708,11 @@ void Map::saveGame(XmlNode *rootNode) const {
 //	string title;
 	mapNode->addAttribute("title",title, mapTagReplacements);
 //	float waterLevel;
-	mapNode->addAttribute("waterLevel",floatToStr(waterLevel), mapTagReplacements);
+	mapNode->addAttribute("waterLevel",floatToStr(waterLevel,6), mapTagReplacements);
 //	float heightFactor;
-	mapNode->addAttribute("heightFactor",floatToStr(heightFactor), mapTagReplacements);
+	mapNode->addAttribute("heightFactor",floatToStr(heightFactor,6), mapTagReplacements);
 //	float cliffLevel;
-	mapNode->addAttribute("cliffLevel",floatToStr(cliffLevel), mapTagReplacements);
+	mapNode->addAttribute("cliffLevel",floatToStr(cliffLevel,6), mapTagReplacements);
 //	int cameraHeight;
 	mapNode->addAttribute("cameraHeight",intToStr(cameraHeight), mapTagReplacements);
 //	int w;
@@ -1742,14 +1737,14 @@ void Map::saveGame(XmlNode *rootNode) const {
 	string exploredList = "";
 	string visibleList = "";
 
-	for(unsigned int i = 0; i < getSurfaceCellArraySize(); ++i) {
+	for(unsigned int i = 0; i < (unsigned int)getSurfaceCellArraySize(); ++i) {
 		SurfaceCell &surfaceCell = surfaceCells[i];
 
 		if(exploredList != "") {
 			exploredList += ",";
 		}
 
-		for(unsigned int j = 0; j < GameConstants::maxPlayers; ++j) {
+		for(unsigned int j = 0; j < (unsigned int)GameConstants::maxPlayers; ++j) {
 			if(j > 0) {
 				exploredList += "|";
 			}
@@ -1761,7 +1756,7 @@ void Map::saveGame(XmlNode *rootNode) const {
 			visibleList += ",";
 		}
 
-		for(unsigned int j = 0; j < GameConstants::maxPlayers; ++j) {
+		for(unsigned int j = 0; j < (unsigned int)GameConstants::maxPlayers; ++j) {
 			if(j > 0) {
 				visibleList += "|";
 			}
@@ -1790,14 +1785,14 @@ void Map::saveGame(XmlNode *rootNode) const {
 	}
 
 //	Vec2i *startLocations;
-	for(unsigned int i = 0; i < maxPlayers; ++i) {
+	for(unsigned int i = 0; i < (unsigned int)maxPlayers; ++i) {
 		XmlNode *startLocationsNode = mapNode->addChild("startLocations");
 		startLocationsNode->addAttribute("location",startLocations[i].getString(), mapTagReplacements);
 	}
 //	Checksum checksumValue;
 //	mapNode->addAttribute("checksumValue",intToStr(checksumValue.getSum()), mapTagReplacements);
 //	float maxMapHeight;
-	mapNode->addAttribute("maxMapHeight",floatToStr(maxMapHeight), mapTagReplacements);
+	mapNode->addAttribute("maxMapHeight",floatToStr(maxMapHeight,6), mapTagReplacements);
 //	string mapFile;
 	mapNode->addAttribute("mapFile",mapFile, mapTagReplacements);
 }
@@ -1823,7 +1818,7 @@ void Map::loadGame(const XmlNode *rootNode, World *world) {
 //	}
 
 //	printf("getSurfaceCellArraySize() = %d\n",getSurfaceCellArraySize());
-	for(unsigned int i = 0; i < getSurfaceCellArraySize(); ++i) {
+	for(unsigned int i = 0; i < (unsigned int)getSurfaceCellArraySize(); ++i) {
 		SurfaceCell &surfaceCell = surfaceCells[i];
 		surfaceCell.loadGame(mapNode,i,world);
 	}
@@ -1868,6 +1863,10 @@ void Map::loadGame(const XmlNode *rootNode, World *world) {
 				string value = tokensExploredValue[k];
 
 				surfaceCell.setExplored(k,strToInt(value) != 0);
+
+				//if(surfaceCell.isExplored(k) == true) {
+				//	printf("Setting cell at index: %d for team: %d to: %d [%s]\n",surfaceCellIndexExplored,k,surfaceCell.isExplored(k),value.c_str());
+				//}
 			}
 			surfaceCellIndexExplored++;
 		}
@@ -1930,7 +1929,6 @@ bool PosCircularIterator::next(){
 #else
 	while(floor(pos.dist(center)) >= (radius+1) || !map->isInside(pos) || !map->isInsideSurface(map->toSurfCoords(pos)) );
 #endif
-	//while(!(pos.dist(center) <= radius && map->isInside(pos)));
 
 	return true;
 }
