@@ -47,6 +47,7 @@ class Resource;
 class TechTree;
 class GameSettings;
 class World;
+
 // =====================================================
 // 	class Cell
 //
@@ -57,7 +58,7 @@ class Cell {
 private:
     Unit *units[fieldCount];	//units on this cell
     Unit *unitsWithEmptyCellMap[fieldCount];	//units with an empty cellmap on this cell
-	float height;
+    float height;
 
 private:
 	Cell(Cell&);
@@ -69,11 +70,11 @@ public:
 	//get
 	inline Unit *getUnit(int field) const		{ if(field >= fieldCount) { throw megaglest_runtime_error("Invalid field value" + intToStr(field));} return units[field];}
 	inline Unit *getUnitWithEmptyCellMap(int field) const		{ if(field >= fieldCount) { throw megaglest_runtime_error("Invalid field value" + intToStr(field));} return unitsWithEmptyCellMap[field];}
-	inline float getHeight() const				{return height;}
+	inline float getHeight() const				{return truncateDecimal<float>(height,6);}
 
 	inline void setUnit(int field, Unit *unit)	{ if(field >= fieldCount) { throw megaglest_runtime_error("Invalid field value" + intToStr(field));} units[field]= unit;}
 	inline void setUnitWithEmptyCellMap(int field, Unit *unit)	{ if(field >= fieldCount) { throw megaglest_runtime_error("Invalid field value" + intToStr(field));} unitsWithEmptyCellMap[field]= unit;}
-	inline void setHeight(float height)		{this->height= height;}
+	inline void setHeight(float height)		{this->height = truncateDecimal<float>(height,6);}
 
 	inline bool isFree(Field field) const {
 		Unit *unit = getUnit(field);
@@ -294,18 +295,17 @@ public:
 	inline int getH() const											{return h;}
 	inline int getSurfaceW() const										{return surfaceW;}
 	inline int getSurfaceH() const										{return surfaceH;}
-	inline int getMaxPlayers() const										{return maxPlayers;}
-	inline float getHeightFactor() const								{return heightFactor;}
-	inline float getWaterLevel() const									{return waterLevel;}
-	inline float getCliffLevel() const									{return cliffLevel;}
+	inline int getMaxPlayers() const									{return maxPlayers;}
+	inline float getHeightFactor() const								{return truncateDecimal<float>(heightFactor,6);}
+	inline float getWaterLevel() const									{return truncateDecimal<float>(waterLevel,6);}
+	inline float getCliffLevel() const									{return truncateDecimal<float>(cliffLevel,6);}
 	inline int getCameraHeight() const									{return cameraHeight;}
-	inline float getMaxMapHeight() const								{return maxMapHeight;}
+	inline float getMaxMapHeight() const								{return truncateDecimal<float>(maxMapHeight,6);}
 	Vec2i getStartLocation(int locationIndex) const;
 	inline bool getSubmerged(const SurfaceCell *sc) const				{return sc->getHeight()<waterLevel;}
 	inline bool getSubmerged(const Cell *c) const						{return c->getHeight()<waterLevel;}
 	inline bool getDeepSubmerged(const SurfaceCell *sc) const			{return sc->getHeight()<waterLevel-(1.5f/heightFactor);}
 	inline bool getDeepSubmerged(const Cell *c) const					{return c->getHeight()<waterLevel-(1.5f/heightFactor);}
-	//float getSurfaceHeight(const Vec2i &pos) const;
 
 	//is
 	inline bool isInside(int x, int y) const {
@@ -320,7 +320,7 @@ public:
 	inline bool isInsideSurface(const Vec2i &sPos) const {
 		return isInsideSurface(sPos.x, sPos.y);
 	}
-	bool isResourceNear(const Vec2i &pos, const ResourceType *rt, Vec2i &resourcePos, int size, Unit *unit=NULL,bool fallbackToPeersHarvestingSameResource=false,Vec2i *resourceClickPos=NULL) const;
+	bool isResourceNear(int frameIndex,const Vec2i &pos, const ResourceType *rt, Vec2i &resourcePos, int size, Unit *unit=NULL,bool fallbackToPeersHarvestingSameResource=false,Vec2i *resourceClickPos=NULL) const;
 
 	//free cells
 	bool isFreeCell(const Vec2i &pos, Field field) const;
@@ -362,7 +362,6 @@ public:
 	//static
 	inline static Vec2i toSurfCoords(const Vec2i &unitPos)		{return unitPos / cellScale;}
 	inline static Vec2i toUnitCoords(const Vec2i &surfPos)		{return surfPos * cellScale;}
-	static string getMapPath(const string &mapName, string scenarioDir="", bool errorOnNotFound=true);
 
 	inline bool isFreeCellOrMightBeFreeSoon(Vec2i originPos, const Vec2i &pos, Field field) const {
 		return
@@ -409,56 +408,33 @@ public:
 		int teamIndex= unit->getTeam();
 		Field field= unit->getCurrField();
 
-		const bool *cachedResult = unit->getFaction()->aproxCanMoveSoonCached(size,field,pos1,pos2);
-		if(cachedResult != NULL) {
-			return *cachedResult;
-		}
-
 		//single cell units
 		if(size == 1) {
-			if(isAproxFreeCellOrMightBeFreeSoon(unit->getPos(),pos2, field, teamIndex) == false) {
-
-				//printf("[%s] Line: %d returning false\n",__FUNCTION__,__LINE__);
-				unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, false);
+			if(isAproxFreeCellOrMightBeFreeSoon(unit->getPosNotThreadSafe(),pos2, field, teamIndex) == false) {
 				return false;
 			}
 			if(pos1.x != pos2.x && pos1.y != pos2.y) {
-				if(isAproxFreeCellOrMightBeFreeSoon(unit->getPos(),Vec2i(pos1.x, pos2.y), field, teamIndex) == false) {
-
-					//Unit *cellUnit = getCell(Vec2i(pos1.x, pos2.y))->getUnit(field);
-					//Object * obj = getSurfaceCell(toSurfCoords(Vec2i(pos1.x, pos2.y)))->getObject();
-
-					//printf("[%s] Line: %d returning false cell [%s] free [%d] cell unitid = %d object class = %d\n",__FUNCTION__,__LINE__,Vec2i(pos1.x, pos2.y).getString().c_str(),this->isFreeCell(Vec2i(pos1.x, pos2.y),field),(cellUnit != NULL ? cellUnit->getId() : -1),(obj != NULL ? obj->getType()->getClass() : -1));
-					//printf("[%s] Line: %d returning false\n",__FUNCTION__,__LINE__);
-					unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, false);
+				if(isAproxFreeCellOrMightBeFreeSoon(unit->getPosNotThreadSafe(),Vec2i(pos1.x, pos2.y), field, teamIndex) == false) {
 					return false;
 				}
-				if(isAproxFreeCellOrMightBeFreeSoon(unit->getPos(),Vec2i(pos2.x, pos1.y), field, teamIndex) == false) {
-					//printf("[%s] Line: %d returning false\n",__FUNCTION__,__LINE__);
-					unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, false);
+				if(isAproxFreeCellOrMightBeFreeSoon(unit->getPosNotThreadSafe(),Vec2i(pos2.x, pos1.y), field, teamIndex) == false) {
 					return false;
 				}
 			}
 
 			bool isBadHarvestPos = false;
-			//if(unit != NULL) {
-				Command *command= unit->getCurrCommand();
-				if(command != NULL) {
-					const HarvestCommandType *hct = dynamic_cast<const HarvestCommandType*>(command->getCommandType());
-					if(hct != NULL && unit->isBadHarvestPos(pos2) == true) {
-						isBadHarvestPos = true;
-					}
+			Command *command= unit->getCurrCommand();
+			if(command != NULL) {
+				const HarvestCommandType *hct = dynamic_cast<const HarvestCommandType*>(command->getCommandType());
+				if(hct != NULL && unit->isBadHarvestPos(pos2) == true) {
+					isBadHarvestPos = true;
 				}
-			//}
+			}
 
 			if(unit == NULL || isBadHarvestPos == true) {
-
-				//printf("[%s] Line: %d returning false\n",__FUNCTION__,__LINE__);
-				unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, false);
 				return false;
 			}
 
-			unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, true);
 			return true;
 		}
 		//multi cell units
@@ -469,18 +445,12 @@ public:
 					Vec2i cellPos = Vec2i(i,j);
 					if(isInside(cellPos) && isInsideSurface(toSurfCoords(cellPos))) {
 						if(getCell(cellPos)->getUnit(unit->getCurrField()) != unit) {
-							if(isAproxFreeCellOrMightBeFreeSoon(unit->getPos(),cellPos, field, teamIndex) == false) {
-
-								//printf("[%s] Line: %d returning false\n",__FUNCTION__,__LINE__);
-								unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, false);
+							if(isAproxFreeCellOrMightBeFreeSoon(unit->getPosNotThreadSafe(),cellPos, field, teamIndex) == false) {
 								return false;
 							}
 						}
 					}
 					else {
-
-						//printf("[%s] Line: %d returning false\n",__FUNCTION__,__LINE__);
-						unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, false);
 						return false;
 					}
 				}
@@ -496,14 +466,10 @@ public:
 			}
 
 			if(isBadHarvestPos == true) {
-
-				//printf("[%s] Line: %d returning false\n",__FUNCTION__,__LINE__);
-				unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, false);
 				return false;
 			}
 
 		}
-		unit->getFaction()->addAproxCanMoveSoonCached(size,field, pos1, pos2, true);
 		return true;
 	}
 
