@@ -23,6 +23,7 @@
 #include "socket.h"
 #include "config.h"
 #include "platform_util.h"
+#include "gen_uuid.h"
 #include "leak_dumper.h"
 
 using namespace Shared::Xml;
@@ -51,10 +52,29 @@ Checksum Scenario::load(const string &path) {
 		string name= cutLastExt(lastDir(path));
 
 		char szBuf[8096]="";
-		snprintf(szBuf,8096,Lang::getInstance().get("LogScreenGameLoadingScenario","",true).c_str(),formatString(name).c_str());
+		snprintf(szBuf,8096,Lang::getInstance().getString("LogScreenGameLoadingScenario","",true).c_str(),formatString(name).c_str());
 		Logger::getInstance().add(szBuf, true);
 
-		Scenario::loadScenarioInfo(path, &info);
+		bool isTutorial = Scenario::isGameTutorial(path);
+
+		//Properties::setTechtreePath();
+
+		//printf("path [%s]\n",path.c_str());
+		string scenarioFolder = cutLastFile(formatPath(path));
+		endPathWithSlash(scenarioFolder);
+
+		if(isTutorial == false) {
+			Properties::setScenarioPath(scenarioFolder);
+
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("==> Set scenario path to [%s]\n",scenarioFolder.c_str());
+		}
+		else {
+			Properties::setTutorialPath(scenarioFolder);
+
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("==> Set tutorial path to [%s]\n",scenarioFolder.c_str());
+		}
+
+		Scenario::loadScenarioInfo(path, &info, isTutorial);
 
 		//parse xml
 		XmlTree xmlTree;
@@ -62,7 +82,7 @@ Checksum Scenario::load(const string &path) {
 		const XmlNode *scenarioNode= xmlTree.getRootNode();
 		const XmlNode *scriptsNode= scenarioNode->getChild("scripts");
 
-		for(int i= 0; i<scriptsNode->getChildCount(); ++i){
+		for(int i= 0; i < (int)scriptsNode->getChildCount(); ++i){
 			const XmlNode *scriptNode = scriptsNode->getChild(i);
 
 			scripts.push_back(Script(getFunctionName(scriptNode), scriptNode->getText()));
@@ -83,7 +103,7 @@ Checksum Scenario::load(const string &path) {
 
 int Scenario::getScenarioPathIndex(const vector<string> dirList, const string &scenarioName) {
     int iIndex = 0;
-    for(int idx = 0; idx < dirList.size(); idx++) {
+    for(int idx = 0; idx < (int)dirList.size(); idx++) {
     	string currentPath = dirList[idx];
     	endPathWithSlash(currentPath);
         string scenarioFile = currentPath + scenarioName + "/" + scenarioName + ".xml";
@@ -98,7 +118,7 @@ int Scenario::getScenarioPathIndex(const vector<string> dirList, const string &s
 
 string Scenario::getScenarioDir(const vector<string> dir, const string &scenarioName) {
     string scenarioDir = "";
-    for(int idx = 0; idx < dir.size(); idx++) {
+    for(int idx = 0; idx < (int)dir.size(); idx++) {
     	string currentPath = dir[idx];
     	endPathWithSlash(currentPath);
     	string scenarioFile = currentPath + scenarioName + "/" + scenarioName + ".xml";
@@ -116,24 +136,30 @@ string Scenario::getScenarioDir(const vector<string> dir, const string &scenario
 
 string Scenario::getScenarioPath(const vector<string> dirList, const string &scenarioName, bool getMatchingRootScenarioPathOnly){
     string scenarioFile = "";
-    for(int idx = 0; idx < dirList.size(); idx++) {
+    for(int idx = 0; idx < (int)dirList.size(); idx++) {
     	string currentPath = dirList[idx];
     	endPathWithSlash(currentPath);
     	scenarioFile = currentPath + scenarioName + "/" + scenarioName + ".xml";
+    	//printf("\n[%s:%s] Line: %d scenarioName [%s] scenarioFile [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioName.c_str(),scenarioFile.c_str());
 
+    	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("\n[%s:%s] Line: %d scenarioName [%s] scenarioFile [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioName.c_str(),scenarioFile.c_str());
     	//printf("\n[%s:%s] Line: %d scenarioName [%s] scenarioFile [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioName.c_str(),scenarioFile.c_str());
 
         if(fileExists(scenarioFile) == true) {
+        	//printf("\n[%s:%s] Line: %d scenarioName [%s] scenarioFile [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioName.c_str(),scenarioFile.c_str());
+
 			if(getMatchingRootScenarioPathOnly == true) {
+				//printf("\n[%s:%s] Line: %d scenarioName [%s] scenarioFile [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioName.c_str(),scenarioFile.c_str());
 				scenarioFile = dirList[idx];
 			}
             break;
         }
 		else {
+			//printf("\n[%s:%s] Line: %d scenarioName [%s] scenarioFile [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioName.c_str(),scenarioFile.c_str());
 			scenarioFile = "";
 		}
     }
-
+    //printf("\n[%s:%s] Line: %d scenarioName [%s] scenarioFile [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioName.c_str(),scenarioFile.c_str());
     return scenarioFile;
 }
 
@@ -147,18 +173,41 @@ string Scenario::getScenarioPath(const string &dir, const string &scenarioName){
 string Scenario::getFunctionName(const XmlNode *scriptNode){
 	string name= scriptNode->getName();
 
-	for(int i= 0; i<scriptNode->getAttributeCount(); ++i){
+	for(int i= 0; i < (int)scriptNode->getAttributeCount(); ++i){
 		name+= "_" + scriptNode->getAttribute(i)->getValue();
 	}
 	return name;
 }
 
-void Scenario::loadScenarioInfo(string file, ScenarioInfo *scenarioInfo) {
+bool Scenario::isGameTutorial(string path) {
+	bool isTutorial = false;
+	Config &config = Config::getInstance();
+    vector<string> tutorialPaths = config.getPathListForType(ptTutorials);
+    if(tutorialPaths.empty() == false) {
+    	for(unsigned int tutorialIndex = 0; tutorialIndex < tutorialPaths.size(); ++tutorialIndex) {
+    		const string &tutorialPath = tutorialPaths[tutorialIndex];
+    		size_t pos = path.find( tutorialPath );
+    		if( pos != path.npos ) {
+    			isTutorial = true;
+    			break;
+    		}
+    	}
+    }
+
+    return isTutorial;
+}
+
+void Scenario::loadScenarioInfo(string file, ScenarioInfo *scenarioInfo, bool isTutorial) {
 	//printf("[%s:%s] Line: %d file [%s]\n",__FILE__,__FUNCTION__,__LINE__,file.c_str());
 	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,file.c_str());
 	//printf("In [%s::%s Line: %d] file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,file.c_str());
 
+	//printf("[%s:%s] Line: %d file [%s]\n",__FILE__,__FUNCTION__,__LINE__,file.c_str());
     Lang &lang= Lang::getInstance();
+	string scenarioDir = cutLastFile(formatPath(file));
+	string scenarioName = extractLastDirectoryFromPath(scenarioDir);
+	scenarioDir = cutLastFile(scenarioDir);
+	//printf("[%s:%s] Line: %d scenarioDir [%s] scenarioName [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioDir.c_str(),scenarioName.c_str());
 
     XmlTree xmlTree;
 	xmlTree.load(file,Properties::getTagReplacementValues());
@@ -287,7 +336,7 @@ void Scenario::loadScenarioInfo(string file, ScenarioInfo *scenarioInfo) {
     }
 
 	//add player info
-    scenarioInfo->desc= lang.get("Player") + ": ";
+    scenarioInfo->desc= lang.getString("PlayerFaction") + ": ";
 	for(int i=0; i<GameConstants::maxPlayers; ++i) {
 		if(scenarioInfo->factionControls[i] == ctHuman) {
 			scenarioInfo->desc+= formatString(scenarioInfo->factionTypeNames[i]);
@@ -299,10 +348,32 @@ void Scenario::loadScenarioInfo(string file, ScenarioInfo *scenarioInfo) {
 	string difficultyString = "Difficulty" + intToStr(scenarioInfo->difficulty);
 
 	scenarioInfo->desc+= "\n";
-    scenarioInfo->desc+= lang.get("Difficulty") + ": " + lang.get(difficultyString) +"\n";
-    scenarioInfo->desc+= lang.get("Map") + ": " + formatString(scenarioInfo->mapName) + "\n";
-    scenarioInfo->desc+= lang.get("Tileset") + ": " + formatString(scenarioInfo->tilesetName) + "\n";
-	scenarioInfo->desc+= lang.get("TechTree") + ": " + formatString(scenarioInfo->techTreeName) + "\n";
+    scenarioInfo->desc+= lang.getString("Difficulty") + ": " + lang.getString(difficultyString) +"\n";
+    scenarioInfo->desc+= lang.getString("Map") + ": " + formatString(scenarioInfo->mapName) + "\n";
+    scenarioInfo->desc+= lang.getString("Tileset") + ": " + formatString(scenarioInfo->tilesetName) + "\n";
+	scenarioInfo->desc+= lang.getString("TechTree") + ": " + formatString(scenarioInfo->techTreeName) + "\n";
+
+	
+	//look for description and append it
+	lang.loadScenarioStrings(scenarioDir,scenarioName.c_str(),isTutorial);
+	//string tmp_description = lang.getScenarioString("DESCRIPTION");
+	string tmp_description = "";
+	if(lang.hasScenarioString("DESCRIPTION") == true) {
+		tmp_description = lang.getScenarioString("DESCRIPTION");
+	}
+	if( tmp_description != "") {
+		scenarioInfo->desc += lang.getString("Description") + ": \n" + tmp_description + "\n";
+	}
+
+	scenarioInfo->namei18n = "";
+	if(lang.hasScenarioString("SCENARIO_NAME") == true) {
+		scenarioInfo->namei18n =lang.getScenarioString("SCENARIO_NAME");
+		//printf("scenarioInfo->namei18n [%s]\n",scenarioInfo->namei18n.c_str());
+	}
+	else if(lang.hasScenarioString("TUTORIAL_NAME") == true) {
+		scenarioInfo->namei18n =lang.getScenarioString("TUTORIAL_NAME");
+		//printf("scenarioInfo->namei18n [%s]\n",scenarioInfo->namei18n.c_str());
+	}
 
 	if(scenarioNode->hasChild("fog-of-war") == true) {
 		if(scenarioNode->getChild("fog-of-war")->getAttribute("value")->getValue() == "explored") {
@@ -364,35 +435,35 @@ string Scenario::controllerTypeToStr(const ControlType &ct) {
 	Lang &lang= Lang::getInstance();
 	switch(ct) {
 		case ctCpuEasy:
-			controlString= lang.get("CpuEasy");
+			controlString= lang.getString("CpuEasy");
 			break;
 		case ctCpu:
-			controlString= lang.get("Cpu");
+			controlString= lang.getString("Cpu");
 			break;
 		case ctCpuUltra:
-			controlString= lang.get("CpuUltra");
+			controlString= lang.getString("CpuUltra");
 			break;
 		case ctCpuMega:
-			controlString= lang.get("CpuMega");
+			controlString= lang.getString("CpuMega");
 			break;
 		case ctNetwork:
-			controlString= lang.get("Network");
+			controlString= lang.getString("Network");
 			break;
 		case ctHuman:
-			controlString= lang.get("Human");
+			controlString= lang.getString("Human");
 			break;
 
 		case ctNetworkCpuEasy:
-			controlString= lang.get("NetworkCpuEasy");
+			controlString= lang.getString("NetworkCpuEasy");
 			break;
 		case ctNetworkCpu:
-			controlString= lang.get("NetworkCpu");
+			controlString= lang.getString("NetworkCpu");
 			break;
 		case ctNetworkCpuUltra:
-			controlString= lang.get("NetworkCpuUltra");
+			controlString= lang.getString("NetworkCpuUltra");
 			break;
 		case ctNetworkCpuMega:
-			controlString= lang.get("NetworkCpuMega");
+			controlString= lang.getString("NetworkCpuMega");
 			break;
 
 		default:
@@ -408,6 +479,11 @@ void Scenario::loadGameSettings(const vector<string> &dirList,
 		const ScenarioInfo *scenarioInfo, GameSettings *gameSettings,
 		string scenarioDescription) {
 	int factionCount= 0;
+
+	if(gameSettings->getGameUUID() == "") {
+		gameSettings->setGameUUID(getUUIDAsString());
+	}
+
 	gameSettings->setDescription(scenarioDescription);
 	gameSettings->setMap(scenarioInfo->mapName);
     gameSettings->setTileset(scenarioInfo->tilesetName);
@@ -427,6 +503,8 @@ void Scenario::loadGameSettings(const vector<string> &dirList,
 				if(gameSettings->getNetworkPlayerName(i) == "") {
 					gameSettings->setNetworkPlayerName(i,Config::getInstance().getString("NetPlayerName",Socket::getHostName().c_str()));
 				}
+				gameSettings->setNetworkPlayerUUID(i,Config::getInstance().getString("PlayerId",""));
+				gameSettings->setNetworkPlayerPlatform(i,getPlatformNameString());
 			}
 			else {
 				if(gameSettings->getNetworkPlayerName(i) == "") {
