@@ -48,7 +48,7 @@ class ScenarioInfo {
 public:
 	ScenarioInfo() {
 		difficulty = 0;
-		for(unsigned int i = 0; i < GameConstants::maxPlayers; ++i) {
+		for(unsigned int i = 0; i < (unsigned int)GameConstants::maxPlayers; ++i) {
 			factionControls[i] = ctClosed;
 			teams[i] = 0;
 			factionTypeNames[i] = "";
@@ -70,6 +70,7 @@ public:
 
 	    file = "";
 	    name = "";
+	    namei18n = "";
 	}
 	int difficulty;
     ControlType factionControls[GameConstants::maxPlayers];
@@ -92,6 +93,7 @@ public:
 
     string file;
     string name;
+    string namei18n;
 };
 
 // =====================================================
@@ -128,17 +130,18 @@ public:
 	Checksum load(const string &path);
 	Checksum * getChecksumValue() { return &checksumValue; }
 
-	int getScriptCount() const				{return scripts.size();}
+	int getScriptCount() const				{return (int)scripts.size();}
 	const Script* getScript(int i) const	{return &scripts[i];}
 
 	ScenarioInfo getInfo() const { return info; }
 
+	static bool isGameTutorial(string path);
 	static string getScenarioPath(const vector<string> dir, const string &scenarioName, bool getMatchingRootScenarioPathOnly=false);
 	static string getScenarioPath(const string &dir, const string &scenarioName);
 	static int getScenarioPathIndex(const vector<string> dirList, const string &scenarioName);
 	static string getScenarioDir(const vector<string> dir, const string &scenarioName);
 
-	static void loadScenarioInfo(string file, ScenarioInfo *scenarioInfo);
+	static void loadScenarioInfo(string file, ScenarioInfo *scenarioInfo,bool isTutorial);
 	static ControlType strToControllerType(const string &str);
 	static string controllerTypeToStr(const ControlType &ct);
 
