@@ -80,10 +80,11 @@ private:
 	PathFinder *pathFinder;
 	Game *game;
 	//RandomGen random;
+	Mutex *mutexAttackWarnings;
 	float attackWarnRange;
 	AttackWarnings attackWarnings;
 
-	Mutex mutexUnitRangeCellsLookupItemCache;
+	Mutex *mutexUnitRangeCellsLookupItemCache;
 	std::map<Vec2i, std::map<int, std::map<int, UnitRangeCellsLookupItem > > > UnitRangeCellsLookupItemCache;
 	//std::map<int,ExploredCellsLookupKey> ExploredCellsLookupItemCacheTimer;
 	//int UnitRangeCellsLookupItemCacheTimerCount;
@@ -101,7 +102,7 @@ public:
     ~UnitUpdater();
 
 	//update skills
-    void updateUnit(Unit *unit);
+    bool updateUnit(Unit *unit);
 
     //update commands
     void updateUnitCommand(Unit *unit, int frameIndex);
@@ -111,6 +112,7 @@ public:
     void updateAttackStopped(Unit *unit, int frameIndex);
     void updateBuild(Unit *unit, int frameIndex);
     void updateHarvest(Unit *unit, int frameIndex);
+    void updateHarvestEmergencyReturn(Unit *unit, int frameIndex);
     void updateRepair(Unit *unit, int frameIndex);
     void updateProduce(Unit *unit, int frameIndex);
     void updateUpgrade(Unit *unit, int frameIndex);
@@ -120,7 +122,7 @@ public:
 	void clearUnitPrecache(Unit *unit);
 	void removeUnitPrecache(Unit *unit);
 
-	inline unsigned int getAttackWarningCount() const { return attackWarnings.size(); }
+	inline unsigned int getAttackWarningCount() const { return (unsigned int)attackWarnings.size(); }
 	std::pair<bool,Unit *> unitBeingAttacked(const Unit *unit);
 	void unitBeingAttacked(std::pair<bool,Unit *> &result, const Unit *unit, const AttackSkillType *ast,float *currentDistToUnit=NULL);
 	vector<Unit*> enemyUnitsOnRange(const Unit *unit,const AttackSkillType *ast);
@@ -134,6 +136,8 @@ public:
 	void saveGame(XmlNode *rootNode);
 	void loadGame(const XmlNode *rootNode);
 
+	void clearCaches();
+
 private:
     //attack
     void hit(Unit *attacker);
@@ -143,10 +147,10 @@ private:
 
 	//misc
     bool searchForResource(Unit *unit, const HarvestCommandType *hct);
-    bool attackerOnSight(Unit *unit, Unit **enemyPtr);
-    bool attackableOnSight(Unit *unit, Unit **enemyPtr, const AttackSkillType *ast);
-    bool attackableOnRange(Unit *unit, Unit **enemyPtr, const AttackSkillType *ast);
-	bool unitOnRange(Unit *unit, int range, Unit **enemyPtr, const AttackSkillType *ast);
+    bool attackerOnSight(Unit *unit, Unit **enemyPtr, bool evalMode=false);
+    bool attackableOnSight(Unit *unit, Unit **enemyPtr, const AttackSkillType *ast, bool evalMode=false);
+    bool attackableOnRange(Unit *unit, Unit **enemyPtr, const AttackSkillType *ast, bool evalMode=false);
+	bool unitOnRange(Unit *unit, int range, Unit **enemyPtr, const AttackSkillType *ast,bool evalMode=false);
 	void enemiesAtDistance(const Unit *unit, const Unit *priorityUnit, int distance, vector<Unit*> &enemies);
 
 	Unit * findPeerUnitBuilder(Unit *unit);
