@@ -49,8 +49,11 @@ class Config;
 class Game;
 class GameSettings;
 class ScriptManager;
+
+namespace Shared{ namespace Sound{
 class StaticSound;
 class StrSound;
+}};
 
 // =====================================================
 // 	class World
@@ -66,25 +69,6 @@ public:
 	int teamIndex;
 };
 
-class ExploredCellsLookupItem {
-public:
-
-	int ExploredCellsLookupItemCacheTimerCountIndex;
-	std::vector<SurfaceCell *> exploredCellList;
-	std::vector<SurfaceCell *> visibleCellList;
-
-	static time_t lastDebug;
-};
-
-class FowAlphaCellsLookupItem {
-public:
-
-	std::vector<Vec2i> surfPosList;
-	std::vector<float> alphaList;
-
-	static time_t lastDebug;
-};
-
 class World {
 private:
 	typedef vector<Faction *> Factions;
@@ -93,19 +77,14 @@ private:
 	std::map<int,ExploredCellsLookupKey> ExploredCellsLookupItemCacheTimer;
 	int ExploredCellsLookupItemCacheTimerCount;
 
-	bool enableFowAlphaCellsLookupItemCache;
-	std::map<Vec2i, std::map<int, FowAlphaCellsLookupItem > > FowAlphaCellsLookupItemCache;
-
 public:
 	static const int generationArea= 100;
-	static const float airHeight;
 	static const int indirectSightRange= 5;
 
 private:
 
 	Map map;
 	Tileset tileset;
-	//TechTree techTree;
 	TechTree *techTree;
 	TimeFlow timeFlow;
 	Scenario scenario;
@@ -125,8 +104,7 @@ private:
 	int thisFactionIndex;
 	int thisTeamIndex;
 	int frameCount;
-	//int nextUnitId;
-	Mutex mutexFactionNextUnitId;
+	Mutex *mutexFactionNextUnitId;
 	std::map<int,int> mapFactionNextUnitId;
 
 	//config
@@ -134,14 +112,15 @@ private:
 	bool fogOfWar;
 	int fogOfWarSmoothingFrameSkip;
 	bool fogOfWarSmoothing;
+	int fogOfWarSkillTypeValue;
+
 	Game *game;
 	Chrono chronoPerfTimer;
 	bool perfTimerEnabled;
 
 	bool unitParticlesEnabled;
-	bool staggeredFactionUpdates;
-	std::map<string,StaticSound *> staticSoundList;
-	std::map<string,StrSound *> streamSoundList;
+	std::map<string,::Shared::Sound::StaticSound *> staticSoundList;
+	std::map<string,::Shared::Sound::StrSound *> streamSoundList;
 
 	uint32 nextCommandGroupId;
 
@@ -152,6 +131,17 @@ private:
 
 	const XmlNode *loadWorldNode;
 
+	MasterSlaveThreadController masterController;
+
+	bool originalGameFogOfWar;
+	std::map<int,std::pair<const Unit *,const FogOfWarSkillType *> > mapFogOfWarUnitList;
+
+	bool animatedTilesetObjectPosListLoaded;
+	std::vector<Vec2i> animatedTilesetObjectPosList;
+
+	bool cacheFowAlphaTexture;
+	bool cacheFowAlphaTextureFogOfWarValue;
+
 public:
 	World();
 	~World();
@@ -159,6 +149,10 @@ public:
 	void end(); //to die before selection does
 	void endScenario(); //to die before selection does
 
+	void addFogOfWarSkillType(const Unit *unit,const FogOfWarSkillType *fowst);
+	void removeFogOfWarSkillType(const Unit *unit);
+	bool removeFogOfWarSkillTypeFromList(const Unit *unit);
+
 	//get
 	inline int getMaxPlayers() const						{return map.getMaxPlayers();}
 	inline int getThisFactionIndex() const					{return thisFactionIndex;}
@@ -166,10 +160,10 @@ public:
 	inline int getThisTeamIndex() const					{return thisTeamIndex;}
 	inline void setThisTeamIndex(int team) 				{ thisTeamIndex=team;}
 
-	inline const Faction *getThisFaction() const			{return factions[thisFactionIndex];}
-	inline Faction *getThisFactionPtr() 					{return factions[thisFactionIndex];}
+	inline const Faction *getThisFaction() const			{return (thisFactionIndex >= 0 ? factions[thisFactionIndex] : NULL);}
+	inline Faction *getThisFactionPtr() 					{return (thisFactionIndex >= 0 ? factions[thisFactionIndex] : NULL);}
 
-	inline int getFactionCount() const						{return factions.size();}
+	inline int getFactionCount() const						{return (int)factions.size();}
 	inline const Map *getMap() const 						{return &map;}
 	inline Map *getMapPtr()  								{return &map;}
 
@@ -183,8 +177,10 @@ public:
 	inline Faction *getFaction(int i) 						{return factions[i];}
 	inline const Minimap *getMinimap() const				{return &minimap;}
 	inline Minimap *getMiniMapObject() 					{return &minimap;}
+
 	inline const Stats *getStats() const					{return &stats;};
 	inline Stats *getStats()								{return &stats;};
+
 	inline const WaterEffects *getWaterEffects() const		{return &waterEffects;}
 	inline const WaterEffects *getAttackEffects() const		{return &attackEffects;}
 	int getNextUnitId(Faction *faction);
@@ -199,7 +195,9 @@ public:
 			std::map<string,vector<pair<string, string> > > &loadedFileList);
 	void clearTileset();
 	Checksum loadTech(const vector<string> pathList, const string &techName,
-			set<string> &factions, Checksum* checksum,std::map<string,vector<pair<string, string> > > &loadedFileList);
+			set<string> &factions, Checksum* checksum,
+			std::map<string,vector<pair<string, string> > > &loadedFileList,
+			bool validationMode=false);
 	Checksum loadMap(const string &path, Checksum* checksum);
 	Checksum loadScenario(const string &path, Checksum* checksum,bool resetCurrentScenario=false,const XmlNode *rootNode=NULL);
 	void setQueuedScenario(string scenarioName,bool keepFactions);
@@ -211,6 +209,7 @@ public:
 	void update();
 	Unit* findUnitById(int id) const;
 	const UnitType* findUnitTypeById(const FactionType* factionType, int id);
+	const UnitType *findUnitTypeByName(const string factionName, const string unitTypeName);
 	bool placeUnit(const Vec2i &startLoc, int radius, Unit *unit, bool spaciated= false);
 	void moveUnitCells(Unit *unit);
 
@@ -283,19 +282,19 @@ public:
 	std::vector<std::string> validateResourceTypes();
 
 	void setFogOfWar(bool value);
+	bool getFogOfWar() const { return fogOfWar; }
+
 	std::string DumpWorldToLog(bool consoleBasicInfoOnly = false) const;
 
 	inline int getUpdateFps(int factionIndex) const {
 		int result = GameConstants::updateFps;
-		//if(factionIndex != -1 && staggeredFactionUpdates == true) {
-		//	result = (GameConstants::updateFps / GameConstants::maxPlayers);
-		//}
 		return result;
 	}
 	bool canTickWorld() const;
 
-	void exploreCells(const Vec2i &newPos, int sightRange, int teamIndex);
-	bool showWorldForPlayer(int factionIndex) const;
+	ExploredCellsLookupItem exploreCells(const Vec2i &newPos, int sightRange, int teamIndex);
+	void exploreCells(int teamIndex,ExploredCellsLookupItem &exploredCellsCache);
+	bool showWorldForPlayer(int factionIndex, bool excludeFogOfWarCheck=false) const;
 
 	inline UnitUpdater * getUnitUpdater() { return &unitUpdater; }
 
@@ -314,6 +313,10 @@ public:
 	void saveGame(XmlNode *rootNode);
 	void loadGame(const XmlNode *rootNode);
 
+	void clearCaches();
+	void refreshAllUnitExplorations();
+
+	bool factionLostGame(int factionIndex);
 private:
 
 	void initCells(bool fogOfWar);
@@ -322,18 +325,17 @@ private:
 	void initMinimap();
 	void initUnits();
 	void initMap();
-	//void initExplorationState();
 
 	//misc
 	void tick();
-	bool canTickFaction(int factionIdx);
-	int tickFactionIndex();
-	void computeFow(int factionIdxToTick=-1);
+	void computeFow();
 
 	void updateAllTilesetObjects();
 	void updateAllFactionUnits();
 	void underTakeDeadFactionUnits();
 	void updateAllFactionConsumableCosts();
+	void restoreExploredFogOfWarCells();
+
 };
 
 }}//end namespace
