@@ -53,6 +53,7 @@ enum StatusItems {
 	siBRUSH_TYPE,
 	siBRUSH_VALUE,
 	siBRUSH_RADIUS,
+	siPOS_VALUE,
 	siCOUNT
 };
 
@@ -79,6 +80,19 @@ const char *surface_descs[] = {
 	"Grass", "Sec. grass", "Road", "Stone", "Ground"
 };
 
+
+class MainToolBar : public wxToolBar {
+private:
+	DECLARE_EVENT_TABLE()
+
+public:
+	
+    MainToolBar(wxWindow *parent,
+              wxWindowID id) : wxToolBar(parent,id) {}
+
+	void onMouseMove(wxMouseEvent &event);
+};
+
 // =====================================================
 //	class MainWindow
 // =====================================================
@@ -130,6 +144,8 @@ private:
 
 		miViewResetZoomAndPos,
 		miViewGrid,
+		miViewHeightMap,
+		miHideWater,
 		miViewAbout,
 		miViewHelp,
 
@@ -196,6 +212,7 @@ public:
 	MainWindow(string appPath);
 	~MainWindow();
 
+	void refreshMapRender();
 	void init(string fname);
 
 	void onClose(wxCloseEvent &event);
@@ -238,6 +255,8 @@ public:
 
 	void onMenuViewResetZoomAndPos(wxCommandEvent &event);
 	void onMenuViewGrid(wxCommandEvent &event);
+	void onMenuViewHeightMap(wxCommandEvent &event);
+	void onMenuHideWater(wxCommandEvent &event);
 	void onMenuViewAbout(wxCommandEvent &event);
 	void onMenuViewHelp(wxCommandEvent &event);
 
