@@ -147,12 +147,26 @@ MapPreview *Program::map = NULL;
 Program::Program(int w, int h) {
 	cellSize = 6;
 	grid=false;
+	heightmap=false;
+	hideWater=false;
 	ofsetX = 0;
 	ofsetY = 0;
 	map = new MapPreview();
 	renderer.initMapSurface(w, h);
 }
 
+void Program::init() {
+	undoStack = ChangeStack();
+	redoStack = ChangeStack();
+	cellSize = 6;
+	grid=false;
+	heightmap=false;
+	hideWater=false;
+	ofsetX = 0;
+	ofsetY = 0;
+	map = NULL;
+}
+
 Program::~Program() {
 	delete map;
 	map = NULL;
@@ -169,6 +183,14 @@ int Program::getObject(int x, int y) {
 	}
 }
 
+int Program::getCellX(int x) {
+	return (x - ofsetX) / cellSize;
+}
+
+int Program::getCellY(int y) {
+	return (y + ofsetY) / cellSize;
+}
+
 int Program::getResource(int x, int y) {
 	int i=(x - ofsetX) / cellSize;
 	int j= (y + ofsetY) / cellSize;
@@ -240,7 +262,7 @@ bool Program::redo() {
 }
 
 void Program::renderMap(int w, int h) {
-	if(map) renderer.renderMap(map, ofsetX, ofsetY, w, h, cellSize, grid);
+	if(map) renderer.renderMap(map, ofsetX, ofsetY, w, h, cellSize, grid,heightmap,hideWater);
 }
 
 void Program::setRefAlt(int x, int y) {
@@ -619,6 +641,14 @@ bool Program::setGridOnOff() {
     grid=!grid;
     return grid;
 }
+bool Program::setHeightMapOnOff() {
+    heightmap=!heightmap;
+    return heightmap;
+}
+bool Program::setHideWaterOnOff() {
+    hideWater=!hideWater;
+    return hideWater;
+}
 
 void Program::setMapAdvanced(int altFactor, int waterLevel, int cliffLevel , int cameraHeight) {
 	if(map) map->setAdvanced(altFactor, waterLevel, cliffLevel, cameraHeight);
