@@ -10,13 +10,13 @@
 	define( 'DB_LINK', db_connect() );
 
 	// allow for automatic refreshing in web browser by appending '?refresh=VALUE', where VALUE is a numeric value in seconds.
-	define( 'REFRESH_INTERVAL', (int) $_GET['refresh'] );
+	if ( isset( $_GET['refresh'] ) ) { define( 'REFRESH_INTERVAL', (int) $_GET['refresh'] ); } else { define( 'REFRESH_INTERVAL', '' ); }
 
 	// allow for filtering by gameserver version
-	define( 'FILTER_VERSION', $_GET['version'] );
+	if ( isset( $_GET['version'] ) ) { define( 'FILTER_VERSION', $_GET['version'] ); } else { define( 'FILTER_VERSION', '' ); }
 
-	define( 'MGG_HOST', $_GET['mgg_host'] );
-	define( 'MGG_PORT', $_GET['mgg_port'] );
+	if ( isset( $_GET['mgg_host'] ) ) { define( 'MGG_HOST', $_GET['mgg_host'] ); } else { define( 'MGG_HOST', '' ); }
+	if ( isset( $_GET['mgg_port'] ) ) { define( 'MGG_PORT', $_GET['mgg_port'] ); } else { define( 'MGG_PORT', '' ); }
 
 	if ( MGG_HOST != '' ) {
 		$body = MGG_HOST . ':' . MGG_PORT;
@@ -31,7 +31,7 @@
 	// consider replacing this by a cron job
 	cleanupServerList();
 
-	$servers_in_db = mysql_query( 'SELECT * FROM glestserver ORDER BY status, connectedClients>0 DESC, (networkSlots - connectedClients) , ip DESC;' );
+	$servers_in_db = mysql_query( 'SELECT a.*,b.framesToCalculatePlaytime FROM glestserver a LEFT JOIN glestgamestats b ON a.gameUUID = b.gameUUID WHERE status <> 3 OR (status = 3 AND a.lasttime > DATE_add(NOW(), INTERVAL - ' . MAX_HOURS_OLD_GAMES . ' hour)) ORDER BY status, a.lasttime DESC, connectedClients > 0 DESC, (networkSlots - connectedClients) , ip DESC;' );
 	$all_servers = array();
 	while ( $server = mysql_fetch_array( $servers_in_db ) )
 	{
@@ -58,14 +58,15 @@
 	echo '		<meta charset="UTF-8" />' . PHP_EOL;
 	echo '		<title>' . htmlspecialchars( PRODUCT_NAME ) . ' gameservers</title>' . PHP_EOL;
 	echo '		<link rel="stylesheet" type="text/css" href="style/screen.css" />' . PHP_EOL;
-	echo '		<link rel="shortcut icon" type="image/x-icon" href="' . htmlspecialchars( strtolower( PRODUCT_NAME ) ) . '.ico" />' . PHP_EOL;
+	echo '		<link rel="shortcut icon" type="image/x-icon" href="images/' . htmlspecialchars( strtolower( PRODUCT_NAME ) ) . '.ico" />' . PHP_EOL;
 	echo '	</head>' . PHP_EOL;
 	echo '	<body>' . PHP_EOL;
-	echo '		<h1>' . htmlspecialchars( PRODUCT_NAME ) . ' gameservers</h1>' . PHP_EOL;
-	echo '		<table>' . PHP_EOL;
+	echo '		<h1><a href="' . htmlspecialchars( PRODUCT_URL ) . '">' . htmlspecialchars( PRODUCT_NAME ) . '</a> gameservers</h1>' . PHP_EOL;
+	echo '		<table id="gamesTable">' . PHP_EOL;
 	echo '			<tr>' . PHP_EOL;
 	echo '				<th title="glestVersion">Version</th>' . PHP_EOL;
 	echo '				<th title="status">Status</th>' . PHP_EOL;
+        echo '				<th title="gameDuration">Game Duration</th>' . PHP_EOL;
 	echo '				<th title="country">Country</th>' . PHP_EOL;
 	echo '				<th title="serverTitle">Title</th>' . PHP_EOL;
 	echo '				<th title="tech">Techtree</th>' . PHP_EOL;
@@ -77,18 +78,19 @@
 	echo '				<th title="ip">IPv4 address</th>' . PHP_EOL;
 	echo '				<th title="externalServerPort">Game protocol port</th>' . PHP_EOL;
 	echo '				<th title="platform">Platform</th>' . PHP_EOL;
-	echo '				<th title="binaryCompileDate">Build date</th>' . PHP_EOL;
+	echo '				<th title="lasttime">Play date</th>' . PHP_EOL;
 	echo '			</tr>' . PHP_EOL;
 
+        $games_with_stats = 0;
 	foreach( $all_servers as $server )
 	{
 		# Filter by version if requested
-		if ( FILTER_VERSION == $server['glestVersion'] OR FILTER_VERSION == '' )
+		if ( FILTER_VERSION == $server['glestVersion'] or FILTER_VERSION == '' )
 		{
 			echo "\t\t\t" . '<tr>' . PHP_EOL;
 
 			// glestVersion
-			printf( "\t\t\t\t<td><a href=\"?version=%s\">%s</a></td>%s", htmlspecialchars( $server['glestVersion'], ENT_QUOTES ), htmlspecialchars( $server['glestVersion'], ENT_QUOTES ), PHP_EOL );
+			printf( "\t\t\t\t<td><a href=\"?version=%s\" rel=\"nofollow\">%s</a></td>%s", htmlspecialchars( $server['glestVersion'], ENT_QUOTES ), htmlspecialchars( $server['glestVersion'], ENT_QUOTES ), PHP_EOL );
 
 			// status
 			$status_code = $server['status'];
@@ -122,7 +124,22 @@
 					$status_title = 'unknown';
 					$status_class = 'unknown';
 			}
-			printf( "\t\t\t\t<td title=\"%s\" class=\"%s\">%s</td>%s", $server['status'], $status_class, htmlspecialchars( $status_title, ENT_QUOTES ), PHP_EOL );
+
+                        if (($status_code == 2 || $status_code == 3) && $server['gameUUID'] != "")
+                        {
+                                $games_with_stats++;
+                                printf( "\t\t\t\t<td title=\"%s\" class=\"%s\"><a id=\"gameStats_%d\" href=\"#\" gameuuid=\"%s\">%s</a>", $server['status'], $status_class, $games_with_stats, $server['gameUUID'], htmlspecialchars( $status_title, ENT_QUOTES ) );
+                                printf( "</td>%s", PHP_EOL );
+                        }
+                        else
+                        {
+        			printf( "\t\t\t\t<td title=\"%s\" class=\"%s\">%s</td>%s", $server['status'], $status_class, htmlspecialchars( $status_title, ENT_QUOTES ), PHP_EOL );
+                        }
+
+		        // Game Stats
+                        $gameDuration = $server['framesToCalculatePlaytime'];
+                        $gameDuration = getTimeString($gameDuration);
+                        printf( "\t\t\t\t<td>%s</td>%s", htmlspecialchars( $gameDuration,        ENT_QUOTES ), PHP_EOL );
 
 			// country
 			if ( $server['country'] !== '' ) {
@@ -167,10 +184,18 @@
 			// platform
 			printf( "\t\t\t\t<td>%s</td>%s", htmlspecialchars( $server['platform'],           ENT_QUOTES ), PHP_EOL );
 
-			// binaryCompileDate
-			printf( "\t\t\t\t<td>%s</td>%s", htmlspecialchars( $server['binaryCompileDate'],  ENT_QUOTES ), PHP_EOL );
+			// game play time
+			printf( "\t\t\t\t<td>%s</td>%s", htmlspecialchars( $server['lasttime'],  ENT_QUOTES ), PHP_EOL );
 
 			echo "\t\t\t" . '</tr>' . PHP_EOL;
+
+                        if (($status_code == 2 || $status_code == 3) && $server['gameUUID'] != "")
+                        {
+                                //echo "\t\t\t" . '<tr>' . PHP_EOL;
+                                printf( "\t\t\t\t<tr width='100%%' class='fullyhide' id='content_row_%s'>%s", $server['gameUUID'], PHP_EOL );
+                                printf( "<td width='100%%' colspan='100'></td>%s", PHP_EOL );
+                                echo "\t\t\t" . '</tr>' . PHP_EOL;
+                        }
 		}
 	}
 
@@ -180,19 +205,23 @@
 	echo '			<br />' . PHP_EOL;
 	echo '		</p>' . PHP_EOL;
 
+        echo '		<script language="javascript" type="text/javascript" src="scripts/utils.js"></script>' . PHP_EOL;
+
 	if ( FILTER_VERSION != '' )
 	{
 		echo "\t\t<p>Filters active:</p>" . PHP_EOL;
 		echo "\t\t<ul>" . PHP_EOL;
-		printf( "\t\t\t<li>Version <a href=\"?\">%s</a></li>%s", htmlspecialchars( FILTER_VERSION, ENT_QUOTES ), PHP_EOL );
+		printf( "\t\t\t<li>Version <a href=\"?\" rel=\"nofollow\">%s</a></li>%s", htmlspecialchars( FILTER_VERSION, ENT_QUOTES ), PHP_EOL );
 		echo "\t\t</ul>" . PHP_EOL;
 	}
 
 	echo '		<p>Usage:</p>' . PHP_EOL;
-	echo '		<ul>' . PHP_EOL;
-	echo '			<li>You can have this page auto <a href="?refresh=60">refresh every 60 seconds</a> by appending <code>?refresh=60</code> to the URL. Minimum refresh time is 10 seconds.</li>' . PHP_EOL;
+	echo '		<ul id="noJsUsage">' . PHP_EOL;
+	echo '			<li>You can have this page auto <a href="?refresh=60" rel="nofollow">refresh every 60 seconds</a> by appending <code>?refresh=60</code> to the URL. Minimum refresh time is 10 seconds.</li>' . PHP_EOL;
 	echo '			<li>The parameters used by the masterserver API will display when you move your mouse pointer over any of the table headings.</li>' . PHP_EOL;
 	echo '		</ul>' . PHP_EOL;
+	echo '		<script src="scripts/json2.js"></script>' . PHP_EOL;
+	echo '		<script src="scripts/desktop_notifications.js"></script>' . PHP_EOL;
 	echo '	</body>' . PHP_EOL;
 	echo '</html>' . PHP_EOL;
 
