@@ -158,7 +158,11 @@ static inline T* readFromFileReaders(vector<FileReader<T> const *>* readers, con
 			FileReader<T> const * reader = *i;
 			ret = reader->read(file, filepath); //It is guaranteed that at least the filepath matches ...
 		}
+#if defined(WIN32)
+		catch (megaglest_runtime_error) {
+#else
 		catch (megaglest_runtime_error &ex) {
+#endif
 			throw;
 		}
 		catch (...) {
@@ -167,14 +171,14 @@ static inline T* readFromFileReaders(vector<FileReader<T> const *>* readers, con
 		if (ret != NULL) {
 			file.close();
 #if defined(WIN32) && !defined(__MINGW32__)
-			fclose(fp);
+			if(fp) fclose(fp);
 #endif
 			return ret;
 		}
 	}
 	file.close();
 #if defined(WIN32) && !defined(__MINGW32__)
-	fclose(fp);
+	if(fp) fclose(fp);
 #endif
 
 	return NULL;
@@ -206,7 +210,11 @@ static inline T* readFromFileReaders(vector<FileReader<T> const *>* readers, con
 			FileReader<T> const * reader = *i;
 			ret = reader->read(file, filepath, object); //It is guaranteed that at least the filepath matches ...
 		}
+#if defined(WIN32)
+		catch (megaglest_runtime_error) {
+#else
 		catch (megaglest_runtime_error &ex) {
+#endif
 			throw;
 		}
 		catch (...) {
@@ -215,14 +223,14 @@ static inline T* readFromFileReaders(vector<FileReader<T> const *>* readers, con
 		if (ret != NULL) {
 			file.close();
 #if defined(WIN32) && !defined(__MINGW32__)
-			fclose(fp);
+			if(fp) fclose(fp);
 #endif
 			return ret;
 		}
 	}
 	file.close();
 #if defined(WIN32) && !defined(__MINGW32__)
-	fclose(fp);
+	if(fp) fclose(fp);
 #endif
 	return NULL;
 }
@@ -329,7 +337,11 @@ bool FileReader<T>::canRead(ifstream& file) const {
 		delete wouldRead;
 		return ret;
 	}
+#if defined(WIN32)
+	catch (megaglest_runtime_error) {
+#else
 	catch (megaglest_runtime_error &ex) {
+#endif
 		throw;
 	}
 	catch (...) {
@@ -356,7 +368,7 @@ T* FileReader<T>::read(const string& filepath) const {
 	T* ret = read(file,filepath);
 	file.close();
 #if defined(WIN32) && !defined(__MINGW32__)
-	fclose(fp);
+	if(fp) fclose(fp);
 #endif
 
 	return ret;
@@ -381,7 +393,7 @@ T* FileReader<T>::read(const string& filepath, T* object) const {
 	T* ret = read(file,filepath,object);
 	file.close();
 #if defined(WIN32) && !defined(__MINGW32__)
-	fclose(fp);
+	if(fp) fclose(fp);
 #endif
 
 	return ret;
