@@ -20,6 +20,7 @@
 #include "randomgen.h"
 #include "xml_parser.h"
 #include "leak_dumper.h"
+#include "interpolation.h"
 
 using std::list;
 using Shared::Util::RandomGen;
@@ -75,7 +76,7 @@ public:
 //	class ParticleObserver
 // =====================================================
 
-class ParticleObserver{
+class ParticleObserver {
 public:
 	virtual ~ParticleObserver(){};
 	virtual void update(ParticleSystem *particleSystem)= 0;
@@ -83,6 +84,13 @@ public:
 	virtual void loadGame(const XmlNode *rootNode, void *genericData) = 0;
 };
 
+class ParticleOwner {
+public:
+	virtual ~ParticleOwner() {}
+	virtual void end(ParticleSystem *particleSystem)= 0;
+	virtual void logParticleInfo(string info)= 0;
+};
+
 // =====================================================
 //	class ParticleSystem
 // =====================================================
@@ -145,6 +153,7 @@ protected:
 	int alternations;
 	int particleSystemStartDelay;
 	ParticleObserver *particleObserver;
+	ParticleOwner *particleOwner;
 
 public:
 	//conmstructor and destructor
@@ -198,15 +207,21 @@ public:
 	virtual void fade();
 	int isEmpty() const;
 	
+	virtual void setParticleOwner(ParticleOwner *particleOwner) { this->particleOwner = particleOwner;}
+	virtual ParticleOwner * getParticleOwner() { return this->particleOwner;}
+	virtual void callParticleOwnerEnd(ParticleSystem *particleSystem);
+
 	//children
 	virtual int getChildCount() { return 0; }
 	virtual ParticleSystem* getChild(int i);
 
-	string toString() const;
+	virtual string toString() const;
 
 	virtual void saveGame(XmlNode *rootNode);
 	virtual void loadGame(const XmlNode *rootNode);
 
+	virtual Checksum getCRC();
+
 protected:
 	//protected
 	Particle *createParticle();
@@ -242,6 +257,10 @@ public:
 
 	virtual void saveGame(XmlNode *rootNode);
 	virtual void loadGame(const XmlNode *rootNode);
+
+	virtual string toString() const;
+
+	virtual Checksum getCRC();
 };
 
 // =====================================================
@@ -279,6 +298,10 @@ public:
 	virtual void saveGame(XmlNode *rootNode);
 	virtual void loadGame(const XmlNode *rootNode);
 
+	virtual string toString() const;
+
+	virtual Checksum getCRC();
+
 protected:
 	typedef std::vector<UnitParticleSystem*> Children;
 	Children children;
@@ -312,8 +335,8 @@ private:
 	Vec3f fixedAddition;
     Vec3f oldPosition;
     bool energyUp;
-	float startTime;
-	float endTime;
+    float startTime;
+    float endTime;
     
 public:
 	enum Shape{
@@ -329,6 +352,8 @@ public:
 	float sizeNoEnergy;
 	float gravity;
 	float rotation;
+	const Model *unitModel;
+	string meshName;
 	bool isVisibleAtNight;
 	bool isVisibleAtDay;
 	bool isDaylightAffected;
@@ -369,6 +394,8 @@ public:
 	void setSizeNoEnergy(float sizeNoEnergy)			{this->sizeNoEnergy= sizeNoEnergy;}
 	void setGravity(float gravity)						{this->gravity= gravity;}
 	void setRotation(float rotation);
+	const void setUnitModel(const Model* unitModel)					{this->unitModel= unitModel;}
+	void setMeshName(string meshName)						{this->meshName= meshName;}
 	void setRelative(bool relative)						{this->relative= relative;}
 	void setRelativeDirection(bool relativeDirection)	{this->relativeDirection= relativeDirection;}
 	void setFixed(bool fixed)							{this->fixed= fixed;}
@@ -390,6 +417,10 @@ public:
 
 	virtual void saveGame(XmlNode *rootNode);
 	virtual void loadGame(const XmlNode *rootNode);
+
+	virtual string toString() const;
+
+	virtual Checksum getCRC();
 };
 
 // =====================================================
@@ -412,7 +443,11 @@ public:
 	virtual bool deathTest(Particle *p);
 
 	void setRadius(float radius);
-	void setWind(float windAngle, float windSpeed);	
+	void setWind(float windAngle, float windSpeed);
+
+	virtual string toString() const;
+
+	virtual Checksum getCRC();
 };
 
 // =====================================================
@@ -433,7 +468,11 @@ public:
 	virtual bool deathTest(Particle *p);
 
 	void setRadius(float radius);
-	void setWind(float windAngle, float windSpeed);	
+	void setWind(float windAngle, float windSpeed);
+
+	virtual string toString() const;
+
+	virtual Checksum getCRC();
 };
 
 // ===========================================================================
@@ -459,6 +498,10 @@ public:
 
 	virtual void saveGame(XmlNode *rootNode);
 	virtual void loadGame(const XmlNode *rootNode);
+
+	virtual string toString() const;
+
+	virtual Checksum getCRC();
 };
 
 // =====================================================
@@ -493,7 +536,8 @@ private:
 	//parabolic
 	float trajectoryScale;
 	float trajectoryFrequency;
-	
+
+	float arriveDestinationDistance;
 	void rotateChildren();
 
 public:
@@ -519,6 +563,10 @@ public:
 
 	virtual void saveGame(XmlNode *rootNode);
 	virtual void loadGame(const XmlNode *rootNode);
+
+	virtual string toString() const;
+
+	virtual Checksum getCRC();
 };
 
 // =====================================================
@@ -558,7 +606,10 @@ public:
 
 	virtual void saveGame(XmlNode *rootNode);
 	virtual void loadGame(const XmlNode *rootNode);
+
+	virtual string toString() const;
 	
+	virtual Checksum getCRC();
 };
 
 // =====================================================
@@ -581,6 +632,7 @@ public:
 	void cleanupUnitParticleSystems(vector<UnitParticleSystem *> &particleSystems);
 	int findParticleSystems(ParticleSystem *psFind, const vector<ParticleSystem *> &particleSystems) const;
 	bool validateParticleSystemStillExists(ParticleSystem * particleSystem) const;
+	void removeParticleSystemsForParticleOwner(ParticleOwner * particleOwner);
 	bool hasActiveParticleSystem(ParticleSystem::ParticleSystemType type) const;
 }; 
 
