@@ -123,7 +123,7 @@ private:
 	Uint32 lastStartCount;
 	Uint32 lastTickCount;
 	int64  lastResult;
-	int lastMultiplier;
+	int64 lastMultiplier;
 	bool lastStopped;
 
 public:
@@ -135,11 +135,12 @@ public:
 	int64 getMillis();
 	int64 getSeconds();
 
+	bool isStarted() const;
     static int64 getCurTicks();
     static int64 getCurMillis();
 
 private:
-	int64 queryCounter(int multiplier);
+	int64 queryCounter(int64 multiplier);
 };
 
 // =====================================================
@@ -185,6 +186,11 @@ void findAll(const vector<string> &paths, const string &fileFilter, vector<strin
 void findAll(const string &path, vector<string> &results, bool cutExtension=false, bool errorOnNotFound=true);
 vector<string> getFolderTreeContentsListRecursively(const string &path, const string &filterFileExt, bool includeFolders=false, vector<string> *recursiveMap=NULL);
 
+string getGameVersion();
+string getGameGITVersion();
+void setGameVersion(string version);
+void setGameGITVersion(string git);
+
 string getCRCCacheFilePath();
 void setCRCCacheFilePath(string path);
 
@@ -213,7 +219,7 @@ string extractExtension(const string& filename);
 
 void getFullscreenVideoModes(vector<ModeInfo> *modeinfos,bool isFullscreen);
 void getFullscreenVideoInfo(int &colorBits,int &screenWidth,int &screenHeight,bool isFullscreen);
-bool changeVideoMode(int resH, int resW, int colorBits, int refreshFrequency);
+void changeVideoModeFullScreen(bool value);
 void restoreVideoMode(bool exitingApp=false);
 
 bool StartsWith(const std::string &str, const std::string &key);
@@ -246,6 +252,8 @@ bool isKeyDown(int virtualKey);
 //bool isKeyDown(SDLKey key);
 string getCommandLine();
 
+string getUserHome();
+
 #define SPACES " "
 
 inline string trim_at_delim (const string & s, const string &t)  {
@@ -284,6 +292,9 @@ inline string trim (const string & s, const string & t = SPACES) {
 
 string getFullFileArchiveExtractCommand(string fileArchiveExtractCommand,
 		string fileArchiveExtractCommandParameters, string outputpath, string archivename);
+string getFullFileArchiveCompressCommand(string fileArchiveCompressCommand,
+		string fileArchiveCompressCommandParameters, string archivename, string archivefiles);
+
 bool executeShellCommand(string cmd,int expectedResult=IGNORE_CMD_RESULT_VALUE,ShellCommandOutputCallbackInterface *cb=NULL);
 string executable_path(string exeName,bool includeExeNameInPath=false);
 
@@ -293,6 +304,8 @@ bool valid_utf8_file(const char* file_name);
 
 string getFileTextContents(string path);
 
+string safeCharPtrCopy(const char *ptr, int maxLength=-1);
+
 class ValueCheckerVault {
 
 protected:
