@@ -55,8 +55,11 @@ class IRCThread : public BaseThread
 public:
 	static bool debugEnabled;
 protected:
+	static const char *globalCacheContainerName;
 
     std::vector<string> argv;
+
+    Mutex mutexIRCSession;
     irc_session_t *ircSession;
 
     string execute_cmd_onconnect;
@@ -68,7 +71,10 @@ protected:
 	string playerName;
 
     bool hasJoinedChannel;
+
+    Mutex mutexEventDataDone;
     bool eventDataDone;
+
     Mutex mutexNickList;
     time_t lastNickListUpdate;
     std::vector<string> eventData;
@@ -78,13 +84,18 @@ protected:
 
     bool wantToLeaveChannel;
 
+    int irc_run_session(irc_session_t * session);
+
 public:
 
 	IRCThread(const std::vector<string> &argv,IRCCallbackInterface *callbackObj);
+	virtual ~IRCThread();
     virtual void execute();
     virtual void signalQuit();
     virtual bool shutdownAndWait();
 
+    static void setGlobalCacheContainerName(const char *name) { globalCacheContainerName = name; }
+
     void setPlayerName(string value) { playerName = value; }
     string getPlayerName() const { return playerName; }
 
@@ -92,12 +103,12 @@ public:
 
     void SendIRCCmdMessage(string target, string msg);
     std::vector<string> getNickList();
-    bool isConnected();
+    bool isConnected(bool mutexLockRequired=true);
 
     std::vector<string> GetIRCConnectedNickList(string target, bool waitForCompletion);
 
-    bool getEventDataDone() const { return eventDataDone; }
-    void setEventDataDone(bool value) { eventDataDone=value; }
+    bool getEventDataDone();
+    void setEventDataDone(bool value);
 
     bool getHasJoinedChannel() const { return hasJoinedChannel; }
     void setHasJoinedChannel(bool value) { hasJoinedChannel=value; }
@@ -124,6 +135,7 @@ public:
     void joinChannel();
     void leaveChannel();
     void connectToHost();
+    void disconnect();
 };
 
 }}//end namespace
