@@ -42,6 +42,7 @@ const char  *GAME_ARGS[] = {
 	"--opengl-info",
 	"--sdl-info",
 	"--lua-info",
+	"--lua-debug",
 	"--curl-info",
 	"--xerces-info",
 
@@ -50,6 +51,8 @@ const char  *GAME_ARGS[] = {
 	"--validate-scenario",
 	"--validate-tileset",
 
+	"--translate-techtrees",
+
 	"--list-maps",
 	"--list-techtrees",
 	"--list-scenarios",
@@ -71,6 +74,7 @@ const char  *GAME_ARGS[] = {
 	"--disable-backtrace",
 	"--disable-sigsegv-handler",
 	"--disable-vbo",
+	"--disable-vertex-interpolation",
 	"--disable-sound",
 	"--enable-legacyfonts",
 	"--force-ftglfonts",
@@ -89,8 +93,12 @@ const char  *GAME_ARGS[] = {
 	"--disable-opengl-checks",
 	"--disable-streflop-checks",
 	"--debug-network-packets",
+	"--debug-network-packet-sizes",
+	"--debug-network-packet-stats",
 	"--enable-new-protocol",
 
+	"--create-data-archives",
+
 	"--verbose"
 
 };
@@ -115,6 +123,7 @@ enum GAME_ARG_TYPE {
 	GAME_ARG_OPENGL_INFO,
 	GAME_ARG_SDL_INFO,
 	GAME_ARG_LUA_INFO,
+	GAME_ARG_LUA_DEBUG,
 	GAME_ARG_CURL_INFO,
 	GAME_ARG_XERCES_INFO,
 
@@ -123,6 +132,8 @@ enum GAME_ARG_TYPE {
 	GAME_ARG_VALIDATE_SCENARIO,
 	GAME_ARG_VALIDATE_TILESET,
 
+	GAME_ARG_TRANSLATE_TECHTREES,
+
 	GAME_ARG_LIST_MAPS,
 	GAME_ARG_LIST_TECHTRESS,
 	GAME_ARG_LIST_SCENARIOS,
@@ -146,6 +157,7 @@ enum GAME_ARG_TYPE {
 	GAME_ARG_DISABLE_BACKTRACE,
 	GAME_ARG_DISABLE_SIGSEGV_HANDLER,
 	GAME_ARG_DISABLE_VBO,
+	GAME_ARG_DISABLE_VERTEX_INTERPOLATION,
 	GAME_ARG_DISABLE_SOUND,
 	GAME_ARG_ENABLE_LEGACYFONTS,
 	GAME_ARG_FORCE_FTGLFONTS,
@@ -165,8 +177,12 @@ enum GAME_ARG_TYPE {
 	GAME_ARG_DISABLE_STREFLOP_CAPS_CHECK,
 
 	GAME_ARG_DEBUG_NETWORK_PACKETS,
+	GAME_ARG_DEBUG_NETWORK_PACKET_SIZES,
+	GAME_ARG_DEBUG_NETWORK_PACKET_STATS,
 	GAME_ARG_ENABLE_NEW_PROTOCOL,
 
+	GAME_ARG_CREATE_DATA_ARCHIVES,
+
 	GAME_ARG_VERBOSE_MODE,
 
 	GAME_ARG_END
@@ -242,10 +258,11 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n%s\t\t\tdisplays your video driver's OpenGL info.",GAME_ARGS[GAME_ARG_OPENGL_INFO]);
 	printf("\n%s\t\t\tdisplays your SDL version information.",GAME_ARGS[GAME_ARG_SDL_INFO]);
 	printf("\n%s\t\t\tdisplays your LUA version information.",GAME_ARGS[GAME_ARG_LUA_INFO]);
+	printf("\n%s\t\t\tdisplays LUA debug information.",GAME_ARGS[GAME_ARG_LUA_DEBUG]);
 	printf("\n%s\t\t\tdisplays your CURL version information.",GAME_ARGS[GAME_ARG_CURL_INFO]);
 	printf("\n%s\t\t\tdisplays your XERCES version information.",GAME_ARGS[GAME_ARG_XERCES_INFO]);
 
-	printf("\n%s=x=purgeunused=purgeduplicates=svndelete=hideduplicates",GAME_ARGS[GAME_ARG_VALIDATE_TECHTREES]);
+	printf("\n%s=x=purgeunused=purgeduplicates=gitdelete=hideduplicates",GAME_ARGS[GAME_ARG_VALIDATE_TECHTREES]);
 	printf("\n                     \t\tdisplay a report detailing any known problems");
 	printf("\n                     \t\trelated to your selected techtrees game data.");
 	printf("\n                     \t\tWhere x is a comma-delimited list of techtrees");
@@ -257,9 +274,9 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\tWhere purgeduplicates is an optional parameter");
 	printf("\n                     \t\t      telling the validation to merge");
 	printf("\n                     \t\t      duplicate files in the techtree.");
-	printf("\n                     \t\tWhere svndelete is an optional parameter");
+	printf("\n                     \t\tWhere gitdelete is an optional parameter");
 	printf("\n                     \t\t      telling the validation to call");
-	printf("\n                     \t\t      svn delete on duplicate / unused");
+	printf("\n                     \t\t      git rm on duplicate / unused");
 	printf("\n                     \t\t      files in the techtree.");
 	printf("\n                     \t\tWhere hideduplicates is an optional parameter");
 	printf("\n                     \t\t      telling the validation to NOT SHOW");
@@ -288,7 +305,7 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\texample:");
 	printf("\n                     %s %s=tech,egypt",extractFileFromDirectoryPath(argv0).c_str(),GAME_ARGS[GAME_ARG_VALIDATE_FACTIONS]);
 
-	printf("\n%s=x=purgeunused=svndelete",GAME_ARGS[GAME_ARG_VALIDATE_SCENARIO]);
+	printf("\n%s=x=purgeunused=gitdelete",GAME_ARGS[GAME_ARG_VALIDATE_SCENARIO]);
 	printf("\n                     \t\tdisplay a report detailing any known problems");
 	printf("\n                     \t\trelated to your selected scenario game data.");
 	printf("\n                     \t\tWhere x is a single scenario to validate.");
@@ -298,7 +315,7 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\texample:");
 	printf("\n                     %s %s=stranded",extractFileFromDirectoryPath(argv0).c_str(),GAME_ARGS[GAME_ARG_VALIDATE_SCENARIO]);
 
-	printf("\n%s=x=purgeunused=svndelete",GAME_ARGS[GAME_ARG_VALIDATE_TILESET]);
+	printf("\n%s=x=purgeunused=gitdelete",GAME_ARGS[GAME_ARG_VALIDATE_TILESET]);
 	printf("\n                     \t\tdisplay a report detailing any known problems");
 	printf("\n                     \t\trelated to your selected tileset game data.");
 	printf("\n                     \t\tWhere x is a single tileset to validate.");
@@ -308,6 +325,11 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\texample:");
 	printf("\n                     %s %s=desert2",extractFileFromDirectoryPath(argv0).c_str(),GAME_ARGS[GAME_ARG_VALIDATE_TILESET]);
 
+	printf("\n%s=x",GAME_ARGS[GAME_ARG_TRANSLATE_TECHTREES]);
+	printf("\n                     \t\tProduces a default lng file for the specified techtree");
+	printf("\n                     \t\tto prepare for translation into other languages.");
+	printf("\n                     \t\tWhere x is a techtree name");
+
 	printf("\n%s=x",GAME_ARGS[GAME_ARG_LIST_MAPS]);
 	printf("\n                     \t\tdisplay a list of game content: maps");
 	printf("\n                     \t\twhere x is an optional name filter.");
@@ -413,6 +435,7 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 
 
 	printf("\n%s\t\t\tdisables trying to use Vertex Buffer Objects.",GAME_ARGS[GAME_ARG_DISABLE_VBO]);
+	printf("\n%s\t\t\tdisables interpolating animations to make them smoother.",GAME_ARGS[GAME_ARG_DISABLE_VERTEX_INTERPOLATION]);
 	printf("\n%s\t\t\tdisables the sound system.",GAME_ARGS[GAME_ARG_DISABLE_SOUND]);
 
 	printf("\n%s\t\tenables using the legacy font system.",GAME_ARGS[GAME_ARG_ENABLE_LEGACYFONTS]);
@@ -468,6 +491,12 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n%s\t\tdisables opengl capability checks (for corrupt or flaky video drivers).",GAME_ARGS[GAME_ARG_DISABLE_OPENGL_CAPS_CHECK]);
 
 
+	printf("\n%s=x=y\t\t\tcompress selected game data into archives for network sharing.",GAME_ARGS[GAME_ARG_CREATE_DATA_ARCHIVES]);
+	printf("\n                     \t\tWhere x is one of the following data items to compress.");
+	printf("\n                     \t\ttechtrees, tilesets or all.");
+	printf("\n                     \t\tWhere y = include_main to include main (non mod) data.");
+	printf("\n                     \t\texample: %s %s=all",extractFileFromDirectoryPath(argv0).c_str(),GAME_ARGS[GAME_ARG_CREATE_DATA_ARCHIVES]);
+
 	printf("\n%s\t\t\tdisplays verbose information in the console.",GAME_ARGS[GAME_ARG_VERBOSE_MODE]);
 
 	printf("\n\n");
@@ -479,11 +508,11 @@ bool hasCommandArgument(int argc, char** argv,const string argName, int *foundIn
 	if(foundIndex != NULL) {
 		*foundIndex = -1;
 	}
-	int compareLen = strlen(argName.c_str());
+	int compareLen = (int)strlen(argName.c_str());
 
 	for(int idx = startLookupIndex; idx < argc; idx++) {
 		if(useArgParamLen == true) {
-			compareLen = strlen(argv[idx]);
+			compareLen = (int)strlen(argv[idx]);
 		}
 		if(_strnicmp(argName.c_str(),argv[idx],compareLen) == 0) {
 			result = true;
