@@ -121,6 +121,7 @@ private:
     static void setKeystate(SDL_keysym state)					{ keystate = state; }
 
     //static bool masterserverMode;
+    static map<wchar_t,bool> mapAllowedKeys;
 
 protected:
 	int w, h;
@@ -132,6 +133,7 @@ protected:
 public:
 	static bool handleEvent();
 	static void revertMousePos();
+	static Vec2i getOldMousePos();
 	static bool isKeyDown() { return isKeyPressedDown; }
 	static void setupGraphicsScreen(int depthBits=-1, int stencilBits=-1, bool hardware_acceleration=false, bool fullscreen_anti_aliasing=false);
 	static const bool getIsFullScreen() { return isFullScreen; }
@@ -143,6 +145,14 @@ public:
 	Window();
 	virtual ~Window();
 
+	static void addAllowedKeys(string keyList);
+	static void clearAllowedKeys();
+	static bool isAllowedKey(wchar_t key);
+
+	virtual bool ChangeVideoMode(bool preserveContext,int resWidth, int resHeight,
+			bool fullscreenWindow, int colorBits, int depthBits, int stencilBits,
+            bool hardware_acceleration, bool fullscreen_anti_aliasing,
+            float gammaValue) = 0;
 	//static void setMasterserverMode(bool value) { Window::masterserverMode = value;}
 	//static bool getMasterserverMode() { return Window::masterserverMode;}
 
@@ -209,6 +219,8 @@ private:
 	//static char getKey(SDL_keysym keysym, bool skipSpecialKeys=false);
 	//static char getNormalKey(SDL_keysym keysym,bool skipSpecialKeys=false);
 	static void toggleFullscreen();
+
+	static wchar_t convertStringtoSDLKey(const string &value);
 };
 
 bool isKeyPressed(SDLKey compareKey, SDL_KeyboardEvent input, vector<int> modifiersToCheck);
