@@ -47,15 +47,30 @@ private:
 
 	string path;
 	static string applicationPath;
+	static string applicationDataPath;
 	static string gameVersion;
 
+	static string techtreePath;
+	static string scenarioPath;
+	static string tutorialPath;
+
 public:
 	static void setApplicationPath(string value) { applicationPath=value; }
 	static string getApplicationPath() { return applicationPath; }
 
+	static void setApplicationDataPath(string value) { applicationDataPath=value; }
+	static string getApplicationDataPath() { return applicationDataPath; }
+
 	static void setGameVersion(string value) { gameVersion=value; }
 	static string getGameVersion() { return gameVersion; }
 
+	static void setTechtreePath(string value) { techtreePath=value; }
+	static string getTechtreePath() { return techtreePath; }
+	static void setScenarioPath(string value) { scenarioPath=value; }
+	static string getScenarioPath() { return scenarioPath; }
+	static void setTutorialPath(string value) { tutorialPath=value; }
+	static string getTutorialPath() { return tutorialPath; }
+
 	void clear();
 	void load(const string &path,bool clearCurrentProperties=true);
 	void save(const string &path);
@@ -85,7 +100,7 @@ public:
 
 	bool hasString(const string &key) const;
 
-	static bool applyTagsToValue(string &value, std::map<string,string> *mapTagReplacementValues=NULL);
+	static bool applyTagsToValue(string &value, const std::map<string,string> *mapTagReplacementValues=NULL);
 	static std::map<string,string> getTagReplacementValues(std::map<string,string> *mapExtraTagReplacementValues=NULL);
 
 	string getpath() const { return path;}
