@@ -19,6 +19,7 @@
 #include <curl/curl.h>
 #include <cstdio>
 #include <assert.h>
+#include <stdexcept>
 #include "leak_dumper.h"
 
 using std::string;
@@ -51,7 +52,7 @@ public:
 	static uint64 getFlags() { return flags; }
 
 	static void setFlag(GlobalStaticFlagTypes flag) { flags |= flag; }
-	static bool isFlagSet(GlobalStaticFlagTypes flag) { return (flags & flag) == flag; }
+	static bool isFlagSet(GlobalStaticFlagTypes flag) { return (flags & (unsigned int)flag) == (unsigned int)flag; }
 
 protected:
 	static bool isNonGraphicalMode;
@@ -166,6 +167,9 @@ public:
 			if(result != NULL) {
 				return *result;
 			}
+			else if(SystemFlags::debugLogFileList == NULL) {
+				throw std::runtime_error("unknown return value for SystemFlagsType!");
+			}
 		}
 
 		return (*debugLogFileList)[type];
@@ -228,12 +232,20 @@ void copyStringToBuffer(char *buffer, int bufferSize, const string& s);
 //numeric fcs
 int clamp(int value, int min, int max);
 float clamp(float value, float min, float max);
+int64 clamp(int64 value, int64 min, int64 max);
 float saturate(float value);
 int round(float f);
 
 //misc
 bool checkVersionComptability(string clientVersionString, string serverVersionString);
 
+template<typename T>
+void enforceMinimumValue(T minValue, T &value) {
+	if(value < minValue) {
+		value = minValue;
+	}
+}
+
 template<typename T>
 void deleteValues(T beginIt, T endIt){
 	for(T it= beginIt; it!=endIt; ++it){
