@@ -28,8 +28,16 @@ namespace XERCES_CPP_NAMESPACE{
 	class DOMDocument;
 	class DOMNode;
 	class DOMElement;
+
+#if XERCES_VERSION_MAJOR < 3
+	class DOMBuilder;
+#else
+	class DOMLSParser;
+#endif
 }
 
+XERCES_CPP_NAMESPACE_USE
+
 namespace Shared { namespace Xml {
 
 enum xml_engine_parser_type {
@@ -37,7 +45,6 @@ enum xml_engine_parser_type {
 	XML_RAPIDXML_ENGINE = 1
 } ;
 
-static xml_engine_parser_type DEFAULT_XML_ENGINE = XML_RAPIDXML_ENGINE;
 const int strSize= 8094;
 
 class XmlIo;
@@ -51,37 +58,57 @@ class XmlAttribute;
 ///	Wrapper for Xerces C++
 // =====================================================
 
-class XmlIo{
+class XmlIo {
 private:
 	static bool initialized;
 	XERCES_CPP_NAMESPACE::DOMImplementation *implementation;
+#if XERCES_VERSION_MAJOR < 3
+	XERCES_CPP_NAMESPACE::DOMBuilder *parser;
+#else
+	XERCES_CPP_NAMESPACE::DOMLSParser *parser;
+#endif
 
-private:
+protected:
 	XmlIo();
+	virtual ~XmlIo();
+
+	void init();
+
+#if XERCES_VERSION_MAJOR < 3
+	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * getRootDOMDocument(const string &path, DOMBuilder *parser, bool noValidation);
+#else
+	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * getRootDOMDocument(const string &path, DOMLSParser *parser, bool noValidation);
+#endif
+
+	DOMNode *loadDOMNode(const string &path, bool noValidation=false);
+	virtual void releaseDOMParser();
 
 public:
 	static XmlIo &getInstance();
-	~XmlIo();
+
+	static bool isInitialized();
 	void cleanup();
 
-	XmlNode *load(const string &path, std::map<string,string> mapTagReplacementValues,bool noValidation=false, bool skipStackCheck=false);
+	XmlNode *load(const string &path, const std::map<string,string> &mapTagReplacementValues,bool noValidation=false,bool skipStackTrace=false);
 	void save(const string &path, const XmlNode *node);
 };
 
 class XmlIoRapid {
 private:
 	static bool initialized;
-	xml_document<> *doc;
 
 private:
 	XmlIoRapid();
+	void init();
 
 public:
 	static XmlIoRapid &getInstance();
 	~XmlIoRapid();
+
+	static bool isInitialized();
 	void cleanup();
 
-	XmlNode *load(const string &path, std::map<string,string> mapTagReplacementValues,bool noValidation=false,bool skipStackCheck=false);
+	XmlNode *load(const string &path, const std::map<string,string> &mapTagReplacementValues,bool noValidation=false,bool skipStackTrace=false);
 	void save(const string &path, const XmlNode *node);
 };
 
@@ -98,13 +125,14 @@ private:
 private:
 	XmlTree(XmlTree&);
 	void operator =(XmlTree&);
+	void clearRootNode();
 
 public:
-	XmlTree(xml_engine_parser_type engine_type = DEFAULT_XML_ENGINE);
+	XmlTree(xml_engine_parser_type engine_type = XML_RAPIDXML_ENGINE);
 	~XmlTree();
 
 	void init(const string &name);
-	void load(const string &path, std::map<string,string> mapTagReplacementValues, bool noValidation=false,bool skipStackCheck=false);
+	void load(const string &path, const std::map<string,string> &mapTagReplacementValues, bool noValidation=false,bool skipStackCheck=false,bool skipStackTrace=false);
 	void save(const string &path);
 
 	XmlNode *getRootNode() const	{return rootNode;}
@@ -126,9 +154,12 @@ private:
 	XmlNode(XmlNode&);
 	void operator =(XmlNode&);
 
+	string getTreeString() const;
+	bool hasChildNoSuper(const string& childName) const;
+
 public:
-	XmlNode(XERCES_CPP_NAMESPACE::DOMNode *node, std::map<string,string> mapTagReplacementValues);
-	XmlNode(xml_node<> *node, std::map<string,string> mapTagReplacementValues);
+	XmlNode(XERCES_CPP_NAMESPACE::DOMNode *node, const std::map<string,string> &mapTagReplacementValues);
+	XmlNode(xml_node<> *node, const std::map<string,string> &mapTagReplacementValues);
 	XmlNode(const string &name);
 	~XmlNode();
 	
@@ -145,21 +176,19 @@ public:
 
 	XmlNode *getChild(unsigned int i) const;
 	XmlNode *getChild(const string &childName, unsigned int childIndex=0) const;
+	XmlNode *getChildWithAliases(vector<string> childNameList, unsigned int childIndex=0) const;
 	vector<XmlNode *> getChildList(const string &childName) const;
 	bool hasChildAtIndex(const string &childName, int childIndex=0) const;
 	bool hasChild(const string &childName) const;
-	XmlNode *getParent() const;
+	bool hasChildWithAliases(vector<string> childNameList) const;
+	int clearChild(const string &childName);
 
 
-	XmlNode *addChild(const string &name);
-	XmlAttribute *addAttribute(const string &name, const string &value, std::map<string,string> mapTagReplacementValues);
+	XmlNode *addChild(const string &name, const string text = "");
+	XmlAttribute *addAttribute(const string &name, const string &value, const std::map<string,string> &mapTagReplacementValues);
 
 	XERCES_CPP_NAMESPACE::DOMElement *buildElement(XERCES_CPP_NAMESPACE::DOMDocument *document) const;
 	xml_node<>* buildElement(xml_document<> *document) const;
-
-private:
-	string getTreeString() const;
-	bool hasChildNoSuper(const string& childName) const;
 };
 
 // =====================================================
@@ -179,9 +208,9 @@ private:
 	void operator =(XmlAttribute&);
 
 public:
-	XmlAttribute(XERCES_CPP_NAMESPACE::DOMNode *attribute, std::map<string,string> mapTagReplacementValues);
-	XmlAttribute(xml_attribute<> *attribute, std::map<string,string> mapTagReplacementValues);
-	XmlAttribute(const string &name, const string &value, std::map<string,string> mapTagReplacementValues);
+	XmlAttribute(XERCES_CPP_NAMESPACE::DOMNode *attribute, const std::map<string,string> &mapTagReplacementValues);
+	XmlAttribute(xml_attribute<> *attribute, const std::map<string,string> &mapTagReplacementValues);
+	XmlAttribute(const string &name, const string &value, const std::map<string,string> &mapTagReplacementValues);
 
 public:
 	const string getName() const	{return name;}
