@@ -11,6 +11,7 @@
 
 #include "font.h"
 #include <stdexcept>
+#include <string.h>
 #include "conversion.h"
 
 #ifdef USE_FTGL
@@ -34,6 +35,12 @@ using namespace Shared::Graphics::Gl;
 #include "util.h"
 #include "platform_common.h"
 #include "platform_util.h"
+
+#ifdef	HAVE_FRIBIDI
+#include <fribidi.h>
+#endif
+
+#include <iterator>
 #include "leak_dumper.h"
 
 using namespace std;
@@ -49,6 +56,7 @@ bool Font::fontIsMultibyte 					= false;
 bool Font::forceLegacyFonts					= false;
 bool Font::fontIsRightToLeft				= false;
 bool Font::forceFTGLFonts					= false;
+bool Font::fontSupportMixedRightToLeft		= false;
 
 // This value is used to scale the font text rendering
 // in 3D render mode
@@ -70,7 +78,7 @@ void Font::resetToDefaults() {
 	Font::fontIsMultibyte 			= false;
 	//Font::forceLegacyFonts			= false;
 	Font::fontIsRightToLeft			= false;
-
+	Font::fontSupportMixedRightToLeft = false;
 	// This value is used to scale the font text rendering
 	// in 3D render mode
 	Font::scaleFontValue					= 0.80f;
@@ -158,7 +166,7 @@ float FontMetrics::getTextWidth(const string &str) {
 				width+= (widths[97]); // This is the letter a which is a normal wide character and good to use for spacing
 			}
 			else {
-				width+= widths[longestLine[i]];
+				width+= widths[(int)longestLine[i]];
 			}
 		}
 		return width;
@@ -178,7 +186,7 @@ float FontMetrics::getHeight(const string &str) const {
 
 string FontMetrics::wordWrapText(string text, int maxWidth) {
 	// Strip newlines from source
-    //replaceAll(text, "\n", " ");
+    replaceAll(text, "\n", " \n ");
 
 	// Get all words (space separated text)
 	vector<string> words;
@@ -189,13 +197,19 @@ string FontMetrics::wordWrapText(string text, int maxWidth) {
 
     for(unsigned int i = 0; i < words.size(); ++i) {
     	string word = words[i];
-        float wordWidth = this->getTextWidth(word);
-        if (lineWidth + wordWidth > maxWidth) {
-            wrappedText += "\n";
-            lineWidth = 0;
-        }
-        lineWidth += wordWidth;
-        wrappedText += word + " ";
+    	if(word=="\n"){
+    		wrappedText += word;
+    		lineWidth = 0;
+    	}
+    	else {
+    		float wordWidth = this->getTextWidth(word);
+    		if (lineWidth + wordWidth > maxWidth) {
+    			wrappedText += "\n";
+    			lineWidth = 0;
+    		}
+    		lineWidth += this->getTextWidth(word+" ");
+    		wrappedText += word + " ";
+    	}
     }
 
     return wrappedText;
@@ -206,7 +220,9 @@ string FontMetrics::wordWrapText(string text, int maxWidth) {
 // ===============================================
 
 Font::Font(FontTextHandlerType type) {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
 
 	inited		= false;
 	this->type	= fontTypeName;
@@ -219,13 +235,6 @@ Font::Font(FontTextHandlerType type) {
 	if(Font::forceLegacyFonts == false) {
 		try {
 #if defined(USE_FREETYPEGL)
-			//if(Font::forceFTGLFonts == false) {
-			//	textHandler = NULL;
-			//	textHandler = new TextFreetypeGL(type);
-				//printf("TextFreetypeGL is ON\n");
-			//}
-
-			//else
 #endif
 			{
 				TextFTGL::faceResolution = Font::faceResolution;
@@ -233,7 +242,6 @@ Font::Font(FontTextHandlerType type) {
 
 				textHandler = NULL;
 				textHandler = new TextFTGL(type);
-				//printf("TextFTGL is ON\n");
 
 				TextFTGL::faceResolution = Font::faceResolution;
 				TextFTGL::langHeightText = Font::langHeightText;
@@ -303,6 +311,254 @@ void Font::setSize(int size)	{
 	}
 }
 
+bool is_non_ASCII(const int &c) {
+	return (c < 0) || (c >= 128);
+}
+bool is_ASCII(const int &c) {
+	return !is_non_ASCII(c);
+}
+
+void Font::bidi_cvt(string &str_) {
+
+/*
+#ifdef	HAVE_FRIBIDI
+	printf("BEFORE:   [%s]\n",str_.c_str());
+
+	char		*c_str = const_cast<char *>(str_.c_str());	// fribidi forgot const...
+	FriBidiStrIndex	len = (int)str_.length();
+	FriBidiChar	*bidi_logical = new FriBidiChar[len * 4];
+	FriBidiChar	*bidi_visual = new FriBidiChar[len * 4];
+	char		*utf8str = new char[4*len + 1];	//assume worst case here (all 4 Byte characters)
+	FriBidiCharType	base_dir = FRIBIDI_TYPE_ON;
+	FriBidiStrIndex n;
+
+
+#ifdef	OLD_FRIBIDI
+	n = fribidi_utf8_to_unicode (c_str, len, bidi_logical);
+#else
+	n = fribidi_charset_to_unicode(FRIBIDI_CHAR_SET_UTF8, c_str, len, bidi_logical);
+#endif
+	fribidi_boolean log2vis = fribidi_log2vis(bidi_logical, n, &base_dir, bidi_visual, NULL, NULL, NULL);
+	// If convertion was successful
+	//if (log2vis == true) {
+		// Remove bidi marks (that we don't need) from the output text
+		//n = fribidi_remove_bidi_marks (bidi_visual, n, NULL, NULL, NULL);
+
+		// Convert unicode string back to the encoding the input string was in
+		//fribidi_unicode_to_charset (char_set_num, visual, len, op);
+#ifdef	OLD_FRIBIDI
+		fribidi_unicode_to_utf8 (bidi_visual, n, utf8str);
+#else
+		fribidi_unicode_to_charset(FRIBIDI_CHAR_SET_UTF8, bidi_visual, n, utf8str);
+#endif
+
+		// Insert the output string into the output QString
+		str_ = std::string(utf8str);
+	//}
+	//is_rtl_ = base_dir == FRIBIDI_TYPE_RTL;
+	//fontIsRightToLeft = base_dir == FRIBIDI_TYPE_RTL;
+	fontIsRightToLeft = false;
+
+	delete[] bidi_logical;
+	delete[] bidi_visual;
+	delete[] utf8str;
+
+	printf("NEW:      [%s]\n",str_.c_str());
+
+#endif
+*/
+
+
+#ifdef	HAVE_FRIBIDI
+	const bool debugFribidi = false;
+	if(debugFribidi == true) printf("BEFORE:   [%s]\n",str_.c_str());
+
+	string new_value = "";
+	bool hasSoftNewLines = false;
+	bool hasHardNewLines = false;
+	vector<string> lines;
+
+	if(str_.find("\\n") != str_.npos) {
+		Tokenize(str_,lines,"\\n");
+		hasSoftNewLines = true;
+	}
+	else if(str_.find("\n") != str_.npos) {
+		Tokenize(str_,lines,"\n");
+		hasHardNewLines = true;
+	}
+	else {
+		lines.push_back(str_);
+	}
+
+	if(debugFribidi == true) printf("Lines: %d\n",(int)lines.size());
+
+	for(int lineIndex = 0; lineIndex < (int)lines.size(); ++lineIndex) {
+		if(lineIndex > 0) {
+			if(hasSoftNewLines == true) {
+				new_value += "\\n";
+			}
+			else if(hasHardNewLines == true) {
+				new_value += "\n";
+			}
+		}
+		str_ = lines[lineIndex];
+		if(debugFribidi == true) printf("Line: %d [%s] Font::fontSupportMixedRightToLeft = %d\n",lineIndex,str_.c_str(),Font::fontSupportMixedRightToLeft);
+
+		vector<string> words;
+		if(Font::fontSupportMixedRightToLeft == true) {
+			if(str_.find(" ") != str_.npos) {
+				Tokenize(str_,words," ");
+
+			}
+			else {
+				words.push_back(str_);
+			}
+		}
+		else {
+			words.push_back(str_);
+		}
+
+		vector<string> wordList;
+		wordList.reserve(words.size());
+		vector<string> nonASCIIWordList;
+		nonASCIIWordList.reserve(words.size());
+
+		for(int wordIndex = 0; wordIndex < (int)words.size(); ++wordIndex) {
+			str_ = words[wordIndex];
+
+			if(debugFribidi == true) printf("Word: %d [%s]\n",wordIndex,str_.c_str());
+
+			//FriBidi C string holding the original text (that is probably with logical hebrew)
+			FriBidiChar *logical = NULL;
+			//FriBidi C string for the output text (that should be visual hebrew)
+			FriBidiChar *visual = NULL;
+
+			FriBidiStrIndex *ltov = NULL;
+			FriBidiStrIndex *vtol = NULL;
+
+			//C string holding the originall text (not nnecessarily as unicode)
+			char *ip = NULL;
+			//C string for the output text (not necessarily as unicode)
+			char *op = NULL;
+
+			//Size to allocate for the char arrays
+			int size = (int)str_.size() + 2;
+
+			//Allocate memory:
+			//It's probably way too much, but at least it's not too little
+			logical = new FriBidiChar[size * 3];
+			visual = new FriBidiChar[size * 3];
+			ip = new char[size * 3];
+			op = new char[size * 3];
+
+			ltov = new FriBidiStrIndex[size * 3];
+			vtol = new FriBidiStrIndex[size * 3];
+
+			FriBidiCharType base;
+			size_t len;
+
+			//A bool type to see if conversion succeded
+			fribidi_boolean log2vis;
+
+			//Holds information telling fribidi to use UTF-8
+			FriBidiCharSet char_set_num;
+			char_set_num = fribidi_parse_charset ("UTF-8");
+
+			//Copy the given to string into the ip string
+			strcpy(ip, str_.c_str());
+
+			//Find length of originall text
+			len = strlen( ip );
+
+			//Insert ip to logical as unicode (and find it's size now)
+			len = fribidi_charset_to_unicode (char_set_num, ip, (FriBidiStrIndex)len, logical);
+
+			base = FRIBIDI_TYPE_ON;
+
+			//printf("STRIPPED: [%s]\n",str_.c_str());
+
+			//Convert logical text to visual
+			log2vis = fribidi_log2vis (logical, (FriBidiStrIndex)len, &base, visual, ltov, vtol, NULL);
+
+			bool is_converted = false;
+			//If convertion was successful
+			if(log2vis)
+			{
+				//Remove bidi marks (that we don't need) from the output text
+				len = fribidi_remove_bidi_marks (visual, (FriBidiStrIndex)len, ltov, vtol, NULL);
+
+				//Convert unicode string back to the encoding the input string was in
+				fribidi_unicode_to_charset ( char_set_num, visual, (FriBidiStrIndex)len ,op);
+
+				if(string(op) != str_) {
+					is_converted = true;
+				}
+				//Insert the output string into the result
+				str_ = op;
+
+				if(debugFribidi == true) printf("LOG2VIS:  [%s]\n",str_.c_str());
+			}
+			//printf("AFTER:    [%s]\n",str_.c_str());
+
+			//Free allocated memory
+			delete [] ltov;
+			delete [] vtol;
+			delete [] visual;
+			delete [] logical;
+			delete [] ip;
+			delete [] op;
+
+			if(Font::fontSupportMixedRightToLeft == true) {
+				if(is_converted == true) {
+					nonASCIIWordList.push_back(str_);
+
+					if(wordIndex+1 == (int)words.size()) {
+						if(nonASCIIWordList.size() > 1) {
+							std::reverse(nonASCIIWordList.begin(),nonASCIIWordList.end());
+							copy(nonASCIIWordList.begin(), nonASCIIWordList.end(), std::inserter(wordList, wordList.begin()));
+						}
+						else {
+							if(wordList.empty() == false) {
+								copy(nonASCIIWordList.begin(), nonASCIIWordList.end(), std::inserter(wordList, wordList.begin()+wordList.size()));
+							}
+							else {
+								wordList = nonASCIIWordList;
+							}
+						}
+					}
+				}
+				else {
+					if(nonASCIIWordList.size() > 1) {
+						std::reverse(nonASCIIWordList.begin(),nonASCIIWordList.end());
+					}
+
+					copy(nonASCIIWordList.begin(), nonASCIIWordList.end(), std::inserter(wordList, wordList.begin()));
+					nonASCIIWordList.clear();
+					wordList.push_back(str_);
+				}
+			}
+			else {
+				wordList.push_back(str_);
+			}
+		}
+
+		if(debugFribidi == true) printf("Building New Line: %d [%s]\n",lineIndex,new_value.c_str());
+		for(int wordIndex = 0; wordIndex < (int)wordList.size(); ++wordIndex) {
+			if(debugFribidi == true) printf("wordIndex: %d [%s]\n",wordIndex,wordList[wordIndex].c_str());
+			if(wordIndex > 0) {
+				new_value += " ";
+			}
+			new_value += wordList[wordIndex];
+		}
+		if(debugFribidi == true) printf("New Line: %d [%s]\n",lineIndex,new_value.c_str());
+	}
+	str_ = new_value;
+	if(debugFribidi == true) printf("NEW:      [%s]\n",str_.c_str());
+
+#endif
+
+}
+
 // ===============================================
 //	class Font2D
 // ===============================================
@@ -349,14 +605,13 @@ string findFontFamily(const char* font, const char *fontFamily) {
 		fs = FcFontSetCreate();
 		match = FcFontMatch(0, pat, &result);
 
-		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Trying fontconfig for fontfamily [%s]\n",(fontFamily != NULL ? fontFamily : "null"));
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Trying fontconfig for fontfamily [%s]\n",fontFamily);
 
 		if (match) FcFontSetAdd(fs, match);
 		if (pat) FcPatternDestroy(pat);
 		if(fs) {
 			FcChar8* file;
 			if( FcPatternGetString (fs->fonts[0], FC_FILE, 0, &file) == FcResultMatch ) {
-				//CHECK_FONT_PATH((const char*)file,NULL)
 				resultFile = (const char*)file;
 			}
 			FcFontSetDestroy(fs);
@@ -376,33 +631,34 @@ string findFontFamily(const char* font, const char *fontFamily) {
 	return resultFile;
 }
 
+void CHECK_FONT_PATH(const char *filename,const char *fontFamily,const char **font,const char **path) {
+	*path = filename;
+	if( *font == NULL && *path != NULL && strlen(*path) > 0 && fileExists(*path) == true ) {
+		*font = strdup(*path);
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 candidate font file exists [%s]\n",(*font != NULL ? *font : "null"));
+	}
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 Searching for font file [%s] result [%s]\n",(*path != NULL ? *path : "null"),(*font != NULL ? *font : "null"));
+	if( *font == NULL && fontFamily != NULL && strlen(fontFamily) > 0) {
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 Searching for font [%s] family [%s]\n",(*font != NULL ? *font : "null"),fontFamily);
+		string fileFound = findFontFamily(*font, fontFamily);
+		if(fileFound != "") {
+			//*path = fileFound.c_str();
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 candidate font file found [%s]\n",fileFound.c_str());
+			if( fileFound.length() > 0 && fileExists(fileFound) == true ) {
+				if(*font) free((void*)*font);
+				*font = strdup(fileFound.c_str());
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 candidate font file has been set[%s]\n",(*font != NULL ? *font : "null"));
+			}
+			*path = NULL;
+		}
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 Searching for font family [%s] result [%s]\n",fontFamily,(*font != NULL ? *font : "null"));
+	}
+}
+
 const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry) {
 	const char* font = NULL;
 	const char* path = NULL;
 
-	#define CHECK_FONT_PATH(filename,fontFamily) \
-	{ \
-		path = filename; \
-		if( font == NULL && path != NULL && strlen(path) > 0 && fileExists(path) == true ) { \
-			font = strdup(path); \
-			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 candidate font file exists [%s]\n",(font != NULL ? font : "null")); \
-		} \
-		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 Searching for font file [%s] result [%s]\n",(path != NULL ? path : "null"),(font != NULL ? font : "null")); \
-		if( font == NULL && fontFamily != NULL && strlen(fontFamily) > 0) { \
-			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 Searching for font [%s] family [%s]\n",(font != NULL ? font : "null"),(fontFamily != NULL ? fontFamily : "null")); \
-			string fileFound = findFontFamily(font, fontFamily); \
-			if(fileFound != "") { \
-				path = fileFound.c_str(); \
-				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 candidate font file found [%s]\n",fileFound.c_str()); \
-				if( path != NULL && strlen(path) > 0 && fileExists(path) == true ) { \
-					if(font) free((void*)font); \
-					font = strdup(path); \
-					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 candidate font file has been set[%s]\n",(font != NULL ? font : "null")); \
-				} \
-			} \
-			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 Searching for font family [%s] result [%s]\n",(fontFamily != NULL ? fontFamily : "null"),(font != NULL ? font : "null")); \
-		} \
-	}
 
 	string tryFont = "";
 	if(firstFontToTry != NULL || firstFontFamilyToTry != NULL) {
@@ -413,18 +669,19 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 			#endif
 
 
-			CHECK_FONT_PATH(tryFont.c_str(),firstFontFamilyToTry)
+			CHECK_FONT_PATH(tryFont.c_str(),firstFontFamilyToTry,&font,&path);
 		}
 		else {
-			CHECK_FONT_PATH(NULL,firstFontFamilyToTry)
+			CHECK_FONT_PATH(NULL,firstFontFamilyToTry,&font,&path);
 		}
 	}
 
 	// Get user-specified font path
-	if(getenv("MEGAGLEST_FONT") != NULL || getenv("MEGAGLEST_FONT_FAMILY") != NULL) {
-
-		if(getenv("MEGAGLEST_FONT") != NULL) {
-			tryFont = getenv("MEGAGLEST_FONT");
+	string megaglest_font = safeCharPtrCopy(getenv("MEGAGLEST_FONT"),8095);
+	string megaglest_font_family = safeCharPtrCopy(getenv("MEGAGLEST_FONT_FAMILY"),8095);
+	if(megaglest_font != "" || megaglest_font_family != "") {
+		if(megaglest_font != "") {
+			tryFont = megaglest_font;
 
 			if(Text::DEFAULT_FONT_PATH_ABSOLUTE != "") {
 				tryFont = Text::DEFAULT_FONT_PATH_ABSOLUTE + "/" + extractFileFromDirectoryPath(tryFont);
@@ -433,10 +690,10 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 			  replaceAll(tryFont, "/", "\\");
 			#endif
 
-			CHECK_FONT_PATH(tryFont.c_str(),getenv("MEGAGLEST_FONT_FAMILY"))
+			CHECK_FONT_PATH(tryFont.c_str(),megaglest_font_family.c_str(),&font,&path);
 		}
 		else {
-			CHECK_FONT_PATH(NULL,getenv("MEGAGLEST_FONT_FAMILY"))
+			CHECK_FONT_PATH(NULL,megaglest_font_family.c_str(),&font,&path);
 		}
 	}
 
@@ -450,14 +707,14 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 	#ifdef WIN32
 	  replaceAll(tryFont, "/", "\\");
 	#endif
-	CHECK_FONT_PATH(tryFont.c_str(),"Linux Biolinum O:style=Bold")
+	CHECK_FONT_PATH(tryFont.c_str(),"Linux Biolinum O:style=Bold",&font,&path);
 
 	#ifdef FONT_PATH
 	// Get distro-specified font path
-	CHECK_FONT_PATH(FONT_PATH)
+	CHECK_FONT_PATH(FONT_PATH,NULL,&font,&path);
 	#endif
 
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothub__.ttf","Gothic Uralic:style=Regular")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothub__.ttf","Gothic Uralic:style=Regular",&font,&path);
 
 	// Check a couple of common paths for Gothic Uralic/bold as a last resort
 	// Debian
@@ -466,55 +723,55 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 	font that contains all the Unicode characters in use in	your translation.
 	If the font is available in Debian it should be the Debian path.
 	*/
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothub__.ttf","Gothic Uralic:style=Regular")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothub__.ttf","Gothic Uralic:style=Regular",&font,&path);
 	/*
 	TRANSLATORS: If using the FTGL backend, this should be the path of a
 	font that contains all the Unicode characters in use in	your translation.
 	If the font is available in Debian it should be the Debian path.
 	*/
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothu___.ttf","Gothic Uralic:style=Regular")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothu___.ttf","Gothic Uralic:style=Regular",&font,&path);
 	// Mandrake
 	/*
 	TRANSLATORS: If using the FTGL backend, this should be the path of a bold
 	font that contains all the Unicode characters in use in	your translation.
 	If the font is available in Mandrake it should be the Mandrake path.
 	*/
-	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHUB__.TTF","Gothic Uralic:style=Bold")
+	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHUB__.TTF","Gothic Uralic:style=Bold",&font,&path);
 	/*
 	TRANSLATORS: If using the FTGL backend, this should be the path of a
 	font that contains all the Unicode characters in use in	your translation.
 	If the font is available in Mandrake it should be the Mandrake path.
 	*/
-	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHU___.TTF","Gothic Uralic:style=Regular")
+	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHU___.TTF","Gothic Uralic:style=Regular",&font,&path);
 
 	// Check the non-translated versions of the above
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothub__.ttf","Gothic Uralic:style=Regular")
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothu___.ttf","Gothic Uralic:style=Regular")
-	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHUB__.TTF","Gothic Uralic:style=Regular")
-	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHU___.TTF","Gothic Uralic:style=Regular")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothub__.ttf","Gothic Uralic:style=Regular",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothu___.ttf","Gothic Uralic:style=Regular",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHUB__.TTF","Gothic Uralic:style=Regular",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/TTF/uralic/GOTHU___.TTF","Gothic Uralic:style=Regular",&font,&path);
 
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/linux-libertine/LinLibertine_Re.ttf","Linux Libertine O:style=Regular")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/linux-libertine/LinLibertine_Re.ttf","Linux Libertine O:style=Regular",&font,&path);
 
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeSerif.ttf","FreeSerif")
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeSans.ttf","FreeSans")
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeMono.ttf","FreeMono")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeSerif.ttf","FreeSerif",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeSans.ttf","FreeSans",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeMono.ttf","FreeMono",&font,&path);
 
 	//openSUSE
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/LinBiolinum_RB.otf","Bold")
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeSerif.ttf","FreeSerif")
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeSans.ttf","FreeSans")
-	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeMono.ttf","FreeMono")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/LinBiolinum_RB.otf","Bold",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeSerif.ttf","FreeSerif",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeSans.ttf","FreeSans",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeMono.ttf","FreeMono",&font,&path);
 
 	// gentoo paths
-	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeSerif.ttf","FreeSerif")
-	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeSans.ttf","FreeSans")
-	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeMono.ttf","FreeMono")
+	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeSerif.ttf","FreeSerif",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeSans.ttf","FreeSans",&font,&path);
+	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeMono.ttf","FreeMono",&font,&path);
 
 #ifdef _WIN32
-	CHECK_FONT_PATH("c:\\windows\\fonts\\verdana.ttf",NULL)
-	CHECK_FONT_PATH("c:\\windows\\fonts\\tahoma.ttf",NULL)
-	CHECK_FONT_PATH("c:\\windows\\fonts\\arial.ttf",NULL)
-	CHECK_FONT_PATH("\\windows\\fonts\\arial.ttf",NULL)
+	CHECK_FONT_PATH("c:\\windows\\fonts\\verdana.ttf",NULL,&font,&path);
+	CHECK_FONT_PATH("c:\\windows\\fonts\\tahoma.ttf",NULL,&font,&path);
+	CHECK_FONT_PATH("c:\\windows\\fonts\\arial.ttf",NULL,&font,&path);
+	CHECK_FONT_PATH("\\windows\\fonts\\arial.ttf",NULL,&font,&path);
 #endif
 
 	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Final selection of font file is [%s]\n",(font != NULL ? font : "null")); \
