@@ -85,9 +85,9 @@ struct TargaFileHeader{
 const int tgaUncompressedRgb= 2;
 const int tgaUncompressedBw= 3;
 
-void CalculatePixelsCRC(uint8 *pixels,uint64 pixelByteCount, Checksum &crc) {
+void CalculatePixelsCRC(uint8 *pixels,std::size_t pixelByteCount, Checksum &crc) {
 //	crc = Checksum();
-//	for(uint64 i = 0; i < pixelByteCount; ++i) {
+//	for(std::size_t i = 0; i < pixelByteCount; ++i) {
 //		crc.addByte(pixels[i]);
 //	}
 }
@@ -678,12 +678,14 @@ void PixmapIoJpg::write(uint8 *pixels) {
 	if(components == 4) {
 		int n = w * h;
 		unsigned char *dst = tmpbytes = (unsigned char *) malloc(n*3);
-		const unsigned char *src = pixels;
-		for (int i = 0; i < n; i++) {
-			*dst++ = *src++;
-			*dst++ = *src++;
-			*dst++ = *src++;
-			src++;
+		if(dst != NULL) {
+			const unsigned char *src = pixels;
+			for (int i = 0; i < n; i++) {
+				*dst++ = *src++;
+				*dst++ = *src++;
+				*dst++ = *src++;
+				src++;
+			}
 		}
 		components = 3;
 	}
@@ -718,25 +720,29 @@ void PixmapIoJpg::write(uint8 *pixels) {
     // flip lines.
 	uint8 *flip = (uint8 *)malloc(sizeof(uint8) * w * h * 3);
 
-	for (int y = 0;y < h; ++y) {
-		for (int x = 0;x < w; ++x) {
-			flip[(y * w + x) * 3] = pixels[((h - 1 - y) * w + x) * 3];
-			flip[(y * w + x) * 3 + 1] = pixels[((h - 1 - y) * w + x) * 3 + 1];
-			flip[(y * w + x) * 3 + 2] = pixels[((h - 1 - y) * w + x) * 3 + 2];
+	if(pixels != NULL && flip != NULL) {
+		for (int y = 0;y < h; ++y) {
+			for (int x = 0;x < w; ++x) {
+				flip[(y * w + x) * 3] = pixels[((h - 1 - y) * w + x) * 3];
+				flip[(y * w + x) * 3 + 1] = pixels[((h - 1 - y) * w + x) * 3 + 1];
+				flip[(y * w + x) * 3 + 2] = pixels[((h - 1 - y) * w + x) * 3 + 2];
+			}
+		}
+	
+		/* like reading a file, this time write one row at a time */
+		while( cinfo.next_scanline < cinfo.image_height ) {
+			row_pointer[0] = &flip[ cinfo.next_scanline * cinfo.image_width *  cinfo.input_components];
+			jpeg_write_scanlines( &cinfo, row_pointer, 1 );
 		}
-	 }
-
-	/* like reading a file, this time write one row at a time */
-	while( cinfo.next_scanline < cinfo.image_height ) {
-		row_pointer[0] = &flip[ cinfo.next_scanline * cinfo.image_width *  cinfo.input_components];
-		jpeg_write_scanlines( &cinfo, row_pointer, 1 );
 	}
 	if (tmpbytes) {
 		free(tmpbytes);
 		tmpbytes=NULL;
 	}
-	free(flip);
-	flip=NULL;
+	if(flip) {
+		free(flip);
+		flip=NULL;
+	}
 
 	/* similar to read file, clean up after we're done compressing */
 	jpeg_finish_compress( &cinfo );
@@ -753,7 +759,10 @@ void PixmapIoJpg::write(uint8 *pixels) {
 // ===================== PUBLIC ========================
 
 Pixmap1D::Pixmap1D() {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
 
     w= -1;
 	components= -1;
@@ -761,13 +770,19 @@ Pixmap1D::Pixmap1D() {
 }
 
 Pixmap1D::Pixmap1D(int components) {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
 
 	init(components);
 }
 
 Pixmap1D::Pixmap1D(int w, int components) {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
 
 	init(w, components);
 }
@@ -782,11 +797,11 @@ void Pixmap1D::init(int components){
 void Pixmap1D::init(int w, int components){
 	this->w= w;
 	this->components= components;
-	pixels= new uint8[(std::size_t)getPixelByteCount()];
+	pixels= new uint8[getPixelByteCount()];
 	CalculatePixelsCRC(pixels,0, crc);
 }
 
-uint64 Pixmap1D::getPixelByteCount() const {
+std::size_t Pixmap1D::getPixelByteCount() const {
 	return (w * components);
 }
 
@@ -835,7 +850,7 @@ void Pixmap1D::loadBmp(const string &path) {
 		components= 3;
 	}
 	if(pixels == NULL) {
-		pixels= new uint8[(std::size_t)getPixelByteCount()];
+		pixels= new uint8[getPixelByteCount()];
 	}
 
 	//data
@@ -865,7 +880,7 @@ void Pixmap1D::loadTga(const string &path) {
 		components= fileComponents;
 	}
 	if(pixels == NULL) {
-		pixels= new uint8[(std::size_t)getPixelByteCount()];
+		pixels= new uint8[getPixelByteCount()];
 	}
 
 	//read data
@@ -879,7 +894,10 @@ void Pixmap1D::loadTga(const string &path) {
 // ===================== PUBLIC ========================
 
 Pixmap2D::Pixmap2D() {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
     h= -1;
     w= -1;
 	components= -1;
@@ -887,7 +905,10 @@ Pixmap2D::Pixmap2D() {
 }
 
 Pixmap2D::Pixmap2D(int components) {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
     h= -1;
     w= -1;
 	this->components= -1;
@@ -897,7 +918,10 @@ Pixmap2D::Pixmap2D(int components) {
 }
 
 Pixmap2D::Pixmap2D(int w, int h, int components) {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
     this->h= 0;
     this->w= -1;
     this->components= -1;
@@ -926,11 +950,11 @@ void Pixmap2D::init(int w, int h, int components) {
 		snprintf(szBuf,8096,"Invalid pixmap dimensions for [%s], h = %d, w = %d, components = %d\n",path.c_str(),h,w,components);
 		throw megaglest_runtime_error(szBuf);
 	}
-	pixels= new uint8[(std::size_t)getPixelByteCount()];
+	pixels= new uint8[getPixelByteCount()];
 	CalculatePixelsCRC(pixels,0, crc);
 }
 
-uint64 Pixmap2D::getPixelByteCount() const {
+std::size_t Pixmap2D::getPixelByteCount() const {
 	return (h * w * components);
 }
 
@@ -952,22 +976,21 @@ void Pixmap2D::Scale(int format, int newW, int newH) {
 	uint8 *newpixels= new uint8[newW * newH * useComponents];
 
 	glPixelStorei(GL_PACK_ALIGNMENT, 1);
-	int error = gluScaleImage(	format,
+	GLenum glErr = gluScaleImage(	format,
 				w, h, GL_UNSIGNED_BYTE, pixels,
 				newW, newH, GL_UNSIGNED_BYTE, newpixels);
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 
-	if(error == GL_NO_ERROR) {
+	if(glErr == GL_NO_ERROR) {
 		init(newW,newH,this->components);
 		pixels = newpixels;
 
 		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Scaled image from [%d x %d] to [%d x %d]\n",originalW,originalH,w,h);
 	}
 	else {
-		const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-		printf("ERROR Scaling image from [%d x %d] to [%d x %d] error: %d [%s]\n",originalW,originalH,w,h,error,errorString);
+		const char *errorString= reinterpret_cast<const char*>(gluErrorString(glErr));
+		printf("ERROR Scaling image from [%d x %d] to [%d x %d] error: %u [%s]\n",originalW,originalH,w,h,glErr,errorString);
 
-		GLenum glErr = error;
 		assertGlWithErrorNumber(glErr);
 	}
 
@@ -1038,29 +1061,64 @@ void Pixmap2D::savePng(const string &path) {
 
 void Pixmap2D::getPixel(int x, int y, uint8 *value) const {
 	for(int i=0; i<components; ++i){
-		value[i]= pixels[(w*y+x)*components+i];
+		std::size_t index = (w*y+x)*components+i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		value[i]= pixels[index];
 	}
 }
 
 void Pixmap2D::getPixel(int x, int y, float32 *value) const {
-	for(int i=0; i<components; ++i) {
-		value[i]= pixels[(w*y+x)*components+i]/255.f;
+	for(int i = 0; i < components; ++i) {
+		std::size_t index = (w * y + x) * components + i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		value[i] = pixels[index] / 255.f;
 	}
 }
 
 void Pixmap2D::getComponent(int x, int y, int component, uint8 &value) const {
-	value= pixels[(w*y+x)*components+component];
+	std::size_t index = (w*y+x)*components+component;
+	if(index >= getPixelByteCount()) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+
+	value = pixels[index];
 }
 
 void Pixmap2D::getComponent(int x, int y, int component, float32 &value) const {
-	value= pixels[(w*y+x)*components+component]/255.f;
+	std::size_t index = (w*y+x)*components+component;
+	if(index >= getPixelByteCount()) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+
+	value= pixels[index] / 255.f;
 }
 
 //vector get
 Vec4f Pixmap2D::getPixel4f(int x, int y) const {
 	Vec4f v(0.f);
 	for(int i=0; i<components && i<4; ++i){
-		v.ptr()[i]= pixels[(w*y+x)*components+i]/255.f;
+		std::size_t index = (w*y+x)*components+i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		v.ptr()[i]= pixels[index] / 255.f;
 	}
 	return v;
 }
@@ -1068,14 +1126,26 @@ Vec4f Pixmap2D::getPixel4f(int x, int y) const {
 Vec3f Pixmap2D::getPixel3f(int x, int y) const {
 	Vec3f v(0.f);
 	for(int i=0; i<components && i<3; ++i){
-		v.ptr()[i]= pixels[(w*y+x)*components+i]/255.f;
+		std::size_t index = (w*y+x)*components+i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+
+		v.ptr()[i]= pixels[index] / 255.f;
 	}
 	return v;
 }
 
 float Pixmap2D::getPixelf(int x, int y) const {
-	int index = (w*y+x)*components;
-	return pixels[index]/255.f;
+	std::size_t index = (w * y + x) * components;
+	if(index >= getPixelByteCount()) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+	return pixels[index] / 255.f;
 }
 
 float Pixmap2D::getComponentf(int x, int y, int component) const {
@@ -1084,70 +1154,120 @@ float Pixmap2D::getComponentf(int x, int y, int component) const {
 	return c;
 }
 
-void Pixmap2D::setPixel(int x, int y, const uint8 *value) {
-	for(int i=0; i<components; ++i) {
-		int index = (w*y+x)*components+i;
+void Pixmap2D::setPixel(int x, int y, const uint8 *value, int arraySize) {
+	if(arraySize > components) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap arraySize: %d for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",arraySize,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+	for(int i = 0; i < components; ++i) {
+		std::size_t index = (w * y + x) * components + i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+
 		pixels[index]= value[i];
 	}
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
-void Pixmap2D::setPixel(int x, int y, const float32 *value) {
-	for(int i=0; i<components; ++i) {
-		int index = (w*y+x)*components+i;
-		pixels[index]= static_cast<uint8>(value[i]*255.f);
+void Pixmap2D::setPixel(int x, int y, const float32 *value, int arraySize) {
+	if(arraySize > components) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap arraySize: %d for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",arraySize,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+
+	for(int i = 0; i < components; ++i) {
+		std::size_t index = (w*y+x)*components+i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+		pixels[index] = static_cast<uint8>(value[i] * 255.f);
 	}
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
 void Pixmap2D::setComponent(int x, int y, int component, uint8 value) {
-	int index = (w*y+x)*components+component;
+	std::size_t index = (w*y+x)*components+component;
+	if(index >= getPixelByteCount()) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+
 	pixels[index] = value;
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
 void Pixmap2D::setComponent(int x, int y, int component, float32 value) {
-	int index = (w*y+x)*components+component;
-	pixels[index]= static_cast<uint8>(value*255.f);
+	std::size_t index = (w*y+x)*components+component;
+	if(index >= getPixelByteCount()) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+
+	pixels[index] = static_cast<uint8>(value * 255.f);
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
 //vector set
 void Pixmap2D::setPixel(int x, int y, const Vec3f &p) {
 	for(int i = 0; i < components  && i < 3; ++i) {
-		int index = (w*y+x)*components+i;
-		pixels[index]= static_cast<uint8>(p.ptr()[i]*255.f);
+		std::size_t index = (w*y+x)*components+i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+		pixels[index] = static_cast<uint8>(p.ptr()[i] * 255.f);
 	}
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
 void Pixmap2D::setPixel(int x, int y, const Vec4f &p) {
 	for(int i = 0; i < components && i < 4; ++i) {
-		int index = (w*y+x)*components+i;
-		pixels[index]= static_cast<uint8>(p.ptr()[i]*255.f);
+		std::size_t index = (w*y+x)*components+i;
+		if(index >= getPixelByteCount()) {
+			char szBuf[8096];
+			snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+			throw megaglest_runtime_error(szBuf);
+		}
+		pixels[index] = static_cast<uint8>(p.ptr()[i] * 255.f);
 	}
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
 void Pixmap2D::setPixel(int x, int y, float p) {
-	int index = (w*y+x)*components;
-	pixels[index]= static_cast<uint8>(p*255.f);
+	std::size_t index = (w * y + x) * components;
+	if(index >= getPixelByteCount()) {
+		char szBuf[8096];
+		snprintf(szBuf,8096,"Invalid pixmap index: " MG_SIZE_T_SPECIFIER " for [%s], h = %d, w = %d, components = %d x = %d y = %d\n",index,path.c_str(),h,w,components,x,y);
+		throw megaglest_runtime_error(szBuf);
+	}
+
+	pixels[index] = static_cast<uint8>(p * 255.f);
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
-void Pixmap2D::setPixels(const uint8 *value){
-	for(int i=0; i<w; ++i){
-		for(int j=0; j<h; ++j){
-			setPixel(i, j, value);
+void Pixmap2D::setPixels(const uint8 *value, int arraySize) {
+	for(int i = 0; i < w; ++i) {
+		for(int j = 0; j < h; ++j) {
+			setPixel(i, j, value, arraySize);
 		}
 	}
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
 }
 
-void Pixmap2D::setPixels(const float32 *value){
-	for(int i=0; i<w; ++i){
-		for(int j=0; j<h; ++j){
-			setPixel(i, j, value);
+void Pixmap2D::setPixels(const float32 *value, int arraySize) {
+	for(int i = 0; i < w; ++i) {
+		for(int j = 0; j < h; ++j) {
+			setPixel(i, j, value, arraySize);
 		}
 	}
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
@@ -1203,19 +1323,13 @@ void Pixmap2D::splat(const Pixmap2D *leftUp, const Pixmap2D *rightUp, const Pixm
 			float distRd= splatDist(Vec2i(i, j), Vec2i(w, h));
 
 			const float powFactor= 2.0f;
-#ifdef USE_STREFLOP
-			distLu= streflop::pow(static_cast<streflop::Simple>(distLu), static_cast<streflop::Simple>(powFactor));
-			distRu= streflop::pow(static_cast<streflop::Simple>(distRu), static_cast<streflop::Simple>(powFactor));
-			distLd= streflop::pow(static_cast<streflop::Simple>(distLd), static_cast<streflop::Simple>(powFactor));
-			distRd= streflop::pow(static_cast<streflop::Simple>(distRd), static_cast<streflop::Simple>(powFactor));
-			avg= streflop::pow(static_cast<streflop::Simple>(avg), static_cast<streflop::Simple>(powFactor));
-#else
-			distLu= pow(distLu, powFactor);
-			distRu= pow(distRu, powFactor);
-			distLd= pow(distLd, powFactor);
-			distRd= pow(distRd, powFactor);
-			avg= pow(avg, powFactor);
-#endif
+
+			distLu	= std::pow(distLu, powFactor);
+			distRu	= std::pow(distRu, powFactor);
+			distLd	= std::pow(distLd, powFactor);
+			distRd	= std::pow(distRd, powFactor);
+			avg		= std::pow(avg, powFactor);
+
 			float lu= distLu>avg? 0: ((avg-distLu))*random.randRange(0.5f, 1.0f);
 			float ru= distRu>avg? 0: ((avg-distRu))*random.randRange(0.5f, 1.0f);
 			float ld= distLd>avg? 0: ((avg-distLd))*random.randRange(0.5f, 1.0f);
@@ -1269,10 +1383,31 @@ void Pixmap2D::subCopy(int x, int y, const Pixmap2D *sourcePixmap){
 
 	uint8 *pixel= new uint8[components];
 
-	for(int i=0; i<sourcePixmap->getW(); ++i){
-		for(int j=0; j<sourcePixmap->getH(); ++j){
+	for(int i = 0; i < sourcePixmap->getW(); ++i) {
+		for(int j = 0; j < sourcePixmap->getH(); ++j) {
 			sourcePixmap->getPixel(i, j, pixel);
-			setPixel(i+x, j+y, pixel);
+			setPixel(i+x, j+y, pixel, components);
+		}
+	}
+	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
+
+	delete [] pixel;
+}
+
+// uses a a part of a bigger source image to fill this image.
+void Pixmap2D::copyImagePart(int x, int y, const Pixmap2D *sourcePixmap){
+	assert(components==sourcePixmap->getComponents());
+
+	if(x+w>sourcePixmap->getW() && y+h>sourcePixmap->getH()){
+		throw megaglest_runtime_error("Pixmap2D::copyImagePart(), bad dimensions");
+	}
+
+	uint8 *pixel= new uint8[components];
+
+	for(int i = x; i < x + w; ++i) {
+		for(int j = y; j < y + h; ++j) {
+			sourcePixmap->getPixel(i, j, pixel);
+			setPixel(i-x, j-y, pixel, components);
 		}
 	}
 	CalculatePixelsCRC(pixels,getPixelByteCount(), crc);
@@ -1289,7 +1424,10 @@ bool Pixmap2D::doDimensionsAgree(const Pixmap2D *pixmap){
 // =====================================================
 
 Pixmap3D::Pixmap3D() {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
 
 	w= -1;
 	h= -1;
@@ -1300,14 +1438,20 @@ Pixmap3D::Pixmap3D() {
 }
 
 Pixmap3D::Pixmap3D(int w, int h, int d, int components) {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
 	pixels = NULL;
 	slice=0;
 	init(w, h, d, components);
 }
 
 Pixmap3D::Pixmap3D(int d, int components) {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
 	pixels = NULL;
 	slice=0;
 	init(d, components);
@@ -1318,11 +1462,11 @@ void Pixmap3D::init(int w, int h, int d, int components){
 	this->h= h;
 	this->d= d;
 	this->components= components;
-	pixels= new uint8[(std::size_t)getPixelByteCount()];
+	pixels= new uint8[getPixelByteCount()];
 	CalculatePixelsCRC(pixels,0, crc);
 }
 
-uint64 Pixmap3D::getPixelByteCount() const {
+std::size_t Pixmap3D::getPixelByteCount() const {
 	return (h * w * d * components);
 }
 
@@ -1394,7 +1538,7 @@ void Pixmap3D::loadSliceBmp(const string &path, int slice){
 		components= 3;
 	}
 	if(pixels==NULL){
-		pixels= new uint8[(std::size_t)getPixelByteCount()];
+		pixels= new uint8[getPixelByteCount()];
 	}
 
 	//data
@@ -1407,34 +1551,16 @@ void Pixmap3D::loadSliceTga(const string &path, int slice){
 	//deletePixels();
 	FileReader<Pixmap3D>::readPath(path,this);
 	//printf("Loading 3D pixmap TGA [%s] this [%p]\n",path.c_str(),this);
-
-/*
-	PixmapIoTga plt;
-	plt.openRead(path);
-
-	//header
-	int fileComponents= plt.getComponents();
-
-	//init
-	w= plt.getW();
-	h= plt.getH();
-	if(components==-1){
-		components= fileComponents;
-	}
-	if(pixels==NULL){
-		pixels= new uint8[(std::size_t)getPixelByteCount()];
-	}
-
-	//read data
-	plt.read(&pixels[slice*w*h*components], components);
-*/
 }
 
 // =====================================================
 //	class PixmapCube
 // =====================================================
 PixmapCube::PixmapCube() {
-	assert(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false);
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
+		throw megaglest_runtime_error("Loading graphics in headless server mode not allowed!");
+	}
+
 }
 
 PixmapCube::~PixmapCube() {
@@ -1453,8 +1579,8 @@ void PixmapCube::init(int components) {
 	}
 }
 
-uint64 PixmapCube::getPixelByteCount() const {
-	uint64 result = 0;
+std::size_t PixmapCube::getPixelByteCount() const {
+	std::size_t result = 0;
 	for(int i=0; i<6; ++i) {
 		result += faces[i].getPixelByteCount();
 	}
