@@ -46,6 +46,7 @@ public:
 // =====================================================
 
 bool LuaScript::disableSandbox = false;
+bool LuaScript::debugModeEnabled = false;
 
 LuaScript::LuaScript() {
 	Lua_STREFLOP_Wrapper streflopWrapper;
@@ -121,16 +122,18 @@ void LuaScript::DumpGlobals()
 {
 	LuaHandle *L = luaState;
 	// push the first key (nil = beginning of table)
+#if LUA_VERSION_NUM <= 501
 	lua_pushnil(L);
-
+#endif
 	// lua_next will:
 	// 1 - pop the key
 	// 2 - push the next key
 	// 3 - push the value at that key
 	// ... so the key will be at index -2 and the value at index -1
 #if LUA_VERSION_NUM > 501
-		lua_rawgeti(L, LUA_REGISTRYINDEX, LUA_RIDX_GLOBALS);
-		for (lua_pushnil(L); lua_next(L, -2); lua_pop(L, 1)) {
+	lua_pushglobaltable(L);
+	lua_pushnil(L);
+	while (lua_next(L, -2) != 0) {
 #else
 	while (lua_next(L, LUA_GLOBALSINDEX) != 0) {
 #endif
@@ -197,20 +200,24 @@ void LuaScript::DumpGlobals()
 		// call luaL_dostring with that line.
 		//SaveLine(key_string + " = " + value_string);		// Pop the value so the index remains on top of the stack for the next iteration
 		printf("Found global LUA var: %s = %s\n",key_string.c_str(),value_string.c_str());
-#if LUA_VERSION_NUM <= 501
+
+		lua_pop(L, 1);
+	}
+#if LUA_VERSION_NUM > 501
 		lua_pop(L, 1);
 #endif
-	}
+
 }
 
 void LuaScript::saveGame(XmlNode *rootNode) {
 	std::map<string,string> mapTagReplacements;
 
-	const bool debugLuaDump = false;
 	//try{
 	LuaHandle *L = luaState;
 	// push the first key (nil = beginning of table)
+#if LUA_VERSION_NUM <= 501
 	lua_pushnil(L);
+#endif
 
 	// lua_next will:
 	// 1 - pop the key
@@ -219,8 +226,9 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 	// ... so the key will be at index -2 and the value at index -1
 
 #if LUA_VERSION_NUM > 501
-	lua_rawgeti(L, LUA_REGISTRYINDEX, LUA_RIDX_GLOBALS);
-	for (lua_pushnil(L); lua_next(L, -2); lua_pop(L, 1)) {
+	lua_pushglobaltable(L);
+	lua_pushnil(L);
+	while (lua_next(L, -2) != 0) {
 #else
 	while (lua_next(L, LUA_GLOBALSINDEX) != 0) {
 #endif
@@ -228,7 +236,7 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 		int key_type = lua_type(L, -2);
 		int value_type = lua_type(L, -1);
 
-		if(debugLuaDump == true) printf("LUA save key_type = %d, value_type = %d for var [%s]\n",key_type,value_type,lua_tostring(L, -2));
+		if(LuaScript::debugModeEnabled == true) printf("LUA save key_type = %d, value_type = %d for var [%s]\n",key_type,value_type,lua_tostring(L, -2));
 
 		// support only string keys
 		// globals aren't likely to have a non-string key, but just to be certain ...
@@ -249,6 +257,7 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 		// get the key as a string
 		string key_string = lua_tostring(L, -2); // no copy required - we already know this is a string
 
+		if(LuaScript::debugModeEnabled == true) printf("key_string [%s]\n",key_string.c_str());
 		// do not support variables that start with '_'
 		// lua has some predefined values like _VERSION. They all start with underscore
 
@@ -285,14 +294,18 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 			break;
 		case LUA_TTABLE:
 			{
-			if(debugLuaDump == true) printf("LUA TABLE DETECTED - START\n");
+
+			int tableItemCount = 0;
+			if(LuaScript::debugModeEnabled == true) printf("================ LUA TABLE DETECTED - START =================\n");
 			for (lua_pushnil(L); lua_next(L, -2) ;) {
-				if(debugLuaDump == true) printf("LUA TABLE loop A\n");
+				tableItemCount++;
+
+				if(LuaScript::debugModeEnabled == true) printf("LUA TABLE loop A\n");
 
 				int tableKeyType = lua_type(L, -2);
 				int tableValueType = lua_type(L, -1);
 
-				if(debugLuaDump == true) printf("LUA TABLE loop item type [%s]\n",lua_typename(L, tableValueType));
+				if(LuaScript::debugModeEnabled == true) printf("LUA TABLE loop item type [%s] key: %d value type: %d\n",lua_typename(L, tableValueType),tableKeyType,tableValueType);
 
 				switch (tableValueType) {
 					case LUA_TSTRING:
@@ -307,21 +320,26 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 					// Stack: value, key, table
 					std :: string value = "";
 					if(!lua_isnil(L, -1)) {
-						if(debugLuaDump == true) printf("LUA TABLE loop B\n");
+						if(LuaScript::debugModeEnabled == true) printf("LUA TABLE loop B\n");
 
 						lua_pushvalue(L, -1);
 
-						if(debugLuaDump == true) printf("LUA TABLE loop C\n");
+						if(LuaScript::debugModeEnabled == true) printf("LUA TABLE loop C\n");
 
-						value = lua_tostring (L, -1);
+						if(tableValueType == LUA_TBOOLEAN ) {
+							value = lua_toboolean(L, -1) == 0 ? "false" : "true";
+						}
+						else {
+							value = lua_tostring (L, -1);
+						}
 
-						if(debugLuaDump == true) printf("LUA TABLE loop D\n");
+						if(LuaScript::debugModeEnabled == true) printf("LUA TABLE loop D\n");
 
 						lua_pop (L, 1);
 					}
 					lua_pop (L, 1);
 
-					if(debugLuaDump == true) printf("LUA TABLE value [%s]\n",value.c_str());
+					if(LuaScript::debugModeEnabled == true) printf("LUA TABLE value [%s]\n",value.c_str());
 
 					// Stack: key, table
 					lua_pushvalue(L, -1);
@@ -332,7 +350,7 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 
 					// Stack: key, table
 					//std :: cout << key << "" << value << "\ n";
-					if(debugLuaDump == true) printf("[%s] [%s]\n",key.c_str(),value.c_str());
+					if(LuaScript::debugModeEnabled == true) printf("[%s] [%s]\n",key.c_str(),value.c_str());
 
 					if(value_string != "") {
 						value_string += "|||";
@@ -344,9 +362,17 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 					tableList.push_back(make_pair(make_pair(tableKeyType,key),make_pair(tableValueType,value)));
 				}
 				else {
+					if(LuaScript::debugModeEnabled == true) printf("***WARNING*** SKIPPING LUA TABLE because it has an unsupported embedded type: %d [%s]\n",tableValueType, lua_typename(L, tableValueType));
+
 					lua_pop(L, 1);
 				}
 			}
+			if(tableItemCount < 1) {
+				if(LuaScript::debugModeEnabled == true) printf("Skipping lua table tableItemCount = %d\n",tableItemCount);
+
+				skipTable = true;
+			}
+			if(LuaScript::debugModeEnabled == true) printf("---------------------------- LUA TABLE DETECTED - END ----------------------------\n");
 			}
 			break;
 		}
@@ -357,7 +383,7 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 		//}
 
 		if(skipTable == true) {
-			if(debugLuaDump == true) printf("#2 SKIPPING TABLE\n");
+			if(LuaScript::debugModeEnabled == true) printf("#2 SKIPPING TABLE\n");
 		}
 		else {
 			//vector<pair<pair<int,string>, pair<int,string>> > tableList;
@@ -387,10 +413,12 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 				luaScriptNode->addAttribute("value_type",intToStr(value_type), mapTagReplacements);
 			}
 		}
-#if LUA_VERSION_NUM <= 501
+		lua_pop(L, 1);
+	}
+
+#if LUA_VERSION_NUM > 501
 		lua_pop(L, 1);
 #endif
-	}
 
 	//}
 	//catch(const exception &ex) {
@@ -399,57 +427,84 @@ void LuaScript::saveGame(XmlNode *rootNode) {
 }
 
 void LuaScript::loadGame(const XmlNode *rootNode) {
+	if(LuaScript::debugModeEnabled) printf("START [%s::%s] Line: %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
 	vector<XmlNode *> luaScriptNodeList = rootNode->getChildList("LuaScript");
+
+	if(LuaScript::debugModeEnabled) printf("luaScriptNodeList.size(): %d\n",(int)luaScriptNodeList.size());
+
 	for(unsigned int i = 0; i < luaScriptNodeList.size(); ++i) {
 		XmlNode *node = luaScriptNodeList[i];
 
 		string variable = node->getAttribute("variable")->getValue();
 		int value_type = node->getAttribute("value_type")->getIntValue();
 
+		if(LuaScript::debugModeEnabled) printf("  index #: %u [%s] [%d]\n",i,variable.c_str(),value_type);
+
 		switch (value_type) {
 			case LUA_TSTRING:
+				if(LuaScript::debugModeEnabled) printf("  value [%s]\n",node->getAttribute("value")->getValue().c_str());
 				lua_pushstring( luaState, node->getAttribute("value")->getValue().c_str() );
 				break;
 			case LUA_TNUMBER:
-				lua_pushnumber( luaState, node->getAttribute("value")->getIntValue() );
+				if(LuaScript::debugModeEnabled) printf("  value [%s]\n",node->getAttribute("value")->getValue().c_str());
+				lua_pushnumber( luaState, node->getAttribute("value")->getFloatValue() );
 				break;
 			case LUA_TBOOLEAN:
+				if(LuaScript::debugModeEnabled) printf("  value [%s]\n",node->getAttribute("value")->getValue().c_str());
 				lua_pushboolean( luaState, node->getAttribute("value")->getBoolValue() );
 				break;
 			case LUA_TTABLE:
 				{
+					if(LuaScript::debugModeEnabled == true) printf("================ LUA TABLE DETECTED - START =================\n");
+
 					lua_newtable(luaState);    /* We will pass a table */
 					vector<XmlNode *> luaScriptTableNode = node->getChildList("Table");
+
+					if(LuaScript::debugModeEnabled) printf("luaScriptTableNode.size(): %d\n",(int)luaScriptTableNode.size());
+
 					for(unsigned int j = 0; j < luaScriptTableNode.size(); ++j) {
 						XmlNode *nodeTable = luaScriptTableNode[j];
 
 						int key_type = nodeTable->getAttribute("key_type")->getIntValue();
+						if(LuaScript::debugModeEnabled == true) printf("Table item key_type: %d [%s]\n",key_type,lua_typename(luaState, key_type));
+
 						switch (key_type) {
 							case LUA_TSTRING:
+								if(LuaScript::debugModeEnabled) printf("  table item key [%s]\n",nodeTable->getAttribute("key")->getValue().c_str());
 								lua_pushstring( luaState, nodeTable->getAttribute("key")->getValue().c_str() );
 								break;
 							case LUA_TNUMBER:
-								lua_pushnumber( luaState, nodeTable->getAttribute("key")->getIntValue() );
+								if(LuaScript::debugModeEnabled) printf("  table item key [%s]\n",nodeTable->getAttribute("key")->getValue().c_str());
+								lua_pushnumber( luaState, nodeTable->getAttribute("key")->getFloatValue() );
 								break;
 							case LUA_TBOOLEAN:
-								lua_pushboolean( luaState, nodeTable->getAttribute("key")->getIntValue() );
+								if(LuaScript::debugModeEnabled) printf("  table item key [%s]\n",nodeTable->getAttribute("key")->getValue().c_str());
+								lua_pushboolean( luaState, nodeTable->getAttribute("key")->getBoolValue() );
 								break;
 						}
+
 						int value_type = nodeTable->getAttribute("value_type")->getIntValue();
+						if(LuaScript::debugModeEnabled == true) printf("Table item value_type: %d [%s]\n",value_type,lua_typename(luaState, value_type));
+
 						switch (value_type) {
 							case LUA_TSTRING:
+								if(LuaScript::debugModeEnabled) printf("  table item value [%s]\n",nodeTable->getAttribute("value")->getValue().c_str());
 								lua_pushstring( luaState, nodeTable->getAttribute("value")->getValue().c_str() );
 								break;
 							case LUA_TNUMBER:
-								lua_pushnumber( luaState, nodeTable->getAttribute("value")->getIntValue() );
+								if(LuaScript::debugModeEnabled) printf("  table item value [%s]\n",nodeTable->getAttribute("value")->getValue().c_str());
+								lua_pushnumber( luaState, nodeTable->getAttribute("value")->getFloatValue() );
 								break;
 							case LUA_TBOOLEAN:
-								lua_pushboolean( luaState, nodeTable->getAttribute("value")->getIntValue() );
+								if(LuaScript::debugModeEnabled) printf("  table item value [%s]\n",nodeTable->getAttribute("value")->getValue().c_str());
+								lua_pushboolean( luaState, nodeTable->getAttribute("value")->getBoolValue() );
 								break;
 						}
 
 						lua_rawset(luaState, -3);      /* Stores the pair in the table */
 					}
+					if(LuaScript::debugModeEnabled == true) printf("----------------- LUA TABLE DETECTED - END -----------------\n");
 				}
 				break;
 
@@ -620,7 +675,7 @@ string LuaScript::errorToString(int errorCode) {
 			error+= "Error while running the error handler"; 
 			break;
 		default:
-			error+= "Unknown error";
+			error+= "Unknown LUA error" + intToStr(errorCode);
 			break;
 	}
 
@@ -860,7 +915,7 @@ Vec2i LuaArguments::getVec2i(int argumentIndex) const{
 	//printf("xa = %s argumentIndex = %d\n",lua_tostring(luaState, argumentIndex),argumentIndex);
 
 	//v.x= luaL_checkint(luaState, argumentIndex);
-	v.x= lua_tointeger(luaState, argumentIndex);
+	v.x= (int)lua_tointeger(luaState, argumentIndex);
 	lua_pop(luaState, 1);
 
 	//printf("X = %d\n",v.x);
@@ -869,7 +924,7 @@ Vec2i LuaArguments::getVec2i(int argumentIndex) const{
 	//printf("ya = %s\n",lua_tostring(luaState, argumentIndex));
 
 	//v.y= luaL_checkint(luaState, argumentIndex);
-	v.y= lua_tointeger(luaState, argumentIndex);
+	v.y= (int)lua_tointeger(luaState, argumentIndex);
 	lua_pop(luaState, 1);
 
 	//printf("Y = %d\n",v.y);
@@ -920,6 +975,13 @@ void LuaArguments::returnInt(int value){
 	lua_pushinteger(luaState, value);
 }
 
+void LuaArguments::returnFloat(float value){
+	Lua_STREFLOP_Wrapper streflopWrapper;
+
+	++returnCount;
+	lua_pushnumber(luaState, value);
+}
+
 void LuaArguments::returnString(const string &value){
 	Lua_STREFLOP_Wrapper streflopWrapper;
 
