@@ -52,6 +52,7 @@ MapPreview::MapPreview() {
 	title = "";
 	desc = "";
 	author = "";
+	maxFactions = 0;
 	refAlt = DEFAULT_MAP_CELL_HEIGHT;
 	hasChanged = false;
 }
@@ -561,7 +562,7 @@ void MapPreview::reset(int w, int h, float alt, MapSurfaceType surf) {
 
 	this->w = w;
 	this->h = h;
-	this->maxFactions = maxFactions;
+	//this->maxFactions = maxFactions;
 
 	//cells = new Cell*[w];
 	cells.resize(w);
@@ -608,7 +609,7 @@ void MapPreview::resize(int w, int h, float alt, MapSurfaceType surf) {
 	int oldH = this->h;
 	this->w = w;
 	this->h = h;
-	this->maxFactions = maxFactions;
+	//this->maxFactions = maxFactions;
 
 	//create new cells
 	//Cell **oldCells = cells;
@@ -755,10 +756,10 @@ void toEndianMapFileHeader(MapFileHeader &header) {
 	header.height = Shared::PlatformByteOrder::toCommonEndian(header.height);
 	header.heightFactor = Shared::PlatformByteOrder::toCommonEndian(header.heightFactor);
 	header.waterLevel = Shared::PlatformByteOrder::toCommonEndian(header.waterLevel);
-	for(unsigned int i =0; i < MAX_TITLE_LENGTH; ++i) {
+	for(unsigned int i =0; i < (unsigned int)MAX_TITLE_LENGTH; ++i) {
 		header.title[i] = Shared::PlatformByteOrder::toCommonEndian(header.title[i]);
 	}
-	for(unsigned int i =0; i < MAX_DESCRIPTION_LENGTH; ++i) {
+	for(unsigned int i =0; i < (unsigned int)MAX_DESCRIPTION_LENGTH; ++i) {
 		header.description[i] = Shared::PlatformByteOrder::toCommonEndian(header.description[i]);
 	}
 }
@@ -769,10 +770,10 @@ void fromEndianMapFileHeader(MapFileHeader &header) {
 	header.height = Shared::PlatformByteOrder::fromCommonEndian(header.height);
 	header.heightFactor = Shared::PlatformByteOrder::fromCommonEndian(header.heightFactor);
 	header.waterLevel = Shared::PlatformByteOrder::fromCommonEndian(header.waterLevel);
-	for(unsigned int i =0; i < MAX_TITLE_LENGTH; ++i) {
+	for(unsigned int i =0; i < (unsigned int)MAX_TITLE_LENGTH; ++i) {
 		header.title[i] = Shared::PlatformByteOrder::fromCommonEndian(header.title[i]);
 	}
-	for(unsigned int i =0; i < MAX_DESCRIPTION_LENGTH; ++i) {
+	for(unsigned int i =0; i < (unsigned int)MAX_DESCRIPTION_LENGTH; ++i) {
 		header.description[i] = Shared::PlatformByteOrder::fromCommonEndian(header.description[i]);
 	}
 }
@@ -882,7 +883,7 @@ void MapPreview::loadFromFile(const string &path) {
 			}
 		}
 
-		fclose(f1);
+		if(f1) fclose(f1);
 
 		fileLoaded = true;
 		mapFileLoaded = path;
@@ -919,8 +920,11 @@ void MapPreview::saveToFile(const string &path) {
 		header.heightFactor = heightFactor;
 		header.waterLevel = waterLevel;
 		strncpy(header.title, title.c_str(), MAX_TITLE_LENGTH);
+		header.title[MAX_TITLE_LENGTH-1] = 0;
 		strncpy(header.author, author.c_str(), MAX_AUTHOR_LENGTH);
+		header.author[MAX_AUTHOR_LENGTH-1] = 0;
 		strncpy(header.version2.short_desc, desc.c_str(), MAX_DESCRIPTION_LENGTH_VERSION2);
+		header.version2.short_desc[MAX_DESCRIPTION_LENGTH_VERSION2-1] = 0;
 		header.version2.magic= 0x01020304;
 		header.version2.cliffLevel= cliffLevel;
 		header.version2.cameraHeight= cameraHeight;
@@ -960,7 +964,7 @@ void MapPreview::saveToFile(const string &path) {
 			}
 		}
 
-		fclose(f1);
+		if(f1) fclose(f1);
 
 		hasChanged = false;
 	}
