@@ -24,7 +24,17 @@ namespace Shared { namespace PlatformCommon {
 Mutex BaseThread::mutexMasterThreadList;
 std::map<void *,int> BaseThread::masterThreadList;
 
-BaseThread::BaseThread() : Thread(), ptr(NULL) {
+BaseThread::BaseThread() : Thread(),
+		mutexRunning(new Mutex(CODE_AT_LINE)),
+		mutexQuit(new Mutex(CODE_AT_LINE)),
+		mutexBeginExecution(new Mutex(CODE_AT_LINE)),
+		mutexDeleteSelfOnExecutionDone(new Mutex(CODE_AT_LINE)),
+	    mutexThreadObjectAccessor(new Mutex(CODE_AT_LINE)),
+	    mutexThreadOwnerValid(new Mutex(CODE_AT_LINE)),
+		mutexExecutingTask(new Mutex(CODE_AT_LINE)),
+		mutexStarted(new Mutex(CODE_AT_LINE)),
+		ptr(NULL), genericData(NULL) {
+
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	ptr = this;
@@ -36,6 +46,7 @@ BaseThread::BaseThread() : Thread(), ptr(NULL) {
 
 	setQuitStatus(false);
 	setRunningStatus(false);
+	setStarted(false);
 	setHasBeginExecution(false);
 	setExecutingTask(false);
 	setDeleteSelfOnExecutionDone(false);
@@ -44,22 +55,104 @@ BaseThread::BaseThread() : Thread(), ptr(NULL) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 }
 
+Mutex * BaseThread::getMutexThreadObjectAccessor() {
+	return mutexThreadObjectAccessor;
+}
+
 BaseThread::~BaseThread() {
+
+	//printf("In ~BaseThread Line: %d uniqueID [%s]\n",__LINE__,uniqueID.c_str());
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s]\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str());
+
+	//BaseThread *base_thread = dynamic_cast<BaseThread *>(this);
+	if(this->getStarted() == false) {
+		time_t elapsed = time(NULL);
+		for(;this->getStarted() == false &&
+    		difftime((long int)time(NULL),elapsed) <= 3;) {
+			sleep(5);
+		}
+	}
+
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s]\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str());
 	bool ret = shutdownAndWait();
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s] ret [%d] END\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str(),ret);
 
+	//printf("In ~BaseThread Line: %d uniqueID [%s]\n",__LINE__,uniqueID.c_str());
+
 	MutexSafeWrapper safeMutexMasterList(&mutexMasterThreadList);
+
+	//printf("In ~BaseThread Line: %d uniqueID [%s]\n",__LINE__,uniqueID.c_str());
+
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s] ret [%d] END\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str(),ret);
+
+	//printf("In ~BaseThread Line: %d uniqueID [%s]\n",__LINE__,uniqueID.c_str());
+
 	if(masterThreadList.find(this) == masterThreadList.end()) {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s] ret [%d] END\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str(),ret);
+
 		char szBuf[8096]="";
 		snprintf(szBuf,8096,"invalid thread delete for ptr: %p",this);
 		throw megaglest_runtime_error(szBuf);
 	}
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s] ret [%d] END\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str(),ret);
+
+	//printf("In ~BaseThread Line: %d uniqueID [%s]\n",__LINE__,uniqueID.c_str());
+
 	masterThreadList[this]--;
 	if(masterThreadList[this] <= 0) {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s] ret [%d] END\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str(),ret);
 		masterThreadList.erase(this);
 	}
+
+	//printf("In ~BaseThread Line: %d uniqueID [%s]\n",__LINE__,uniqueID.c_str());
+
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s] ret [%d] END\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str(),ret);
+
 	safeMutexMasterList.ReleaseLock();
+
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s] ret [%d] END\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str(),ret);
+
+
+	delete mutexRunning;
+	mutexRunning = NULL;
+	delete mutexQuit;
+	mutexQuit = NULL;
+	delete mutexBeginExecution;
+	mutexBeginExecution = NULL;
+	delete mutexDeleteSelfOnExecutionDone;
+	mutexDeleteSelfOnExecutionDone = NULL;
+	delete mutexThreadObjectAccessor;
+	mutexThreadObjectAccessor = NULL;
+    delete mutexThreadOwnerValid;
+    mutexThreadOwnerValid = NULL;
+    delete mutexExecutingTask;
+    mutexExecutingTask = NULL;
+	delete mutexStarted;
+	mutexStarted = NULL;
+
+	//printf("In ~BaseThread Line: %d uniqueID [%s] [%p]\n",__LINE__,uniqueID.c_str(),this);
+}
+
+bool BaseThread::getStarted() {
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexStarted,mutexOwnerId);
+	mutexStarted->setOwnerId(mutexOwnerId);
+	bool retval = started;
+	safeMutex.ReleaseLock();
+
+	return retval;
+}
+
+void BaseThread::setStarted(bool value) {
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s]\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str());
+
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexStarted,mutexOwnerId);
+	mutexStarted->setOwnerId(mutexOwnerId);
+	started = value;
+	safeMutex.ReleaseLock();
+
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s]\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str());
 }
 
 bool BaseThread::isThreadDeleted(void *ptr) {
@@ -74,23 +167,23 @@ bool BaseThread::isThreadDeleted(void *ptr) {
 
 Mutex * BaseThread::getMutexThreadOwnerValid() {
 	if(getThreadOwnerValid() == true) {
-	    return &mutexThreadOwnerValid;
+	    return mutexThreadOwnerValid;
 	}
     return NULL;
 }
 
 void BaseThread::setThreadOwnerValid(bool value) {
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexThreadOwnerValid,mutexOwnerId);
-	mutexThreadOwnerValid.setOwnerId(mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexThreadOwnerValid,mutexOwnerId);
+	mutexThreadOwnerValid->setOwnerId(mutexOwnerId);
 	threadOwnerValid = value;
 	safeMutex.ReleaseLock();
 }
 
 bool BaseThread::getThreadOwnerValid() {
 	//bool ret = false;
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexThreadOwnerValid,mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexThreadOwnerValid,mutexOwnerId);
 	//mutexThreadOwnerValid.setOwnerId(mutexOwnerId);
 	bool ret = threadOwnerValid;
 	safeMutex.ReleaseLock();
@@ -105,9 +198,9 @@ void BaseThread::signalQuit() {
 void BaseThread::setQuitStatus(bool value) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s]\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str());
 
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexQuit,mutexOwnerId);
-	mutexQuit.setOwnerId(mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexQuit,mutexOwnerId);
+	mutexQuit->setOwnerId(mutexOwnerId);
 	quit = value;
 	safeMutex.ReleaseLock();
 
@@ -116,8 +209,8 @@ void BaseThread::setQuitStatus(bool value) {
 
 bool BaseThread::getQuitStatus() {
 	//bool retval = false;
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexQuit,mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexQuit,mutexOwnerId);
 	//mutexQuit.setOwnerId(mutexOwnerId);
 	bool retval = quit;
 	safeMutex.ReleaseLock();
@@ -127,8 +220,8 @@ bool BaseThread::getQuitStatus() {
 
 bool BaseThread::getHasBeginExecution() {
 	//bool retval = false;
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexBeginExecution,mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexBeginExecution,mutexOwnerId);
 	//mutexBeginExecution.setOwnerId(mutexOwnerId);
 	bool retval = hasBeginExecution;
 	safeMutex.ReleaseLock();
@@ -139,9 +232,9 @@ bool BaseThread::getHasBeginExecution() {
 void BaseThread::setHasBeginExecution(bool value) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] uniqueID [%s]\n",__FILE__,__FUNCTION__,__LINE__,uniqueID.c_str());
 
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexBeginExecution,mutexOwnerId);
-	mutexBeginExecution.setOwnerId(mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexBeginExecution,mutexOwnerId);
+	mutexBeginExecution->setOwnerId(mutexOwnerId);
 	hasBeginExecution = value;
 	safeMutex.ReleaseLock();
 
@@ -151,8 +244,8 @@ void BaseThread::setHasBeginExecution(bool value) {
 bool BaseThread::getRunningStatus() {
 	//bool retval = false;
 
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexRunning,mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexRunning,mutexOwnerId);
 	bool retval = running;
 	safeMutex.ReleaseLock();
 
@@ -164,9 +257,9 @@ bool BaseThread::getRunningStatus() {
 }
 
 void BaseThread::setRunningStatus(bool value) {
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexRunning,mutexOwnerId);
-	mutexRunning.setOwnerId(mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexRunning,mutexOwnerId);
+	mutexRunning->setOwnerId(mutexOwnerId);
 	running = value;
 	if(value == true) {
 		setHasBeginExecution(true);
@@ -175,17 +268,17 @@ void BaseThread::setRunningStatus(bool value) {
 }
 
 void BaseThread::setExecutingTask(bool value) {
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexExecutingTask,mutexOwnerId);
-	mutexExecutingTask.setOwnerId(mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexExecutingTask,mutexOwnerId);
+	mutexExecutingTask->setOwnerId(mutexOwnerId);
 	executingTask = value;
 	safeMutex.ReleaseLock();
 }
 
 bool BaseThread::getExecutingTask() {
 	//bool retval = false;
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-	MutexSafeWrapper safeMutex(&mutexExecutingTask,mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+	MutexSafeWrapper safeMutex(mutexExecutingTask,mutexOwnerId);
 	bool retval = executingTask;
 	safeMutex.ReleaseLock();
 
@@ -194,8 +287,8 @@ bool BaseThread::getExecutingTask() {
 
 bool BaseThread::getDeleteSelfOnExecutionDone() {
     //bool retval = false;
-    static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-    MutexSafeWrapper safeMutex(&mutexDeleteSelfOnExecutionDone,mutexOwnerId);
+    static string mutexOwnerId = CODE_AT_LINE;
+    MutexSafeWrapper safeMutex(mutexDeleteSelfOnExecutionDone,mutexOwnerId);
     bool retval = deleteSelfOnExecutionDone;
     safeMutex.ReleaseLock();
 
@@ -203,16 +296,16 @@ bool BaseThread::getDeleteSelfOnExecutionDone() {
 }
 
 void BaseThread::setDeleteSelfOnExecutionDone(bool value) {
-	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
-    MutexSafeWrapper safeMutex(&mutexDeleteSelfOnExecutionDone,mutexOwnerId);
-    mutexDeleteSelfOnExecutionDone.setOwnerId(mutexOwnerId);
+	static string mutexOwnerId = CODE_AT_LINE;
+    MutexSafeWrapper safeMutex(mutexDeleteSelfOnExecutionDone,mutexOwnerId);
+    mutexDeleteSelfOnExecutionDone->setOwnerId(mutexOwnerId);
     deleteSelfOnExecutionDone = value;
 }
 
 void BaseThread::deleteSelfIfRequired() {
     if(getDeleteSelfOnExecutionDone() == true) {
     	if(isThreadDeleted(this->ptr) == false) {
-    		delete this;
+    		this->setDeleteAfterExecute(true);
     	}
         return;
     }
