@@ -65,12 +65,21 @@
 #include "platform_util.h"
 #include "utf8.h"
 #include "byte_order.h"
-#include "leak_dumper.h"
+
+#if _BSD_SOURCE || _SVID_SOURCE || _XOPEN_SOURCE >= 500 || _XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED
+#include <unistd.h>
+#include <sys/types.h>
+#include <pwd.h>
+#endif
 
 #ifdef __APPLE__
 #include <mach-o/dyld.h>
+#include <sys/param.h>
 #endif
 
+#include "leak_dumper.h"
+
+
 using namespace Shared::Platform;
 using namespace Shared::Util;
 using namespace std;
@@ -82,6 +91,9 @@ namespace Shared { namespace PlatformCommon {
 const time_t REFRESH_CRC_DAY_SECONDS = 60 * 60 * 24;
 static string crcCachePath = "";
 
+static string gameVersion = "";
+static string gameGITVersion = "";
+
 namespace Private {
 
 bool shouldBeFullscreen = false;
@@ -94,29 +106,29 @@ int ScreenHeight = 600;
 //          PerformanceTimer
 // =====================================
 
-void PerformanceTimer::init(float fps, int maxTimes){
-	times= 0;
-	this->maxTimes= maxTimes;
-
-	lastTicks= SDL_GetTicks();
-
-	updateTicks= static_cast<int>(1000./fps);
+void PerformanceTimer::init(float fps, int maxTimes) {
+	this->times			= 0;
+	this->maxTimes		= maxTimes;
+	this->lastTicks		= SDL_GetTicks();
+	this->updateTicks	= static_cast<int>(1000.0f / fps);
 }
 
-bool PerformanceTimer::isTime(){
+bool PerformanceTimer::isTime() {
 	Uint32 thisTicks = SDL_GetTicks();
 
-	if((thisTicks-lastTicks)>=updateTicks && times<maxTimes){
-		lastTicks+= updateTicks;
+	if((thisTicks - lastTicks) >= updateTicks &&
+		times < maxTimes) {
+
+		lastTicks += updateTicks;
 		times++;
 		return true;
 	}
-	times= 0;
+	times = 0;
 	return false;
 }
 
-void PerformanceTimer::reset(){
-	lastTicks= SDL_GetTicks();
+void PerformanceTimer::reset() {
+	lastTicks = SDL_GetTicks();
 }
 
 // =====================================
@@ -124,41 +136,45 @@ void PerformanceTimer::reset(){
 // =====================================
 
 Chrono::Chrono(bool autoStart) {
-	freq = 1000;
-	stopped= true;
-	accumCount= 0;
+	freq 			= 1000;
+	stopped			= true;
+	accumCount		= 0;
 
-	lastStartCount = 0;
-	lastTickCount = 0;
-	lastResult = 0;
-	lastMultiplier = 0;
-	lastStopped = false;
+	lastStartCount 	= 0;
+	lastTickCount 	= 0;
+	lastResult 		= 0;
+	lastMultiplier 	= 0;
+	lastStopped 	= false;
+	startCount 		= 0;
 
 	if(autoStart == true) {
 		start();
 	}
 }
 
+bool Chrono::isStarted() const {
+	return (stopped == false);
+}
+
 void Chrono::start() {
-	stopped= false;
-	startCount = SDL_GetTicks();
+	stopped		= false;
+	startCount 	= SDL_GetTicks();
 }
 
 void Chrono::stop() {
-	Uint32 endCount;
-	endCount = SDL_GetTicks();
-	accumCount += endCount-startCount;
-	stopped= true;
+	Uint32 endCount	= SDL_GetTicks();
+	accumCount 		+= endCount - startCount;
+	stopped			= true;
 }
 
 void Chrono::reset() {
-	accumCount = 0;
-	lastStartCount = 0;
-	lastTickCount = 0;
-	lastResult = 0;
-	lastMultiplier = 0;
+	accumCount 		= 0;
+	lastStartCount 	= 0;
+	lastTickCount 	= 0;
+	lastResult 		= 0;
+	lastMultiplier 	= 0;
 
-	startCount = SDL_GetTicks();
+	startCount 		= SDL_GetTicks();
 }
 
 int64 Chrono::getMicros() {
@@ -173,13 +189,13 @@ int64 Chrono::getSeconds() {
 	return queryCounter(1);
 }
 
-int64 Chrono::queryCounter(int multiplier) {
+int64 Chrono::queryCounter(int64 multiplier) {
 
 	if(	multiplier == lastMultiplier &&
 		stopped == lastStopped &&
 		lastStartCount == startCount) {
 
-		if(stopped) {
+		if(stopped == true) {
 			return lastResult;
 		}
 		else {
@@ -191,19 +207,19 @@ int64 Chrono::queryCounter(int multiplier) {
 	}
 
 	int64 result = 0;
-	if(stopped) {
-		result = multiplier*accumCount/freq;
+	if(stopped == true) {
+		result = multiplier * accumCount / freq;
 	}
 	else {
 		Uint32 endCount = SDL_GetTicks();
-		result = multiplier*(accumCount+endCount-startCount)/freq;
+		result = multiplier * (accumCount + endCount - startCount) / freq;
 		lastTickCount = endCount;
 	}
 
-	lastStartCount = startCount;
-	lastResult = result;
-	lastMultiplier = multiplier;
-	lastStopped = stopped;
+	lastStartCount 	= startCount;
+	lastResult 		= result;
+	lastMultiplier 	= multiplier;
+	lastStopped 	= stopped;
 
 	return result;
 }
@@ -250,7 +266,7 @@ void Tokenize(const string& str,vector<string>& tokens,const string& delimiters)
 			break;
 		}
 		tokens.push_back( string( textLine.substr( pos, nextPos - pos ) ) );
-		pos = nextPos + 1;
+		pos = nextPos + delimiters.size();
 	}
 
 }
@@ -336,8 +352,8 @@ void findAll(const string &path, vector<string> &results, bool cutExtension, boo
 	/** Stupid win32 is searching for all files without extension when *. is
 	 * specified as wildcard
 	 */
-	if(mypath.size() >= 2 && mypath.compare(max((size_t)0,mypath.size() - 2), 2, "*.") == 0) {
-		mypath = mypath.substr(0, max((size_t)0,mypath.size() - 2));
+	if((int)mypath.size() >= 2 && mypath.compare(max((int)0,(int)mypath.size() - 2), 2, "*.") == 0) {
+		mypath = mypath.substr(0, max((int)0,(int)mypath.size() - 2));
 		mypath += "*";
 	}
 
@@ -354,7 +370,7 @@ void findAll(const string &path, vector<string> &results, bool cutExtension, boo
 		}
 	}
 	else {
-		for(int i = 0; i < globbuf.gl_pathc; ++i) {
+		for(int i = 0; i < (int)globbuf.gl_pathc; ++i) {
 			const char* p = globbuf.gl_pathv[i];
 			const char* begin = p;
 			for( ; *p != 0; ++p) {
@@ -362,7 +378,7 @@ void findAll(const string &path, vector<string> &results, bool cutExtension, boo
 				if(*p == '/')
 					begin = p+1;
 			}
-			if(!(strcmp(".", begin)==0 || strcmp("..", begin)==0 || strcmp(".svn", begin)==0)) {
+			if(!(strcmp(".", begin)==0 || strcmp("..", begin)==0 || strcmp(".git", begin)==0)) {
 				results.push_back(begin);
 			}
 			else {
@@ -423,6 +439,8 @@ bool isdir(const char *path)
 }
 
 bool fileExists(const string &path) {
+	 if (path.size() == 0) return false;
+
 #ifdef WIN32
 	wstring wstr = utf8_decode(path);
 	FILE* file= _wfopen(wstr.c_str(), L"rb");
@@ -433,14 +451,14 @@ bool fileExists(const string &path) {
 		fclose(file);
 		return true;
 	}
-	else {
+//	else {
 		//int fileErrno = errno;
-#ifdef WIN32
-        int fileErrno = errno;
-		DWORD error = GetLastError();
-		string strError = "[#6] Could not open file, result: " + intToStr(error) + " - " + intToStr(fileErrno) + " " + strerror(fileErrno) + " [" + path + "]";
-#endif
-	}
+//#ifdef WIN32
+        //int fileErrno = errno;
+		//DWORD error = GetLastError();
+		//string strError = "[#6] Could not open file, result: " + intToStr(error) + " - " + intToStr(fileErrno) + " " + strerror(fileErrno) + " [" + path + "]";
+//#endif
+//	}
 	return false;
 }
 
@@ -456,10 +474,10 @@ void removeFolder(const string &path) {
     // First delete files
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] DELETE FILES\n",__FILE__,__FUNCTION__,__LINE__);
 
-    for(int i = results.size() -1; i >= 0; --i) {
+    for(int i = (int)results.size() -1; i >= 0; --i) {
         string item = results[i];
 
-        //if(item.find(".svn") != string::npos) {
+        //if(item.find(".git") != string::npos) {
         //	printf("!!!!!!!!!!!!!!!!!! FOUND SPECIAL FOLDER [%s] in [%s]\n",item.c_str(),path.c_str());
         //}
 
@@ -471,7 +489,7 @@ void removeFolder(const string &path) {
     // Now delete folders
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] DELETE FOLDERS\n",__FILE__,__FUNCTION__,__LINE__);
 
-    for(int i = results.size() -1; i >= 0; --i) {
+    for(int i = (int)results.size() -1; i >= 0; --i) {
         string item = results[i];
         if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] item [%s] isdir(item.c_str()) = %d\n",__FILE__,__FUNCTION__,__LINE__,item.c_str(), isdir(item.c_str()));
         if(isdir(item.c_str()) == true) {
@@ -511,7 +529,7 @@ bool EndsWith(const string &str, const string& key)
 {
     bool result = false;
     if (str.length() >= key.length()) {
-    	result = (0 == str.compare(max((size_t)0,str.length() - key.length()), key.length(), key));
+    	result = (0 == str.compare(max((int)0,(int)str.length() - (int)key.length()), key.length(), key));
     }
 
     return result;
@@ -544,8 +562,36 @@ void trimPathWithStartingSlash(string &path) {
 }
 
 void updatePathClimbingParts(string &path) {
+	// Update paths with /./
+	string::size_type pos = path.find("/./");
+	if(pos != string::npos && pos != 0) {
+		string orig = path;
+		path.erase(pos,2);
+		//pos--;
+
+		pos = path.find("/./");
+		if(pos != string::npos && pos != 0) {
+			updatePathClimbingParts(path);
+		}
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("CHANGED relative path from [%s] to [%s]\n",orig.c_str(),path.c_str());
+	}
+	pos = path.find("\\.\\");
+	if(pos != string::npos && pos != 0) {
+		string orig = path;
+		path.erase(pos,2);
+		//pos--;
+
+		pos = path.find("\\.\\");
+		if(pos != string::npos && pos != 0) {
+			updatePathClimbingParts(path);
+		}
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("CHANGED relative path from [%s] to [%s]\n",orig.c_str(),path.c_str());
+	}
+
 	// Update paths with ..
-	string::size_type pos = path.find("..");
+	pos = path.find("..");
 	if(pos != string::npos && pos != 0) {
 		string orig = path;
 		path.erase(pos,2);
@@ -554,10 +600,10 @@ void updatePathClimbingParts(string &path) {
 			path.erase(pos,1);
 		}
 
-		for(int x = pos; x >= 0; --x) {
+		for(int x = (int)pos; x >= 0; --x) {
 			//printf("x [%d][%c] pos [%ld][%c] [%s]\n",x,path[x],(long int)pos,path[pos],path.substr(0,x+1).c_str());
 
-			if((path[x] == '/' || path[x] == '\\') && x != pos) {
+			if((path[x] == '/' || path[x] == '\\') && x != (int)pos) {
 				path.erase(x,pos-x);
 				break;
 			}
@@ -570,7 +616,6 @@ void updatePathClimbingParts(string &path) {
 		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("CHANGED relative path from [%s] to [%s]\n",orig.c_str(),path.c_str());
 	}
 
-
 /*
 	string::size_type pos = path.rfind("..");
 	if(pos != string::npos && pos != 0) {
@@ -603,21 +648,31 @@ void setCRCCacheFilePath(string path) {
 	crcCachePath = path;
 }
 
-string getFormattedCRCCacheFileName(std::pair<string,string> cacheKeys) {
+string getGameVersion() {
+	return gameVersion;
+}
+string getGameGITVersion() {
+	return gameGITVersion;
+}
+void setGameVersion(string version) {
+	gameVersion = version;
+}
+void setGameGITVersion(string git) {
+	gameGITVersion = git;
+}
+
+string getCRCCacheFileName(std::pair<string,string> cacheKeys) {
 	string crcCacheFile = cacheKeys.first + cacheKeys.second;
-	//replaceAll(crcCacheFile, "/", "_");
-	//replaceAll(crcCacheFile, "\\", "_");
-	//replaceAll(crcCacheFile, "*", "_");
-	//replaceAll(crcCacheFile, ".", "_");
-	//return getCRCCacheFilePath() + crcCacheFile;
+	return crcCacheFile;
+}
+string getFormattedCRCCacheFileName(std::pair<string,string> cacheKeys) {
+	string crcCacheFile = getCRCCacheFileName(cacheKeys);
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] cacheKeys.first = [%s] cacheKeys.second [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,cacheKeys.first.c_str(),cacheKeys.second.c_str());
 
 	Checksum checksum;
 	checksum.addString(crcCacheFile);
-	//checksum.addString(lastFile(crcCacheFile));
 	string result = getCRCCacheFilePath() + "CRC_CACHE_" + uIntToStr(checksum.getSum());
-	//printf("result [%s]\n",result.c_str());
 	return result;
 }
 std::pair<string,string> getFolderTreeContentsCheckSumCacheKey(vector<string> paths, string pathSearchString, const string &filterFileExt) {
@@ -654,10 +709,24 @@ pair<bool,time_t> hasCachedFileCRCValue(string crcCacheFile, uint32 &value) {
 				SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d for Cache file [%s]\n",__FILE__,__FUNCTION__,__LINE__,crcCacheFile.c_str());
 			}
 
-			int readbytes = fscanf(fp,"%20ld,%20u,%20ld",&refreshDate,&crcValue,&lastUpdateDate);
+			char gameVer[500]="";
+			char gitVer[500]="";
+			char actualFilePath[8096]="";
+			int readbytes = fscanf(fp,"%20ld,%20u,%20ld\n%499s\n%499s\n%8095s",
+					&refreshDate,
+					&crcValue,
+					&lastUpdateDate,
+					&gameVer[0],
+					&gitVer[0],
+					&actualFilePath[0]);
 			refreshDate = Shared::PlatformByteOrder::fromCommonEndian(refreshDate);
 			crcValue = Shared::PlatformByteOrder::fromCommonEndian(crcValue);
 			lastUpdateDate = Shared::PlatformByteOrder::fromCommonEndian(lastUpdateDate);
+			string readGameVer = Shared::PlatformByteOrder::fromCommonEndian(string(gameVer));
+			string readGitVer = Shared::PlatformByteOrder::fromCommonEndian(string(gitVer));
+			string readActualFilePath = Shared::PlatformByteOrder::fromCommonEndian(string(actualFilePath));
+
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("CRC readGameVer [%s] [%s]\n%s\n",readGameVer.c_str(),readGitVer.c_str(),readActualFilePath.c_str());
 
 			if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) {
 				SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d for Cache file [%s] readbytes = %d\n",__FILE__,__FUNCTION__,__LINE__,crcCacheFile.c_str(),readbytes);
@@ -680,7 +749,8 @@ pair<bool,time_t> hasCachedFileCRCValue(string crcCacheFile, uint32 &value) {
 			time_t tBadCRCDate = mktime( &future );
 
 			result.second = lastUpdateDate;
-			if(	refreshDate > 0 &&
+			if(	readGameVer != "" && readGitVer != "" &&
+					refreshDate > 0 &&
 				refreshDate > tBadCRCDate &&
 				time(NULL) < refreshDate) {
 
@@ -729,7 +799,7 @@ pair<bool,time_t> hasCachedFileCRCValue(string crcCacheFile, uint32 &value) {
 	return result;
 }
 
-void writeCachedFileCRCValue(string crcCacheFile, uint32 &crcValue) {
+void writeCachedFileCRCValue(string crcCacheFile, uint32 &crcValue, string actualFileName) {
 #ifdef WIN32
 		FILE *fp = _wfopen(utf8_decode(crcCacheFile).c_str(), L"w");
 #else
@@ -750,7 +820,20 @@ void writeCachedFileCRCValue(string crcCacheFile, uint32 &crcValue) {
         char szBuf1[100]="";
         strftime(szBuf1,100,"%Y-%m-%d %H:%M:%S",loctime);
 
-		fprintf(fp,"%ld,%u,%ld",Shared::PlatformByteOrder::toCommonEndian(refreshDate),Shared::PlatformByteOrder::toCommonEndian(crcValue),Shared::PlatformByteOrder::toCommonEndian(now));
+		string writeGameVer = Shared::PlatformByteOrder::toCommonEndian(gameVersion);
+		string writeGameGITVersion = Shared::PlatformByteOrder::toCommonEndian(gameGITVersion);
+		string writeActualFileName = Shared::PlatformByteOrder::toCommonEndian(actualFileName);
+
+		fprintf(fp,"%20ld,%20u,%20ld",
+				Shared::PlatformByteOrder::toCommonEndian(refreshDate),
+				Shared::PlatformByteOrder::toCommonEndian(crcValue),
+				Shared::PlatformByteOrder::toCommonEndian(now));
+
+		fprintf(fp,"\n%s\n%s\n%s",
+				writeGameVer.c_str(),
+				writeGameGITVersion.c_str(),
+				writeActualFileName.c_str());
+
 		fclose(fp);
 
 		//if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"========== Writing CRC Cache offset [%d] refreshDate = %ld [%s], crcValue = %u, file [%s]\n",offset,refreshDate,szBuf1,crcValue,crcCacheFile.c_str());
@@ -783,8 +866,8 @@ time_t getFolderTreeContentsCheckSumRecursivelyLastGenerated(vector<string> path
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"-------------- In [%s::%s Line: %d] Calculating CRC for [%s] -----------\n",__FILE__,__FUNCTION__,__LINE__,pathSearchString.c_str());
 
 	std::pair<string,string> cacheKeys = getFolderTreeContentsCheckSumCacheKey(paths, pathSearchString, filterFileExt);
-	string cacheLookupId =  cacheKeys.first;
-	std::map<string,uint32> &crcTreeCache = CacheManager::getCachedItem< std::map<string,uint32> >(cacheLookupId);
+	//string cacheLookupId =  cacheKeys.first;
+	//std::map<string,uint32> &crcTreeCache = CacheManager::getCachedItem< std::map<string,uint32> >(cacheLookupId);
 
 	string cacheKey = cacheKeys.second;
 	string crcCacheFile = getFormattedCRCCacheFileName(cacheKeys);
@@ -865,7 +948,7 @@ uint32 getFolderTreeContentsCheckSumRecursively(vector<string> paths, string pat
 	uint32 result = checksum.getFinalFileListSum();
 	//if(forceNoCache == false) {
 	crcTreeCache[cacheKey] = result;
-	writeCachedFileCRCValue(crcCacheFile, crcTreeCache[cacheKey]);
+	writeCachedFileCRCValue(crcCacheFile, crcTreeCache[cacheKey],getCRCCacheFileName(cacheKeys));
 	//}
 	return result;
 }
@@ -938,8 +1021,8 @@ uint32 getFolderTreeContentsCheckSumRecursively(const string &path, const string
 	// Stupid win32 is searching for all files without extension when *. is
 	// specified as wildcard
 	//
-	if(mypath.size() >= 2 && mypath.compare(max((size_t)0,mypath.size() - 2), 2, "*.") == 0) {
-		mypath = mypath.substr(0, max((size_t)0,mypath.size() - 2));
+	if((int)mypath.size() >= 2 && mypath.compare(max((int)0,(int)mypath.size() - 2), 2, "*.") == 0) {
+		mypath = mypath.substr(0, max((int)0,(int)mypath.size() - 2));
 		mypath += "*";
 	}
 
@@ -956,13 +1039,13 @@ uint32 getFolderTreeContentsCheckSumRecursively(const string &path, const string
 
 	int fileLoopCount = 0;
 	int fileMatchCount = 0;
-	for(int i = 0; i < globbuf.gl_pathc; ++i) {
+	for(int i = 0; i < (int)globbuf.gl_pathc; ++i) {
 		const char* p = globbuf.gl_pathv[i];
 		//printf("Line: %d p [%s]\n",__LINE__,p);
 
 		if(isdir(p) == false) {
             bool addFile = true;
-            if(EndsWith(p, ".") == true || EndsWith(p, "..") == true || EndsWith(p, ".svn") == true) {
+            if(EndsWith(p, ".") == true || EndsWith(p, "..") == true || EndsWith(p, ".git") == true) {
             	addFile = false;
             }
             else if(filterFileExt != "") {
@@ -994,7 +1077,7 @@ uint32 getFolderTreeContentsCheckSumRecursively(const string &path, const string
 	}
 #endif
 
-	for(int i = 0; i < globbuf.gl_pathc; ++i) {
+	for(int i = 0; i < (int)globbuf.gl_pathc; ++i) {
 #if defined(__APPLE__) || defined(__FreeBSD__)
 		struct stat statStruct;
 		// only process if dir..
@@ -1025,7 +1108,7 @@ uint32 getFolderTreeContentsCheckSumRecursively(const string &path, const string
 		//if(forceNoCache == false) {
 		crcTreeCache[cacheKey] = result;
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] scanning [%s] ending checksum = %d for cacheKey [%s] fileMatchCount = %d, fileLoopCount = %d\n",__FILE__,__FUNCTION__,path.c_str(),crcTreeCache[cacheKey],cacheKey.c_str(),fileMatchCount,fileLoopCount);
-		writeCachedFileCRCValue(crcCacheFile, crcTreeCache[cacheKey]);
+		writeCachedFileCRCValue(crcCacheFile, crcTreeCache[cacheKey],getCRCCacheFileName(cacheKeys));
 		//}
 
 		return result;
@@ -1108,8 +1191,8 @@ vector<string> getFolderTreeContentsListRecursively(const string &path, const st
 	/** Stupid win32 is searching for all files without extension when *. is
 	 * specified as wildcard
 	 */
-	if(mypath.size() >= 2 && mypath.compare(max((size_t)0,mypath.size() - 2), 2, "*.") == 0) {
-		mypath = mypath.substr(0, max((size_t)0,mypath.size() - 2));
+	if((int)mypath.size() >= 2 && mypath.compare(max((int)0,(int)mypath.size() - 2), 2, "*.") == 0) {
+		mypath = mypath.substr(0, max((int)0,(int)mypath.size() - 2));
 		mypath += "*";
 	}
 
@@ -1134,7 +1217,7 @@ vector<string> getFolderTreeContentsListRecursively(const string &path, const st
 		throw megaglest_runtime_error(msg.str());
 	}
 #endif
-	for(int i = 0; i < globbuf.gl_pathc; ++i) {
+	for(int i = 0; i < (int)globbuf.gl_pathc; ++i) {
 		const char* p = globbuf.gl_pathv[i];
 
 		bool skipItem = (EndsWith(p, ".") == true || EndsWith(p, "..") == true);
@@ -1175,7 +1258,7 @@ vector<string> getFolderTreeContentsListRecursively(const string &path, const st
 	}
 #endif
 
-	for(int i = 0; i < globbuf.gl_pathc; ++i) {
+	for(int i = 0; i < (int)globbuf.gl_pathc; ++i) {
 #if defined(__APPLE__) || defined(__FreeBSD__)
 		struct stat statStruct;
 		// only get if dir..
@@ -1259,8 +1342,8 @@ vector<std::pair<string,uint32> > getFolderTreeContentsCheckSumListRecursively(c
 	/** Stupid win32 is searching for all files without extension when *. is
 	 * specified as wildcard
 	 */
-	if(mypath.size() >= 2 && mypath.compare(max((size_t)0,mypath.size() - 2), 2, "*.") == 0) {
-		mypath = mypath.substr(0, max((size_t)0,mypath.size() - 2));
+	if((int)mypath.size() >= 2 && mypath.compare(max((int)0,(int)mypath.size() - 2), 2, "*.") == 0) {
+		mypath = mypath.substr(0, max((int)0,(int)mypath.size() - 2));
 		mypath += "*";
 	}
 
@@ -1276,12 +1359,12 @@ vector<std::pair<string,uint32> > getFolderTreeContentsCheckSumListRecursively(c
 	}
 #endif
 
-	for(int i = 0; i < globbuf.gl_pathc; ++i) {
+	for(int i = 0; i < (int)globbuf.gl_pathc; ++i) {
 		const char* p = globbuf.gl_pathv[i];
 
 		if(isdir(p) == false) {
             bool addFile = true;
-            if(EndsWith(p, ".") == true || EndsWith(p, "..") == true || EndsWith(p, ".svn") == true) {
+            if(EndsWith(p, ".") == true || EndsWith(p, "..") == true || EndsWith(p, ".git") == true) {
             	addFile = false;
             }
             else if(filterFileExt != "") {
@@ -1314,7 +1397,7 @@ vector<std::pair<string,uint32> > getFolderTreeContentsCheckSumListRecursively(c
 	}
 #endif
 
-	for(int i = 0; i < globbuf.gl_pathc; ++i) {
+	for(int i = 0; i < (int)globbuf.gl_pathc; ++i) {
 #if defined(__APPLE__) || defined(__FreeBSD__)
 		struct stat statStruct;
 		// only get if dir..
@@ -1373,7 +1456,7 @@ string extractLastDirectoryFromPath(string Path) {
 			result = Path.erase( 0, lastDirectory + 1);
 		}
 		else {
-			for(int i = lastDirectory-1; i >= 0; --i) {
+			for(int i = (int)lastDirectory-1; i >= 0; --i) {
 				if((Path[i] == '/' || Path[i] == '\\') && i > 0) {
 					result = Path.erase( 0, i);
 					break;
@@ -1529,10 +1612,10 @@ void getFullscreenVideoModes(vector<ModeInfo> *modeinfos, bool isFullscreen) {
 					format.BitsPerPixel = 16;
 					bpp = 16;
 					break;
-				case 3://8 bpp
-					format.BitsPerPixel = 8;
-					bpp = 8;
-					break;
+//				case 3://8 bpp
+//					format.BitsPerPixel = 8;
+//					bpp = 8;
+//					break;
 			}
 
 			/* Get available fullscreen/hardware modes */
@@ -1687,16 +1770,16 @@ void getFullscreenVideoModes(vector<ModeInfo> *modeinfos, bool isFullscreen) {
 				    }
 			   }
 		  }
-	} while(++loops != 4);
+	//} while(++loops != 4);
+	} while(++loops != 3);
 
 	std::sort(modeinfos->begin(),modeinfos->end());
 }
 
 
 
-bool changeVideoMode(int resW, int resH, int colorBits, int ) {
-	Private::shouldBeFullscreen = true;
-	return true;
+void changeVideoModeFullScreen(bool value) {
+	Private::shouldBeFullscreen = value;
 }
 
 void restoreVideoMode(bool exitingApp) {
@@ -1799,48 +1882,20 @@ bool isKeyDown(int virtualKey) {
 }
 
 string replaceAllHTMLEntities(string& context) {
-//	quotation mark = APL quote	&quot;	&#34;	&#x22;	"	"	"
-//	ampersand	&amp;	&#38;	&#x26;	&	&	&
-//	less-than sign	&lt;	&#60;	&#x3C;	<	<	<
-//	greater-than sign	&gt;	&#62;	&#x3E;	>	>	>
-//	Latin capital ligature OE	&OElig;	&#338;	&#x152;	Œ	Œ	Œ
-//	Latin small ligature oe	&oelig;	&#339;	&#x153;	œ	œ	œ
-//	Latin capital letter S with caron	&Scaron;	&#352;	&#x160;	Š	Š	Š
-//	Latin small letter s with caron	&scaron;	&#353;	&#x161;	š	š	š
-//	Latin capital letter Y with diaeresis	&Yuml;	&#376;	&#x178;	Ÿ	Ÿ	Ÿ
-//	modifier letter circumflex accent	&circ;	&#710;	&#x2C6;	ˆ	ˆ	ˆ
-//	small tilde	&tilde;	&#732;	&#x2DC;	˜	˜	˜
-//	en space	&ensp;	&#8194;	&#x2002;
-//	em space	&emsp;	&#8195;	&#x2003;
-//	thin space	&thinsp;	&#8201;	&#x2009;
-//	zero width non-joiner	&zwnj;	&#8204;	&#x200C;	‌	‌	‌
-//	zero width joiner	&zwj;	&#8205;	&#x200D;	‍	‍	‍
-//	left-to-right mark	&lrm;	&#8206;	&#x200E;	‎	‎	‎
-//	right-to-left mark	&rlm;	&#8207;	&#x200F;	‏	‏	‏
-//	en dash	&ndash;	&#8211;	&#x2013;	–	–	–
-//	em dash	&mdash;	&#8212;	&#x2014;	—	—	—
-//	left single quotation mark	&lsquo;	&#8216;	&#x2018;	‘	‘	‘
-//	right single quotation mark	&rsquo;	&#8217;	&#x2019;	’	’	’
-//	single low-9 quotation mark	&sbquo;	&#8218;	&#x201A;	‚	‚	‚
-//	left double quotation mark	&ldquo;	&#8220;	&#x201C;	“	“	“
-//	right double quotation mark	&rdquo;	&#8221;	&#x201D;	”	”	”
-//	double low-9 quotation mark	&bdquo;	&#8222;	&#x201E;	„	„	„
-//	dagger	&dagger;	&#8224;	&#x2020;	†	†	†
-//	double dagger	&Dagger;	&#8225;	&#x2021;	‡	‡	‡
-//	per mille sign	&permil;	&#8240;	&#x2030;	‰	‰	‰
-//	single left-pointing angle quotation mark	&lsaquo;	&#8249;	&#x2039;	‹	‹	‹
-//	single right-pointing angle quotation mark	&rsaquo;	&#8250;	&#x203A;	›	›	›
-//	euro sign	&euro;	&#8364;	&#x20AC;	€	€	€
-
 	replaceAll(context,"&quot;","\"");
 	replaceAll(context,"&amp;","&");
 	replaceAll(context,"&lt;","<");
 	replaceAll(context,"&gt;",">");
-	replaceAll(context,"&OElig;","Œ");
-	replaceAll(context,"&oelig;","œ");
-	replaceAll(context,"&Scaron;","Š");
-	replaceAll(context,"&scaron;","š");
-	replaceAll(context,"&Yuml;","Ÿ");
+	//replaceAll(context,"&OElig;","Œ");
+	replaceAll(context,"&OElig;","\xC5\x92\0");
+	//replaceAll(context,"&oelig;","œ");
+	replaceAll(context,"&oelig;","\xC5\x93\0");
+	//replaceAll(context,"&Scaron;","Š");
+	replaceAll(context,"&Scaron;","\xC5\xA0\0");
+	//replaceAll(context,"&scaron;","š");
+	replaceAll(context,"&scaron;","\xC5\xA1\0");
+	//replaceAll(context,"&Yuml;","Ÿ");
+	replaceAll(context,"&Yuml;","\xC5\xB8\0");
 	replaceAll(context,"&circ;","ˆ");
 	replaceAll(context,"&tilde;","˜");
 	replaceAll(context,"&ensp;"," ");
@@ -1848,18 +1903,30 @@ string replaceAllHTMLEntities(string& context) {
 	replaceAll(context,"&thinsp;"," ");
 	replaceAll(context,"&ndash;","-");
 	replaceAll(context,"&mdash;","-");
-	replaceAll(context,"&lsquo;","‘");
-	replaceAll(context,"&rsquo;","’");
-	replaceAll(context,"&sbquo;","‚");
-	replaceAll(context,"&ldquo;","“");
-	replaceAll(context,"&rdquo;","”");
-	replaceAll(context,"&bdquo;","„");
-	replaceAll(context,"&dagger;","†");
-	replaceAll(context,"&Dagger;","‡");
-	replaceAll(context,"&permil;","‰");
-	replaceAll(context,"&lsaquo;","‹");
-	replaceAll(context,"&rsaquo;","›");
-	replaceAll(context,"&euro;","€");
+	//replaceAll(context,"&lsquo;","‘");
+	replaceAll(context,"&lsquo;","\xE2\x80\x98\0");
+	//replaceAll(context,"&rsquo;","’");
+	replaceAll(context,"&rsquo;","\xE2\x80\x99\0");
+	//replaceAll(context,"&sbquo;","‚");
+	replaceAll(context,"&sbquo;","\xE2\x80\x9A\0");
+	//replaceAll(context,"&ldquo;","“");
+	replaceAll(context,"&ldquo;","\xE2\x80\x9C\0");
+	//replaceAll(context,"&rdquo;","”");
+	replaceAll(context,"&rdquo;","\xE2\x80\x9D\0");
+	//replaceAll(context,"&bdquo;","„");
+	replaceAll(context,"&bdquo;","\xE2\x80\x9E\0");
+	//replaceAll(context,"&dagger;","†");
+	replaceAll(context,"&dagger;","\xE2\x80\xA0\0");
+	//replaceAll(context,"&Dagger;","‡");
+	replaceAll(context,"&Dagger;","\xE2\x80\xA1\0");
+	//replaceAll(context,"&permil;","‰");
+	replaceAll(context,"&permil;","\xE2\x80\xB0\0");
+	//replaceAll(context,"&lsaquo;","‹");
+	replaceAll(context,"&lsaquo;","\xE2\x80\xB9\0");
+	//replaceAll(context,"&rsaquo;","›");
+	replaceAll(context,"&rsaquo;","\xE2\x80\xBA\0");
+	//replaceAll(context,"&euro;","€");
+	replaceAll(context,"&euro;","\xE2\x82\xAC\0");
 
 	return context;
 }
@@ -1943,6 +2010,30 @@ string getFullFileArchiveExtractCommand(string fileArchiveExtractCommand,
 	return result;
 }
 
+string getFullFileArchiveCompressCommand(string fileArchiveCompressCommand,
+		string fileArchiveCompressCommandParameters,
+		string archivename, string archivefiles) {
+	string parsedArchivename = archivename;
+	string parsedArchivefiles = archivefiles;
+
+// This is required for execution on win32
+#if defined(WIN32)
+	replaceAll(parsedArchivename, "\\\\", "\\");
+	replaceAll(parsedArchivename, "/", "\\");
+	replaceAll(parsedArchivefiles, "\\\\", "\\");
+	replaceAll(parsedArchivefiles, "/", "\\");
+
+#endif
+
+	string result = fileArchiveCompressCommand;
+	result += " ";
+	string args = replaceAll(fileArchiveCompressCommandParameters, "{archivename}", parsedArchivename);
+	args = replaceAll(args, "{archivefiles}", parsedArchivefiles);
+	result += args;
+
+	return result;
+}
+
 bool executeShellCommand(string cmd, int expectedResult, ShellCommandOutputCallbackInterface *cb) {
 	bool result = false;
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"About to run [%s]", cmd.c_str());
@@ -2335,5 +2426,45 @@ void ValueCheckerVault::checkItemInVault(const void *ptr,int value) const {
 
 }
 
+string getUserHome() {
+	string home_folder = "";
+	home_folder = safeCharPtrCopy(getenv("HOME"),8095);
+	if(home_folder == "") {
+#if _BSD_SOURCE || _SVID_SOURCE || _XOPEN_SOURCE >= 500 || _XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED
+		struct passwd *pw = getpwuid(getuid());
+		home_folder = safeCharPtrCopy(pw->pw_dir,8095);
+#endif
+	}
+	return home_folder;
+}
+
+string safeCharPtrCopy(const char *ptr,int maxLength) {
+	string result = "";
+	if(ptr == NULL) {
+		return result;
+	}
+	if(maxLength <= 0) {
+		maxLength = 8096;
+	}
+
+	int ptrLength = (int)strlen(ptr);
+	if(ptrLength >= maxLength) {
+		char *pBuffer = new char[maxLength+1];
+		memset(pBuffer,0,maxLength+1);
+#ifdef WIN32
+		memcpy(pBuffer,ptr,min((int)strlen(ptr),maxLength));
+#else
+		memcpy(pBuffer,ptr,std::min((int)strlen(ptr),maxLength));
+#endif
+
+		result = pBuffer;
+		delete [] pBuffer;
+	}
+	else {
+		result = ptr;
+	}
+	return result;
+}
+
 
 }}//end namespace
