@@ -13,13 +13,19 @@
 #include "ircclient.h"
 #include "util.h"
 #include "platform_common.h"
-#include <libircclient.h>
+#include "socket.h"
+#include "cache_manager.h"
 
+#if !defined(DISABLE_IRCCLIENT)
+
+#include <libircclient.h>
 // upstream moved some defines into new headers as of 1.6
 #ifndef LIBIRCCLIENT_PRE1_6
 #include <libirc_rfcnumeric.h>
 #endif
 
+#endif
+
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -31,10 +37,12 @@ using namespace Shared::PlatformCommon;
 
 namespace Shared { namespace PlatformCommon {
 
+const char *IRCThread::globalCacheContainerName = NULL;
 const int IRC_SERVER_PORT = 6667;
 //bool IRCThread::debugEnabled = true;
 bool IRCThread::debugEnabled = false;
 
+#if !defined(DISABLE_IRCCLIENT)
 void addlog (const char * fmt, ...) {
 	//FILE * fp;
 	char buf[8096];
@@ -67,7 +75,11 @@ void dump_event (irc_session_t * session, const char * event, const char * origi
 		if ( cnt ) {
 			strcat (buf, "|");
         }
-		strcat (buf, params[cnt]);
+#ifdef WIN32
+		strncat (buf, params[cnt],min((int)strlen(params[cnt]),511));
+#else
+		strncat (buf, params[cnt],std::min((int)strlen(params[cnt]),511));
+#endif
 	}
 
 	addlog ("Event \"%s\", origin: \"%s\", params: %d [%s]", event, origin ? origin : "NULL", cnt, buf);
@@ -126,24 +138,50 @@ void event_notice (irc_session_t * session, const char * event, const char * ori
 }
 
 void event_join(irc_session_t * session, const char * event, const char * origin, const char ** params, unsigned int count) {
+
+	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
 	dump_event (session, event, origin, params, count);
 
+	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
     IRCThread *ctx = (IRCThread *)irc_get_ctx(session);
+
+    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
 	if(ctx != NULL) {
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
         if(ctx->getHasJoinedChannel() == false) {
+
+        	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
             irc_cmd_user_mode (session, "+i");
+
+            if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
             //irc_cmd_msg (session, params[0], "MG Bot says hello!");
             ctx->setHasJoinedChannel(true);
 
-            ctx->GetIRCConnectedNickList(ctx->getChannel(),true);
+            if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
+            //ctx->GetIRCConnectedNickList(ctx->getChannel(),true);
+            ctx->GetIRCConnectedNickList(ctx->getChannel(),false);
+
+            if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
         }
         else {
+        	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
             char realNick[128]="";
             get_nickname(origin,realNick,127);
             if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: user joined channel realNick [%s] origin [%s]\n", realNick,origin);
 
             bool foundNick = false;
 
+            if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
             MutexSafeWrapper safeMutex(ctx->getMutexNickList(),string(__FILE__) + "_" + intToStr(__LINE__));
             std::vector<string> nickList = ctx->getCachedNickList();
             for(unsigned int i = 0;
@@ -155,14 +193,22 @@ void event_join(irc_session_t * session, const char * event, const char * origin
                     break;
                 }
             }
+            if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
             if(foundNick == false) {
                 nickList.push_back(realNick);
             }
+
+            if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
         }
 
+        if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
         if(ctx->getWantToLeaveChannel() == true) {
         	ctx->leaveChannel();
         }
+
+        if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
     }
 }
 
@@ -194,8 +240,6 @@ void event_privmsg (irc_session_t * session, const char * event, const char * or
 }
 
 void dcc_recv_callback (irc_session_t * session, irc_dcc_t id, int status, void * ctx, const char * data, unsigned int length) {
-	static int count = 1;
-	char buf[12]="";
 
 	switch (status)
 	{
@@ -210,6 +254,9 @@ void dcc_recv_callback (irc_session_t * session, irc_dcc_t id, int status, void
 		}
 		else {
 			if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("DCC %d: %s\n", id, data);
+
+			static int count = 1;
+			char buf[12]="";
 			sprintf (buf, "DCC [%d]: %d", id, count++);
 			irc_dcc_msg	(session, id, buf);
 		}
@@ -240,13 +287,13 @@ void dcc_file_recv_callback (irc_session_t * session, irc_dcc_t id, int status,
 		if ( ctx ) {
 			fwrite (data, 1, length, (FILE*) ctx);
 		}
-		if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("File sent progress: %d\n", length);
+		if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("File sent progress: %u\n", length);
 	}
 }
 
 void event_channel(irc_session_t * session, const char * event, const char * origin, const char ** params, unsigned int count) {
 	//IRC: Event "433", origin: "leguin.freenode.net", params: 3 [*|softcoder|Nickname is already in use.]
-	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf("In [%s::%s] Line: %d count = %d origin = %s\n",__FILE__,__FUNCTION__,__LINE__,count,(origin ? origin : "null"));
+	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf("In [%s::%s] Line: %d count = %u origin = %s\n",__FILE__,__FUNCTION__,__LINE__,count,(origin ? origin : "null"));
 
 	if ( count != 2 )
 		return;
@@ -361,7 +408,9 @@ void event_leave(irc_session_t *session, const char *event, const char *origin,
 }
 void event_numeric(irc_session_t * session, unsigned int event, const char * origin, const char ** params, unsigned int count) {
 	char buf[24]="";
-	sprintf (buf, "%d", event);
+	sprintf (buf, "%u", event);
+
+	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d event = %u\n", __LINE__, event);
 
     switch (event) {
 		case LIBIRC_RFC_ERR_NICKNAMEINUSE :
@@ -409,7 +458,7 @@ void event_numeric(irc_session_t * session, unsigned int event, const char * ori
             {
                 if(event == LIBIRC_RFC_RPL_NAMREPLY) {
 
-                	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: LIBIRC_RFC_RPL_NAMREPLY count = %d\n", count);
+                	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: LIBIRC_RFC_RPL_NAMREPLY count = %u\n", count);
 
                     std::vector<string> nickList;
                     if(count >= 4) {
@@ -441,10 +490,15 @@ void event_numeric(irc_session_t * session, unsigned int event, const char * ori
             }
         case LIBIRC_RFC_RPL_ENDOFNAMES:
             {
+            	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("!!!!! ===> IRC: Line: %d event = %u\n", __LINE__, event);
+
                 IRCThread *ctx = (IRCThread *)irc_get_ctx(session);
                 if(ctx != NULL) {
+                	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d event = %u\n", __LINE__, event);
+
                     ctx->setEventDataDone(true);
                 }
+                if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d event = %u\n", __LINE__, event);
             }
             break;
     }
@@ -452,28 +506,79 @@ void event_numeric(irc_session_t * session, unsigned int event, const char * ori
 	dump_event (session, buf, origin, params, count);
 }
 
+#endif
+
+bool IRCThread::getEventDataDone() {
+	MutexSafeWrapper safeMutex(&mutexEventDataDone,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool result = eventDataDone;
+	safeMutex.ReleaseLock();
+
+	return result;
+}
+void IRCThread::setEventDataDone(bool value) {
+	MutexSafeWrapper safeMutex(&mutexEventDataDone,string(__FILE__) + "_" + intToStr(__LINE__));
+	eventDataDone=value;
+}
+
 IRCThread::IRCThread(const std::vector<string> &argv, IRCCallbackInterface *callbackObj) : BaseThread() {
+	uniqueID = "IRCThread";
     this->argv = argv;
     this->callbackObj = callbackObj;
     ircSession = NULL;
     eventData.clear();
-    eventDataDone = false;
+    setEventDataDone(false);
     hasJoinedChannel = false;
     lastNickListUpdate = time(NULL);
     wantToLeaveChannel = false;
     playerName = "";
 }
 
-void IRCThread::signalQuit() {
-    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: signalQuit [%p]\n",ircSession);
+void IRCThread::disconnect() {
+#if !defined(DISABLE_IRCCLIENT)
 
-    if(ircSession != NULL) {
+	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool validSession = (ircSession != NULL);
+	safeMutex.ReleaseLock();
+
+    if(validSession == true) {
         setCallbackObj(NULL);
         if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Quitting Channel\n");
-        irc_cmd_quit(ircSession, "MG Bot is closing!");
+
+        MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+        irc_disconnect(ircSession);
+        safeMutex1.ReleaseLock();
+
         BaseThread::signalQuit();
         hasJoinedChannel = false;
     }
+#else
+    BaseThread::signalQuit();
+#endif
+}
+
+void IRCThread::signalQuit() {
+    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: signalQuit [%p]\n",ircSession);
+
+#if !defined(DISABLE_IRCCLIENT)
+
+	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool validSession = (ircSession != NULL);
+	safeMutex.ReleaseLock();
+
+    if(validSession == true) {
+        setCallbackObj(NULL);
+        if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Quitting Channel\n");
+
+        MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+        irc_cmd_quit(ircSession, "MG Bot is closing!");
+        safeMutex1.ReleaseLock();
+		hasJoinedChannel = false;
+    }
+    BaseThread::signalQuit();
+
+#else
+    BaseThread::signalQuit();
+#endif
 }
 
 bool IRCThread::shutdownAndWait() {
@@ -484,41 +589,98 @@ bool IRCThread::shutdownAndWait() {
 }
 
 void IRCThread::SendIRCCmdMessage(string target, string msg) {
-    if(ircSession != NULL && hasJoinedChannel == true) {
+	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool validSession = (ircSession != NULL);
+	safeMutex.ReleaseLock();
+
+    if(validSession == true && hasJoinedChannel == true) {
     	if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] sending IRC command to [%s] cmd [%s]\n",__FILE__,__FUNCTION__,__LINE__,target.c_str(),msg.c_str());
+
+#if !defined(DISABLE_IRCCLIENT)
+    	MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
         int ret = irc_cmd_msg (ircSession, target.c_str(), msg.c_str());
+        safeMutex1.ReleaseLock();
+
         if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] sending IRC command to [%s] cmd [%s] ret = %d\n",__FILE__,__FUNCTION__,__LINE__,target.c_str(),msg.c_str(),ret);
+#endif
     }
 }
 
 std::vector<string> IRCThread::GetIRCConnectedNickList(string target, bool waitForCompletion) {
-    eventDataDone = false;
-    if(ircSession != NULL && hasJoinedChannel == true) {
+    setEventDataDone(false);
+
+    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+	MutexSafeWrapper safeMutexSession(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool validSession = (ircSession != NULL);
+	safeMutexSession.ReleaseLock();
+
+	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
+    if(validSession == true && hasJoinedChannel == true) {
     	if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] sending IRC nick list command to [%s]\n",__FILE__,__FUNCTION__,__LINE__,target.c_str());
+
+#if !defined(DISABLE_IRCCLIENT)
+
+    	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
+    	MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+
+    	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
         int ret = irc_cmd_names (ircSession, target.c_str());
+        safeMutex1.ReleaseLock();
+
+        if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
         if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] sending IRC nick list command to [%s] ret = %d\n",__FILE__,__FUNCTION__,__LINE__,target.c_str(),ret);
 
+        if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
         if(waitForCompletion == true) {
+
+        	if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
             for(time_t tElapsed = time(NULL);
-                eventDataDone == false &&
+                getEventDataDone() == false &&
                 this->getQuitStatus() == false &&
                 difftime(time(NULL),tElapsed) <= 5;) {
                 sleep(50);
             }
+
+            if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
         }
+
+        if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+#endif
     }
 
+    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
     MutexSafeWrapper safeMutex(&mutexNickList,string(__FILE__) + "_" + intToStr(__LINE__));
     std::vector<string> nickList = eventData;
     safeMutex.ReleaseLock();
 
+    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC: Line: %d\n", __LINE__);
+
     return nickList;
 }
 
-bool IRCThread::isConnected() {
+bool IRCThread::isConnected(bool mutexLockRequired) {
     bool ret = false;
-    if(ircSession != NULL) {
+
+	MutexSafeWrapper safeMutex(NULL,string(__FILE__) + "_" + intToStr(__LINE__));
+	if(mutexLockRequired == true) {
+		safeMutex.setMutex(&mutexIRCSession);
+	}
+	bool validSession = (ircSession != NULL);
+	safeMutex.ReleaseLock();
+
+    if(validSession == true) {
+#if !defined(DISABLE_IRCCLIENT)
+    	MutexSafeWrapper safeMutex1(NULL,string(__FILE__) + "_" + intToStr(__LINE__));
+    	if(mutexLockRequired == true) {
+    		safeMutex1.setMutex(&mutexIRCSession);
+    	}
         ret = (irc_is_connected(ircSession) != 0);
+#endif
     }
 
     return ret;
@@ -556,8 +718,12 @@ void IRCThread::execute() {
         if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"IRC thread is running\n");
 
         try	{
+#if !defined(DISABLE_IRCCLIENT)
             irc_callbacks_t	callbacks;
-            ircSession=NULL;
+
+        	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+        	ircSession=NULL;
+        	safeMutex.ReleaseLock(true);
 
             if(argv.size() != 5) {
                 if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC Usage: <server> <nick> <channel> : got params [%ld]\n",(long int)argv.size());
@@ -590,12 +756,16 @@ void IRCThread::execute() {
             if(this->getQuitStatus() == true) {
                 return;
             }
+            safeMutex.Lock();
             ircSession = irc_create_session (&callbacks);
+        	bool validSession = (ircSession != NULL);
 
-            if(!ircSession) {
+            if(validSession == false) {
+            	safeMutex.ReleaseLock();
                 if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC Could not create session\n");
                 return;
             }
+            safeMutex.ReleaseLock(true);
 
 //            this->execute_cmd_onconnect = "";
 //            if(argv.size() >= 5) {
@@ -612,16 +782,22 @@ void IRCThread::execute() {
             }
             this->channel = argv[2];
             this->nick    = argv[1];
+
+            safeMutex.Lock();
             irc_set_ctx(ircSession, this);
+            safeMutex.ReleaseLock(true);
 
             if(this->getQuitStatus() == true) {
                 return;
             }
 
+            safeMutex.Lock();
             if(irc_connect(ircSession, argv[0].c_str(), IRC_SERVER_PORT, 0, this->nick.c_str(), this->username.c_str(), "megaglest")) {
+            	safeMutex.ReleaseLock();
                 if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC Could not connect: %s\n", irc_strerror (irc_errno(ircSession)));
                 return;
             }
+            safeMutex.ReleaseLock();
 
             if(this->getQuitStatus() == true) {
                 return;
@@ -631,15 +807,29 @@ void IRCThread::execute() {
                 return;
             }
 
+			//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
+
             for(int iAttempts=1;
-                this->getQuitStatus() == false && iAttempts <= 5;
-                ++iAttempts) {
-                if(irc_run(ircSession)) {
-                    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC Could not run the session: %s\n", irc_strerror (irc_errno(ircSession)));
+                this->getQuitStatus() == false && iAttempts <= 7; ++iAttempts) {
+                //if(irc_run(ircSession)) {
+
+            	int run_result = irc_run_session(ircSession);
+            	if(run_result && this->getQuitStatus() == false) {
+					//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
+
+                    if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC Could not run the session: %s run_result = %d\n", irc_strerror (irc_errno(ircSession)), run_result);
+                    printf ("===> IRC Could not run the session: %s run_result = %d\n", irc_strerror (irc_errno(ircSession)), run_result);
                 }
             }
 
+			//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
+#else
+            for(;this->getQuitStatus() == false;) {
+            	sleep(50);
+            }
+#endif
             if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC exiting IRC CLient!\n");
+			//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
         }
         catch(const exception &ex) {
             SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
@@ -651,18 +841,95 @@ void IRCThread::execute() {
         }
 
         if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] IRC thread is exiting\n",__FILE__,__FUNCTION__,__LINE__);
+		//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
     }
 
+	//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
     // Delete ourself when the thread is done (no other actions can happen after this
     // such as the mutex which modifies the running status of this method
     MutexSafeWrapper safeMutex(&mutexIRCCB,string(__FILE__) + "_" + intToStr(__LINE__));
     IRCCallbackInterface *cb = getCallbackObj(false);
     if(cb != NULL) {
+		//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
        cb->IRC_CallbackEvent(IRC_evt_exitThread, NULL, NULL, 0);
     }
     safeMutex.ReleaseLock();
 
-	delete this;
+	//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
+
+    if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In IRCThread() calling delete ...\n");
+
+	//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
+	setDeleteAfterExecute(true);
+}
+
+int IRCThread::irc_run_session(irc_session_t * session) {
+//	if ( session->state != LIBIRC_STATE_CONNECTING )
+//	{
+//		session->lasterror = LIBIRC_ERR_STATE;
+//		return 1;
+//	}
+
+	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+
+	if ( isConnected(false) == false ) {
+		//session->lasterror = LIBIRC_ERR_STATE;
+		return 1;
+	}
+	safeMutex.ReleaseLock(true);
+
+	bool isSessionConnected = true;
+	while ( isSessionConnected == true && this->getQuitStatus() == false) {
+		struct timeval tv;
+		fd_set in_set, out_set;
+		int maxfd = 0;
+
+		tv.tv_usec = 250000;
+		tv.tv_sec = 0;
+
+		// Init sets
+		FD_ZERO (&in_set);
+		FD_ZERO (&out_set);
+
+		safeMutex.Lock();
+		irc_add_select_descriptors (session, &in_set, &out_set, &maxfd);
+		safeMutex.ReleaseLock(true);
+
+		if ( select (maxfd + 1, &in_set, &out_set, 0, &tv) < 0 ) {
+			int lastSocketError = Socket::getLastSocketError();
+			if ( lastSocketError == PLATFORM_SOCKET_INTERRUPTED ) {
+				continue;
+			}
+
+			//session->lasterror = LIBIRC_ERR_TERMINATED;
+			return 2;
+		}
+
+		safeMutex.Lock();
+		if ( irc_process_select_descriptors (session, &in_set, &out_set) ) {
+			return 3;
+		}
+
+		isSessionConnected = isConnected(false);
+		safeMutex.ReleaseLock(true);
+	}
+
+	return 0;
+}
+
+IRCThread::~IRCThread() {
+	//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
+
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In ~IRCThread() ...\n");
+
+	if(IRCThread::globalCacheContainerName != NULL) {
+		//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
+
+		IRCThread * &ircClient = CacheManager::getCachedItem< IRCThread * >(IRCThread::globalCacheContainerName);
+		ircClient = NULL;
+	}
+
+	//printf("In ~IRCThread Line: %d [%p]\n",__LINE__,this);
 }
 
 void normalizeNick(char *nick) {
@@ -714,47 +981,77 @@ void normalizeNick(char *nick) {
 
 void IRCThread::connectToHost() {
 	bool connectRequired = false;
-	if(ircSession == NULL) {
+
+	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool validSession = (ircSession != NULL);
+	safeMutex.ReleaseLock();
+
+	if(validSession == false) {
 		connectRequired = true;
 	}
 	else {
+#if !defined(DISABLE_IRCCLIENT)
+
+		MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
 		int result = irc_is_connected(ircSession);
 		if(result != 1) {
 			connectRequired = true;
 		}
+#endif
 	}
 
 	if(connectRequired == false) {
+#if !defined(DISABLE_IRCCLIENT)
+		MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
         if(irc_connect(ircSession, argv[0].c_str(), IRC_SERVER_PORT, 0, this->nick.c_str(), this->username.c_str(), "megaglest")) {
             if(SystemFlags::VERBOSE_MODE_ENABLED || IRCThread::debugEnabled) printf ("===> IRC Could not connect: %s\n", irc_strerror (irc_errno(ircSession)));
             return;
         }
+#endif
 	}
 }
 
 void IRCThread::joinChannel() {
 	wantToLeaveChannel = false;
 	connectToHost();
-	if(ircSession != NULL) {
+
+	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool validSession = (ircSession != NULL);
+	safeMutex.ReleaseLock();
+
+	if(validSession == true) {
+#if !defined(DISABLE_IRCCLIENT)
+
+		MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
 		IRCThread *ctx = (IRCThread *)irc_get_ctx(ircSession);
 		if(ctx != NULL) {
 			eventData.clear();
-			eventDataDone = false;
+			setEventDataDone(false);
 			hasJoinedChannel = false;
 			lastNickListUpdate = time(NULL);
 
 			irc_cmd_join(ircSession, ctx->getChannel().c_str(), 0);
 		}
+#endif
 	}
 }
 
 void IRCThread::leaveChannel() {
 	wantToLeaveChannel = true;
-	if(ircSession != NULL) {
+
+	MutexSafeWrapper safeMutex(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
+	bool validSession = (ircSession != NULL);
+	safeMutex.ReleaseLock();
+
+	if(validSession == true) {
+#if !defined(DISABLE_IRCCLIENT)
+
+		MutexSafeWrapper safeMutex1(&mutexIRCSession,string(__FILE__) + "_" + intToStr(__LINE__));
 		IRCThread *ctx = (IRCThread *)irc_get_ctx(ircSession);
 		if(ctx != NULL) {
 			irc_cmd_part(ircSession,ctx->getChannel().c_str());
 		}
+#endif
 	}
 }
 
