@@ -25,14 +25,16 @@ using namespace Shared::PlatformCommon;
 
 namespace Shared { namespace PlatformCommon {
 
-const char *FTP_MAPS_CUSTOM_USERNAME        = "maps_custom";
-const char *FTP_MAPS_USERNAME               = "maps";
-const char *FTP_TILESETS_CUSTOM_USERNAME    = "tilesets_custom";
-const char *FTP_TILESETS_USERNAME           = "tilesets";
-const char *FTP_TECHTREES_CUSTOM_USERNAME   = "techtrees_custom";
-const char *FTP_TECHTREES_USERNAME          = "techtrees";
+static const char *FTP_MAPS_CUSTOM_USERNAME        = "maps_custom";
+static const char *FTP_MAPS_USERNAME               = "maps";
+static const char *FTP_TILESETS_CUSTOM_USERNAME    = "tilesets_custom";
+static const char *FTP_TILESETS_USERNAME           = "tilesets";
+static const char *FTP_TECHTREES_CUSTOM_USERNAME   = "techtrees_custom";
+static const char *FTP_TECHTREES_USERNAME          = "techtrees";
 
-const char *FTP_COMMON_PASSWORD             = "mg_ftp_server";
+static const char *FTP_TEMPFILES_USERNAME          = "temp";
+
+static const char *FTP_COMMON_PASSWORD             = "mg_ftp_server";
 
 /*
  * This is an example showing how to get a single file from an FTP server.
@@ -64,6 +66,7 @@ static size_t my_fwrite(void *buffer, size_t size, size_t nmemb, void *stream) {
     }
 
     if(SystemFlags::VERBOSE_MODE_ENABLED) printf ("===> FTP Client thread writing to file [%s]\n",fullFilePath.c_str());
+    //printf ("===> FTP Client thread writing to file [%s]\n",fullFilePath.c_str());
 
     // Abort file xfer and delete partial file
     if(out && out->ftpServer && out->ftpServer->getQuitStatus() == true) {
@@ -77,7 +80,7 @@ static size_t my_fwrite(void *buffer, size_t size, size_t nmemb, void *stream) {
 
 
         removeFile(fullFilePath);
-        return -1;
+        return 0;
     }
 
     if(out && out->stream == NULL) {
@@ -94,7 +97,7 @@ static size_t my_fwrite(void *buffer, size_t size, size_t nmemb, void *stream) {
           if(SystemFlags::VERBOSE_MODE_ENABLED) printf ("===> FTP Client thread FAILED to open file for writing [%s]\n",fullFilePath.c_str());
           if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"===> FTP Client thread FAILED to open file for writing [%s]\n",fullFilePath.c_str());
           SystemFlags::OutputDebug(SystemFlags::debugError,"===> FTP Client thread FAILED to open file for writing [%s]\n",fullFilePath.c_str());
-          return -1; /* failure, can't open file to write */
+          return 0; /* failure, can't open file to write */
         }
 
         out->isValidXfer = true;
@@ -103,7 +106,7 @@ static size_t my_fwrite(void *buffer, size_t size, size_t nmemb, void *stream) {
         if(SystemFlags::VERBOSE_MODE_ENABLED) printf ("===> #2 FTP Client thread FAILED to open file for writing [%s]\n",fullFilePath.c_str());
         if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"===> #2 FTP Client thread FAILED to open file for writing [%s]\n",fullFilePath.c_str());
         SystemFlags::OutputDebug(SystemFlags::debugError,"===> #2 FTP Client thread FAILED to open file for writing [%s]\n",fullFilePath.c_str());
-        return -1; /* failure, can't open file to write */
+        return 0; /* failure, can't open file to write */
     }
 
     size_t result = fwrite(buffer, size, nmemb, out->stream);
@@ -237,7 +240,10 @@ FTPClientThread::FTPClientThread(int portNumber, string serverUrl,
 		string fileArchiveExtension,
 		string fileArchiveExtractCommand,
 		string fileArchiveExtractCommandParameters,
-		int fileArchiveExtractCommandSuccessResult) : BaseThread() {
+		int fileArchiveExtractCommandSuccessResult,
+		string tempFilesPath) : BaseThread() {
+
+	uniqueID = "FTPClientThread";
     this->portNumber    = portNumber;
     this->serverUrl     = serverUrl;
     this->mapsPath      = mapsPath;
@@ -251,6 +257,7 @@ FTPClientThread::FTPClientThread(int portNumber, string serverUrl,
     this->fileArchiveExtractCommand = fileArchiveExtractCommand;
     this->fileArchiveExtractCommandParameters = fileArchiveExtractCommandParameters;
     this->fileArchiveExtractCommandSuccessResult = fileArchiveExtractCommandSuccessResult;
+    this->tempFilesPath = tempFilesPath;
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line %d] Using FTP port #: %d, serverUrl [%s]\n",__FILE__,__FUNCTION__,__LINE__,portNumber,serverUrl.c_str());
 }
@@ -323,7 +330,10 @@ pair<FTP_Client_ResultType,string> FTPClientThread::getMapFromServer(pair<string
 
 
         // Max 10 minutes to transfer
-        curl_easy_setopt(curl, CURLOPT_TIMEOUT, 600);
+        //curl_easy_setopt(curl, CURLOPT_TIMEOUT, 600);
+        // Max 60 minutes to transfer
+        curl_easy_setopt(curl, CURLOPT_TIMEOUT, 3600L);
+        curl_easy_setopt(curl, CURLOPT_FTP_RESPONSE_TIMEOUT, 120L);
 
         /* Switch on full protocol/debug output */
         if(SystemFlags::VERBOSE_MODE_ENABLED) curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
@@ -449,26 +459,42 @@ void FTPClientThread::addFileToRequests(string fileName,string URL) {
     }
 }
 
+void FTPClientThread::addTempFileToRequests(string fileName,string URL) {
+	std::pair<string,string> item = make_pair(fileName,URL);
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+    MutexSafeWrapper safeMutex(&mutexTempFileList,mutexOwnerId);
+    mutexTempFileList.setOwnerId(mutexOwnerId);
+    if(std::find(tempFileList.begin(),tempFileList.end(),item) == tempFileList.end()) {
+    	tempFileList.push_back(item);
+    }
+}
+
 void FTPClientThread::getTilesetFromServer(pair<string,string> tileSetName) {
 	bool findArchive = executeShellCommand(
 			this->fileArchiveExtractCommand,
 			this->fileArchiveExtractCommandSuccessResult);
 
 	pair<FTP_Client_ResultType,string> result = make_pair(ftp_crt_FAIL,"");
-	if(tileSetName.second != "") {
-		result = getTilesetFromServer(tileSetName, "", "", "", findArchive);
-	}
-	else {
-		result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_CUSTOM_USERNAME, FTP_COMMON_PASSWORD, findArchive);
-		if(result.first == ftp_crt_FAIL && this->getQuitStatus() == false) {
-			if(findArchive == true) {
-				result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_CUSTOM_USERNAME, FTP_COMMON_PASSWORD, false);
-			}
+	if(findArchive == true) {
+		if(tileSetName.second != "") {
+			//result = getTilesetFromServer(tileSetName, "", "", "", findArchive);
+			result = getTilesetFromServer(tileSetName, "", "", "", true);
+		}
+		else {
+			//result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_CUSTOM_USERNAME, FTP_COMMON_PASSWORD, findArchive);
+			result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_CUSTOM_USERNAME, FTP_COMMON_PASSWORD, true);
 			if(result.first == ftp_crt_FAIL && this->getQuitStatus() == false) {
-				result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_USERNAME, FTP_COMMON_PASSWORD, findArchive);
+				//if(findArchive == true) {
+					//result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_CUSTOM_USERNAME, FTP_COMMON_PASSWORD, false);
+				result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_CUSTOM_USERNAME, FTP_COMMON_PASSWORD, true);
+				//}
+				if(result.first == ftp_crt_FAIL && this->getQuitStatus() == false) {
+				//	result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_USERNAME, FTP_COMMON_PASSWORD, findArchive);
+					result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_USERNAME, FTP_COMMON_PASSWORD, true);
 
-				if(findArchive == true) {
-					result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_USERNAME, FTP_COMMON_PASSWORD, false);
+				//	if(findArchive == true) {
+				//		result = getTilesetFromServer(tileSetName, "", FTP_TILESETS_USERNAME, FTP_COMMON_PASSWORD, false);
+				//	}
 				}
 			}
 		}
@@ -594,7 +620,7 @@ pair<FTP_Client_ResultType,string> FTPClientThread::getTilesetFromServer(
     			for(unsigned int i = 0; i < wantDirListOnly.size(); ++i) {
     				string fileFromList = wantDirListOnly[i];
 
-    				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("fileFromList [%s] i [%d]\n",fileFromList.c_str(),i);
+    				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("fileFromList [%s] i [%u]\n",fileFromList.c_str(),i);
 
     				if( fileFromList != "models" && fileFromList != "textures" &&
     					fileFromList != "sounds") {
@@ -840,6 +866,63 @@ pair<FTP_Client_ResultType,string>  FTPClientThread::getFileInternalFromServer(p
     return result;
 }
 
+void FTPClientThread::getTempFileFromServer(pair<string,string> fileName) {
+	pair<FTP_Client_ResultType,string> result = make_pair(ftp_crt_FAIL,"");
+
+	bool findArchive = true;
+	string ext = extractExtension(fileName.first);
+	if(("." + ext) == this->fileArchiveExtension) {
+		findArchive = executeShellCommand(
+				this->fileArchiveExtractCommand,
+				this->fileArchiveExtractCommandSuccessResult);
+	}
+	if(findArchive == true) {
+		result = getTempFileInternalFromServer(fileName);
+	}
+
+	static string mutexOwnerId = string(__FILE__) + string("_") + intToStr(__LINE__);
+    MutexSafeWrapper safeMutex(this->getProgressMutex(),mutexOwnerId);
+    this->getProgressMutex()->setOwnerId(mutexOwnerId);
+    if(this->pCBObject != NULL) {
+        this->pCBObject->FTPClient_CallbackEvent(fileName.first,ftp_cct_TempFile,result,NULL);
+    }
+}
+
+pair<FTP_Client_ResultType,string>  FTPClientThread::getTempFileInternalFromServer(pair<string,string> fileName) {
+	string destFile = fileName.first;
+	//string destFileSaveAs = fileName.first;
+    string destFileSaveAs = tempFilesPath;
+
+	endPathWithSlash(destFileSaveAs);
+	destFileSaveAs += fileName.first;
+
+	string remotePath = fileName.second;
+	fileName.second = "";
+
+    pair<FTP_Client_ResultType,string> result = getFileFromServer(ftp_cct_TempFile,
+    		fileName,remotePath, destFileSaveAs, FTP_TEMPFILES_USERNAME, FTP_COMMON_PASSWORD);
+
+    // Extract the archive
+    if(result.first == ftp_crt_SUCCESS) {
+    	string ext = extractExtension(destFileSaveAs);
+    	if(("." + ext) == fileArchiveExtension) {
+    		string destRootArchiveFolder = extractDirectoryPathFromFile(destFileSaveAs);
+			string extractCmd = getFullFileArchiveExtractCommand(
+					this->fileArchiveExtractCommand,
+					this->fileArchiveExtractCommandParameters,
+					destRootArchiveFolder,
+					destFileSaveAs);
+
+			if(executeShellCommand(extractCmd,this->fileArchiveExtractCommandSuccessResult) == false) {
+				result.first = ftp_crt_FAIL;
+				result.second = "failed to extract archive!";
+			}
+    	}
+    }
+
+    return result;
+}
+
 pair<FTP_Client_ResultType,string>  FTPClientThread::getFileFromServer(FTP_Client_CallbackType downloadType,
 		pair<string,string> fileNameTitle,
 		string remotePath, string destFileSaveAs,
@@ -884,6 +967,8 @@ pair<FTP_Client_ResultType,string>  FTPClientThread::getFileFromServer(FTP_Clien
         	snprintf(szBuf,8096,"ftp://%s:%s@%s:%d/%s",ftpUser.c_str(),ftpUserPassword.c_str(),serverUrl.c_str(),portNumber,remotePath.c_str());
         }
 
+        //printf("===> Getting ftp file: %s\n",szBuf);
+
         curl_easy_setopt(curl, CURLOPT_URL,szBuf);
         curl_easy_setopt(curl, CURLOPT_FTP_USE_EPSV, 0L);
 
@@ -911,7 +996,10 @@ pair<FTP_Client_ResultType,string>  FTPClientThread::getFileFromServer(FTP_Clien
         curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &ftpfile);
 
         // Max 10 minutes to transfer
-        curl_easy_setopt(curl, CURLOPT_TIMEOUT, 600);
+        //curl_easy_setopt(curl, CURLOPT_TIMEOUT, 600);
+        // Max 60 minutes to transfer
+        curl_easy_setopt(curl, CURLOPT_TIMEOUT, 3600L);
+        curl_easy_setopt(curl, CURLOPT_FTP_RESPONSE_TIMEOUT, 120L);
 
         // Switch on full protocol/debug output
         if(SystemFlags::VERBOSE_MODE_ENABLED) curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
@@ -1104,6 +1192,20 @@ void FTPClientThread::execute() {
                     safeMutex5.ReleaseLock();
                 }
 
+                static string mutexOwnerId6 = string(__FILE__) + string("_") + intToStr(__LINE__);
+                MutexSafeWrapper safeMutex6(&mutexTempFileList,mutexOwnerId6);
+                mutexTempFileList.setOwnerId(mutexOwnerId6);
+                if(tempFileList.size() > 0) {
+                	pair<string,string> file = tempFileList[0];
+                	tempFileList.erase(tempFileList.begin() + 0);
+                    safeMutex6.ReleaseLock();
+
+                    getTempFileFromServer(file);
+                }
+                else {
+                    safeMutex6.ReleaseLock();
+                }
+
                 if(this->getQuitStatus() == false) {
                     sleep(25);
                 }
@@ -1126,7 +1228,6 @@ void FTPClientThread::execute() {
 
     // Delete ourself when the thread is done (no other actions can happen after this
     // such as the mutex which modifies the running status of this method
-	//delete this;
     deleteSelfIfRequired();
 }
 
