@@ -33,17 +33,80 @@ namespace Shared{ namespace Platform{
 string PlatformExceptionHandler::application_binary="";
 bool PlatformExceptionHandler::disableBacktrace = false;
 
-void message(string message) {
+const char * getDialogCommand() {
+
+/*
+  if (::system(NULL)) {
+    if (::system("which gdialog") == 0)
+      return "gdialog";
+    else if (::system("which kdialog") == 0)
+      return "kdialog";
+  }
+*/
+
+  FILE *file = popen("which zenity","r");
+  //printf("File #1 [%p]\n",file);
+  if (file != NULL) {
+	  pclose(file);
+	  return "zenity";
+  }
+
+  file = popen("which gdialog","r");
+  //printf("File #1 [%p]\n",file);
+  if (file != NULL) {
+	  pclose(file);
+	  return "gdialog";
+  }
+  file = popen("which kdialog","r");
+  //printf("File #2 [%p]\n",file);
+  if (file != NULL) {
+	  pclose(file);
+	  return "kdialog";
+  }
+
+  return NULL;
+}
+
+bool showMessage(std::string warning,string writepath) {
+	bool guiMessage = false;
+	const char * dialogCommand = getDialogCommand();
+	if (dialogCommand) {
+		std::string command = dialogCommand;
+
+		string text_file = writepath + "/mg_dialog_text.txt";
+		{
+			FILE *fp = fopen(text_file.c_str(),"wt");
+			if (fp != NULL) {
+				fputs(warning.c_str(),fp);
+				fclose(fp);
+			}
+		}
+
+		//command += " --title \"Error\" --msgbox \"`printf \"" + warning + "\"`\"";
+		command += " --title \"Error\" --text-info --filename=" + text_file;
+
+		//printf("\n\n\nzenity command [%s]\n\n\n",command.c_str());
+
+		FILE *fp = popen(command.c_str(),"r");
+		if (fp != 0) {
+			guiMessage = true;
+			pclose(fp);
+		}
+	}
+
+	return guiMessage;
+}
+
+void message(string message, bool isNonGraphicalModeEnabled,string writepath) {
+	std::cerr << "\n\n\n";
 	std::cerr << "******************************************************\n";
 	std::cerr << "    " << message << "\n";
 	std::cerr << "******************************************************\n";
-}
+	std::cerr << "\n\n\n";
 
-bool ask(string message) {
-	std::cerr << "Confirmation: " << message << "\n";
-	int res;
-	std::cin >> res;
-	return res != 0;
+	if(isNonGraphicalModeEnabled == false) {
+		showMessage(message,writepath);
+	}
 }
 
 void exceptionMessage(const exception &excp) {
@@ -59,55 +122,60 @@ static int getFileAndLine(char *function, void *address, char *file, size_t flen
 
 			// prepare command to be executed
 			// our program need to be passed after the -e parameter
+#if __APPLE_CC__
+			snprintf(buf, 8096,"xcrun atos -v -o %s %p",PlatformExceptionHandler::application_binary.c_str(),address);
+            fprintf(stderr,"> %s\n",buf);
+#else
 			snprintf(buf, 8096,"addr2line -C -e %s -f -i %p",PlatformExceptionHandler::application_binary.c_str(),address);
-
+#endif
 			FILE* f = popen (buf, "r");
 			if (f == NULL) {
 				perror (buf);
 				return 0;
 			}
+#if __APPLE_CC__
+            //### TODO Will: still working this out
+            int len = fread(buf,1,maxbufSize,f);
+            buf[len] = 0;
+            fprintf(stderr,"< %s",buf);
+            return -1;
+#endif
+            for(;function != NULL && function[0] != '\0';) {
+                // get function name
+                char *ret = fgets (buf, maxbufSize, f);
+                if(ret == NULL) {
+                    pclose(f);
+                    return line;
+                }
+                //printf("Looking for [%s] Found [%s]\n",function,ret);
+                if(strstr(ret,function) != NULL) {
+                    break;
+                }
+                // get file and line
+                ret = fgets (buf, maxbufSize, f);
+                if(ret == NULL) {
+                    pclose(f);
+                    return line;
+                }
 
-			if(function != NULL && function[0] != '\0') {
-				line = 0;
-				for(;function != NULL && function[0] != '\0';) {
-					// get function name
-					char *ret = fgets (buf, maxbufSize, f);
-					if(ret == NULL) {
-						pclose(f);
-						return line;
-					}
-					//printf("Looking for [%s] Found [%s]\n",function,ret);
-					if(strstr(ret,function) != NULL) {
-						break;
-					}
+                if(strlen(buf) > 0 && buf[0] != '?') {
+                    //int l;
+                    char *p = buf;
+                    // file name is until ':'
+                    while(*p != 0 && *p != ':') {
+                        p++;
+                    }
 
-					// get file and line
-					ret = fgets (buf, maxbufSize, f);
-					if(ret == NULL) {
-						pclose(f);
-						return line;
-					}
-
-					if(strlen(buf) > 0 && buf[0] != '?') {
-						//int l;
-						char *p = buf;
-
-						// file name is until ':'
-						while(*p != 0 && *p != ':') {
-							p++;
-						}
-
-						*p++ = 0;
-						// after file name follows line number
-						strcpy (file , buf);
-						sscanf (p,"%d", &line);
-					}
-					else {
-						strcpy (file,"unknown");
-						line = 0;
-					}
-				}
-			}
+                    *p++ = 0;
+                    // after file name follows line number
+                    strcpy (file , buf);
+                    sscanf (p,"%10d", &line);
+                }
+                else {
+                    strcpy (file,"unknown");
+                    line = 0;
+                }
+            }
 
 			// get file and line
 			char *ret = fgets (buf, maxbufSize, f);
@@ -128,7 +196,7 @@ static int getFileAndLine(char *function, void *address, char *file, size_t flen
 				*p++ = 0;
 				// after file name follows line number
 				strcpy (file , buf);
-				sscanf (p,"%d", &line);
+				sscanf (p,"%10d", &line);
 			}
 			else {
 				strcpy (file,"unknown");
