@@ -59,6 +59,8 @@ int Window::lastShowMouseState = 0;
 
 bool Window::tryVSynch = false;
 
+map<wchar_t,bool> Window::mapAllowedKeys;
+
 //bool Window::masterserverMode = false;
 
 // ========== PUBLIC ==========
@@ -359,6 +361,10 @@ void Window::revertMousePos() {
 	SDL_WarpMouse(oldX, oldY);
 }
 
+Vec2i Window::getOldMousePos() {
+	return Vec2i(oldX, oldY);
+}
+
 string Window::getText() {
 	char* c = 0;
 	SDL_WM_GetCaption(&c, 0);
@@ -475,11 +481,7 @@ void Window::setupGraphicsScreen(int depthBits, int stencilBits, bool hardware_a
 		//const float lodBias = std::max(std::min( configHandler->Get("TextureLODBias", 0.0f) , 4.0f), -4.0f);
 		const float lodBias = max(min(0.0f,4.0f),-4.0f);
 		//if(SystemFlags::VERBOSE_MODE_ENABLED) printf ("\n\n\n\n\n$$$$ In [%s::%s Line: %d] lodBias = %f\n\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,lodBias);
-	#ifdef USE_STREFLOP
-		if (streflop::fabs(static_cast<streflop::Simple>(lodBias)) > 0.01f) {
-	#else
-		if (fabs(lodBias) > 0.01f) {
-	#endif
+		if (std::fabs(lodBias) > 0.01f) {
 			glTexEnvf(GL_TEXTURE_FILTER_CONTROL,GL_TEXTURE_LOD_BIAS, lodBias );
 		}
 		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
@@ -628,7 +630,6 @@ void Window::toggleFullscreen() {
 
 void Window::handleMouseDown(SDL_Event event) {
 	static const Uint32 DOUBLECLICKTIME = 500;
-	static const int DOUBLECLICKDELTA = 5;
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
@@ -659,6 +660,7 @@ void Window::handleMouseDown(SDL_Event event) {
 	if(n >= 0 && n < mbCount) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
+		static const int DOUBLECLICKDELTA = 5;
 		if(ticks - lastMouseDown[n] < DOUBLECLICKTIME
 				&& abs(lastMouseX[n] - event.button.x) < DOUBLECLICKDELTA
 				&& abs(lastMouseY[n] - event.button.y) < DOUBLECLICKDELTA) {
@@ -702,6 +704,79 @@ MouseButton Window::getMouseButton(int sdlButton) {
 	}
 }
 
+wchar_t Window::convertStringtoSDLKey(const string &value) {
+	wchar_t result = SDLK_UNKNOWN;
+
+	if(value.length() >= 1) {
+		if(value.length() == 3 && value[0] == '\'' && value[2] == '\'') {
+			result = (SDLKey)value[1];
+		}
+		else {
+			bool foundKey = false;
+			if(value.length() > 1) {
+				for(int i = SDLK_UNKNOWN; i < SDLK_LAST; ++i) {
+					SDLKey key = static_cast<SDLKey>(i);
+					string keyName = SDL_GetKeyName(key);
+					if(value == keyName) {
+						result = key;
+						foundKey = true;
+						break;
+					}
+				}
+			}
+
+			if(foundKey == false) {
+				result = (SDLKey)value[0];
+			}
+		}
+	}
+	else {
+		string sError = "Unsupported key name: [" + value + "]";
+		throw megaglest_runtime_error(sError.c_str());
+	}
+
+	// Because SDL is based on lower Ascii
+	//result = tolower(result);
+	return result;
+}
+
+void Window::addAllowedKeys(string keyList) {
+	clearAllowedKeys();
+
+	if(keyList.empty() == false) {
+		vector<string> keys;
+		Tokenize(keyList,keys,",");
+		if(keys.empty() == false) {
+			for(unsigned int keyIndex = 0; keyIndex < keys.size(); ++keyIndex) {
+				string key = trim(keys[keyIndex]);
+
+				wchar_t sdl_key = convertStringtoSDLKey(key);
+				if(sdl_key != SDLK_UNKNOWN) {
+					mapAllowedKeys[sdl_key] = true;
+				}
+
+				if(SystemFlags::VERBOSE_MODE_ENABLED)  printf("key: %d [%s] IS ALLOWED\n",sdl_key, key.c_str());
+				//printf("key: %d [%s] IS ALLOWED\n",sdl_key, key.c_str());
+			}
+		}
+	}
+}
+void Window::clearAllowedKeys() {
+	mapAllowedKeys.clear();
+}
+
+bool Window::isAllowedKey(wchar_t key) {
+	map<wchar_t,bool>::const_iterator iterFind = mapAllowedKeys.find(key);
+	bool result =(iterFind != mapAllowedKeys.end());
+
+	if(SystemFlags::VERBOSE_MODE_ENABLED) {
+		string keyName = SDL_GetKeyName((SDLKey)key);
+		printf("key: %d [%s] allowed result: %d\n",key,keyName.c_str(),result);
+	}
+
+	return result;
+}
+
 bool isKeyPressed(SDLKey compareKey, SDL_KeyboardEvent input,bool modifiersAllowed) {
 	vector<int> modifiersToCheck;
 	if(modifiersAllowed == false) {
@@ -940,7 +1015,7 @@ wchar_t extractKeyPressedUnicode(SDL_KeyboardEvent input) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] returning key [%d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,c);
 
 	string pressKeyName = SDL_GetKeyName((SDLKey)c);
-	string inputKeyName = SDL_GetKeyName(input.keysym.sym);
+	//string inputKeyName = SDL_GetKeyName(input.keysym.sym);
 
 	//printf ("PRESS pressed key [%d - %s] input.keysym.sym [%d] input.keysym.unicode [%d] mod = %d\n",
 	//		c,pressKeyName.c_str(),input.keysym.sym,input.keysym.unicode,input.keysym.mod);
@@ -959,7 +1034,7 @@ vector<int> extractKeyPressedUnicodeLength(string text) {
 		wchar_t keyW = c;
 		wchar_t textAppend[] = { keyW, 0 };
 
-		if(textAppend) {
+		if(*textAppend) {
 			wchar_t newKey = textAppend[0];
 			if (newKey < 0x80) {
 				result.push_back(1);
@@ -1006,7 +1081,7 @@ SDLKey extractKeyPressed(SDL_KeyboardEvent input) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] returning key [%d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,c);
 
 	string pressKeyName = SDL_GetKeyName((SDLKey)c);
-	string inputKeyName = SDL_GetKeyName(input.keysym.sym);
+	//string inputKeyName = SDL_GetKeyName(input.keysym.sym);
 
 	//printf ("PRESS pressed key [%d - %s] input.keysym.sym [%d] input.keysym.unicode [%d] mod = %d\n",
 	//		c,pressKeyName.c_str(),input.keysym.sym,input.keysym.unicode,input.keysym.mod);
@@ -1018,6 +1093,10 @@ SDLKey extractKeyPressed(SDL_KeyboardEvent input) {
 }
 
 bool isAllowedInputTextKey(wchar_t &key) {
+	if(Window::isAllowedKey(key) == true) {
+		return true;
+	}
+
 	bool result = (
 	key != SDLK_DELETE &&
 	key != SDLK_BACKSPACE &&
@@ -1078,6 +1157,10 @@ bool isAllowedInputTextKey(wchar_t &key) {
 }
 
 bool isAllowedInputTextKey(SDLKey key) {
+	if(Window::isAllowedKey(key) == true) {
+		return true;
+	}
+
 	bool result = (
 	key != SDLK_DELETE &&
 	key != SDLK_BACKSPACE &&
@@ -1173,7 +1256,7 @@ wchar_t Window::extractLastKeyPressed() {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] returning key [%d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,c);
 
 	string pressKeyName = SDL_GetKeyName((SDLKey)c);
-	string inputKeyName = SDL_GetKeyName(keystate.sym);
+	//string inputKeyName = SDL_GetKeyName(keystate.sym);
 
 	//printf ("PRESS pressed key [%d - %s] input.keysym.sym [%d] input.keysym.unicode [%d] mod = %d\n",
 	//		c,pressKeyName.c_str(),input.keysym.sym,input.keysym.unicode,input.keysym.mod);
