@@ -182,6 +182,7 @@ StreamSoundSource::StreamSoundSource()
 	sound = 0;
 	fadeState = NoFading;
 	format = 0;
+	fade = 0;
 	alGenBuffers(STREAMFRAGMENTS, buffers);
 	SoundPlayerOpenAL::checkAlError(string(__FILE__) + string(" ") + string(__FUNCTION__) + string(" ") + intToStr(__LINE__));
 }
@@ -376,6 +377,18 @@ bool SoundPlayerOpenAL::init(const SoundPlayerParams* params) {
 	this->params = *params;
 
 	try {
+
+		if(context != 0) {
+			alcMakeContextCurrent( NULL );
+			alcDestroyContext(context);
+			context = 0;
+		}
+
+		if(device != 0) {
+			alcCloseDevice(device);
+			device = 0;
+		}
+
 		// Allows platforms to specify which sound device to use
 		// using the environment variable: MEGAGLEST_SOUND_DEVICE
 		char *deviceName = getenv("MEGAGLEST_SOUND_DEVICE");
@@ -430,6 +443,7 @@ void SoundPlayerOpenAL::end() {
 			i != staticSources.end(); ++i) {
 		delete *i;
     }
+	staticSources.clear();
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
@@ -437,12 +451,14 @@ void SoundPlayerOpenAL::end() {
 			i != streamSources.end(); ++i) {
 		delete *i;
     }
+	streamSources.clear();
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-    alcMakeContextCurrent( NULL );
-    SoundPlayerOpenAL::checkAlcError(string(__FILE__) + string(" ") + string(__FUNCTION__) + string(" ") + intToStr(__LINE__));
-
+	if(context != 0) {
+		alcMakeContextCurrent( NULL );
+		SoundPlayerOpenAL::checkAlcError(string(__FILE__) + string(" ") + string(__FUNCTION__) + string(" ") + intToStr(__LINE__));
+	}
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	if(context != 0) {
