@@ -22,13 +22,14 @@
 #include "randomgen.h"
 
 #ifdef WIN32
+
 #include <shlwapi.h>
 #include <shlobj.h>
+
 #endif
 
 #include "utf8.h"
 #include "font.h"
-
 #include "string_utils.h"
 
 #include "leak_dumper.h"
@@ -41,8 +42,13 @@ using namespace Shared::Graphics;
 namespace Shared{ namespace Util{
 
 string Properties::applicationPath = "";
+string Properties::applicationDataPath = "";
 string Properties::gameVersion = "";
 
+string Properties::techtreePath = "";
+string Properties::scenarioPath = "";
+string Properties::tutorialPath = "";
+
 // =====================================================
 //	class Properties
 // =====================================================
@@ -174,17 +180,20 @@ std::map<string,string> Properties::getTagReplacementValues(std::map<string,stri
 	// First add the standard tags
 	//
 #ifdef WIN32
-	char *homeDir = getenv("USERPROFILE");
+	const char *homeDirX = getenv("USERPROFILE");
 #else
-	char *homeDir = getenv("HOME");
+	string home = getUserHome();
+	const char *homeDirX = home.c_str();
 #endif
 
-	mapTagReplacementValues["~/"] = (homeDir != NULL ? homeDir : "");
-	mapTagReplacementValues["$HOME"] = (homeDir != NULL ? homeDir : "");
-	mapTagReplacementValues["%%HOME%%"] = (homeDir != NULL ? homeDir : "");
-	mapTagReplacementValues["%%USERPROFILE%%"] = (homeDir != NULL ? homeDir : "");
-	mapTagReplacementValues["%%HOMEPATH%%"] = (homeDir != NULL ? homeDir : "");
-	mapTagReplacementValues["{HOMEPATH}"] = (homeDir != NULL ? homeDir : "");
+	string homeDir = safeCharPtrCopy(homeDirX, 8096);
+
+	mapTagReplacementValues["~/"] 				= homeDir;
+	mapTagReplacementValues["$HOME"] 			= homeDir;
+	mapTagReplacementValues["%%HOME%%"] 		= homeDir;
+	mapTagReplacementValues["%%USERPROFILE%%"] 	= homeDir;
+	mapTagReplacementValues["%%HOMEPATH%%"] 	= homeDir;
+	mapTagReplacementValues["{HOMEPATH}"] 		= homeDir;
 
 	// For win32 we allow use of the appdata variable since that is the recommended
 	// place for application data in windows platform
@@ -226,12 +235,16 @@ std::map<string,string> Properties::getTagReplacementValues(std::map<string,stri
 	mapTagReplacementValues["{APPLICATIONDATAPATH}"] = formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH));
 
 #else
-	mapTagReplacementValues["$APPLICATIONDATAPATH"] = Properties::applicationPath;
-	mapTagReplacementValues["%%APPLICATIONDATAPATH%%"] = Properties::applicationPath;
-	mapTagReplacementValues["{APPLICATIONDATAPATH}"] = Properties::applicationPath;
+	mapTagReplacementValues["$APPLICATIONDATAPATH"] = Properties::applicationDataPath;
+	mapTagReplacementValues["%%APPLICATIONDATAPATH%%"] = Properties::applicationDataPath;
+	mapTagReplacementValues["{APPLICATIONDATAPATH}"] = Properties::applicationDataPath;
 
 #endif
 
+	mapTagReplacementValues["{TECHTREEPATH}"] = Properties::techtreePath;
+	mapTagReplacementValues["{SCENARIOPATH}"] = Properties::scenarioPath;
+	mapTagReplacementValues["{TUTORIALPATH}"] = Properties::tutorialPath;
+
 	mapTagReplacementValues["$GAMEVERSION"] = Properties::gameVersion;
 
 	//
@@ -248,7 +261,7 @@ std::map<string,string> Properties::getTagReplacementValues(std::map<string,stri
 	return mapTagReplacementValues;
 }
 
-bool Properties::applyTagsToValue(string &value, std::map<string,string> *mapTagReplacementValues) {
+bool Properties::applyTagsToValue(string &value, const std::map<string,string> *mapTagReplacementValues) {
 	string originalValue = value;
 
 	//if(originalValue.find("$APPLICATIONDATAPATH") != string::npos) {
@@ -256,24 +269,35 @@ bool Properties::applyTagsToValue(string &value, std::map<string,string> *mapTag
 	//}
 
 	if(mapTagReplacementValues != NULL) {
-		for(std::map<string,string>::iterator iterMap = mapTagReplacementValues->begin();
+		for(std::map<string,string>::const_iterator iterMap = mapTagReplacementValues->begin();
 				iterMap != mapTagReplacementValues->end(); ++iterMap) {
+
+//			if(value.find("{SCENARIOPATH}") != string::npos) {
+//				printf("\n\n** WILL REPLACE [%s]\n",value.c_str());
+//				replaceAll(value, "{SCENARIOPATH}",		Properties::scenarioPath);
+//				printf("** REPLACED [%s]\n\n",value.c_str());
+//			}
+//			else {
 			replaceAll(value, iterMap->first, iterMap->second);
+//			}
 		}
 	}
 	else {
 #ifdef WIN32
-	char *homeDir = getenv("USERPROFILE");
+	const char *homeDirX = getenv("USERPROFILE");
 #else
-	char *homeDir = getenv("HOME");
+	string home = getUserHome();
+	const char *homeDirX = home.c_str();
 #endif
 
-	replaceAll(value, "~/", 			(homeDir != NULL ? homeDir : ""));
-	replaceAll(value, "$HOME", 			(homeDir != NULL ? homeDir : ""));
-	replaceAll(value, "%%HOME%%", 		(homeDir != NULL ? homeDir : ""));
-	replaceAll(value, "%%USERPROFILE%%",(homeDir != NULL ? homeDir : ""));
-	replaceAll(value, "%%HOMEPATH%%",	(homeDir != NULL ? homeDir : ""));
-	replaceAll(value, "{HOMEPATH}",		(homeDir != NULL ? homeDir : ""));
+	string homeDir = safeCharPtrCopy(homeDirX, 8096);
+
+	replaceAll(value, "~/", 			homeDir);
+	replaceAll(value, "$HOME", 			homeDir);
+	replaceAll(value, "%%HOME%%", 		homeDir);
+	replaceAll(value, "%%USERPROFILE%%",homeDir);
+	replaceAll(value, "%%HOMEPATH%%",	homeDir);
+	replaceAll(value, "{HOMEPATH}",		homeDir);
 
 	// For win32 we allow use of the appdata variable since that is the recommended
 	// place for application data in windows platform
@@ -313,12 +337,20 @@ bool Properties::applyTagsToValue(string &value, std::map<string,string> *mapTag
 	replaceAll(value, "{APPLICATIONDATAPATH}",		formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH)));
 
 #else
-	replaceAll(value, "$APPLICATIONDATAPATH", 		Properties::applicationPath);
-	replaceAll(value, "%%APPLICATIONDATAPATH%%",	Properties::applicationPath);
-	replaceAll(value, "{APPLICATIONDATAPATH}",		Properties::applicationPath);
+	replaceAll(value, "$APPLICATIONDATAPATH", 		Properties::applicationDataPath);
+	replaceAll(value, "%%APPLICATIONDATAPATH%%",	Properties::applicationDataPath);
+	replaceAll(value, "{APPLICATIONDATAPATH}",		Properties::applicationDataPath);
 
 #endif
 
+	replaceAll(value, "{TECHTREEPATH}",		Properties::techtreePath);
+//	if(value.find("{SCENARIOPATH}") != string::npos) {
+//		printf("\n\n** WILL REPLACE [%s]\n",value.c_str());
+	replaceAll(value, "{SCENARIOPATH}",		Properties::scenarioPath);
+//		printf("** REPLACED [%s]\n\n",value.c_str());
+//	}
+	replaceAll(value, "{TUTORIALPATH}",		Properties::tutorialPath);
+
 	replaceAll(value, "$GAMEVERSION",		Properties::gameVersion);
 	}
 
@@ -345,7 +377,7 @@ void Properties::save(const string &path){
 
 	fileStream.close();
 #if defined(WIN32) && !defined(__MINGW32__)
-	fclose(fp);
+	if(fp) fclose(fp);
 #endif
 }
 
@@ -399,21 +431,22 @@ int Properties::getInt(const string &key, int min, int max,const char *defaultVa
 }
 
 float Properties::getFloat(const string &key, const char *defaultValueIfNotFound) const{
+	float result = 0.0;
 	try{
-		return strToFloat(getString(key,defaultValueIfNotFound));
+		result = strToFloat(getString(key,defaultValueIfNotFound));
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
 		//throw megaglest_runtime_error("Error accessing value: " + key + " in: " + path + "\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + key + " in: " + path + "\n[" + e.what() + "]");
 	}
-	return 0.0;
+	return result;
 }
 
 float Properties::getFloat(const string &key, float min, float max, const char *defaultValueIfNotFound) const{
 	float f= getFloat(key,defaultValueIfNotFound);
 	if(f<min || f>max){
-		throw megaglest_runtime_error("Value out of range: " + key + ", min: " + floatToStr(min) + ", max: " + floatToStr(max));
+		throw megaglest_runtime_error("Value out of range: " + key + ", min: " + floatToStr(min,16) + ", max: " + floatToStr(max,16));
 	}
 	return f;
 }
@@ -470,7 +503,7 @@ void Properties::setBool(const string &key, bool value){
 }
 
 void Properties::setFloat(const string &key, float value){
-	setString(key, floatToStr(value));
+	setString(key, floatToStr(value,6));
 }
 
 void Properties::setString(const string &key, const string &value){
@@ -516,15 +549,16 @@ int Properties::getInt(const char *key,const char *defaultValueIfNotFound) const
 }
 
 float Properties::getFloat(const char *key, const char *defaultValueIfNotFound) const{
+	float result = 0.0;
 	try{
-		return strToFloat(getString(key,defaultValueIfNotFound));
+		result = strToFloat(getString(key,defaultValueIfNotFound));
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
 		//throw megaglest_runtime_error("Error accessing value: " + string(key) + " in: " + path + "\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + string(key) + " in: " + path + "\n[" + e.what() + "]");
 	}
-	return 0.0;
+	return result;
 }
 
 const string Properties::getString(const char *key, const char *defaultValueIfNotFound) const{
