@@ -378,7 +378,10 @@ void SystemFlags::Close() {
 		SystemFlags::lockFileCountIndex = -1;
 
 		if(SystemFlags::lockfilename != "") {
-			remove(SystemFlags::lockfilename.c_str());
+			int remove_result = remove(SystemFlags::lockfilename.c_str());
+
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] remove_result = %d for file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,remove_result,SystemFlags::lockfilename.c_str());
+
 			SystemFlags::lockfilename = "";
 		}
 	}
@@ -445,12 +448,15 @@ void SystemFlags::logDebugEntry(DebugType type, string debugEntry, time_t debugT
         return;
     }
 
-    // Get the current time.
-//    time_t curtime = time (NULL);
-    // Convert it to local time representation.
-    struct tm *loctime = localtime (&debugTime);
     char szBuf2[100]="";
-    strftime(szBuf2,100,"%Y-%m-%d %H:%M:%S",loctime);
+    //if (type != debugPathFinder && type != debugError && type != debugWorldSynch) {
+    if (type != debugPathFinder && type != debugWorldSynch) {
+		// Get the current time.
+	//    time_t curtime = time (NULL);
+		// Convert it to local time representation.
+		struct tm *loctime = localtime (&debugTime);
+		strftime(szBuf2,100,"%Y-%m-%d %H:%M:%S",loctime);
+    }
 /*
     va_list argList;
     va_start(argList, fmt);
@@ -466,18 +472,20 @@ void SystemFlags::logDebugEntry(DebugType type, string debugEntry, time_t debugT
 
         	// If the file is already open (shared) by another debug type
         	// do not over-write the file but share the stream pointer
-        	for(std::map<SystemFlags::DebugType,SystemFlags::SystemFlagsType>::iterator iterMap = SystemFlags::debugLogFileList->begin();
-        		iterMap != SystemFlags::debugLogFileList->end(); ++iterMap) {
-        		SystemFlags::SystemFlagsType &currentDebugLog2 = iterMap->second;
+        	if(SystemFlags::debugLogFileList != NULL) {
+				for(std::map<SystemFlags::DebugType,SystemFlags::SystemFlagsType>::iterator iterMap = SystemFlags::debugLogFileList->begin();
+					iterMap != SystemFlags::debugLogFileList->end(); ++iterMap) {
+					SystemFlags::SystemFlagsType &currentDebugLog2 = iterMap->second;
 
-        		if(	iterMap->first != type &&
-        			currentDebugLog.debugLogFileName == currentDebugLog2.debugLogFileName &&
-        			currentDebugLog2.fileStream != NULL) {
-					currentDebugLog.fileStream = currentDebugLog2.fileStream;
-					currentDebugLog.fileStreamOwner = false;
-					currentDebugLog.mutex			= currentDebugLog2.mutex;
-        			break;
-        		}
+					if(	iterMap->first != type &&
+						currentDebugLog.debugLogFileName == currentDebugLog2.debugLogFileName &&
+						currentDebugLog2.fileStream != NULL) {
+						currentDebugLog.fileStream = currentDebugLog2.fileStream;
+						currentDebugLog.fileStreamOwner = false;
+						currentDebugLog.mutex			= currentDebugLog2.mutex;
+						break;
+					}
+				}
         	}
 
         	string debugLog = currentDebugLog.debugLogFileName;
@@ -499,6 +507,16 @@ void SystemFlags::logDebugEntry(DebugType type, string debugEntry, time_t debugT
             		int idx = 1;
             		for(idx = 1; idx <= 100; ++idx) {
 						newlockfile = lockfile + intToStr(idx);
+
+						if(SystemFlags::lockFile != -1) {
+#ifndef WIN32
+							close(SystemFlags::lockFile);
+#else
+							_close(SystemFlags::lockFile);
+#endif
+							SystemFlags::lockFile = -1;
+							SystemFlags::lockFileCountIndex = -1;
+						}
 						//SystemFlags::lockFile = open(newlockfile.c_str(), O_WRONLY | O_CREAT | O_EXCL, S_IRUSR|S_IWUSR);
 #ifndef WIN32
 	                    SystemFlags::lockFile = open(newlockfile.c_str(), O_WRONLY | O_CREAT, S_IREAD | S_IWRITE);
@@ -536,7 +554,7 @@ void SystemFlags::logDebugEntry(DebugType type, string debugEntry, time_t debugT
             	currentDebugLog.fileStream->open(debugLog.c_str(), ios_base::out | ios_base::trunc);
 #endif
 				currentDebugLog.fileStreamOwner = true;
-				currentDebugLog.mutex			= new Mutex();
+				currentDebugLog.mutex			= new Mutex(CODE_AT_LINE);
             }
 
             if(SystemFlags::haveSpecialOutputCommandLineOption == false) {
@@ -556,7 +574,8 @@ void SystemFlags::logDebugEntry(DebugType type, string debugEntry, time_t debugT
         assert(currentDebugLog.fileStream != NULL);
 
         if(currentDebugLog.fileStream->is_open() == true) {
-			MutexSafeWrapper safeMutex(currentDebugLog.mutex,string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__));
+        	static string mutexCodeLocation = string(extractFileFromDirectoryPath(__FILE__).c_str()) + "_" + intToStr(__LINE__);
+			MutexSafeWrapper safeMutex(currentDebugLog.mutex,mutexCodeLocation);
 
 			// All items in the if clause we don't want timestamps
 			if (type != debugPathFinder && type != debugError && type != debugWorldSynch) {
@@ -584,7 +603,7 @@ void SystemFlags::logDebugEntry(DebugType type, string debugEntry, time_t debugT
 			printf("[%s] %s", szBuf2, debugEntry.c_str());
 		}
 		else if (type == debugError) {
-			printf("[%s] *ERROR* %s", szBuf2, debugEntry.c_str());
+			printf("*ERROR* [%s] %s", szBuf2,debugEntry.c_str());
 		}
 		else {
 			printf("%s", debugEntry.c_str());
@@ -707,11 +726,11 @@ void copyStringToBuffer(char *buffer, int bufferSize, const string& s){
 
 // ==================== numeric fcs ====================
 
-float saturate(float value){
-	if (value<0.f){
+float saturate(float value) {
+	if (value < 0.f){
         return 0.f;
 	}
-	if (value>1.f){
+	if (value > 1.f){
         return 1.f;
 	}
     return value;
@@ -727,7 +746,7 @@ int clamp(int value, int min, int max){
     return value;
 }
 
-float clamp(float value, float min, float max){
+int64 clamp(int64 value, int64 min, int64 max){
 	if (value<min){
         return min;
 	}
@@ -737,6 +756,16 @@ float clamp(float value, float min, float max){
     return value;
 }
 
+float clamp(float value, float min, float max) {
+	if (value < min) {
+        return min;
+	}
+	if (value > max) {
+        return max;
+	}
+    return value;
+}
+
 int round(float f){
      return (int) f;
 }
@@ -766,14 +795,14 @@ bool checkVersionComptability(string clientVersionString, string serverVersionSt
 					StartsWith(tokensSpecialVersionTags[1],"alpha") == true ||
 					StartsWith(tokensSpecialVersionTags[1],"beta") == true) {
 
-					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND SPECIAL SERVER VERSION TAG [%s] [%s] tokensSpecialVersionTags.size() = " MG_SIZE_T_SPECIFIER "\n",tokensSpecialVersionTags[0].c_str(),tokensSpecialVersionTags[1].c_str(),tokensSpecialVersionTags.size());
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Line Ref: %d FOUND SPECIAL SERVER VERSION TAG [%s] [%s] tokensSpecialVersionTags.size() = " MG_SIZE_T_SPECIFIER "\n",__LINE__,tokensSpecialVersionTags[0].c_str(),tokensSpecialVersionTags[1].c_str(),tokensSpecialVersionTags.size());
 
 					// Chop off platform specific version info
 					if(tokensSpecialVersionTags.size() > 2) {
 						string trimRightDelim = "-" + tokensSpecialVersionTags[2];
 						string newVersionText = trim_at_delim(serverVersionString, trimRightDelim);
 
-						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("NEW SERVER VERSION TAG [%s] OLD TAG [%s] delim [%s]\n",newVersionText.c_str(),serverVersionString.c_str(),trimRightDelim.c_str());
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Line Ref: %d NEW SERVER VERSION TAG [%s] OLD TAG [%s] delim [%s]\n",__LINE__,newVersionText.c_str(),serverVersionString.c_str(),trimRightDelim.c_str());
 						serverVersionString = newVersionText;
 
 						tokensServer.clear();
@@ -793,14 +822,14 @@ bool checkVersionComptability(string clientVersionString, string serverVersionSt
 					StartsWith(tokensSpecialVersionTags[1],"alpha") == true ||
 					StartsWith(tokensSpecialVersionTags[1],"beta") == true) {
 
-					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND SPECIAL CLIENT VERSION TAG [%s] [%s] tokensSpecialVersionTags.size() = " MG_SIZE_T_SPECIFIER "\n",tokensSpecialVersionTags[0].c_str(),tokensSpecialVersionTags[1].c_str(),tokensSpecialVersionTags.size());
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Line Ref: %d FOUND SPECIAL CLIENT VERSION TAG [%s] [%s] tokensSpecialVersionTags.size() = " MG_SIZE_T_SPECIFIER "\n",__LINE__,tokensSpecialVersionTags[0].c_str(),tokensSpecialVersionTags[1].c_str(),tokensSpecialVersionTags.size());
 
 					// Chop off platform specific version info
 					if(tokensSpecialVersionTags.size() > 2) {
 						string trimRightDelim = "-" + tokensSpecialVersionTags[2];
 						string newVersionText = trim_at_delim(clientVersionString, trimRightDelim);
 
-						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("NEW CLIENT VERSION TAG [%s] OLD TAG [%s] delim [%s]\n",newVersionText.c_str(),clientVersionString.c_str(),trimRightDelim.c_str());
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Line Ref: %d NEW CLIENT VERSION TAG [%s] OLD TAG [%s] delim [%s]\n",__LINE__,newVersionText.c_str(),clientVersionString.c_str(),trimRightDelim.c_str());
 						clientVersionString = newVersionText;
 
 						tokens.clear();
@@ -820,31 +849,17 @@ bool checkVersionComptability(string clientVersionString, string serverVersionSt
 		if(SystemFlags::VERBOSE_MODE_ENABLED) {
 			// debug version strings
 			for(unsigned int i = 0; i < tokensServer.size(); ++i) {
-				printf("Server version index = %d str [%s] IsNumeric = %d\n",i,tokensServer[i].c_str(),IsNumeric(tokensServer[i].c_str(),false));
+				printf("Line Ref: %d Server version index = %u str [%s] IsNumeric = %d\n",__LINE__,i,tokensServer[i].c_str(),IsNumeric(tokensServer[i].c_str(),false));
 			}
 			for(unsigned int i = 0; i < tokens.size(); ++i) {
-				printf("Client version index = %d str [%s] IsNumeric = %d\n",i,tokens[i].c_str(),IsNumeric(tokens[i].c_str(),false));
+				printf("Line Ref: %d Client version index = %u str [%s] IsNumeric = %d\n",__LINE__,i,tokens[i].c_str(),IsNumeric(tokens[i].c_str(),false));
 			}
 		}
 		// **NOTE:
 		// after of 3.7.0 we go to 2 digi version compatibility, check if both
 		// client and server are at least 3.7
-		bool compatiblePost3_7_0_Check = false;
 		bool compatiblePre3_7_0_1_Check = true;
 
-//		if(tokens.size() >= 3 && tokensServer.size() >= 3 &&
-//			tokensServer[0] != "" && IsNumeric(tokensServer[0].c_str(),false) && strToInt(tokensServer[0]) == 3 &&
-//			tokensServer[1] != "" && IsNumeric(tokensServer[1].c_str(),false) && strToInt(tokensServer[1]) == 7 &&
-//			(tokensServer[2][0] == '0' || tokensServer[2][0] == '1') &&
-//			tokens[0] != "" && IsNumeric(tokens[0].c_str(),false) && strToInt(tokens[0]) == 3 &&
-//			tokens[1] != "" && IsNumeric(tokens[1].c_str(),false) && strToInt(tokens[1]) == 7 &&
-//			(tokens[2][0] == '0' || tokens[2][0] == '1')) {
-//
-//			compatiblePost3_7_0_Check = false;
-//			compatiblePre3_7_0_1_Check = false;
-//			compatible = true;
-//		}
-//		else if(tokens.size() >= 2 && tokensServer.size() >= 2) {
 		if(tokens.size() >= 2 && tokensServer.size() >= 2) {
 			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
@@ -856,6 +871,7 @@ bool checkVersionComptability(string clientVersionString, string serverVersionSt
 
 				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
+				bool compatiblePost3_7_0_Check = false;
 				// Both are at least 3.7.0, now check if both are > 3.7.0
 				if(tokens.size() >= 3 && tokensServer.size() >= 3) {
 					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
@@ -913,7 +929,7 @@ bool checkVersionComptability(string clientVersionString, string serverVersionSt
 				if(compatiblePost3_7_0_Check == true) {
 					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 					compatiblePre3_7_0_1_Check = false;
-					// only check the first 2 sections with . to compare makor versions #'s
+					// only check the first 2 sections with . to compare major versions #'s
 					compatible = (tokens.size() >= 2 && tokensServer.size() >= 2);
 
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] clientVersionString [%s], serverVersionString [%s] compatible [%d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,clientVersionString.c_str(),serverVersionString.c_str(),compatible);
