@@ -9,6 +9,7 @@
 //	License, or (at your option) any later version
 // ==============================================================
 
+#include "data_types.h"
 #include "xml_parser.h"
 
 #include <fstream>
@@ -21,7 +22,6 @@
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/framework/LocalFileFormatTarget.hpp>
 #include "util.h"
-#include "data_types.h"
 #include "properties.h"
 #include "platform_common.h"
 #include "platform_util.h"
@@ -61,10 +61,22 @@ public:
 //	class XmlIo
 // =====================================================
 
-bool XmlIo::initialized= false;
+bool XmlIo::initialized		= false;
 bool XmlIoRapid::initialized= false;
 
-XmlIo::XmlIo() {
+XmlIo::XmlIo() : parser(NULL) {
+	init();
+}
+
+XmlIo::~XmlIo() {
+	cleanup();
+}
+
+bool XmlIo::isInitialized() {
+	return XmlIo::initialized;
+}
+
+void XmlIo::init() {
 	try{
 		//printf("XmlIo init\n");
 		XMLPlatformUtils::Initialize();
@@ -90,10 +102,14 @@ XmlIo::XmlIo() {
 
 XmlIo &XmlIo::getInstance() {
 	static XmlIo XmlIo;
+	if(XmlIo::initialized == false) {
+		XmlIo.init();
+	}
 	return XmlIo;
 }
 
 void XmlIo::cleanup() {
+	releaseDOMParser();
 	if(XmlIo::initialized == true) {
 		XmlIo::initialized= false;
 		//printf("XmlIo cleanup\n");
@@ -101,11 +117,26 @@ void XmlIo::cleanup() {
 	}
 }
 
-XmlIo::~XmlIo() {
-	cleanup();
+#if XERCES_VERSION_MAJOR < 3
+	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * XmlIo::getRootDOMDocument(const string &path, DOMBuilder *parser, bool noValidation) {
+		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document= parser->parseURI(path.c_str());
+		return document;
+	}
+#else
+	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * XmlIo::getRootDOMDocument(const string &path, DOMLSParser *parser, bool noValidation) {
+		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document= parser->parseURI(path.c_str());
+		return document;
+	}
+#endif
+
+void XmlIo::releaseDOMParser() {
+	if(parser != NULL) {
+		parser->release();
+		parser = NULL;
+	}
 }
 
-XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplacementValues,bool noValidation,bool skipStackCheck) {
+DOMNode *XmlIo::loadDOMNode(const string &path, bool noValidation) {
 	//printf("Load file using Xerces engine [%s]\n",path.c_str());
 
 	try {
@@ -113,15 +144,15 @@ XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplaceme
 
 		ErrorHandler errorHandler;
 #if XERCES_VERSION_MAJOR < 3
- 		DOMBuilder *parser= (static_cast<DOMImplementationLS*>(implementation))->createDOMBuilder(DOMImplementationLS::MODE_SYNCHRONOUS, 0);
- 		parser->setErrorHandler(&errorHandler);
- 		if(noValidation == false) {
+		parser= (static_cast<DOMImplementationLS*>(implementation))->createDOMBuilder(DOMImplementationLS::MODE_SYNCHRONOUS, 0);
+		parser->setErrorHandler(&errorHandler);
+		if(noValidation == false) {
 			parser->setFeature(XMLUni::fgXercesSchema, true);
 			parser->setFeature(XMLUni::fgXercesSchemaFullChecking, true);
 			//parser->setFeature(XMLUni::fgDOMValidateIfSchema, true);
 			parser->setFeature(XMLUni::fgDOMValidation, true);
- 		}
- 		else {
+		}
+		else {
 			//parser->setFeature(XMLUni::fgSAX2CoreValidation, false);
 			//parser->setFeature(XMLUni::fgXercesDynamic, true);
 
@@ -130,29 +161,31 @@ XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplaceme
 			//parser->setFeature(XMLUni::fgDOMValidateIfSchema, true);
 			//parser->setFeature(XMLUni::fgDOMValidation, false);
 
- 			parser->setFeature(XMLUni::fgXercesSchemaFullChecking, false);
- 			parser->setFeature(XMLUni::fgXercesLoadExternalDTD, false);
- 			parser->setFeature(XMLUni::fgXercesCacheGrammarFromParse, true);
- 			parser->setFeature(XMLUni::fgXercesUseCachedGrammarInParse, true);
- 		}
+			parser->setFeature(XMLUni::fgXercesSchemaFullChecking, false);
+			parser->setFeature(XMLUni::fgXercesLoadExternalDTD, false);
+			parser->setFeature(XMLUni::fgXercesCacheGrammarFromParse, true);
+			parser->setFeature(XMLUni::fgXercesUseCachedGrammarInParse, true);
+		}
 #else
-		DOMLSParser *parser = (static_cast<DOMImplementationLS*>(implementation))->createLSParser(DOMImplementationLS::MODE_SYNCHRONOUS, 0);
- 		DOMConfiguration  *config = parser->getDomConfig();
- 		if(noValidation == false) {
+		parser = (static_cast<DOMImplementationLS*>(implementation))->createLSParser(DOMImplementationLS::MODE_SYNCHRONOUS, 0);
+		DOMConfiguration  *config = parser->getDomConfig();
+		if(noValidation == false) {
 			config->setParameter(XMLUni::fgXercesSchema, true);
 			config->setParameter(XMLUni::fgXercesSchemaFullChecking, true);
 			//config->setParameter(XMLUni::fgDOMValidateIfSchema, true);
 			config->setParameter(XMLUni::fgDOMValidate, true);
- 		}
- 		else {
+		}
+		else {
 			config->setParameter(XMLUni::fgXercesSchema, false);
 			config->setParameter(XMLUni::fgXercesSchemaFullChecking, false);
 			config->setParameter(XMLUni::fgXercesLoadExternalDTD, false);
 			config->setParameter(XMLUni::fgXercesCacheGrammarFromParse, true);
 			config->setParameter(XMLUni::fgXercesUseCachedGrammarInParse, true);
- 		}
+		}
 #endif
-		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document= parser->parseURI(path.c_str());
+		//XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document= parser->parseURI(path.c_str());
+		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document = getRootDOMDocument(path, parser, noValidation);
+
 #ifdef WIN32
 		if(document == NULL) {
 			document= parser->parseURI(utf8_decode(path).c_str());
@@ -162,8 +195,7 @@ XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplaceme
 			throw megaglest_runtime_error("Can not parse URL: " + path);
 		}
 
-		XmlNode *rootNode= new XmlNode(document->getDocumentElement(),mapTagReplacementValues);
-		parser->release();
+		DOMNode *rootNode = document->getDocumentElement();
 		return rootNode;
 	}
 	catch(const DOMException &ex) {
@@ -176,10 +208,40 @@ XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplaceme
 	return NULL;
 }
 
+XmlNode *XmlIo::load(const string &path, const std::map<string,string> &mapTagReplacementValues,bool noValidation,bool skipStackTrace) {
+	//printf("Load file using Xerces engine [%s]\n",path.c_str());
+
+	try {
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("XERCES_FULLVERSIONDOT [%s]\nnoValidation = %d\npath [%s]\n",XERCES_FULLVERSIONDOT,noValidation,path.c_str());
+
+		DOMNode *domNode = loadDOMNode(path, noValidation);
+		XmlNode *rootNode= new XmlNode(domNode,mapTagReplacementValues);
+		releaseDOMParser();
+
+		return rootNode;
+	}
+	catch(const DOMException &ex) {
+		char szBuf[8096]="";
+		if(skipStackTrace == false) {
+			snprintf(szBuf,8096,"In [%s::%s Line: %d] Exception while loading: [%s], msg:\n%s",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),XMLString::transcode(ex.msg));
+		}
+		else {
+			snprintf(szBuf,8096,"Error loading: [%s], msg:\n%s",path.c_str(),XMLString::transcode(ex.msg));
+		}
+		SystemFlags::OutputDebug(SystemFlags::debugError,"%s\n",szBuf);
+
+		throw megaglest_runtime_error(szBuf,skipStackTrace);
+	}
+	return NULL;
+}
+
 void XmlIo::save(const string &path, const XmlNode *node){
 	//printf("Saving file using Xerces engine [%s]\n",path.c_str());
 
 	try{
+		if(node == NULL) {
+			throw megaglest_runtime_error("node == NULL during save!");
+		}
 		XMLCh str[strSize];
 		XMLString::transcode(node->getName().c_str(), str, strSize-1);
 
@@ -227,36 +289,35 @@ void XmlIo::save(const string &path, const XmlNode *node){
 //	class XmlIoRapid
 // =====================================================
 XmlIoRapid::XmlIoRapid() {
-	try{
-		//printf("XmlIo init\n");
+	init();
+}
 
+void XmlIoRapid::init() {
+	try{
 		XmlIoRapid::initialized= true;
 	}
 	catch(const exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error initializing XML system, msg %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
 		throw megaglest_runtime_error("Error initializing XML system");
 	}
+}
 
-	try {
-		doc = new xml_document<>();
-	}
-	catch(const DOMException &ex) {
-		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Exception while creating XML parser, msg: %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.getMessage());
-		throw megaglest_runtime_error("Exception while creating XML parser");
-	}
+bool XmlIoRapid::isInitialized() {
+	return XmlIoRapid::initialized;
 }
 
 XmlIoRapid &XmlIoRapid::getInstance() {
 	static XmlIoRapid io;
+	if(XmlIoRapid::initialized == false) {
+		io.init();
+	}
+
 	return io;
 }
 
 void XmlIoRapid::cleanup() {
 	if(XmlIoRapid::initialized == true) {
 		XmlIoRapid::initialized= false;
-		//printf("XmlIo cleanup\n");
-		delete doc;
-		doc = NULL;
 	}
 }
 
@@ -264,14 +325,20 @@ XmlIoRapid::~XmlIoRapid() {
 	cleanup();
 }
 
-XmlNode *XmlIoRapid::load(const string &path, std::map<string,string> mapTagReplacementValues,bool noValidation,bool skipStackCheck) {
+XmlNode *XmlIoRapid::load(const string &path, const std::map<string,string> &mapTagReplacementValues,bool noValidation,bool skipStackTrace) {
+	bool showPerfStats = SystemFlags::VERBOSE_MODE_ENABLED;
 	Chrono chrono;
 	chrono.start();
-	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Using RapidXml to load file [%s]\n",path.c_str());
+	if(SystemFlags::VERBOSE_MODE_ENABLED || showPerfStats) printf("Using RapidXml to load file [%s]\n",path.c_str());
 	//printf("Using RapidXml to load file [%s]\n",path.c_str());
 
 	XmlNode *rootNode = NULL;
 	try {
+
+		if(folderExists(path) == true) {
+			throw megaglest_runtime_error("Can not open file: [" + path + "] as it is a folder!");
+		}
+
 #if defined(WIN32) && !defined(__MINGW32__)
 		FILE *fp = _wfopen(utf8_decode(path).c_str(), L"rb");
 		ifstream xmlFile(fp);
@@ -282,25 +349,51 @@ XmlNode *XmlIoRapid::load(const string &path, std::map<string,string> mapTagRepl
 			throw megaglest_runtime_error("Can not open file: [" + path + "]");
 		}
 
+		if(showPerfStats) printf("In [%s::%s Line: %d] took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
+
 		xmlFile.unsetf(ios::skipws);
 
         // Determine stream size
-		xmlFile.seekg(0, ios::end);
-        streampos size = xmlFile.tellg();
-        xmlFile.seekg(0);
+		int64 file_size = -1;
+		if((double)xmlFile.tellg() != -1) {
+			streampos size1 = xmlFile.tellg();
+			xmlFile.seekg(0, ios::end);
+			if((double)xmlFile.tellg() != -1) {
+				streampos size2 = xmlFile.tellg();
+				xmlFile.seekg(0);
+				file_size = size2 - size1;
+			}
+		}
+
+        if(showPerfStats) printf("In [%s::%s Line: %d] took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
+
+        if(file_size <= 0) {
+        	throw megaglest_runtime_error("Invalid file size for file: [" + path + "] size = " + intToStr(file_size));
+        }
+        //printf("File size is: " MG_I64_SPECIFIER " for [%s]\n",file_size,path.c_str());
 
         // Load data and add terminating 0
         vector<char> buffer;
-        buffer.resize((unsigned int)size + 1);
-        xmlFile.read(&buffer.front(), static_cast<streamsize>(size));
-        buffer[(unsigned int)size] = 0;
+        buffer.resize((unsigned int)file_size + 1);
+        xmlFile.read(&buffer.front(), static_cast<streamsize>(file_size));
+        buffer[(unsigned int)file_size] = 0;
+
+        if(showPerfStats) printf("In [%s::%s Line: %d] took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
 
         // This is required because rapidxml seems to choke when we load lua
         // scenarios that have lua + xml style comments
         replaceAllBetweenTokens(buffer, "<!--","-->", "", true);
-        doc->parse<parse_no_data_nodes>(&buffer.front());
 
-		rootNode= new XmlNode(doc->first_node(),mapTagReplacementValues);
+        if(showPerfStats) printf("In [%s::%s Line: %d] took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
+
+        xml_document<> doc;
+        doc.parse<parse_no_data_nodes>(&buffer.front());
+
+        if(showPerfStats) printf("In [%s::%s Line: %d] took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
+
+		rootNode= new XmlNode(doc.first_node(),mapTagReplacementValues);
+
+		if(showPerfStats) printf("In [%s::%s Line: %d] took msecs: " MG_I64_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis());
 
 #if defined(WIN32) && !defined(__MINGW32__)
 		if(fp) {
@@ -310,10 +403,16 @@ XmlNode *XmlIoRapid::load(const string &path, std::map<string,string> mapTagRepl
 	}
 	catch(const exception &ex) {
 		char szBuf[8096]="";
-		snprintf(szBuf,8096,"In [%s::%s Line: %d] Exception while loading: [%s], msg:\n%s",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),ex.what());
+
+		if(skipStackTrace == false) {
+			snprintf(szBuf,8096,"In [%s::%s Line: %d] Exception while loading: [%s], msg:\n%s",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),ex.what());
+		}
+		else {
+			snprintf(szBuf,8096,"Error loading: [%s], msg:\n%s",path.c_str(),ex.what());
+		}
 		SystemFlags::OutputDebug(SystemFlags::debugError,"%s\n",szBuf);
 
-		throw megaglest_runtime_error(szBuf);
+		throw megaglest_runtime_error(szBuf,skipStackTrace);
 	}
 
 	//if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] took msecs: %ld for file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis(),path.c_str());
@@ -324,6 +423,10 @@ XmlNode *XmlIoRapid::load(const string &path, std::map<string,string> mapTagRepl
 
 void XmlIoRapid::save(const string &path, const XmlNode *node){
 	try {
+		if(node == NULL) {
+			throw megaglest_runtime_error("node == NULL during save!");
+		}
+
 		xml_document<> doc;
 
 		// xml declaration
@@ -390,11 +493,23 @@ void XmlIoRapid::save(const string &path, const XmlNode *node){
 // =====================================================
 XmlTree::XmlTree(xml_engine_parser_type engine_type) {
 	rootNode= NULL;
+
+	switch(engine_type) {
+		case XML_XERCES_ENGINE:
+			break;
+		case XML_RAPIDXML_ENGINE:
+		break;
+
+		default:
+			throw megaglest_runtime_error("Invalid XML parser engine: " + intToStr((int)engine_type));
+	}
+
 	this->engine_type = engine_type;
 	this->skipStackCheck = false;
 }
 
 void XmlTree::init(const string &name){
+	clearRootNode();
 	this->rootNode= new XmlNode(name);
 }
 
@@ -402,9 +517,11 @@ typedef std::vector<XmlTree*> LoadStack;
 //static LoadStack loadStack;
 static string loadStackCacheName = string(__FILE__) + string("_loadStackCacheName");
 
-void XmlTree::load(const string &path, std::map<string,string> mapTagReplacementValues, bool noValidation,bool skipStackCheck) {
+void XmlTree::load(const string &path, const std::map<string,string> &mapTagReplacementValues, bool noValidation,bool skipStackCheck,bool skipStackTrace) {
 	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s] skipStackCheck = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),skipStackCheck);
 
+	clearRootNode();
+
 	this->skipStackCheck = skipStackCheck;
 	if(this->skipStackCheck == false) {
 		//printf("XmlTree::load p [%p]\n",this);
@@ -424,11 +541,12 @@ void XmlTree::load(const string &path, std::map<string,string> mapTagReplacement
 	}
 
 	loadPath = path;
+
 	if(this->engine_type == XML_XERCES_ENGINE) {
-		this->rootNode= XmlIo::getInstance().load(path, mapTagReplacementValues, noValidation,this->skipStackCheck);
+		this->rootNode= XmlIo::getInstance().load(path, mapTagReplacementValues, noValidation,skipStackTrace);
 	}
 	else {
-		this->rootNode= XmlIoRapid::getInstance().load(path, mapTagReplacementValues, noValidation,this->skipStackCheck);
+		this->rootNode= XmlIoRapid::getInstance().load(path, mapTagReplacementValues, noValidation,skipStackTrace);
 	}
 
 	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str());
@@ -443,9 +561,7 @@ void XmlTree::save(const string &path){
 	}
 }
 
-XmlTree::~XmlTree() {
-	//printf("XmlTree::~XmlTree p [%p]\n",this);
-
+void XmlTree::clearRootNode() {
 	if(this->skipStackCheck == false) {
 		LoadStack &loadStack = CacheManager::getCachedItem<LoadStack>(loadStackCacheName);
 		Mutex &mutex = CacheManager::getMutexForItem<LoadStack>(loadStackCacheName);
@@ -459,13 +575,19 @@ XmlTree::~XmlTree() {
 	}
 
 	delete rootNode;
+	rootNode=NULL;
+}
+
+XmlTree::~XmlTree() {
+	//printf("XmlTree::~XmlTree p [%p]\n",this);
+	clearRootNode();
 }
 
 // =====================================================
 //	class XmlNode
 // =====================================================
 
-XmlNode::XmlNode(DOMNode *node, std::map<string,string> mapTagReplacementValues): superNode(NULL) {
+XmlNode::XmlNode(DOMNode *node, const std::map<string,string> &mapTagReplacementValues): superNode(NULL) {
     if(node == NULL || node->getNodeName() == NULL) {
         throw megaglest_runtime_error("XML structure seems to be corrupt!");
     }
@@ -512,13 +634,15 @@ XmlNode::XmlNode(DOMNode *node, std::map<string,string> mapTagReplacementValues)
 	}
 }
 
-XmlNode::XmlNode(xml_node<> *node, std::map<string,string> mapTagReplacementValues) : superNode(NULL) {
+XmlNode::XmlNode(xml_node<> *node, const std::map<string,string> &mapTagReplacementValues) : superNode(NULL) {
 	if(node == NULL || node->name() == NULL) {
         throw megaglest_runtime_error("XML structure seems to be corrupt!");
     }
 
 	//get name
 	name = node->name();
+	children.reserve(1000);
+	attributes.reserve(1000);
 
 	//check document
 	if(node->type() == node_document) {
@@ -546,7 +670,7 @@ XmlNode::XmlNode(xml_node<> *node, std::map<string,string> mapTagReplacementValu
 	//get value
 	if(node->type() == node_element && children.size() == 0) {
 		text = node->value();
-		Properties::applyTagsToValue(this->text);
+		Properties::applyTagsToValue(this->text,&mapTagReplacementValues);
 	}
 }
 
@@ -558,14 +682,16 @@ XmlNode::~XmlNode() {
 	for(unsigned int i=0; i<children.size(); ++i) {
 		delete children[i];
 	}
+	children.clear();
 	for(unsigned int i=0; i<attributes.size(); ++i) {
 		delete attributes[i];
 	}
+	attributes.clear();
 }
 
 XmlAttribute *XmlNode::getAttribute(unsigned int i) const {
 	if(i >= attributes.size()) {
-		throw megaglest_runtime_error(getName()+" node doesn't have "+intToStr(i)+" attributes");
+		throw megaglest_runtime_error(getName()+" node doesn't have " + uIntToStr(i) + " attributes");
 	}
 	return attributes[i];
 }
@@ -594,10 +720,22 @@ bool XmlNode::hasAttribute(const string &name) const {
 	return result;
 }
 
+int XmlNode::clearChild(const string &childName) {
+	int clearChildCount = 0;
+	for(int i = (int)children.size()-1; i >= 0; --i) {
+		if(children[i]->getName() == childName) {
+			delete children[i];
+			children.erase(children.begin()+i);
+			clearChildCount++;
+		}
+	}
+	return clearChildCount;
+}
+
 XmlNode *XmlNode::getChild(unsigned int i) const {
 	assert(!superNode);
 	if(i >= children.size()) {
-		throw megaglest_runtime_error("\"" + getName()+"\" node doesn't have "+intToStr(i+1)+" children");
+		throw megaglest_runtime_error("\"" + getName()+"\" node doesn't have "+ uIntToStr(i+1) + " children");
 	}
 	return children[i];
 }
@@ -613,14 +751,15 @@ vector<XmlNode *> XmlNode::getChildList(const string &childName) const {
 	return list;
 }
 
-XmlNode *XmlNode::getChild(const string &childName, unsigned int i) const{
-	if(superNode && !hasChildNoSuper(childName))
+XmlNode *XmlNode::getChild(const string &childName, unsigned int i) const {
+	if(superNode && hasChildNoSuper(childName) == false) {
 		return superNode->getChild(childName,i);
-	if(i>=children.size()){
-		throw megaglest_runtime_error("\"" + name + "\" node doesn't have "+intToStr(i+1)+" children named \"" + childName + "\"\n\nTree: "+getTreeString());
+	}
+	if(i >= children.size()) {
+		throw megaglest_runtime_error("\"" + name + "\" node doesn't have " + uIntToStr(i+1) +" children named \"" + childName + "\"\n\nTree: "+getTreeString());
 	}
 
-	int count= 0;
+	unsigned int count= 0;
 	for(unsigned int j = 0; j < children.size(); ++j) {
 		if(children[j]->getName() == childName) {
 			if(count == i) {
@@ -630,7 +769,40 @@ XmlNode *XmlNode::getChild(const string &childName, unsigned int i) const{
 		}
 	}
 
-	throw megaglest_runtime_error("Node \""+getName()+"\" doesn't have "+intToStr(i+1)+" children named  \""+childName+"\"\n\nTree: "+getTreeString());
+	throw megaglest_runtime_error("Node \""+getName()+"\" doesn't have " + uIntToStr(i+1) + " children named  \""+childName+"\"\n\nTree: "+getTreeString());
+}
+
+bool XmlNode::hasChildNoSuper(const string &childName) const {
+	//int count= 0;
+	for(unsigned int j = 0; j < children.size(); ++j) {
+		if(children[j]->getName() == childName) {
+            return true;
+		}
+	}
+	return false;
+}
+XmlNode * XmlNode::getChildWithAliases(vector<string> childNameList, unsigned int childIndex) const {
+	for(int aliasIndex = 0; aliasIndex < (int)childNameList.size(); ++aliasIndex) {
+		const string &childName = childNameList[aliasIndex];
+		if(superNode && hasChildNoSuper(childName) == false) {
+			return superNode->getChild(childName,childIndex);
+		}
+		if(childIndex >= children.size()) {
+			throw megaglest_runtime_error("\"" + name + "\" node doesn't have "+intToStr(childIndex+1)+" children named \"" + childName + "\"\n\nTree: "+getTreeString());
+		}
+
+		unsigned int count= 0;
+		for(unsigned int j = 0; j < children.size(); ++j) {
+			if(children[j]->getName() == childName) {
+				if(count == childIndex) {
+					return children[j];
+				}
+				count++;
+			}
+		}
+	}
+
+	throw megaglest_runtime_error("Node \""+getName()+"\" doesn't have "+intToStr(childIndex+1)+" children named  \""+ (childNameList.empty() ? "???" : childNameList[0]) +"\"\n\nTree: "+getTreeString());
 }
 
 bool XmlNode::hasChildAtIndex(const string &childName, int i) const {
@@ -638,6 +810,7 @@ bool XmlNode::hasChildAtIndex(const string &childName, int i) const {
 		return superNode->hasChildAtIndex(childName,i);
 	int count= 0;
 	for(unsigned int j = 0; j < children.size(); ++j) {
+		//printf("Looking for [%s] at index: %d found [%s] index = %d\n",childName.c_str(),i,children[j]->getName().c_str(),j);
 		if(children[j]->getName()==childName) {
             if(count == i) {
 				return true;
@@ -652,25 +825,28 @@ bool XmlNode::hasChildAtIndex(const string &childName, int i) const {
 bool XmlNode::hasChild(const string &childName) const {
 	return hasChildNoSuper(childName) || (superNode && superNode->hasChild(childName));
 }
-	
-bool XmlNode::hasChildNoSuper(const string &childName) const {
-	//int count= 0;
-	for(unsigned int j = 0; j < children.size(); ++j) {
-		if(children[j]->getName() == childName) {
-            return true;
+
+bool XmlNode::hasChildWithAliases(vector<string> childNameList) const {
+	bool result = false;
+	for(int aliasIndex = 0; aliasIndex < (int)childNameList.size(); ++aliasIndex) {
+		const string &childName = childNameList[aliasIndex];
+		result = hasChild(childName);
+		if(result == true) {
+			break;
 		}
 	}
-	return false;
+	return result;
 }
 
-XmlNode *XmlNode::addChild(const string &name){
+XmlNode *XmlNode::addChild(const string &name, const string text) {
 	assert(!superNode);
 	XmlNode *node= new XmlNode(name);
+	node->text = text;
 	children.push_back(node);
 	return node;
 }
 
-XmlAttribute *XmlNode::addAttribute(const string &name, const string &value, std::map<string,string> mapTagReplacementValues) {
+XmlAttribute *XmlNode::addAttribute(const string &name, const string &value, const std::map<string,string> &mapTagReplacementValues) {
 	XmlAttribute *attr= new XmlAttribute(name, value, mapTagReplacementValues);
 	attributes.push_back(attr);
 	return attr;
@@ -737,7 +913,11 @@ string XmlNode::getTreeString() const {
 //	class XmlAttribute
 // =====================================================
 
-XmlAttribute::XmlAttribute(DOMNode *attribute, std::map<string,string> mapTagReplacementValues) {
+XmlAttribute::XmlAttribute(DOMNode *attribute, const std::map<string,string> &mapTagReplacementValues) {
+	if(attribute == NULL || attribute->getNodeName() == NULL) {
+        throw megaglest_runtime_error("XML attribute seems to be corrupt!");
+    }
+
 	skipRestrictionCheck 			= false;
 	usesCommondata 					= false;
 	this->mapTagReplacementValues 	= mapTagReplacementValues;
@@ -752,10 +932,16 @@ XmlAttribute::XmlAttribute(DOMNode *attribute, std::map<string,string> mapTagRep
 	name= str;
 }
 
-XmlAttribute::XmlAttribute(xml_attribute<> *attribute, std::map<string,string> mapTagReplacementValues) {
+XmlAttribute::XmlAttribute(xml_attribute<> *attribute, const std::map<string,string> &mapTagReplacementValues) {
+	if(attribute == NULL || attribute->name() == NULL) {
+        throw megaglest_runtime_error("XML attribute seems to be corrupt!");
+    }
+
 	skipRestrictionCheck 			= false;
 	usesCommondata 					= false;
-	this->mapTagReplacementValues 	= mapTagReplacementValues;
+	if(mapTagReplacementValues.size() > 0) {
+		this->mapTagReplacementValues 	= mapTagReplacementValues;
+	}
 	//char str[strSize]				= "";
 
 	//XMLString::transcode(attribute->getNodeValue(), str, strSize-1);
@@ -767,10 +953,12 @@ XmlAttribute::XmlAttribute(xml_attribute<> *attribute, std::map<string,string> m
 	name= attribute->name();
 }
 
-XmlAttribute::XmlAttribute(const string &name, const string &value, std::map<string,string> mapTagReplacementValues) {
+XmlAttribute::XmlAttribute(const string &name, const string &value, const std::map<string,string> &mapTagReplacementValues) {
 	skipRestrictionCheck 			= false;
 	usesCommondata 					= false;
-	this->mapTagReplacementValues 	= mapTagReplacementValues;
+	if(mapTagReplacementValues.size() > 0) {
+		this->mapTagReplacementValues 	= mapTagReplacementValues;
+	}
 	this->name						= name;
 	this->value						= value;
 
@@ -812,6 +1000,7 @@ float XmlAttribute::getFloatValue() const{
 
 float XmlAttribute::getFloatValue(float min, float max) const{
 	float f= strToFloat(value);
+	//printf("getFloatValue f = %.10f [%s]\n",f,value.c_str());
 	if(f<min || f>max){
 		throw megaglest_runtime_error("Xml attribute float out of range: " + getName() + ": " + value);
 	}
